@@ -28,56 +28,43 @@
  *
  */
 
-/*
- * This macro can be used in .h files to isolate bits that the manager
- * should not (need to) see, such as pthread mutexes etc.
- */
-#define VARNISH_CACHE_CHILD	1
+#ifdef VRT_H_INCLUDED
+#  error "vrt.h included before cache.h - they are exclusive"
+#endif
 
+#ifdef CACHE_H_INCLUDED
+#  error "cache.h included multiple times"
+#endif
+
+#include <math.h>
+#include <pthread.h>
 #include <stdarg.h>
+#include <sys/types.h>
 
-#include "common/common.h"
+#include "vdef.h"
+#include "vrt.h"
+
+#define CACHE_H_INCLUDED
+
+#include "miniobj.h"
+#include "vas.h"
+#include "vqueue.h"
+#include "vtree.h"
 
 #include "vapi/vsl_int.h"
-#include "vapi/vsm_int.h"
-
-#include "waiter/waiter.h"
-
-#include <sys/socket.h>
-
-#include <pthread.h>
-#ifdef HAVE_PTHREAD_NP_H
-#include <pthread_np.h>
-#endif
-#include <string.h>
-#include <limits.h>
-#include <unistd.h>
-#include <math.h>
-
-#include "common/params.h"
-
-/*--------------------------------------------------------------------*/
-
-enum req_fsm_nxt {
-	REQ_FSM_MORE,
-	REQ_FSM_DONE,
-	REQ_FSM_DISEMBARK,
-};
 
 /*--------------------------------------------------------------------*/
 
 enum body_status {
 #define BODYSTATUS(U,l)	BS_##U,
 #include "tbl/body_status.h"
-#undef BODYSTATUS
 };
 
 /*--------------------------------------------------------------------*/
 
 enum req_body_state_e {
 #define REQ_BODY(U)	REQ_BODY_##U,
-#include <tbl/req_body.h>
-#undef REQ_BODY
+#include "tbl/req_body.h"
 };
 
 /*--------------------------------------------------------------------*/
@@ -86,44 +73,35 @@ enum sess_close {
 	SC_NULL = 0,
 #define SESS_CLOSE(nm, stat, err, desc)	SC_##nm,
 #include "tbl/sess_close.h"
-#undef SESS_CLOSE
 };
 
 /*--------------------------------------------------------------------
- * Indicies into http->hd[]
+ * Indices into http->hd[]
  */
 enum {
 #define SLTH(tag, ind, req, resp, sdesc, ldesc)	ind,
 #include "tbl/vsl_tags_http.h"
-#undef SLTH
 };
 
 /*--------------------------------------------------------------------*/
 
-struct VSC_C_lck;
+struct VSC_lck;
+struct VSC_main;
+struct VSC_main_wrk;
+struct backend;
 struct ban;
 struct ban_proto;
-struct backend;
-struct busyobj;
 struct cli;
-struct cli_proto;
-struct director;
-struct iovec;
+struct http_conn;
 struct mempool;
 struct objcore;
-struct object;
 struct objhead;
 struct pool;
-struct poolparam;
-struct transport;
-struct req;
 struct sess;
-struct suckaddr;
-struct vrt_priv;
-struct vsb;
-struct waitinglist;
+struct transport;
 struct worker;
-struct v1l;
+struct listen_sock;
+struct vcf;
 
 #define DIGEST_LEN		32
 
@@ -137,18 +115,19 @@ typedef struct {
 /*--------------------------------------------------------------------*/
 
 enum req_step {
+	R_STP_NONE = 0,
 #define REQ_STEP(l, u, arg)	R_STP_##u,
 #include "tbl/steps.h"
-#undef REQ_STEP
 };
 
 enum fetch_step {
+	F_STP_NONE = 0,
 #define FETCH_STEP(l, U, arg)	F_STP_##U,
 #include "tbl/steps.h"
-#undef FETCH_STEP
 };
 
 /*--------------------------------------------------------------------*/
+
 struct lock { void *priv; };	// Opaque
 
 /*--------------------------------------------------------------------
@@ -187,60 +166,6 @@ struct http {
 	struct ws		*ws;
 	uint16_t		status;
 	uint8_t			protover;
-	uint8_t			conds;		/* If-* headers present */
-};
-
-/*--------------------------------------------------------------------
- * VFP filter state
- */
-
-VTAILQ_HEAD(vfp_entry_s, vfp_entry);
-
-struct vfp_ctx {
-	unsigned		magic;
-#define VFP_CTX_MAGIC		0x61d9d3e5
-	struct busyobj		*bo;
-	struct worker		*wrk;
-	struct objcore		*oc;
-
-	int			failed;
-
-	struct vfp_entry_s	vfp;
-	struct vfp_entry	*vfp_nxt;
-
-	struct http		*http;
-	struct http		*esi_req;
-};
-
-/*--------------------------------------------------------------------
- * HTTP Protocol connection structure
- *
- * This is the protocol independent object for a HTTP connection, used
- * both for backend and client sides.
- *
- */
-
-struct http_conn {
-	unsigned		magic;
-#define HTTP_CONN_MAGIC		0x3e19edd1
-
-	int			fd;
-	enum sess_close		doclose;
-	unsigned		maxbytes;
-	unsigned		maxhdr;
-	enum body_status	body_status;
-	struct ws		*ws;
-	char			*rxbuf_b;
-	char			*rxbuf_e;
-	char			*pipeline_b;
-	char			*pipeline_e;
-	ssize_t			content_length;
-	struct vfp_ctx		vfc[1];
-	void			*priv;
-
-	/* Timeouts */
-	double			first_byte_timeout;
-	double			between_bytes_timeout;
 };
 
 /*--------------------------------------------------------------------*/
@@ -248,7 +173,6 @@ struct http_conn {
 struct acct_req {
 #define ACCT(foo)	uint64_t	foo;
 #include "tbl/acct_fields_req.h"
-#undef ACCT
 };
 
 /*--------------------------------------------------------------------*/
@@ -256,37 +180,14 @@ struct acct_req {
 struct acct_bereq {
 #define ACCT(foo)	uint64_t	foo;
 #include "tbl/acct_fields_bereq.h"
-#undef ACCT
-};
-
-/*--------------------------------------------------------------------*/
-
-#define L0(t, n)
-#define L1(t, n)		t n;
-#define VSC_F(n,t,l,s,f,v,d,e)	L##l(t, n)
-struct dstat {
-	unsigned		summs;
-#include "tbl/vsc_f_main.h"
-};
-#undef VSC_F
-#undef L0
-#undef L1
-
-/*--------------------------------------------------------------------*/
-
-struct exp {
-	double			t_origin;
-	float			ttl;
-	float			grace;
-	float			keep;
 };
 
 /*--------------------------------------------------------------------*/
 
 struct vsl_log {
-	uint32_t		*wlb, *wlp, *wle;
-	unsigned		wlr;
-	unsigned		wid;
+	uint32_t                *wlb, *wlp, *wle;
+	unsigned                wlr;
+	unsigned                wid;
 };
 
 /*--------------------------------------------------------------------*/
@@ -298,10 +199,12 @@ struct vxid_pool {
 
 /*--------------------------------------------------------------------*/
 
+VRBT_HEAD(vrt_priv_tree,vrt_priv);
+
 struct vrt_privs {
 	unsigned		magic;
-#define VRT_PRIVS_MAGIC		0x03ba7501
-	VTAILQ_HEAD(,vrt_priv)	privs;
+#define VRT_PRIVS_MAGIC		0x03ba7502
+	struct vrt_priv_tree	privs;
 };
 
 /* Worker pool stuff -------------------------------------------------*/
@@ -314,13 +217,30 @@ struct pool_task {
 	void				*priv;
 };
 
-enum task_how {
+/*
+ * tasks are taken off the queues in this order
+ *
+ * prios up to TASK_QUEUE_RESERVE are run from the reserve
+ *
+ * TASK_QUEUE_{REQ|STR} are new req's (H1/H2), and subject to queue limit.
+ *
+ * TASK_QUEUE_RUSH is req's returning from waiting list, they are
+ * not subject to TASK_QUEUE_CLIENT because we cannot safely clean
+ * them up if scheduling them fails.
+ */
+enum task_prio {
 	TASK_QUEUE_BO,
+#define TASK_QUEUE_RESERVE	TASK_QUEUE_BO
+	TASK_QUEUE_RUSH,
 	TASK_QUEUE_REQ,
+	TASK_QUEUE_STR,
 	TASK_QUEUE_VCA,
 	TASK_QUEUE_END
 };
 
+#define TASK_QUEUE_CLIENT(prio) \
+	(prio == TASK_QUEUE_REQ || prio == TASK_QUEUE_STR)
+
 /*--------------------------------------------------------------------*/
 
 struct worker {
@@ -329,14 +249,14 @@ struct worker {
 	struct pool		*pool;
 	struct objhead		*nobjhead;
 	struct objcore		*nobjcore;
-	struct waitinglist	*nwaitinglist;
 	void			*nhashpriv;
-	struct dstat		stats[1];
+	struct VSC_main_wrk	*stats;
 	struct vsl_log		*vsl;		// borrowed from req/bo
 
 	struct pool_task	task;
 
-	double			lastused;
+	vtim_real		lastused;
+	int			strangelove;
 
 	struct v1l		*v1l;
 
@@ -351,36 +271,42 @@ struct worker {
 	unsigned		cur_method;
 	unsigned		seen_methods;
 	unsigned		handling;
-
-	uintptr_t		stack_start;
-	uintptr_t		stack_end;
-};
-
-/* LRU ---------------------------------------------------------------*/
-
-struct lru {
-	unsigned		magic;
-#define LRU_MAGIC		0x3fec7bb0
-	VTAILQ_HEAD(,objcore)	lru_head;
-	struct lock		mtx;
-	unsigned		flags;
-#define LRU_F_DONTMOVE		(1<<1)
-#define LRU_F_CONDEMMED		(1<<2)
-	unsigned		n_objcore;
 };
 
 /* Stored object -----------------------------------------------------
- * Pointer to a stored object, and the methods it supports
+ * This is just to encapsulate the fields owned by the stevedore
  */
 
 struct storeobj {
-	unsigned		magic;
-#define STOREOBJ_MAGIC		0x6faed850
 	const struct stevedore	*stevedore;
 	void			*priv;
 	uintptr_t		priv2;
 };
 
+/* Busy Objcore structure --------------------------------------------
+ *
+ */
+
+/*
+ * The macro-states we expose outside the fetch code
+ */
+enum boc_state_e {
+#define BOC_STATE(U, l)       BOS_##U,
+#include "tbl/boc_state.h"
+};
+
+struct boc {
+	unsigned		magic;
+#define BOC_MAGIC		0x70c98476
+	unsigned		refcount;
+	struct lock		mtx;
+	pthread_cond_t		cond;
+	void			*stevedore_priv;
+	enum boc_state_e	state;
+	uint8_t			*vary;
+	uint64_t		len_so_far;
+};
+
 /* Object core structure ---------------------------------------------
  * Objects have sideways references in the binary heap and the LRU list
  * and we want to avoid paging in a lot of objects just to move them up
@@ -389,37 +315,57 @@ struct storeobj {
  * housekeeping fields parts of an object.
  */
 
+enum obj_attr {
+#define OBJ_FIXATTR(U, l, s)	OA_##U,
+#define OBJ_VARATTR(U, l)	OA_##U,
+#define OBJ_AUXATTR(U, l)	OA_##U,
+#include "tbl/obj_attr.h"
+				OA__MAX,
+};
+
+enum obj_flags {
+#define OBJ_FLAG(U, l, v)       OF_##U = v,
+#include "tbl/obj_attr.h"
+};
+
+enum oc_flags {
+#define OC_FLAG(U, l, v)	OC_F_##U = v,
+#include "tbl/oc_flags.h"
+};
+
+enum oc_exp_flags {
+#define OC_EXP_FLAG(U, l, v)	OC_EF_##U = v,
+#include "tbl/oc_exp_flags.h"
+};
+
 struct objcore {
 	unsigned		magic;
 #define OBJCORE_MAGIC		0x4d301302
 	int			refcnt;
 	struct storeobj		stobj[1];
 	struct objhead		*objhead;
-	struct busyobj		*busyobj;
-	double			timer_when;
-	long			hits;
+	struct boc		*boc;
+	vtim_real		timer_when;
+	VCL_INT			hits;
 
-	struct exp		exp;
 
-	uint16_t		flags;
-#define OC_F_BUSY		(1<<1)
-#define OC_F_PASS		(1<<2)
-#define OC_F_INCOMPLETE		(1<<3)
-#define OC_F_PRIVATE		(1<<8)
-#define OC_F_FAILED		(1<<9)
+	vtim_real		t_origin;
+	float			ttl;
+	float			grace;
+	float			keep;
 
-	uint16_t		exp_flags;
-#define OC_EF_OFFLRU		(1<<4)
-#define OC_EF_MOVE		(1<<10)
-#define OC_EF_INSERT		(1<<11)
-#define OC_EF_EXP		(1<<12)
-#define OC_EF_DYING		(1<<7)
+	uint8_t			flags;
 
-	unsigned		timer_idx;
-	VTAILQ_ENTRY(objcore)	list;
+	uint8_t			exp_flags;
+
+	uint16_t		oa_present;
+
+	unsigned		timer_idx;	// XXX 4Gobj limit
+	vtim_real		last_lru;
+	VTAILQ_ENTRY(objcore)	hsh_list;
 	VTAILQ_ENTRY(objcore)	lru_list;
-	double			last_lru;
 	VTAILQ_ENTRY(objcore)	ban_list;
+	VSTAILQ_ENTRY(objcore)	exp_list;
 	struct ban		*ban;
 };
 
@@ -432,17 +378,6 @@ struct objcore {
  * streaming delivery will make use of.
  */
 
-/*
- * The macro-states we expose outside the fetch code
- */
-enum busyobj_state_e {
-	BOS_INVALID = 0,	/* don't touch (yet) */
-	BOS_REQ_DONE,		/* beresp.* can be examined */
-	BOS_STREAM,		/* beresp.* can be examined */
-	BOS_FINISHED,		/* object is complete */
-	BOS_FAILED,		/* something went wrong */
-};
-
 enum director_state_e {
 	DIR_S_NULL = 0,
 	DIR_S_HDRS = 1,
@@ -452,27 +387,23 @@ enum director_state_e {
 struct busyobj {
 	unsigned		magic;
 #define BUSYOBJ_MAGIC		0x23b95567
-	struct lock		mtx;
-	pthread_cond_t		cond;
+
 	char			*end;
 
 	/*
-	 * All fields from refcount and down are zeroed when the busyobj
+	 * All fields from retries and down are zeroed when the busyobj
 	 * is recycled.
 	 */
-	unsigned		refcount;
 	int			retries;
 	struct req		*req;
+	struct sess		*sp;
 	struct worker		*wrk;
 
-	uint8_t			*vary;
-
-	struct vfp_ctx		vfc[1];
-
-	enum busyobj_state_e	state;
+	struct vfp_ctx		*vfc;
+	const char		*filter_list;
 
 	struct ws		ws[1];
-	char			*ws_bo;
+	uintptr_t		ws_bo;
 	struct http		*bereq0;
 	struct http		*bereq;
 	struct http		*beresp;
@@ -485,21 +416,20 @@ struct busyobj {
 
 #define BO_FLAG(l, r, w, d) unsigned	l:1;
 #include "tbl/bo_flags.h"
-#undef BO_FLAG
 
 	/* Timeouts */
-	double			connect_timeout;
-	double			first_byte_timeout;
-	double			between_bytes_timeout;
+	vtim_dur		connect_timeout;
+	vtim_dur		first_byte_timeout;
+	vtim_dur		between_bytes_timeout;
 
 	/* Timers */
-	double			t_first;	/* First timestamp logged */
-	double			t_prev;		/* Previous timestamp logged */
+	vtim_real		t_first;	/* First timestamp logged */
+	vtim_real		t_prev;		/* Previous timestamp logged */
 
 	/* Acct */
 	struct acct_bereq	acct;
 
-	const char		*storage_hint;
+	const struct stevedore	*storage;
 	const struct director	*director_req;
 	const struct director	*director_resp;
 	enum director_state_e	director_state;
@@ -514,7 +444,6 @@ struct busyobj {
 
 /*--------------------------------------------------------------------*/
 
-VTAILQ_HEAD(vdp_entry_s, vdp_entry);
 
 struct req {
 	unsigned		magic;
@@ -525,11 +454,11 @@ struct req {
 	enum sess_close		doclose;
 	int			restarts;
 	int			esi_level;
-	struct req		*top;	/* esi_level == 0 request */
+	struct req		*topreq;	/* esi_level == 0 request */
+	struct vcl		*vcl0;
 
 #define REQ_FLAG(l, r, w, d) unsigned	l:1;
 #include "tbl/req_flags.h"
-#undef REQ_FLAG
 
 	uint16_t		err_code;
 	const char		*err_reason;
@@ -555,21 +484,23 @@ struct req {
 
 	uint8_t			digest[DIGEST_LEN];
 
-	double			d_ttl;
+	vtim_dur		d_ttl;
+	vtim_dur		d_grace;
 
-	ssize_t			req_bodybytes;	/* Parsed req bodybytes */
+	const struct stevedore	*storage;
 
 	const struct director	*director_hint;
 	struct vcl		*vcl;
 
-	char			*ws_req;	/* WS above request data */
+	uintptr_t		ws_req;		/* WS above request data */
 
 	/* Timestamps */
-	double			t_first;	/* First timestamp logged */
-	double			t_prev;		/* Previous timestamp logged */
-	double			t_req;		/* Headers complete */
+	vtim_real		t_first;	/* First timestamp logged */
+	vtim_real		t_prev;		/* Previous timestamp logged */
+	vtim_real		t_req;		/* Headers complete */
 
-	struct http_conn	htc[1];
+	struct http_conn	*htc;
+	struct vfp_ctx		*vfc;
 	const char		*client_identity;
 
 	/* HTTP request */
@@ -585,18 +516,12 @@ struct req {
 	struct objcore		*stale_oc;
 
 	/* Deliver pipeline */
-	struct vdp_entry_s	vdp;
-	struct vdp_entry	*vdp_nxt;
-	unsigned		vdp_errval;
+	struct vdp_ctx		*vdc;
+	const char		*filter_list;
 
 	/* Delivery mode */
 	unsigned		res_mode;
-#define RES_LEN			(1<<1)
-#define RES_EOF			(1<<2)
-#define RES_CHUNKED		(1<<3)
 #define RES_ESI			(1<<4)
-#define RES_ESI_CHILD		(1<<5)
-#define RES_GUNZIP		(1<<6)
 #define RES_PIPE		(1<<7)
 
 	/* Transaction VSL buffer */
@@ -604,8 +529,14 @@ struct req {
 
 	/* Temporary accounting */
 	struct acct_req		acct;
+
+	struct vrt_privs	privs[1];
+
+	struct vcf		*vcf;
 };
 
+#define IS_TOPREQ(req) ((req)->topreq == (req))
+
 /*--------------------------------------------------------------------
  * Struct sess is a high memory-load structure because sessions typically
  * hang around the waiter for relatively long time.
@@ -618,8 +549,8 @@ struct req {
 
 enum sess_attr {
 #define SESS_ATTR(UP, low, typ, len)	SA_##UP,
+	SA_TRANSPORT,
 #include "tbl/sess_attr.h"
-#undef SESS_ATTR
 	SA_LAST
 };
 
@@ -627,34 +558,25 @@ struct sess {
 	unsigned		magic;
 #define SESS_MAGIC		0x2c2f9c5a
 
-	enum sess_step		sess_step;
-	struct lock		mtx;
+	uint16_t		sattr[SA_LAST];
+	struct listen_sock	*listen_sock;
+	int			refcnt;
 	int			fd;
 	uint32_t		vxid;
 
-	/* Cross references ------------------------------------------*/
+	struct lock		mtx;
 
 	struct pool		*pool;
 
-	/* Session related fields ------------------------------------*/
-
 	struct ws		ws[1];
 
-	uint16_t		sattr[SA_LAST];
-
-	/* Timestamps, all on TIM_real() timescale */
-	double			t_open;		/* fd accepted */
-	double			t_idle;		/* fd accepted or resp sent */
-
-	struct vrt_privs	privs[1];
+	vtim_real		t_open;		/* fd accepted */
+	vtim_real		t_idle;		/* fd accepted or resp sent */
 
 };
 
 /* Prototypes etc ----------------------------------------------------*/
 
-/* Cross file typedefs */
-
-typedef enum htc_status_e htc_complete_f(struct http_conn *);
 
 /* cache_ban.c */
 
@@ -665,92 +587,16 @@ const char *BAN_AddTest(struct ban_proto *,
 const char *BAN_Commit(struct ban_proto *b);
 void BAN_Abandon(struct ban_proto *b);
 
-/* for stevedoes resurrecting bans */
-void BAN_Hold(void);
-void BAN_Release(void);
-void BAN_Reload(const uint8_t *ban, unsigned len);
-struct ban *BAN_RefBan(struct objcore *oc, double t0);
-double BAN_Time(const struct ban *ban);
-
-/* cache_busyobj.c */
-struct busyobj *VBO_GetBusyObj(struct worker *, const struct req *);
-void VBO_DerefBusyObj(struct worker *wrk, struct busyobj **busyobj);
-void VBO_extend(struct busyobj *, ssize_t);
-ssize_t VBO_waitlen(struct worker *, struct busyobj *, ssize_t l);
-void VBO_setstate(struct busyobj *bo, enum busyobj_state_e next);
-void VBO_waitstate(struct busyobj *bo, enum busyobj_state_e want);
-
 /* cache_cli.c [CLI] */
 extern pthread_t cli_thread;
 #define ASSERT_CLI() do {assert(pthread_self() == cli_thread);} while (0)
 
-/* cache_expire.c */
-void EXP_Clr(struct exp *e);
-
-double EXP_Ttl(const struct req *, const struct exp*);
-double EXP_When(const struct exp *exp);
-void EXP_Insert(struct worker *wrk, struct objcore *oc);
-void EXP_Inject(struct worker *wrk, struct objcore *oc, struct lru *lru);
-void EXP_Rearm(struct objcore *, double now, double ttl, double grace,
-    double keep);
-void EXP_Touch(struct objcore *oc, double now);
-int EXP_NukeOne(struct worker *wrk, struct lru *lru);
-
-enum exp_event_e {
-	EXP_INSERT,
-	EXP_INJECT,
-	EXP_REMOVE,
-};
-typedef void exp_callback_f(struct worker *, struct objcore *,
-    enum exp_event_e, void *priv);
-
-uintptr_t EXP_Register_Callback(exp_callback_f *func, void *priv);
-void EXP_Deregister_Callback(uintptr_t*);
-
-/* cache_fetch.c */
-enum vbf_fetch_mode_e {
-	VBF_NORMAL = 0,
-	VBF_PASS = 1,
-	VBF_BACKGROUND = 2,
-};
-void VBF_Fetch(struct worker *wrk, struct req *req,
-    struct objcore *oc, struct objcore *oldoc, enum vbf_fetch_mode_e);
-
-/* cache_gzip.c */
-struct vgz;
-
-enum vgzret_e {
-	VGZ_ERROR = -1,
-	VGZ_OK = 0,
-	VGZ_END = 1,
-	VGZ_STUCK = 2,
-};
-
-enum vgz_flag { VGZ_NORMAL, VGZ_ALIGN, VGZ_RESET, VGZ_FINISH };
-// struct vgz *VGZ_NewUngzip(struct vsl_log *vsl, const char *id);
-struct vgz *VGZ_NewGzip(struct vsl_log *vsl, const char *id);
-void VGZ_Ibuf(struct vgz *, const void *, ssize_t len);
-int VGZ_IbufEmpty(const struct vgz *vg);
-void VGZ_Obuf(struct vgz *, void *, ssize_t len);
-int VGZ_ObufFull(const struct vgz *vg);
-enum vgzret_e VGZ_Gzip(struct vgz *, const void **, ssize_t *len,
-    enum vgz_flag);
-// enum vgzret_e VGZ_Gunzip(struct vgz *, const void **, ssize_t *len);
-enum vgzret_e VGZ_Destroy(struct vgz **);
-
-enum vgz_ua_e {
-	VUA_UPDATE,		// Update start/stop/last bits if changed
-	VUA_END_GZIP,		// Record uncompressed size
-	VUA_END_GUNZIP,		// Record uncompressed size
-};
-
-void VGZ_UpdateObj(const struct vfp_ctx *, struct vgz*, enum vgz_ua_e);
-
 /* cache_http.c */
 unsigned HTTP_estimate(unsigned nhttp);
-void HTTP_Copy(struct http *to, const struct http * const fm);
-struct http *HTTP_create(void *p, uint16_t nhttp);
-const char *http_Status2Reason(unsigned);
+void HTTP_Clone(struct http *to, const struct http * const fm);
+void HTTP_Dup(struct http *to, const struct http * const fm);
+struct http *HTTP_create(void *p, uint16_t nhttp, unsigned);
+const char *http_Status2Reason(unsigned, const char **);
 unsigned http_EstimateWS(const struct http *fm, unsigned how);
 void http_PutResponse(struct http *to, const char *proto, uint16_t status,
     const char *response);
@@ -759,11 +605,12 @@ void HTTP_Encode(const struct http *fm, uint8_t *, unsigned len, unsigned how);
 int HTTP_Decode(struct http *to, const uint8_t *fm);
 void http_ForceHeader(struct http *to, const char *hdr, const char *val);
 void http_PrintfHeader(struct http *to, const char *fmt, ...)
-    __v_printflike(2, 3);
-void http_TimeHeader(struct http *to, const char *fmt, double now);
+    v_printflike_(2, 3);
+void http_TimeHeader(struct http *to, const char *fmt, vtim_real now);
+void http_Proto(struct http *to);
 void http_SetHeader(struct http *to, const char *hdr);
-void http_SetH(const struct http *to, unsigned n, const char *fm);
-void http_ForceField(const struct http *to, unsigned n, const char *t);
+void http_SetH(struct http *to, unsigned n, const char *fm);
+void http_ForceField(struct http *to, unsigned n, const char *t);
 void HTTP_Setup(struct http *, struct ws *, struct vsl_log *, enum VSL_tag_e);
 void http_Teardown(struct http *ht);
 int http_GetHdr(const struct http *hp, const char *hdr, const char **ptr);
@@ -782,6 +629,7 @@ void http_CopyHome(const struct http *hp);
 void http_Unset(struct http *hp, const char *hdr);
 unsigned http_CountHdr(const struct http *hp, const char *hdr);
 void http_CollectHdr(struct http *hp, const char *hdr);
+void http_CollectHdrSep(struct http *hp, const char *hdr, const char *sep);
 void http_VSL_log(const struct http *hp);
 void HTTP_Merge(struct worker *, struct objcore *, struct http *to);
 uint16_t HTTP_GetStatusPack(struct worker *, struct objcore *oc);
@@ -791,17 +639,14 @@ int HTTP_IterHdrPack(struct worker *, struct objcore *, const char **);
 const char *HTTP_GetHdrPack(struct worker *, struct objcore *, const char *hdr);
 enum sess_close http_DoConnection(struct http *hp);
 
-/* cache_http1_proto.c */
-
-htc_complete_f HTTP1_Complete;
-uint16_t HTTP1_DissectRequest(struct http_conn *, struct http *);
-uint16_t HTTP1_DissectResponse(struct http_conn *, struct http *resp,
-    const struct http *req);
-unsigned HTTP1_Write(const struct worker *w, const struct http *hp, const int*);
+#define HTTPH_R_PASS	(1 << 0)	/* Request (c->b) in pass mode */
+#define HTTPH_R_FETCH	(1 << 1)	/* Request (c->b) for fetch */
+#define HTTPH_A_INS	(1 << 2)	/* Response (b->o) for insert */
+#define HTTPH_A_PASS	(1 << 3)	/* Response (b->o) for pass */
 
 #define HTTPH(a, b, c) extern char b[];
 #include "tbl/http_headers.h"
-#undef HTTPH
+
 extern const char H__Status[];
 extern const char H__Proto[];
 extern const char H__Reason[];
@@ -809,7 +654,6 @@ extern const char H__Reason[];
 /* cache_main.c */
 #define VXID(u) ((u) & VSL_IDENTMASK)
 uint32_t VXID_Get(struct worker *, uint32_t marker);
-extern volatile struct params * cache_param;
 extern pthread_key_t witness_key;
 
 /* cache_lck.c */
@@ -818,157 +662,71 @@ extern pthread_key_t witness_key;
 void Lck__Lock(struct lock *lck, const char *p,  int l);
 void Lck__Unlock(struct lock *lck, const char *p,  int l);
 int Lck__Trylock(struct lock *lck, const char *p,  int l);
-void Lck__New(struct lock *lck, struct VSC_C_lck *, const char *);
-void Lck__Assert(const struct lock *lck, int held);
+void Lck__New(struct lock *lck, struct VSC_lck *, const char *);
+int Lck__Held(const struct lock *lck);
+int Lck__Owned(const struct lock *lck);
 
 /* public interface: */
 void Lck_Delete(struct lock *lck);
-int Lck_CondWait(pthread_cond_t *cond, struct lock *lck, double);
+int Lck_CondWait(pthread_cond_t *cond, struct lock *lck, vtim_real);
 
 #define Lck_New(a, b) Lck__New(a, b, #b)
 #define Lck_Lock(a) Lck__Lock(a, __func__, __LINE__)
 #define Lck_Unlock(a) Lck__Unlock(a, __func__, __LINE__)
 #define Lck_Trylock(a) Lck__Trylock(a, __func__, __LINE__)
-#define Lck_AssertHeld(a) Lck__Assert(a, 1)
+#define Lck_AssertHeld(a)		\
+	do {				\
+		assert(Lck__Held(a));	\
+		assert(Lck__Owned(a));	\
+	} while (0)
 
-struct VSC_C_lck *Lck_CreateClass(const char *name);
+struct VSC_lck *Lck_CreateClass(struct vsc_seg **, const char *);
+void Lck_DestroyClass(struct vsc_seg **);
 
-#define LOCK(nam) extern struct VSC_C_lck *lck_##nam;
+#define LOCK(nam) extern struct VSC_lck *lck_##nam;
 #include "tbl/locks.h"
-#undef LOCK
-
-/* cache_mempool.c */
-void MPL_AssertSane(void *item);
-struct mempool * MPL_New(const char *name, volatile struct poolparam *pp,
-    volatile unsigned *cur_size);
-void MPL_Destroy(struct mempool **mpp);
-void *MPL_Get(struct mempool *mpl, unsigned *size);
-void MPL_Free(struct mempool *mpl, void *item);
 
 /* cache_obj.c */
-enum objiter_status {
-	OIS_DONE,
-	OIS_DATA,
-	OIS_STREAM,
-	OIS_ERROR,
-};
-void *ObjIterBegin(struct worker *, struct objcore *);
-enum objiter_status ObjIter(struct objcore *, void *, void **, ssize_t *);
-void ObjIterEnd(struct objcore *, void **);
-int ObjGetSpace(struct worker *, struct objcore *, ssize_t *sz, uint8_t **ptr);
-void ObjExtend(struct worker *, struct objcore *, ssize_t l);
-void ObjTrimStore(struct worker *, struct objcore *);
-unsigned ObjGetXID(struct worker *, struct objcore *);
-uint64_t ObjGetLen(struct worker *, struct objcore *oc);
-void ObjUpdateMeta(struct worker *, struct objcore *);
-void ObjFreeObj(struct worker *, struct objcore *);
-void ObjSlim(struct worker *, struct objcore *oc);
-struct lru *ObjGetLRU(const struct objcore *);
-void *ObjGetattr(struct worker *wrk, struct objcore *oc, enum obj_attr attr,
+
+int ObjHasAttr(struct worker *, struct objcore *, enum obj_attr);
+const void *ObjGetAttr(struct worker *, struct objcore *, enum obj_attr,
     ssize_t *len);
-void *ObjSetattr(struct worker *, struct objcore *, enum obj_attr attr,
-    ssize_t len, const void *);
-int ObjCopyAttr(struct worker *, struct objcore *, struct objcore *,
-    enum obj_attr attr);
 
-int ObjSetDouble(struct worker *, struct objcore *, enum obj_attr, double);
-int ObjSetU32(struct worker *, struct objcore *, enum obj_attr, uint32_t);
-int ObjSetU64(struct worker *, struct objcore *, enum obj_attr, uint64_t);
+typedef int objiterate_f(void *priv, unsigned flush,
+    const void *ptr, ssize_t len);
+#define OBJ_ITER_FLUSH	0x01
+#define OBJ_ITER_FINAL	0x02
 
+int ObjIterate(struct worker *, struct objcore *,
+    void *priv, objiterate_f *func, int final);
+
+unsigned ObjGetXID(struct worker *, struct objcore *);
+uint64_t ObjGetLen(struct worker *, struct objcore *);
 int ObjGetDouble(struct worker *, struct objcore *, enum obj_attr, double *);
 int ObjGetU32(struct worker *, struct objcore *, enum obj_attr, uint32_t *);
 int ObjGetU64(struct worker *, struct objcore *, enum obj_attr, uint64_t *);
-
-int ObjCheckFlag(struct worker *, struct objcore *oc, enum obj_flags of);
-void ObjSetFlag(struct worker *, struct objcore *, enum obj_flags of, int val);
-
-/* cache_panic.c */
-const char *body_status_2str(enum body_status e);
-const char *sess_close_2str(enum sess_close sc, int want_desc);
-
-/* cache_pool.c */
-int Pool_Task(struct pool *pp, struct pool_task *task, enum task_how how);
-int Pool_Task_Arg(struct worker *, task_func_t *,
-    const void *arg, size_t arg_len);
-void Pool_Sumstat(struct worker *w);
-int Pool_TrySumstat(struct worker *wrk);
-void Pool_PurgeStat(unsigned nobj);
-int Pool_Task_Any(struct pool_task *task, enum task_how how);
-
-/* cache_range.c [VRG] */
-void VRG_dorange(struct req *req, const char *r);
-
-/* cache_req.c */
-struct req *Req_New(const struct worker *, struct sess *);
-void Req_Release(struct req *);
-int Req_Cleanup(struct sess *sp, struct worker *wrk, struct req *req);
-void Req_Fail(struct req *req, enum sess_close reason);
-
-/* cache_req_body.c */
-int VRB_Ignore(struct req *req);
-ssize_t VRB_Cache(struct req *req, ssize_t maxsize);
-typedef int (req_body_iter_f)(struct req *, void *priv, void *ptr, size_t);
-ssize_t VRB_Iterate(struct req *req, req_body_iter_f *func, void *priv);
-void VRB_Free(struct req *req);
-
-/* cache_req_fsm.c [CNT] */
-enum req_fsm_nxt CNT_Request(struct worker *, struct req *);
-void CNT_AcctLogCharge(struct dstat *, struct req *);
-
+int ObjCheckFlag(struct worker *, struct objcore *, enum obj_flags of);
 /* cache_session.c [SES] */
-struct sess *SES_New(struct pool *);
-void SES_Close(struct sess *sp, enum sess_close reason);
-void SES_Wait(struct sess *sp);
-void SES_Delete(struct sess *sp, enum sess_close reason, double now);
-void SES_NewPool(struct pool *pp, unsigned pool_no);
-int SES_Reschedule_Req(struct req *);
-task_func_t SES_Proto_Sess;
-task_func_t SES_Proto_Req;
 
-enum htc_status_e {
-	HTC_S_JUNK =		-5,
-	HTC_S_CLOSE =		-4,
-	HTC_S_TIMEOUT =		-3,
-	HTC_S_OVERFLOW =	-2,
-	HTC_S_EOF =		-1,
-	HTC_S_EMPTY =		 0,
-	HTC_S_MORE =		 1,
-	HTC_S_COMPLETE =	 2,
-	HTC_S_IDLE =		 3,
-};
-
-void SES_RxInit(struct http_conn *htc, struct ws *ws,
-    unsigned maxbytes, unsigned maxhdr);
-void SES_RxReInit(struct http_conn *htc);
-enum htc_status_e SES_RxStuff(struct http_conn *, htc_complete_f *,
-    double *t1, double *t2, double ti, double tn);
-
-#define SESS_ATTR(UP, low, typ, len)				\
-	int SES_Get_##low(const struct sess *sp, typ *dst);	\
-	void SES_Reserve_##low(struct sess *sp, typ *dst);
+#define SESS_ATTR(UP, low, typ, len)					\
+	int SES_Get_##low(const struct sess *sp, typ **dst);
 #include "tbl/sess_attr.h"
-#undef SESS_ATTR
-void SES_Set_String_Attr(struct sess *sp, enum sess_attr a, const char *src);
 const char *SES_Get_String_Attr(const struct sess *sp, enum sess_attr a);
 
 /* cache_shmlog.c */
-extern struct VSC_C_main *VSC_C_main;
-void *VSM_Alloc(unsigned size, const char *class, const char *type,
-    const char *ident);
-void VSM_Free(void *ptr);
-#ifdef VSL_ENDMARKER
 void VSLv(enum VSL_tag_e tag, uint32_t vxid, const char *fmt, va_list va);
 void VSL(enum VSL_tag_e tag, uint32_t vxid, const char *fmt, ...)
-    __v_printflike(3, 4);
+    v_printflike_(3, 4);
 void VSLbv(struct vsl_log *, enum VSL_tag_e tag, const char *fmt, va_list va);
 void VSLb(struct vsl_log *, enum VSL_tag_e tag, const char *fmt, ...)
-    __v_printflike(3, 4);
+    v_printflike_(3, 4);
 void VSLbt(struct vsl_log *, enum VSL_tag_e tag, txt t);
-void VSLb_ts(struct vsl_log *, const char *event, double first, double *pprev,
-    double now);
+void VSLb_ts(struct vsl_log *, const char *event, vtim_real first,
+    vtim_real *pprev, vtim_real now);
+void VSLb_bin(struct vsl_log *, enum VSL_tag_e, ssize_t, const void*);
 
 static inline void
-VSLb_ts_req(struct req *req, const char *event, double now)
+VSLb_ts_req(struct req *req, const char *event, vtim_real now)
 {
 
 	if (isnan(req->t_first) || req->t_first == 0.)
@@ -977,7 +735,7 @@ VSLb_ts_req(struct req *req, const char *event, double now)
 }
 
 static inline void
-VSLb_ts_busyobj(struct busyobj *bo, const char *event, double now)
+VSLb_ts_busyobj(struct busyobj *bo, const char *event, vtim_real now)
 {
 
 	if (isnan(bo->t_first) || bo->t_first == 0.)
@@ -985,38 +743,14 @@ VSLb_ts_busyobj(struct busyobj *bo, const char *event, double now)
 	VSLb_ts(bo->vsl, event, bo->t_first, &bo->t_prev, now);
 }
 
-
-void VSL_Flush(struct vsl_log *, int overflow);
-
-#endif
-
-/* cache_vary.c */
-int VRY_Create(struct busyobj *bo, struct vsb **psb);
-int VRY_Match(struct req *, const uint8_t *vary);
-void VRY_Prep(struct req *);
-void VRY_Clear(struct req *);
-enum vry_finish_flag { KEEP, DISCARD };
-void VRY_Finish(struct req *req, enum vry_finish_flag);
-
 /* cache_vcl.c */
-const char *VCL_Method_Name(unsigned);
 const char *VCL_Name(const struct vcl *);
-void VCL_Ref(struct vcl *);
-void VCL_Refresh(struct vcl **);
-void VCL_Rel(struct vcl **);
-const char *VCL_Return_Name(unsigned);
-
-#define VCL_MET_MAC(l,u,t,b) \
-    void VCL_##l##_method(struct vcl *, struct worker *, struct req *, \
-	struct busyobj *bo, void *specific);
-#include "tbl/vcl_returns.h"
-#undef VCL_MET_MAC
 
 /* cache_vrt.c */
-
 /*
  * These prototypes go here, because we do not want to pollute vrt.h
  * with va_list.  VCC never generates direct calls to them.
+ * XXX: We should deprecate these (ref: STRANDS)
  */
 const char *VRT_String(struct ws *ws, const char *h, const char *p, va_list ap);
 char *VRT_StringList(char *d, unsigned dl, const char *p, va_list ap);
@@ -1031,36 +765,34 @@ void WRK_BgThread(pthread_t *thr, const char *name, bgthread_t *func,
 
 void WS_Init(struct ws *ws, const char *id, void *space, unsigned len);
 unsigned WS_Reserve(struct ws *ws, unsigned bytes);
+unsigned WS_ReserveLumps(struct ws *ws, size_t sz);
 void WS_MarkOverflow(struct ws *ws);
 void WS_Release(struct ws *ws, unsigned bytes);
-void WS_ReleaseP(struct ws *ws, char *ptr);
+void WS_ReleaseP(struct ws *ws, const char *ptr);
 void WS_Assert(const struct ws *ws);
-void WS_Reset(struct ws *ws, char *p);
+void WS_Reset(struct ws *ws, uintptr_t);
 void *WS_Alloc(struct ws *ws, unsigned bytes);
 void *WS_Copy(struct ws *ws, const void *str, int len);
-char *WS_Snapshot(struct ws *ws);
+uintptr_t WS_Snapshot(struct ws *ws);
 int WS_Overflowed(const struct ws *ws);
-void *WS_Printf(struct ws *ws, const char *fmt, ...) __v_printflike(2, 3);
+void *WS_Printf(struct ws *ws, const char *fmt, ...) v_printflike_(2, 3);
+int WS_Inside(const struct ws *, const void *, const void *);
+void WS_Assert_Allocated(const struct ws *ws, const void *ptr, ssize_t len);
+
+static inline char*
+WS_Front(const struct ws *ws)
+{
+	return ws->f;
+}
 
 /* cache_rfc2616.c */
-void RFC2616_Ttl(struct busyobj *, double now);
+void RFC2616_Ttl(struct busyobj *, vtim_real now, vtim_real *t_origin,
+    float *ttl, float *grace, float *keep);
 unsigned RFC2616_Req_Gzip(const struct http *);
 int RFC2616_Do_Cond(const struct req *sp);
 void RFC2616_Weaken_Etag(struct http *hp);
 void RFC2616_Vary_AE(struct http *hp);
-
-/* stevedore.c */
-int STV_NewObject(struct objcore *, struct worker *,
-    const char *hint, unsigned len);
-struct storage *STV_alloc(const struct stevedore *, size_t size, int flags);
-#define LESS_MEM_ALLOCED_IS_OK	1
-void STV_trim(const struct stevedore *, struct storage *, size_t size,
-    int move_ok);
-void STV_free(const struct stevedore *, struct storage *st);
-void STV_open(void);
-void STV_close(void);
-int STV_BanInfo(enum baninfo event, const uint8_t *ban, unsigned len);
-void STV_BanExport(const uint8_t *bans, unsigned len);
+void RFC2616_Response_Body(const struct worker *, const struct busyobj *);
 
 /*
  * A normal pointer difference is signed, but we never want a negative value
@@ -1076,14 +808,11 @@ pdiff(const void *b, const void *e)
 	    ((unsigned)((const unsigned char *)e - (const unsigned char *)b));
 }
 
-static inline void
-Tcheck(const txt t)
-{
-
-	AN(t.b);
-	AN(t.e);
-	assert(t.b <= t.e);
-}
+#define Tcheck(t) do {						\
+		AN((t).b);					\
+		AN((t).e);					\
+		assert((t).b <= (t).e);				\
+	} while(0)
 
 /*
  * unsigned length of a txt
@@ -1103,35 +832,10 @@ Tlen(const txt t)
  */
 #define W_TIM_real(w) ((w)->lastused = VTIM_real())
 
-static inline int
-FEATURE(enum feature_bits x)
-{
-	return (cache_param->feature_bits[(unsigned)x>>3] &
-	    (0x80U >> ((unsigned)x & 7)));
-}
-
-static inline int
-DO_DEBUG(enum debug_bits x)
-{
-	return (cache_param->debug_bits[(unsigned)x>>3] &
-	    (0x80U >> ((unsigned)x & 7)));
-}
-
-#define DSL(debug_bit, id, ...)					\
-	do {							\
-		if (DO_DEBUG(debug_bit))			\
-			VSL(SLT_Debug, (id), __VA_ARGS__);	\
-	} while (0)
-
 #define PAN_CheckMagic(vsb, ptr, exp)					\
 	do {								\
 		if ((ptr)->magic != (exp))				\
 			VSB_printf((vsb),				\
-			    "MAGIC 0x%08x (Should:%s/0x%08x)\n",	\
-			    (ptr)->magic, #exp, exp);			\
+			    "MAGIC at %p is 0x%08x (Should be: %s/0x%08x)\n", \
+			    ptr, (ptr)->magic, #exp, exp);		\
 	} while(0)
-
-#ifdef VARNISHD_IS_NOT_A_VMOD
-#  include "cache/cache_priv.h"
-#endif
-
