@@ -33,12 +33,10 @@
 
 #include "config.h"
 
-#include <errno.h>
 #include <stdlib.h>
 #include <netinet/in.h>
 #include <netinet/tcp.h>
 #include <sys/socket.h>
-#include <sys/un.h>
 
 #include "cache_varnishd.h"
 
@@ -370,12 +368,7 @@ vca_make_session(struct worker *wrk, void *arg)
 	CHECK_OBJ_NOTNULL(wrk, WORKER_MAGIC);
 	CAST_OBJ_NOTNULL(wa, arg, WRK_ACCEPT_MAGIC);
 
-	if (VTCP_blocking(wa->acceptsock)) {
-		closefd(&wa->acceptsock);
-		wrk->stats->sess_drop++;	// XXX Better counter ?
-		WS_Release(wrk->aws, 0);
-		return;
-	}
+	VTCP_blocking(wa->acceptsock);
 
 	/* Turn accepted socket into a session */
 	AN(wrk->aws->r);
@@ -387,7 +380,7 @@ vca_make_session(struct worker *wrk, void *arg)
 		 * to send an intelligent message back.
 		 */
 		vca_pace_bad();
-		(void)VTCP_nonblocking(wa->acceptsock);
+		VTCP_nonblocking(wa->acceptsock);
 		closefd(&wa->acceptsock);
 		wrk->stats->sess_drop++;
 		WS_Release(wrk->aws, 0);
@@ -462,6 +455,10 @@ vca_accept_task(struct worker *wrk, void *arg)
 	while (!pool_accepting)
 		VTIM_sleep(.1);
 
+	/* Dont hold on to (possibly) discarded VCLs */
+	if (wrk->vcl != NULL)
+		VCL_Rel(&wrk->vcl);
+
 	while (!ps->pool->die) {
 		INIT_OBJ(&wa, WRK_ACCEPT_MAGIC);
 		wa.acceptlsock = ls;
@@ -521,12 +518,12 @@ vca_accept_task(struct worker *wrk, void *arg)
 				strcpy(lport, "0");
 			} else {
 				VTCP_myname(ls->sock, laddr, VTCP_ADDRBUFSIZE,
-				    lport, VTCP_ADDRBUFSIZE);
+				    lport, VTCP_PORTBUFSIZE);
 			}
 
 			VSL(SLT_SessError, 0, "%s %s %s %d %d %s",
 			    wa.acceptlsock->name, laddr, lport,
-			    ls->sock, i, strerror(i));
+			    ls->sock, i, vstrerror(i));
 			(void)Pool_TrySumstat(wrk);
 			continue;
 		}
@@ -549,12 +546,6 @@ vca_accept_task(struct worker *wrk, void *arg)
 		if (!ps->pool->die && DO_DEBUG(DBG_SLOW_ACCEPTOR))
 			VTIM_sleep(2.0);
 
-		/*
-		 * We were able to hand off, so release this threads VCL
-		 * reference (if any) so we don't hold on to discarded VCLs.
-		 */
-		if (wrk->vcl != NULL)
-			VCL_Rel(&wrk->vcl);
 	}
 }
 
@@ -650,12 +641,12 @@ ccf_start(struct cli *cli, const char * const *av, void *priv)
 			if (i)
 				VSL(SLT_Error, 0,
 				    "Kernel TCP Fast Open: sock=%d, ret=%d %s",
-				    ls->sock, i, strerror(errno));
+				    ls->sock, i, vstrerror(errno));
 		}
 		if (listen(ls->sock, cache_param->listen_depth)) {
 			VCLI_SetResult(cli, CLIS_CANT);
 			VCLI_Out(cli, "Listen failed on socket '%s': %s",
-			    ls->endpoint, strerror(errno));
+			    ls->endpoint, vstrerror(errno));
 			return;
 		}
 		vca_tcp_opt_set(ls->sock, ls->uds, 1);
@@ -665,7 +656,7 @@ ccf_start(struct cli *cli, const char * const *av, void *priv)
 			if (i)
 				VSL(SLT_Error, 0,
 				    "Kernel filtering: sock=%d, ret=%d %s",
-				    ls->sock, i, strerror(errno));
+				    ls->sock, i, vstrerror(errno));
 		}
 	}
 
