@@ -39,7 +39,6 @@
 
 #include <poll.h>
 #include <stdio.h>
-#include <errno.h>
 #include <string.h>
 #include <stdlib.h>
 
@@ -153,9 +152,8 @@ vbp_has_poked(struct vbp_target *vt)
 void
 VBP_Update_Backend(struct vbp_target *vt)
 {
-	unsigned i = 0;
+	unsigned i = 0, chg;
 	char bits[10];
-	const char *logmsg;
 
 	CHECK_OBJ_NOTNULL(vt, VBP_TARGET_MAGIC);
 
@@ -176,27 +174,21 @@ VBP_Update_Backend(struct vbp_target *vt)
 		return;
 	}
 
-	if (vt->good >= vt->threshold) {
-		if (vt->backend->director->sick) {
-			logmsg = "Back healthy";
-			VRT_SetHealth(vt->backend->director, 1);
-		} else {
-			logmsg = "Still healthy";
-		}
-	} else {
-		if (vt->backend->director->sick) {
-			logmsg = "Still sick";
-		} else {
-			logmsg = "Went sick";
-			VRT_SetHealth(vt->backend->director, 0);
-		}
-	}
-	VSL(SLT_Backend_health, 0, "%s %s %s %u %u %u %.6f %.6f %s",
-	    vt->backend->director->vcl_name, logmsg, bits,
+	i = (vt->good < vt->threshold);
+	chg = (i != vt->backend->sick);
+	vt->backend->sick = i;
+
+	VSL(SLT_Backend_health, 0, "%s %s %s %s %u %u %u %.6f %.6f %s",
+	    vt->backend->director->vcl_name, chg ? "Went" : "Still",
+	    i ? "sick" : "healthy", bits,
 	    vt->good, vt->threshold, vt->window,
 	    vt->last, vt->avg, vt->resp_buf);
 	VBE_SetHappy(vt->backend, vt->happy);
 
+	if (chg) {
+		vt->backend->changed = VTIM_real();
+		VRT_SetChanged(vt->backend->director, vt->backend->changed);
+	}
 	Lck_Unlock(&vbp_mtx);
 }
 
@@ -236,11 +228,11 @@ vbp_write(struct vbp_target *vt, int *sock, const void *buf, size_t len)
 		if (i < 0) {
 			vt->err_xmit |= 1;
 			bprintf(vt->resp_buf, "Write error %d (%s)",
-				errno, strerror(errno));
+				errno, vstrerror(errno));
 		} else {
 			bprintf(vt->resp_buf,
 				"Short write (%d/%zu) error %d (%s)",
-				i, len, errno, strerror(errno));
+				i, len, errno, vstrerror(errno));
 		}
 		VTCP_close(sock);
 		return (-1);
@@ -289,7 +281,7 @@ vbp_poke(struct vbp_target *vt)
 
 	s = VTP_Open(vt->tcp_pool, t_end - t_now, (const void **)&sa, &err);
 	if (s < 0) {
-		bprintf(vt->resp_buf, "Open error %d (%s)", err, strerror(err));
+		bprintf(vt->resp_buf, "Open error %d (%s)", err, vstrerror(err));
 		Lck_Lock(&vbp_mtx);
 		if (vt->backend)
 			VBE_Connect_Error(vt->backend->vsc, err);
@@ -342,6 +334,7 @@ vbp_poke(struct vbp_target *vt)
 	/* Send the request */
 	if (vbp_write(vt, &s, vt->req, vt->req_len) != 0)
 		return;
+
 	vt->good_xmit |= 1;
 
 	pfd->fd = s;
@@ -350,21 +343,21 @@ vbp_poke(struct vbp_target *vt)
 		pfd->events = POLLIN;
 		pfd->revents = 0;
 		tmo = (int)round((t_end - t_now) * 1e3);
-		if (tmo > 0)
-			i = poll(pfd, 1, tmo);
-		if (i == 0) {
-			vt->err_recv |= 1;
-			bprintf(vt->resp_buf, "Poll error %d (%s)",
-				errno, strerror(errno));
-			VTCP_close(&s);
-			return;
-		}
 		if (tmo <= 0) {
 			bprintf(vt->resp_buf,
-				"Poll (read) timeout %.3fs exceeded by %.3fs",
-				vt->timeout, t_now - t_end);
-			VTCP_close(&s);
-			return;
+			    "Poll timeout %.3fs exceeded by %.3fs",
+			    vt->timeout, t_now - t_end);
+			i = -1;
+			break;
+		}
+		i = poll(pfd, 1, tmo);
+		if (i <= 0) {
+			if (!i)
+				errno = ETIMEDOUT;
+			bprintf(vt->resp_buf, "Poll error %d (%s)",
+			    errno, vstrerror(errno));
+			i = -1;
+			break;
 		}
 		if (rlen < sizeof vt->resp_buf)
 			i = read(s, vt->resp_buf + rlen,
@@ -374,7 +367,7 @@ vbp_poke(struct vbp_target *vt)
 		if (i <= 0) {
 			if (i < 0)
 				bprintf(vt->resp_buf, "Read error %d (%s)",
-					errno, strerror(errno));
+					errno, vstrerror(errno));
 			break;
 		}
 		rlen += i;
@@ -515,19 +508,19 @@ void
 VBP_Status(struct vsb *vsb, const struct backend *be, int details, int json)
 {
 	struct vbp_target *vt;
-	char buf[12];
 
 	CHECK_OBJ_NOTNULL(be, BACKEND_MAGIC);
 	vt = be->probe;
 	CHECK_OBJ_NOTNULL(vt, VBP_TARGET_MAGIC);
 
 	if (!details) {
-		bprintf(buf, "%u/%u %s", vt->good, vt->window,
-		    vt->backend->director->sick ? "bad" : "good");
 		if (json)
-			VSB_printf(vsb, "\"%s\"", buf);
+			VSB_printf(vsb, "[%u, %u, \"%s\"]",
+			    vt->good, vt->window,
+			    vt->backend->sick ? "sick" : "healthy");
 		else
-			VSB_printf(vsb, "%-10s", buf);
+			VSB_printf(vsb, "%u/%u\t%s", vt->good, vt->window,
+			    vt->backend->sick ? "sick" : "healthy");
 		return;
 	}
 
@@ -547,7 +540,7 @@ VBP_Status(struct vsb *vsb, const struct backend *be, int details, int json)
 	}
 
 	VSB_printf(vsb,
-	    "\nCurrent states  good: %2u threshold: %2u window: %2u\n",
+	    "\n Current states  good: %2u threshold: %2u window: %2u\n",
 	    vt->good, vt->threshold, vt->window);
 	VSB_printf(vsb,
 	    "  Average response time of good probes: %.6f\n", vt->avg);
@@ -690,7 +683,7 @@ VBP_Remove(struct backend *be)
 	CHECK_OBJ_NOTNULL(vt, VBP_TARGET_MAGIC);
 
 	Lck_Lock(&vbp_mtx);
-	VRT_SetHealth(be->director, 1);
+	be->sick = 1;
 	be->probe = NULL;
 	vt->backend = NULL;
 	if (vt->running) {
