@@ -31,6 +31,7 @@
 #include "config.h"
 
 #include <pcre.h>
+#include <stdio.h>
 
 #include "cache_varnishd.h"
 #include "cache_ban.h"
@@ -40,6 +41,10 @@
 #include "vend.h"
 #include "vmb.h"
 
+/* cache_ban_build.c */
+void BAN_Build_Init(void);
+void BAN_Build_Fini(void);
+
 struct lock ban_mtx;
 int ban_shutdown;
 struct banhead_s ban_head = VTAILQ_HEAD_INITIALIZER(ban_head);
@@ -55,9 +60,18 @@ struct ban_test {
 	uint8_t			arg1;
 	const char		*arg1_spec;
 	const char		*arg2;
+	double			arg2_double;
 	const void		*arg2_spec;
 };
 
+static const char * const arg_name[BAN_ARGARRSZ + 1] = {
+#define PVAR(a, b, c) [BAN_ARGIDX(c)] = (a),
+#include "tbl/ban_vars.h"
+	[BAN_ARGARRSZ] = NULL
+};
+
+extern const char * const oper[BAN_OPERARRSZ + 1];
+
 /*--------------------------------------------------------------------
  * Storage handling of bans
  */
@@ -118,10 +132,10 @@ BAN_Release(void)
  * Extract time and length from ban-spec
  */
 
-double
+vtim_real
 ban_time(const uint8_t *banspec)
 {
-	double t;
+	vtim_real t;
 
 	assert(sizeof t == (BANS_LENGTH - BANS_TIMESTAMP));
 	memcpy(&t, banspec, sizeof t);
@@ -148,11 +162,14 @@ ban_equal(const uint8_t *bs1, const uint8_t *bs2)
 	u = vbe32dec(bs1 + BANS_LENGTH);
 	if (u != vbe32dec(bs2 + BANS_LENGTH))
 		return (0);
+	if (bs1[BANS_FLAGS] & BANS_FLAG_NODEDUP)
+		return (0);
+
 	return (!memcmp(bs1 + BANS_LENGTH, bs2 + BANS_LENGTH, u - BANS_LENGTH));
 }
 
 void
-ban_mark_completed(struct ban *b, struct VSC_main *stats)
+ban_mark_completed(struct ban *b)
 {
 	unsigned ln;
 
@@ -166,12 +183,8 @@ ban_mark_completed(struct ban *b, struct VSC_main *stats)
 		b->spec[BANS_FLAGS] |= BANS_FLAG_COMPLETED;
 		VWMB();
 		vbe32enc(b->spec + BANS_LENGTH, BANS_HEAD_LEN);
-		stats->bans_completed++;
+		VSC_C_main->bans_completed++;
 		bans_persisted_fragmentation += ln - ban_len(b->spec);
-		/*
-		 * XXX absolute update of gauges - may be inaccurate for
-		 * Pool_Sumstat race
-		 */
 		VSC_C_main->bans_persisted_fragmentation =
 		    bans_persisted_fragmentation;
 	}
@@ -204,16 +217,25 @@ ban_get_lump(const uint8_t **bs)
 static void
 ban_iter(const uint8_t **bs, struct ban_test *bt)
 {
+	uint64_t dtmp;
 
 	memset(bt, 0, sizeof *bt);
+	bt->arg2_double = nan("");
 	bt->arg1 = *(*bs)++;
-	if (bt->arg1 == BANS_ARG_REQHTTP || bt->arg1 == BANS_ARG_OBJHTTP) {
+	if (BANS_HAS_ARG1_SPEC(bt->arg1)) {
 		bt->arg1_spec = (const char *)*bs;
 		(*bs) += (*bs)[0] + 2;
 	}
+	if (BANS_HAS_ARG2_DOUBLE(bt->arg1)) {
+		dtmp = vbe64dec(ban_get_lump(bs));
+		bt->oper = *(*bs)++;
+
+		memcpy(&bt->arg2_double, &dtmp, sizeof dtmp);
+		return;
+	}
 	bt->arg2 = ban_get_lump(bs);
 	bt->oper = *(*bs)++;
-	if (bt->oper == BANS_OPER_MATCH || bt->oper == BANS_OPER_NMATCH)
+	if (BANS_HAS_ARG2_SPEC(bt->oper))
 		bt->arg2_spec = ban_get_lump(bs);
 }
 
@@ -261,10 +283,10 @@ BAN_DestroyObj(struct objcore *oc)
  */
 
 struct ban *
-BAN_FindBan(double t0)
+BAN_FindBan(vtim_real t0)
 {
 	struct ban *b;
-	double t1;
+	vtim_real t1;
 
 	assert(ban_holds > 0);
 	VTAILQ_FOREACH(b, &ban_head, list) {
@@ -319,10 +341,6 @@ ban_export(void)
 	assert(VSB_len(vsb) == ln);
 	STV_BanExport((const uint8_t *)VSB_data(vsb), VSB_len(vsb));
 	VSB_destroy(&vsb);
-	/*
-	 * XXX absolute update of gauges - may be inaccurate for Pool_Sumstat
-	 * race
-	 */
 	VSC_C_main->bans_persisted_bytes =
 	    bans_persisted_bytes = ln;
 	VSC_C_main->bans_persisted_fragmentation =
@@ -366,8 +384,7 @@ ban_reload(const uint8_t *ban, unsigned len)
 {
 	struct ban *b, *b2;
 	int duplicate = 0;
-	double t0, t1, t2 = 9e99;
-	struct VSC_main *stats = VSC_C_main;	// XXX accurate?
+	vtim_real t0, t1, t2 = 9e99;
 	ASSERT_CLI();
 	Lck_AssertHeld(&ban_mtx);
 
@@ -386,8 +403,8 @@ ban_reload(const uint8_t *ban, unsigned len)
 			duplicate = 1;
 	}
 
-	stats->bans++;
-	stats->bans_added++;
+	VSC_C_main->bans++;
+	VSC_C_main->bans_added++;
 
 	b2 = ban_alloc();
 	AN(b2);
@@ -395,22 +412,18 @@ ban_reload(const uint8_t *ban, unsigned len)
 	AN(b2->spec);
 	memcpy(b2->spec, ban, len);
 	if (ban[BANS_FLAGS] & BANS_FLAG_REQ) {
-		stats->bans_req++;
+		VSC_C_main->bans_req++;
 		b2->flags |= BANS_FLAG_REQ;
 	}
 	if (duplicate)
-		stats->bans_dups++;
+		VSC_C_main->bans_dups++;
 	if (duplicate || (ban[BANS_FLAGS] & BANS_FLAG_COMPLETED))
-		ban_mark_completed(b2, stats);
+		ban_mark_completed(b2);
 	if (b == NULL)
 		VTAILQ_INSERT_TAIL(&ban_head, b2, list);
 	else
 		VTAILQ_INSERT_BEFORE(b, b2, list);
 	bans_persisted_bytes += len;
-	/*
-	 * XXX absolute update of gauges - may be inaccurate for Pool_Sumstat
-	 * race
-	 */
 	VSC_C_main->bans_persisted_bytes = bans_persisted_bytes;
 
 	/* Hunt down older duplicates */
@@ -418,8 +431,8 @@ ban_reload(const uint8_t *ban, unsigned len)
 		if (b->flags & BANS_FLAG_COMPLETED)
 			continue;
 		if (ban_equal(b->spec, ban)) {
-			ban_mark_completed(b, stats);
-			stats->bans_dups++;
+			ban_mark_completed(b);
+			VSC_C_main->bans_dups++;
 		}
 	}
 }
@@ -453,7 +466,7 @@ BAN_Reload(const uint8_t *ptr, unsigned len)
  * Get a bans timestamp
  */
 
-double
+vtim_real
 BAN_Time(const struct ban *b)
 {
 
@@ -469,20 +482,32 @@ BAN_Time(const struct ban *b)
  */
 
 int
-ban_evaluate(struct worker *wrk, const uint8_t *bs, struct objcore *oc,
+ban_evaluate(struct worker *wrk, const uint8_t *bsarg, struct objcore *oc,
     const struct http *reqhttp, unsigned *tests)
 {
 	struct ban_test bt;
-	const uint8_t *be;
+	const uint8_t *bs, *be;
 	const char *p;
 	const char *arg1;
+	double darg1, darg2;
 
+	/*
+	 * for ttl and age, fix the point in time such that banning refers to
+	 * the same point in time when the ban is evaluated
+	 *
+	 * for grace/keep, we assume that the absolute values are pola and that
+	 * users will most likely also specify a ttl criterion if they want to
+	 * fix a point in time (such as "obj.ttl > 5h && obj.keep > 3h")
+	 */
+
+	bs = bsarg;
 	be = bs + ban_len(bs);
 	bs += BANS_HEAD_LEN;
 	while (bs < be) {
 		(*tests)++;
 		ban_iter(&bs, &bt);
 		arg1 = NULL;
+		darg1 = darg2 = nan("");
 		switch (bt.arg1) {
 		case BANS_ARG_URL:
 			AN(reqhttp);
@@ -499,18 +524,42 @@ ban_evaluate(struct worker *wrk, const uint8_t *bs, struct objcore *oc,
 		case BANS_ARG_OBJSTATUS:
 			arg1 = HTTP_GetHdrPack(wrk, oc, H__Status);
 			break;
+		case BANS_ARG_OBJTTL:
+			darg1 = oc->ttl + oc->t_origin;
+			darg2 = bt.arg2_double + ban_time(bsarg);
+			break;
+		case BANS_ARG_OBJAGE:
+			darg1 = 0.0 - oc->t_origin;
+			darg2 = 0.0 - (ban_time(bsarg) - bt.arg2_double);
+			break;
+		case BANS_ARG_OBJGRACE:
+			darg1 = oc->grace;
+			darg2 = bt.arg2_double;
+			break;
+		case BANS_ARG_OBJKEEP:
+			darg1 = oc->keep;
+			darg2 = bt.arg2_double;
+			break;
 		default:
 			WRONG("Wrong BAN_ARG code");
 		}
 
 		switch (bt.oper) {
 		case BANS_OPER_EQ:
-			if (arg1 == NULL || strcmp(arg1, bt.arg2))
+			if (arg1 == NULL) {
+				if (isnan(darg1) || darg1 != darg2)
+					return (0);
+			} else if (strcmp(arg1, bt.arg2)) {
 				return (0);
+			}
 			break;
 		case BANS_OPER_NEQ:
-			if (arg1 != NULL && !strcmp(arg1, bt.arg2))
+			if (arg1 == NULL) {
+				if (! isnan(darg1) && darg1 == darg2)
+					return (0);
+			} else if (!strcmp(arg1, bt.arg2)) {
 				return (0);
+			}
 			break;
 		case BANS_OPER_MATCH:
 			if (arg1 == NULL ||
@@ -524,6 +573,30 @@ ban_evaluate(struct worker *wrk, const uint8_t *bs, struct objcore *oc,
 			    0, 0, NULL, 0) >= 0)
 				return (0);
 			break;
+		case BANS_OPER_GT:
+			AZ(arg1);
+			assert(! isnan(darg1));
+			if (!(darg1 > darg2))
+				return (0);
+			break;
+		case BANS_OPER_GTE:
+			AZ(arg1);
+			assert(! isnan(darg1));
+			if (!(darg1 >= darg2))
+				return (0);
+			break;
+		case BANS_OPER_LT:
+			AZ(arg1);
+			assert(! isnan(darg1));
+			if (!(darg1 < darg2))
+				return (0);
+			break;
+		case BANS_OPER_LTE:
+			AZ(arg1);
+			assert(! isnan(darg1));
+			if (!(darg1 <= darg2))
+				return (0);
+			break;
 		default:
 			WRONG("Wrong BAN_OPER code");
 		}
@@ -548,7 +621,6 @@ BAN_CheckObject(struct worker *wrk, struct objcore *oc, struct req *req)
 	struct vsl_log *vsl;
 	struct ban *b0, *bn;
 	unsigned tests;
-	struct VSC_main *stats;
 
 	CHECK_OBJ_NOTNULL(wrk, WORKER_MAGIC);
 	CHECK_OBJ_NOTNULL(oc, OBJCORE_MAGIC);
@@ -557,7 +629,6 @@ BAN_CheckObject(struct worker *wrk, struct objcore *oc, struct req *req)
 	assert(oc->refcnt > 0);
 
 	vsl = req->vsl;
-	stats = wrk->stats;
 
 	CHECK_OBJ_NOTNULL(oc->ban, BAN_MAGIC);
 
@@ -600,8 +671,8 @@ BAN_CheckObject(struct worker *wrk, struct objcore *oc, struct req *req)
 
 	Lck_Lock(&ban_mtx);
 	bn->refcount--;
-	stats->bans_tested++;
-	stats->bans_tests_tested += tests;
+	VSC_C_main->bans_tested++;
+	VSC_C_main->bans_tests_tested += tests;
 
 	if (b == bn) {
 		/* not banned */
@@ -612,6 +683,8 @@ BAN_CheckObject(struct worker *wrk, struct objcore *oc, struct req *req)
 		oc->ban = b0;
 		b = NULL;
 	}
+	if (b != NULL)
+		VSC_C_main->bans_obj_killed++;
 
 	if (VTAILQ_LAST(&ban_head, banhead_s)->refcount == 0)
 		ban_kick_lurker();
@@ -624,7 +697,6 @@ BAN_CheckObject(struct worker *wrk, struct objcore *oc, struct req *req)
 		return (0);
 	} else {
 		VSLb(vsl, SLT_ExpBan, "%u banned lookup", ObjGetXID(wrk, oc));
-		stats->bans_obj_killed++;
 		return (1);
 	}
 }
@@ -672,7 +744,7 @@ ccf_ban(struct cli *cli, const char * const *av, void *priv)
 
 	if (err == NULL) {
 		// XXX racy - grab wstat lock?
-		err = BAN_Commit(bp, VSC_C_main);
+		err = BAN_Commit(bp);
 	}
 
 	if (err != NULL) {
@@ -682,63 +754,78 @@ ccf_ban(struct cli *cli, const char * const *av, void *priv)
 	}
 }
 
+#define Ms 60
+#define Hs (Ms * 60)
+#define Ds (Hs * 24)
+#define Ws (Ds * 7)
+#define Ys (Ds * 365)
+
+#define Xfmt(buf, var, s, unit)						\
+	((var) >= s && (var) % s == 0)					\
+		bprintf((buf), "%ju" unit, (var) / s)
+
+// XXX move to VTIM?
+#define vdur_render(buf, dur) do {					\
+	uintmax_t dec = (uintmax_t)floor(dur);				\
+	uintmax_t frac = (uintmax_t)floor((dur) * 1e3) % UINTMAX_C(1000); \
+	if (dec == 0 && frac == 0)					\
+		(void) strncpy(buf, "0s", sizeof(buf));			\
+	else if (dec == 0)						\
+		bprintf((buf), "%jums", frac);				\
+	else if (frac != 0)						\
+		bprintf((buf), "%ju.%03jus", dec, frac);		\
+	else if Xfmt(buf, dec, Ys, "y");				\
+	else if Xfmt(buf, dec, Ws, "w");				\
+	else if Xfmt(buf, dec, Ds, "d");				\
+	else if Xfmt(buf, dec, Hs, "h");				\
+	else if Xfmt(buf, dec, Ms, "m");				\
+	else								\
+		bprintf((buf), "%jus", dec);				\
+	} while (0)
+
 static void
-ban_render(struct cli *cli, const uint8_t *bs)
+ban_render(struct cli *cli, const uint8_t *bs, int quote)
 {
 	struct ban_test bt;
 	const uint8_t *be;
+	char buf[64];
 
 	be = bs + ban_len(bs);
 	bs += BANS_HEAD_LEN;
 	while (bs < be) {
 		ban_iter(&bs, &bt);
-		switch (bt.arg1) {
-		case BANS_ARG_URL:
-			VCLI_Out(cli, "req.url");
-			break;
-		case BANS_ARG_REQHTTP:
-			VCLI_Out(cli, "req.http.%.*s",
+		ASSERT_BAN_ARG(bt.arg1);
+		ASSERT_BAN_OPER(bt.oper);
+
+		if (BANS_HAS_ARG1_SPEC(bt.arg1))
+			VCLI_Out(cli, "%s.%.*s",
+			    arg_name[BAN_ARGIDX(bt.arg1)],
 			    bt.arg1_spec[0] - 1, bt.arg1_spec + 1);
-			break;
-		case BANS_ARG_OBJHTTP:
-			VCLI_Out(cli, "obj.http.%.*s",
-			    bt.arg1_spec[0] - 1, bt.arg1_spec + 1);
-			break;
-		case BANS_ARG_OBJSTATUS:
-			VCLI_Out(cli, "obj.status");
-			break;
-		default:
-			WRONG("Wrong BANS_ARG");
+		else
+			VCLI_Out(cli, "%s", arg_name[BAN_ARGIDX(bt.arg1)]);
+
+		VCLI_Out(cli, " %s ", oper[BAN_OPERIDX(bt.oper)]);
+
+		if (BANS_HAS_ARG2_DOUBLE(bt.arg1)) {
+			vdur_render(buf, bt.arg2_double);
+			VCLI_Out(cli, "%s", buf);
+		} else if (quote) {
+			VCLI_Quote(cli, bt.arg2);
+		} else {
+			VCLI_Out(cli, "%s", bt.arg2);
 		}
-		switch (bt.oper) {
-		case BANS_OPER_EQ:	VCLI_Out(cli, " == "); break;
-		case BANS_OPER_NEQ:	VCLI_Out(cli, " != "); break;
-		case BANS_OPER_MATCH:	VCLI_Out(cli, " ~ "); break;
-		case BANS_OPER_NMATCH:	VCLI_Out(cli, " !~ "); break;
-		default:
-			WRONG("Wrong BANS_OPER");
-		}
-		VCLI_Out(cli, "%s", bt.arg2);
+
 		if (bs < be)
 			VCLI_Out(cli, " && ");
 	}
 }
 
-static void v_matchproto_(cli_func_t)
-ccf_ban_list(struct cli *cli, const char * const *av, void *priv)
+static void
+ban_list(struct cli *cli, struct ban *bl)
 {
-	struct ban *b, *bl;
+	struct ban *b;
 	int64_t o;
 
-	(void)av;
-	(void)priv;
-
-	/* Get a reference so we are safe to traverse the list */
-	Lck_Lock(&ban_mtx);
-	bl = VTAILQ_LAST(&ban_head, banhead_s);
-	bl->refcount++;
-	Lck_Unlock(&ban_mtx);
-
 	VCLI_Out(cli, "Present bans:\n");
 	VTAILQ_FOREACH(b, &ban_head, list) {
 		o = bl == b ? 1 : 0;
@@ -752,7 +839,7 @@ ccf_ban_list(struct cli *cli, const char * const *av, void *priv)
 			    b);
 		}
 		VCLI_Out(cli, "  ");
-		ban_render(cli, b->spec);
+		ban_render(cli, b->spec, 0);
 		VCLI_Out(cli, "\n");
 		if (VCLI_Overflow(cli))
 			break;
@@ -764,6 +851,80 @@ ccf_ban_list(struct cli *cli, const char * const *av, void *priv)
 			Lck_Unlock(&ban_mtx);
 		}
 	}
+}
+
+static void
+ban_list_json(struct cli *cli, const char * const *av, struct ban *bl)
+{
+	struct ban *b;
+	int64_t o;
+	int n = 0;
+	int ocs;
+
+	VCLI_JSON_begin(cli, 2, av);
+	VCLI_Out(cli, ",\n");
+	VTAILQ_FOREACH(b, &ban_head, list) {
+		o = bl == b ? 1 : 0;
+		VCLI_Out(cli, "%s", n ? ",\n" : "");
+		n++;
+		VCLI_Out(cli, "{\n");
+		VSB_indent(cli->sb, 2);
+		VCLI_Out(cli, "\"time\": %.6f,\n", ban_time(b->spec));
+		VCLI_Out(cli, "\"refs\": %ju,\n", (intmax_t)(b->refcount - o));
+		VCLI_Out(cli, "\"completed\": %s,\n",
+			 b->flags & BANS_FLAG_COMPLETED ? "true" : "false");
+		VCLI_Out(cli, "\"spec\": \"");
+		ban_render(cli, b->spec, 1);
+		VCLI_Out(cli, "\"");
+
+		if (DO_DEBUG(DBG_LURKER)) {
+			VCLI_Out(cli, ",\n");
+			VCLI_Out(cli, "\"req_tests\": %s,\n",
+				 b->flags & BANS_FLAG_REQ ? "true" : "false");
+			VCLI_Out(cli, "\"obj_tests\": %s,\n",
+				 b->flags & BANS_FLAG_OBJ ? "true" : "false");
+			VCLI_Out(cli, "\"pointer\": \"%p\",\n", b);
+			if (VCLI_Overflow(cli))
+				break;
+
+			ocs = 0;
+			VCLI_Out(cli, "\"objcores\": [\n");
+			VSB_indent(cli->sb, 2);
+			Lck_Lock(&ban_mtx);
+			struct objcore *oc;
+			VTAILQ_FOREACH(oc, &b->objcore, ban_list) {
+				if (ocs)
+					VCLI_Out(cli, ",\n");
+				VCLI_Out(cli, "%p", oc);
+				ocs++;
+			}
+			Lck_Unlock(&ban_mtx);
+			VSB_indent(cli->sb, -2);
+			VCLI_Out(cli, "\n]");
+		}
+		VSB_indent(cli->sb, -2);
+		VCLI_Out(cli, "\n}");
+	}
+	VCLI_JSON_end(cli);
+}
+
+static void v_matchproto_(cli_func_t)
+ccf_ban_list(struct cli *cli, const char * const *av, void *priv)
+{
+	struct ban *bl;
+
+	(void)priv;
+
+	/* Get a reference so we are safe to traverse the list */
+	Lck_Lock(&ban_mtx);
+	bl = VTAILQ_LAST(&ban_head, banhead_s);
+	bl->refcount++;
+	Lck_Unlock(&ban_mtx);
+
+	if (av[2] != NULL && strcmp(av[2], "-j") == 0)
+		ban_list_json(cli, av, bl);
+	else
+		ban_list(cli, bl);
 
 	Lck_Lock(&ban_mtx);
 	bl->refcount--;
@@ -773,7 +934,8 @@ ccf_ban_list(struct cli *cli, const char * const *av, void *priv)
 
 static struct cli_proto ban_cmds[] = {
 	{ CLICMD_BAN,				"", ccf_ban },
-	{ CLICMD_BAN_LIST,			"", ccf_ban_list },
+	{ CLICMD_BAN_LIST,			"", ccf_ban_list,
+	  ccf_ban_list },
 	{ NULL }
 };
 
@@ -812,6 +974,7 @@ BAN_Init(void)
 {
 	struct ban_proto *bp;
 
+	BAN_Build_Init();
 	Lck_New(&ban_mtx, lck_ban);
 	CLI_AddFuncs(ban_cmds);
 
@@ -821,9 +984,9 @@ BAN_Init(void)
 	bp = BAN_Build();
 	AN(bp);
 	AZ(pthread_cond_init(&ban_lurker_cond, NULL));
-	AZ(BAN_Commit(bp, VSC_C_main));
+	AZ(BAN_Commit(bp));
 	Lck_Lock(&ban_mtx);
-	ban_mark_completed(VTAILQ_FIRST(&ban_head), VSC_C_main);
+	ban_mark_completed(VTAILQ_FIRST(&ban_head));
 	Lck_Unlock(&ban_mtx);
 }
 
@@ -852,4 +1015,6 @@ BAN_Shutdown(void)
 	/* Export the ban list to compact it */
 	ban_export();
 	Lck_Unlock(&ban_mtx);
+
+	BAN_Build_Fini();
 }
