@@ -73,16 +73,53 @@
 #define BANS_FLAG_OBJ		(1<<1)
 #define BANS_FLAG_COMPLETED	(1<<2)
 #define BANS_FLAG_HTTP		(1<<3)
+#define BANS_FLAG_DURATION	(1<<4)
+#define BANS_FLAG_NODEDUP	(1<<5)
 
 #define BANS_OPER_EQ		0x10
+#define _BANS_OPER_OFF		BANS_OPER_EQ
 #define BANS_OPER_NEQ		0x11
 #define BANS_OPER_MATCH		0x12
 #define BANS_OPER_NMATCH	0x13
+#define BANS_OPER_GT		0x14
+#define BANS_OPER_GTE		0x15
+#define BANS_OPER_LT		0x16
+#define BANS_OPER_LTE		0x17
+#define _BANS_OPER_LIM		(BANS_OPER_LTE + 1)
+
+#define BAN_OPERIDX(x) ((x) - _BANS_OPER_OFF)
+#define BAN_OPERARRSZ  (_BANS_OPER_LIM - _BANS_OPER_OFF)
+#define ASSERT_BAN_OPER(x) assert((x) >= _BANS_OPER_OFF && (x) < _BANS_OPER_LIM)
 
 #define BANS_ARG_URL		0x18
+#define _BANS_ARG_OFF		BANS_ARG_URL
 #define BANS_ARG_REQHTTP	0x19
 #define BANS_ARG_OBJHTTP	0x1a
 #define BANS_ARG_OBJSTATUS	0x1b
+#define BANS_ARG_OBJTTL	0x1c
+#define BANS_ARG_OBJAGE	0x1d
+#define BANS_ARG_OBJGRACE	0x1e
+#define BANS_ARG_OBJKEEP	0x1f
+#define _BANS_ARG_LIM		(BANS_ARG_OBJKEEP + 1)
+
+#define BAN_ARGIDX(x) ((x) - _BANS_ARG_OFF)
+#define BAN_ARGARRSZ  (_BANS_ARG_LIM - _BANS_ARG_OFF)
+#define ASSERT_BAN_ARG(x) assert((x) >= _BANS_ARG_OFF && (x) < _BANS_ARG_LIM)
+
+// has an arg1_spec (BANS_FLAG_HTTP at build time)
+#define BANS_HAS_ARG1_SPEC(arg)	\
+	((arg) == BANS_ARG_REQHTTP ||	\
+	 (arg) == BANS_ARG_OBJHTTP)
+
+// has an arg2_spec (regex)
+#define BANS_HAS_ARG2_SPEC(oper)	\
+	((oper) == BANS_OPER_MATCH ||	\
+	 (oper) == BANS_OPER_NMATCH)
+
+// has an arg2_double (BANS_FLAG_DURATION at build time)
+#define BANS_HAS_ARG2_DOUBLE(arg)	\
+	((arg) >= BANS_ARG_OBJTTL &&	\
+	 (arg) <= BANS_ARG_OBJKEEP)
 
 /*--------------------------------------------------------------------*/
 
@@ -109,14 +146,14 @@ extern pthread_cond_t	ban_lurker_cond;
 extern uint64_t bans_persisted_bytes;
 extern uint64_t bans_persisted_fragmentation;
 
-void ban_mark_completed(struct ban *b, struct VSC_main *stats);
+void ban_mark_completed(struct ban *);
 unsigned ban_len(const uint8_t *banspec);
 void ban_info_new(const uint8_t *ban, unsigned len);
 void ban_info_drop(const uint8_t *ban, unsigned len);
 
 int ban_evaluate(struct worker *wrk, const uint8_t *bs, struct objcore *oc,
     const struct http *reqhttp, unsigned *tests);
-double ban_time(const uint8_t *banspec);
+vtim_real ban_time(const uint8_t *banspec);
 int ban_equal(const uint8_t *bs1, const uint8_t *bs2);
 void BAN_Free(struct ban *b);
 void ban_kick_lurker(void);
