@@ -146,7 +146,7 @@ VBO_ReleaseBusyObj(struct worker *wrk, struct busyobj **pbo)
 {
 	struct busyobj *bo;
 
-	CHECK_OBJ_ORNULL(wrk, WORKER_MAGIC);
+	CHECK_OBJ_NOTNULL(wrk, WORKER_MAGIC);
 	TAKE_OBJ_NOTNULL(bo, pbo, BUSYOBJ_MAGIC);
 	CHECK_OBJ_ORNULL(bo->fetch_objcore, OBJCORE_MAGIC);
 
@@ -167,8 +167,10 @@ VBO_ReleaseBusyObj(struct worker *wrk, struct busyobj **pbo)
 
 	AZ(bo->htc);
 
+	if (WS_Overflowed(bo->ws))
+		wrk->stats->ws_backend_overflow++;
+
 	if (bo->fetch_objcore != NULL) {
-		AN(wrk);
 		(void)HSH_DerefObjCore(wrk, &bo->fetch_objcore,
 		    HSH_RUSH_POLICY);
 	}
