@@ -66,7 +66,7 @@ VDP_bytes(struct req *req, enum vdp_action act, const void *ptr, ssize_t len)
 
 	assert(act > VDP_NULL || len > 0);
 	/* Call the present layer, while pointing to the next layer down */
-	retval = vdpe->vdp->func(req, act, &vdpe->priv, ptr, len);
+	retval = vdpe->vdp->bytes(req, act, &vdpe->priv, ptr, len);
 	if (retval && (vdc->retval == 0 || retval < vdc->retval))
 		vdc->retval = retval; /* Latch error value */
 	vdc->nxt = vdpe;
@@ -74,7 +74,7 @@ VDP_bytes(struct req *req, enum vdp_action act, const void *ptr, ssize_t len)
 }
 
 int
-VDP_push(struct req *req, const struct vdp *vdp, void *priv, int bottom)
+VDP_Push(struct req *req, const struct vdp *vdp, void *priv)
 {
 	struct vdp_entry *vdpe;
 	struct vdp_ctx *vdc;
@@ -83,7 +83,7 @@ VDP_push(struct req *req, const struct vdp *vdp, void *priv, int bottom)
 	vdc = req->vdc;
 	AN(vdp);
 	AN(vdp->name);
-	AN(vdp->func);
+	AN(vdp->bytes);
 
 	if (vdc->retval)
 		return (vdc->retval);
@@ -100,14 +100,17 @@ VDP_push(struct req *req, const struct vdp *vdp, void *priv, int bottom)
 	INIT_OBJ(vdpe, VDP_ENTRY_MAGIC);
 	vdpe->vdp = vdp;
 	vdpe->priv = priv;
-	if (bottom)
-		VTAILQ_INSERT_TAIL(&vdc->vdp, vdpe, list);
-	else
-		VTAILQ_INSERT_HEAD(&vdc->vdp, vdpe, list);
+	VTAILQ_INSERT_TAIL(&vdc->vdp, vdpe, list);
 	vdc->nxt = VTAILQ_FIRST(&vdc->vdp);
 
 	AZ(vdc->retval);
-	vdc->retval = vdpe->vdp->func(req, VDP_INIT, &vdpe->priv, NULL, 0);
+	if (vdpe->vdp->init != NULL)
+		vdc->retval = vdpe->vdp->init(req, &vdpe->priv);
+	if (vdc->retval > 0) {
+		VTAILQ_REMOVE(&vdc->vdp, vdpe, list);
+		vdc->nxt = VTAILQ_FIRST(&vdc->vdp);
+		vdc->retval = 0;
+	}
 	return (vdc->retval);
 }
 
@@ -124,11 +127,11 @@ VDP_close(struct req *req)
 		if (vdc->retval >= 0)
 			AN(vdpe);
 		if (vdpe != NULL) {
-			CHECK_OBJ_NOTNULL(vdpe, VDP_ENTRY_MAGIC);
-			VTAILQ_REMOVE(&vdc->vdp, vdpe, list);
-			AZ(vdpe->vdp->func(req, VDP_FINI, &vdpe->priv,
-			    NULL, 0));
+			CHECK_OBJ(vdpe, VDP_ENTRY_MAGIC);
+			if (vdpe->vdp->fini != NULL)
+				AZ(vdpe->vdp->fini(req, &vdpe->priv));
 			AZ(vdpe->priv);
+			VTAILQ_REMOVE(&vdc->vdp, vdpe, list);
 		}
 		vdc->nxt = VTAILQ_FIRST(&vdc->vdp);
 	}
@@ -137,8 +140,9 @@ VDP_close(struct req *req)
 /*--------------------------------------------------------------------*/
 
 static int v_matchproto_(objiterate_f)
-vdp_objiterator(void *priv, int flush, const void *ptr, ssize_t len)
+vdp_objiterator(void *priv, unsigned flush, const void *ptr, ssize_t len)
 {
+
 	return (VDP_bytes(priv, flush ? VDP_FLUSH : VDP_NULL, ptr, len));
 }
 
@@ -146,11 +150,12 @@ vdp_objiterator(void *priv, int flush, const void *ptr, ssize_t len)
 int
 VDP_DeliverObj(struct req *req)
 {
-	int r;
+	int r, final;
 
 	CHECK_OBJ_NOTNULL(req, REQ_MAGIC);
-	r = ObjIterate(req->wrk, req->objcore, req, vdp_objiterator,
-	    req->objcore->flags & OC_F_PRIVATE ? 1 : 0);
+	final = req->objcore->flags & (OC_F_PRIVATE | OC_F_HFM | OC_F_HFP)
+	    ? 1 : 0;
+	r = ObjIterate(req->wrk, req->objcore, req, vdp_objiterator, final);
 	if (r < 0)
 		return (r);
 	return (0);
