@@ -60,7 +60,6 @@ vdi_str2ahealth(const char *t)
 #define VBE_AHEALTH(l,u,h) if (!strcasecmp(t, #l)) return (VDI_AH_##u);
 VBE_AHEALTH_LIST
 #undef VBE_AHEALTH
-	if (!strcasecmp(t, "auto")) return (VDI_AH_PROBE);
 	return (NULL);
 }
 
@@ -70,7 +69,11 @@ VDI_Ahealth(const struct director *d)
 
 	CHECK_OBJ_NOTNULL(d, DIRECTOR_MAGIC);
 	AN(d->vdir->admin_health);
-	return (d->vdir->admin_health->name);
+	if (d->vdir->admin_health != VDI_AH_AUTO)
+		return (d->vdir->admin_health->name);
+	if (d->vdir->methods->healthy != NULL)
+		return ("probe");
+	return ("healthy");
 }
 
 /* Resolve director --------------------------------------------------*/
@@ -95,11 +98,17 @@ VDI_Resolve(VRT_CTX)
 			VSLb(bo->vsl, SLT_FetchError,
 			    "Director %s returned no backend", d->vcl_name);
 	}
+
 	CHECK_OBJ_ORNULL(d, DIRECTOR_MAGIC);
-	if (d == NULL)
+	if (d == NULL) {
 		VSLb(bo->vsl, SLT_FetchError, "No backend");
-	else
+	} else {
 		AN(d->vdir);
+
+		if (d->vdir->admin_health->health == 0)
+			d = NULL;
+	}
+
 	return (d);
 }
 
@@ -223,12 +232,29 @@ VRT_Healthy(VRT_CTX, VCL_BACKEND d, VCL_TIME *changed)
 	if (d->vdir->methods->healthy == NULL) {
 		if (changed != NULL)
 			*changed = d->vdir->health_changed;
-		return (!d->sick);
+		return (1);
 	}
 
 	return (d->vdir->methods->healthy(ctx, d, changed));
 }
 
+/*--------------------------------------------------------------------
+ * Update health_changed.
+ */
+VCL_VOID
+VRT_SetChanged(VCL_BACKEND d, VCL_TIME changed)
+{
+	if (d == NULL)
+		return;
+
+	CHECK_OBJ_NOTNULL(d, DIRECTOR_MAGIC);
+
+	if (changed <= d->vdir->health_changed)
+		return;
+
+	d->vdir->health_changed = changed;
+}
+
 /* Send Event ----------------------------------------------------------
  */
 
@@ -252,7 +278,6 @@ VDI_Panic(const struct director *d, struct vsb *vsb, const char *nm)
 	VSB_printf(vsb, "%s = %p {\n", nm, d);
 	VSB_indent(vsb, 2);
 	VSB_printf(vsb, "cli_name = %s,\n", d->vdir->cli_name);
-	VSB_printf(vsb, "health = %s,\n", d->sick ?  "sick" : "healthy");
 	VSB_printf(vsb, "admin_health = %s, changed = %f,\n",
 	    VDI_Ahealth(d), d->vdir->health_changed);
 	VSB_printf(vsb, "type = %s {\n", d->vdir->methods->type);
@@ -272,35 +297,54 @@ struct list_args {
 	unsigned	magic;
 #define LIST_ARGS_MAGIC	0x7e7cefeb
 	int		p;
-	int		v;
 	int		j;
 	const char	*jsep;
+	struct vsb	*vsb;
 };
 
+static const char *
+cli_health(VRT_CTX, const struct director *d)
+{
+	VCL_BOOL healthy = VRT_Healthy(ctx, d, NULL);
+
+	return (healthy ? "healthy" : "sick");
+}
+
 static int v_matchproto_(vcl_be_func)
 do_list(struct cli *cli, struct director *d, void *priv)
 {
 	char time_str[VTIM_FORMAT_SIZE];
 	struct list_args *la;
+	struct vrt_ctx *ctx;
 
+	AN(cli);
 	CAST_OBJ_NOTNULL(la, priv, LIST_ARGS_MAGIC);
+	AN(la->vsb);
 	CHECK_OBJ_NOTNULL(d, DIRECTOR_MAGIC);
 
 	if (d->vdir->admin_health == VDI_AH_DELETED)
 		return (0);
 
-	// XXX admin health "probe" for the no-probe case is confusing
-	VCLI_Out(cli, "\n%-30s %-7s ", d->vdir->cli_name, VDI_Ahealth(d));
+	ctx = VCL_Get_CliCtx(0);
+
+	VSB_printf(la->vsb, "%s\t%s\t", d->vdir->cli_name, VDI_Ahealth(d));
 
 	if (d->vdir->methods->list != NULL)
-		d->vdir->methods->list(d, cli->sb, 0, 0, 0);
+		d->vdir->methods->list(ctx, d, la->vsb, 0, 0);
+	else if (d->vdir->methods->healthy != NULL)
+		VSB_printf(la->vsb, "0/0\t%s", cli_health(ctx, d));
 	else
-		VCLI_Out(cli, "%-10s", d->sick ? "sick" : "healthy");
+		VSB_cat(la->vsb, "0/0\thealthy");
 
 	VTIM_format(d->vdir->health_changed, time_str);
-	VCLI_Out(cli, " %s", time_str);
-	if ((la->p || la->v) && d->vdir->methods->list != NULL)
-		d->vdir->methods->list(d, cli->sb, la->p, la->v, 0);
+	VSB_printf(la->vsb, "\t%s", time_str);
+	if (la->p && d->vdir->methods->list != NULL)
+		d->vdir->methods->list(ctx, d, la->vsb, la->p, 0);
+
+	VSB_printf(la->vsb, "\n");
+	VCL_Rel_CliCtx(&ctx);
+	AZ(ctx);
+
 	return (0);
 }
 
@@ -308,6 +352,7 @@ static int v_matchproto_(vcl_be_func)
 do_list_json(struct cli *cli, struct director *d, void *priv)
 {
 	struct list_args *la;
+	struct vrt_ctx *ctx;
 
 	CAST_OBJ_NOTNULL(la, priv, LIST_ARGS_MAGIC);
 	CHECK_OBJ_NOTNULL(d, DIRECTOR_MAGIC);
@@ -315,6 +360,8 @@ do_list_json(struct cli *cli, struct director *d, void *priv)
 	if (d->vdir->admin_health == VDI_AH_DELETED)
 		return (0);
 
+	ctx = VCL_Get_CliCtx(0);
+
 	VCLI_Out(cli, "%s", la->jsep);
 	la->jsep = ",\n";
 	// XXX admin health "probe" for the no-probe case is confusing
@@ -325,18 +372,25 @@ do_list_json(struct cli *cli, struct director *d, void *priv)
 	VCLI_Out(cli, "\"admin_health\": \"%s\",\n", VDI_Ahealth(d));
 	VCLI_Out(cli, "\"probe_message\": ");
 	if (d->vdir->methods->list != NULL)
-		d->vdir->methods->list(d, cli->sb, 0, 0, 1);
+		d->vdir->methods->list(ctx, d, cli->sb, 0, 1);
+	else if (d->vdir->methods->healthy != NULL)
+		VCLI_Out(cli, "[0, 0, \"%s\"]", cli_health(ctx, d));
 	else
-		VCLI_Out(cli, "\"%s\"", d->sick ? "sick" : "healthy");
+		VCLI_Out(cli, "[0, 0, \"healthy\"]");
+
 	VCLI_Out(cli, ",\n");
 
-	if ((la->p || la->v) && d->vdir->methods->list != NULL) {
+	if (la->p && d->vdir->methods->list != NULL) {
 		VCLI_Out(cli, "\"probe_details\": ");
-		d->vdir->methods->list(d, cli->sb, la->p, la->v, 1);
+		d->vdir->methods->list(ctx, d, cli->sb, la->p, 1);
 	}
 	VCLI_Out(cli, "\"last_change\": %.3f\n", d->vdir->health_changed);
 	VSB_indent(cli->sb, -2);
 	VCLI_Out(cli, "}");
+
+	VCL_Rel_CliCtx(&ctx);
+	AZ(ctx);
+
 	return (0);
 }
 
@@ -356,7 +410,6 @@ cli_backend_list(struct cli *cli, const char * const *av, void *priv)
 			switch(*p) {
 			case 'j': la->j = 1; break;
 			case 'p': la->p = !la->p; break;
-			case 'v': la->p = !la->p; break;
 			default:
 				VCLI_Out(cli, "Invalid flag %c", *p);
 				VCLI_SetResult(cli, CLIS_PARAM);
@@ -370,7 +423,7 @@ cli_backend_list(struct cli *cli, const char * const *av, void *priv)
 		return;
 	}
 	if (la->j) {
-		VCLI_JSON_begin(cli, 2, av);
+		VCLI_JSON_begin(cli, 3, av);
 		VCLI_Out(cli, ",\n");
 		VCLI_Out(cli, "{\n");
 		VSB_indent(cli->sb, 2);
@@ -380,9 +433,12 @@ cli_backend_list(struct cli *cli, const char * const *av, void *priv)
 		VCLI_Out(cli, "}");
 		VCLI_JSON_end(cli);
 	} else {
-		VCLI_Out(cli, "%-30s %-7s %-10s %s",
-		    "Backend name", "Admin", "Probe", "Last change");
+		la->vsb = VSB_new_auto();
+		AN(la->vsb);
+		VSB_printf(la->vsb, "%s\t%s\t%s\t%s\t%s\n",
+		    "Backend name", "Admin", "Probe", "Health", "Last change");
 		(void)VCL_IterDirector(cli, av[i], do_list, la);
+		VCLI_VTE(cli, &la->vsb, 80);
 	}
 }
 
@@ -407,8 +463,6 @@ do_set_health(struct cli *cli, struct director *d, void *priv)
 	if (d->vdir->admin_health != sh->ah) {
 		d->vdir->health_changed = VTIM_real();
 		d->vdir->admin_health = sh->ah;
-		d->sick &= ~0x02;
-		d->sick |= sh->ah->health ? 0 : 0x02;
 	}
 	return (0);
 }
