@@ -64,15 +64,25 @@ struct ecx {
 	int		woken;
 
 	struct req	*preq;
+	struct ecx	*pecx;
 	ssize_t		l_crc;
 	uint32_t	crc;
 };
 
+static int v_matchproto_(vtr_minimal_response_f)
+ved_minimal_response(struct req *req, uint16_t status)
+{
+	(void)req;
+	(void)status;
+	WRONG("esi:includes should not try minimal responses");
+}
+
 static const struct transport VED_transport = {
-	.magic =	TRANSPORT_MAGIC,
-	.name =		"ESI_INCLUDE",
-	.deliver =	ved_deliver,
-	.reembark =	ved_reembark,
+	.magic =		TRANSPORT_MAGIC,
+	.name =			"ESI_INCLUDE",
+	.deliver =		ved_deliver,
+	.reembark =		ved_reembark,
+	.minimal_response =	ved_minimal_response,
 };
 
 /*--------------------------------------------------------------------*/
@@ -103,86 +113,81 @@ ved_include(struct req *preq, const char *src, const char *host,
 	enum req_fsm_nxt s;
 
 	CHECK_OBJ_NOTNULL(preq, REQ_MAGIC);
+	CHECK_OBJ_NOTNULL(preq->topreq, REQ_MAGIC);
 	sp = preq->sp;
 	CHECK_OBJ_NOTNULL(sp, SESS_MAGIC);
 	CHECK_OBJ_NOTNULL(ecx, ECX_MAGIC);
 	wrk = preq->wrk;
 
-	if (preq->esi_level >= cache_param->max_esi_depth)
+	if (preq->esi_level >= cache_param->max_esi_depth) {
+		VSLb(preq->vsl, SLT_VCL_Error,
+		    "ESI depth limit reach (param max_esi_depth = %u)",
+		    cache_param->max_esi_depth);
 		return;
-
-	req = Req_New(wrk, sp);
-	SES_Ref(sp);
-	req->req_body_status = REQ_BODY_NONE;
-	AZ(req->vsl->wid);
-	req->vsl->wid = VXID_Get(wrk, VSL_CLIENTMARKER);
-	VSLb(req->vsl, SLT_Begin, "req %u esi", VXID(preq->vsl->wid));
-	VSLb(preq->vsl, SLT_Link, "req %u esi", VXID(req->vsl->wid));
-	req->esi_level = preq->esi_level + 1;
-
-	if (preq->esi_level == 0)
-		assert(preq->top == preq);
-	else
-		CHECK_OBJ_NOTNULL(preq->top, REQ_MAGIC);
-
-	req->top = preq->top;
-
-	HTTP_Copy(req->http0, preq->http0);
-
-	req->http0->ws = req->ws;
-	req->http0->vsl = req->vsl;
-	req->http0->logtag = SLT_ReqMethod;
-	req->http0->conds = 0;
-
-	http_SetH(req->http0, HTTP_HDR_URL, src);
-	if (host != NULL && *host != '\0')  {
-		http_Unset(req->http0, H_Host);
-		http_SetHeader(req->http0, host);
 	}
 
-	http_ForceField(req->http0, HTTP_HDR_METHOD, "GET");
-	http_ForceField(req->http0, HTTP_HDR_PROTO, "HTTP/1.1");
+	req = Req_New(wrk, sp);
+	AN(req);
+	THR_SetRequest(req);
+	AZ(req->vsl->wid);
+	req->vsl->wid = VXID_Get(wrk, VSL_CLIENTMARKER);
+
+	VSLb(req->vsl, SLT_Begin, "req %u esi", VXID(preq->vsl->wid));
+	VSLb(preq->vsl, SLT_Link, "req %u esi", VXID(req->vsl->wid));
+
+	VSLb_ts_req(req, "Start", W_TIM_real(wrk));
+
+	req->esi_level = preq->esi_level + 1;
+
+	req->topreq = preq->topreq;
+
+	HTTP_Setup(req->http, req->ws, req->vsl, SLT_ReqMethod);
+	HTTP_Dup(req->http, preq->http0);
+
+	http_SetH(req->http, HTTP_HDR_URL, src);
+	if (host != NULL && *host != '\0')  {
+		http_Unset(req->http, H_Host);
+		http_SetHeader(req->http, host);
+	}
+
+	http_ForceField(req->http, HTTP_HDR_METHOD, "GET");
+	http_ForceField(req->http, HTTP_HDR_PROTO, "HTTP/1.1");
 
 	/* Don't allow conditionals, we can't use a 304 */
-	http_Unset(req->http0, H_If_Modified_Since);
-	http_Unset(req->http0, H_If_None_Match);
+	http_Unset(req->http, H_If_Modified_Since);
+	http_Unset(req->http, H_If_None_Match);
 
 	/* Don't allow Range */
-	http_Unset(req->http0, H_Range);
+	http_Unset(req->http, H_Range);
 
 	/* Set Accept-Encoding according to what we want */
-	http_Unset(req->http0, H_Accept_Encoding);
+	http_Unset(req->http, H_Accept_Encoding);
 	if (ecx->isgzip)
-		http_ForceHeader(req->http0, H_Accept_Encoding, "gzip");
+		http_ForceHeader(req->http, H_Accept_Encoding, "gzip");
 
 	/* Client content already taken care of */
-	http_Unset(req->http0, H_Content_Length);
-
-	/* Reset request to status before we started messing with it */
-	HTTP_Copy(req->http, req->http0);
+	http_Unset(req->http, H_Content_Length);
+	req->req_body_status = REQ_BODY_NONE;
 
 	AZ(req->vcl);
-	req->vcl = preq->vcl;
-	preq->vcl = NULL;
+	if (req->topreq->vcl0)
+		req->vcl = req->topreq->vcl0;
+	else
+		req->vcl = preq->vcl;
+	VCL_Ref(req->vcl);
 
-	req->req_step = R_STP_RECV;
+	req->req_step = R_STP_TRANSPORT;
 	req->t_req = preq->t_req;
-	assert(isnan(req->t_first));
-	assert(isnan(req->t_prev));
 
 	req->transport = &VED_transport;
 	req->transport_priv = ecx;
 
-	THR_SetRequest(req);
-
-	VSLb_ts_req(req, "Start", W_TIM_real(wrk));
-
-	req->ws_req = WS_Snapshot(req->ws);
+	CNT_Embark(wrk, req);
+	VCL_TaskEnter(req->vcl, req->privs);
 
 	while (1) {
-		req->wrk = wrk;
 		ecx->woken = 0;
-		s = CNT_Request(wrk, req);
+		s = CNT_Request(req);
 		if (s == REQ_FSM_DONE)
 			break;
 		DSL(DBG_WAITINGLIST, req->vsl->wid,
@@ -193,20 +198,17 @@ ved_include(struct req *preq, const char *src, const char *host,
 			(void)Lck_CondWait(
 			    &ecx->preq->wrk->cond, &sp->mtx, 0);
 		Lck_Unlock(&sp->mtx);
-		ecx->woken = 0;
 		AZ(req->wrk);
+		CNT_Embark(wrk, req);
 	}
 
-	AZ(preq->vcl);
-	preq->vcl = req->vcl;
-	req->vcl = NULL;
+	VCL_Rel(&req->vcl);
 
 	req->wrk = NULL;
 	THR_SetRequest(preq);
 
-	Req_AcctLogCharge(wrk->stats, req);
+	Req_Cleanup(sp, wrk, req);
 	Req_Release(req);
-	SES_Rel(sp);
 }
 
 /*--------------------------------------------------------------------*/
@@ -247,8 +249,48 @@ ved_decode_len(struct req *req, const uint8_t **pp)
 /*---------------------------------------------------------------------
  */
 
-static int v_matchproto_(vdp_bytes)
-ved_vdp(struct req *req, enum vdp_action act, void **priv,
+static int v_matchproto_(vdp_init_f)
+ved_vdp_esi_init(struct req *req, void **priv)
+{
+	struct ecx *ecx;
+
+	AN(priv);
+	AZ(*priv);
+	ALLOC_OBJ(ecx, ECX_MAGIC);
+	AN(ecx);
+	assert(sizeof gzip_hdr == 10);
+	ecx->preq = req;
+	*priv = ecx;
+	RFC2616_Weaken_Etag(req->resp);
+
+	req->res_mode |= RES_ESI;
+	if (req->resp_len != 0)
+		req->resp_len = -1;
+	if (req->esi_level > 0) {
+		assert(req->transport == &VED_transport);
+		CAST_OBJ_NOTNULL(ecx->pecx, req->transport_priv, ECX_MAGIC);
+		if (!ecx->pecx->isgzip)
+			ecx->pecx = NULL;
+	}
+
+	return (0);
+}
+
+static int v_matchproto_(vdp_fini_f)
+ved_vdp_esi_fini(struct req *req, void **priv)
+{
+	struct ecx *ecx;
+
+	(void)req;
+	AN(priv);
+	CAST_OBJ_NOTNULL(ecx, *priv, ECX_MAGIC);
+	FREE_OBJ(ecx);
+	*priv = NULL;
+	return (0);
+}
+
+static int v_matchproto_(vdp_bytes_f)
+ved_vdp_esi_bytes(struct req *req, enum vdp_action act, void **priv,
     const void *ptr, ssize_t len)
 {
 	uint8_t *q, *r;
@@ -256,37 +298,13 @@ ved_vdp(struct req *req, enum vdp_action act, void **priv,
 	uint32_t icrc = 0;
 	uint8_t tailbuf[8 + 5];
 	const uint8_t *pp;
-	struct ecx *ecx, *pecx = NULL;
+	struct ecx *ecx;
 	int retval = 0;
 
-	if (act == VDP_INIT) {
-		AZ(*priv);
-		ALLOC_OBJ(ecx, ECX_MAGIC);
-		AN(ecx);
-		assert(sizeof gzip_hdr == 10);
-		ecx->preq = req;
-		*priv = ecx;
-		RFC2616_Weaken_Etag(req->resp);
-		req->res_mode |= RES_ESI;
-		if (req->resp_len != 0)
-			req->resp_len = -1;
-		return (0);
-	}
+	AN(priv);
 	CAST_OBJ_NOTNULL(ecx, *priv, ECX_MAGIC);
-	if (act == VDP_FINI) {
-		FREE_OBJ(ecx);
-		*priv = NULL;
-		return (0);
-	}
 	pp = ptr;
 
-	if (req->esi_level > 0) {
-		assert(req->transport == &VED_transport);
-		CAST_OBJ_NOTNULL(pecx, req->transport_priv, ECX_MAGIC);
-		if (!pecx->isgzip)
-			pecx = NULL;
-	}
-
 	while (1) {
 		switch (ecx->state) {
 		case 0:
@@ -297,7 +315,7 @@ ved_vdp(struct req *req, enum vdp_action act, void **priv,
 			ecx->e = ecx->p + l;
 
 			if (*ecx->p == VEC_GZ) {
-				if (pecx == NULL)
+				if (ecx->pecx == NULL)
 					retval = VDP_bytes(req, VDP_NULL,
 					    gzip_hdr, 10);
 				ecx->l_crc = 0;
@@ -368,7 +386,7 @@ ved_vdp(struct req *req, enum vdp_action act, void **priv,
 			}
 			break;
 		case 2:
-			if (ecx->isgzip && pecx == NULL) {
+			if (ecx->isgzip && ecx->pecx == NULL) {
 				/*
 				 * We are bytealigned here, so simply emit
 				 * a gzip literal block with finish bit set.
@@ -386,10 +404,10 @@ ved_vdp(struct req *req, enum vdp_action act, void **priv,
 				vle32enc(tailbuf + 9, ecx->l_crc);
 
 				(void)VDP_bytes(req, VDP_NULL, tailbuf, 13);
-			} else if (pecx != NULL) {
-				pecx->crc = crc32_combine(pecx->crc,
+			} else if (ecx->pecx != NULL) {
+				ecx->pecx->crc = crc32_combine(ecx->pecx->crc,
 				    ecx->crc, ecx->l_crc);
-				pecx->l_crc += ecx->l_crc;
+				ecx->pecx->l_crc += ecx->l_crc;
 			}
 			retval = VDP_bytes(req, VDP_FLUSH, NULL, 0);
 			ecx->state = 99;
@@ -431,7 +449,9 @@ ved_vdp(struct req *req, enum vdp_action act, void **priv,
 
 const struct vdp VDP_esi = {
 	.name =		"esi",
-	.func =		ved_vdp,
+	.init =		ved_vdp_esi_init,
+	.bytes =	ved_vdp_esi_bytes,
+	.fini =		ved_vdp_esi_fini,
 };
 
 /*
@@ -439,11 +459,11 @@ const struct vdp VDP_esi = {
  * Push bytes to preq
  */
 static inline int
-ved_bytes(struct req *req, struct req *preq, enum vdp_action act,
+ved_bytes(struct req *req, struct ecx *ecx, enum vdp_action act,
     const void *ptr, ssize_t len)
 {
 	req->acct.resp_bodybytes += len;
-	return (VDP_bytes(preq, act, ptr, len));
+	return (VDP_bytes(ecx->preq, act, ptr, len));
 }
 
 /*---------------------------------------------------------------------
@@ -464,29 +484,29 @@ ved_bytes(struct req *req, struct req *preq, enum vdp_action act,
  * the stream with a bit more overhead.
  */
 
-static int v_matchproto_(vdp_bytes)
-ved_pretend_gzip(struct req *req, enum vdp_action act, void **priv,
+static int v_matchproto_(vdp_fini_f)
+ved_pretend_gzip_fini(struct req *req, void **priv)
+{
+	(void)req;
+	*priv = NULL;
+	return (0);
+}
+
+static int v_matchproto_(vdp_bytes_f)
+ved_pretend_gzip_bytes(struct req *req, enum vdp_action act, void **priv,
     const void *pv, ssize_t l)
 {
 	uint8_t buf1[5], buf2[5];
 	const uint8_t *p;
 	uint16_t lx;
 	struct ecx *ecx;
-	struct req *preq;
 
 	CHECK_OBJ_NOTNULL(req, REQ_MAGIC);
 	CAST_OBJ_NOTNULL(ecx, *priv, ECX_MAGIC);
-	preq = ecx->preq;
 
 	(void)priv;
-	if (act == VDP_INIT)
-		return (0);
-	if (act == VDP_FINI) {
-		*priv = NULL;
-		return (0);
-	}
 	if (l == 0)
-		return (ved_bytes(req, ecx->preq, act, pv, l));
+		return (ved_bytes(req, ecx, act, pv, l));
 
 	p = pv;
 
@@ -502,71 +522,93 @@ ved_pretend_gzip(struct req *req, enum vdp_action act, void **priv,
 	while (l > 0) {
 		if (l >= 65535) {
 			lx = 65535;
-			if (ved_bytes(req, preq, VDP_NULL, buf1, sizeof buf1))
+			if (ved_bytes(req, ecx, VDP_NULL, buf1, sizeof buf1))
 				return (-1);
 		} else {
 			lx = (uint16_t)l;
 			buf2[0] = 0;
 			vle16enc(buf2 + 1, lx);
 			vle16enc(buf2 + 3, ~lx);
-			if (ved_bytes(req, preq, VDP_NULL, buf2, sizeof buf2))
+			if (ved_bytes(req, ecx, VDP_NULL, buf2, sizeof buf2))
 				return (-1);
 		}
-		if (ved_bytes(req, preq, VDP_NULL, p, lx))
+		if (ved_bytes(req, ecx, VDP_NULL, p, lx))
 			return (-1);
 		l -= lx;
 		p += lx;
 	}
-	/* buf2 is local, have to flush */
-	return (ved_bytes(req, preq, VDP_FLUSH, NULL, 0));
+	/* buf1 & buf2 is local, have to flush */
+	return (ved_bytes(req, ecx, VDP_FLUSH, NULL, 0));
 }
 
-static const struct vdp ved_vdp_pgz = {
+static const struct vdp ved_pretend_gz = {
 	.name =		"PGZ",
-	.func =		ved_pretend_gzip,
+	.bytes =	ved_pretend_gzip_bytes,
+	.fini =		ved_pretend_gzip_fini,
 };
 
 /*---------------------------------------------------------------------
- * Include an object in a gzip'ed ESI object delivery
+ * Include a gzip'ed object in a gzip'ed ESI object delivery
  *
- * This is not written as a VDP (yet) because it relies on the
- * OA_GZIPBITS which only becomes available when the input side
- * has fully digested the object and located the magic bit positions.
+ * This is the interesting case: Deliver all the deflate blocks, stripping
+ * the "LAST" bit of the last one and padding it, as necessary, to a byte
+ * boundary.
  *
- * We can improve this two ways.
- *
- * One is to run a gunzip instance here, to find the stopbit ourselves,
- * but that would be double work, in particular when passing a gziped
- * object, where we would have two null-gunzips.
- *
- * The other is to have the input side guarantee that OA_GZIPBITS::stopbit
- * always is committed before the chunk of data containing it.  We would
- * be required to poll OA_GZIPBITS on every chunk presented, but that is
- * much cheaper than running a gunzip instance.
  */
 
 struct ved_foo {
 	unsigned		magic;
 #define VED_FOO_MAGIC		0x6a5a262d
-	struct req		*req;
-	struct req		*preq;
-	ssize_t start, last, stop, lpad;
-	ssize_t ll;
-	uint64_t olen;
-	uint8_t *dbits;
-	uint8_t tailbuf[8];
+	struct ecx		*ecx;
+	ssize_t			start, last, stop, lpad;
+	ssize_t			ll;
+	uint64_t		olen;
+	uint8_t			dbits[8];
+	uint8_t			tailbuf[8];
 };
 
-static int
-ved_objiterate(void *priv, int flush, const void *ptr, ssize_t len)
+static int v_matchproto_(vdp_fini_f)
+ved_gzgz_init(struct req *req, void **priv)
+{
+	ssize_t l;
+	const char *p;
+	struct ved_foo *foo;
+
+	CHECK_OBJ_NOTNULL(req, REQ_MAGIC);
+	CAST_OBJ_NOTNULL(foo, *priv, VED_FOO_MAGIC);
+
+	memset(foo->tailbuf, 0xdd, sizeof foo->tailbuf);
+
+	AN(ObjCheckFlag(req->wrk, req->objcore, OF_GZIPED));
+
+	p = ObjGetAttr(req->wrk, req->objcore, OA_GZIPBITS, &l);
+	AN(p);
+	assert(l == 32);
+	foo->start = vbe64dec(p);
+	foo->last = vbe64dec(p + 8);
+	foo->stop = vbe64dec(p + 16);
+	foo->olen = ObjGetLen(req->wrk, req->objcore);
+	assert(foo->start > 0 && foo->start < foo->olen * 8);
+	assert(foo->last > 0 && foo->last < foo->olen * 8);
+	assert(foo->stop > 0 && foo->stop < foo->olen * 8);
+	assert(foo->last >= foo->start);
+	assert(foo->last < foo->stop);
+
+	/* The start bit must be byte aligned. */
+	AZ(foo->start & 7);
+	return (0);
+}
+
+static int v_matchproto_(vdp_bytes_f)
+ved_gzgz_bytes(struct req *req, enum vdp_action act, void **priv,
+    const void *ptr, ssize_t len)
 {
 	struct ved_foo *foo;
 	const uint8_t *pp;
 	ssize_t dl;
 	ssize_t l;
 
-	CAST_OBJ_NOTNULL(foo, priv, VED_FOO_MAGIC);
-	(void)flush;
+	CAST_OBJ_NOTNULL(foo, *priv, VED_FOO_MAGIC);
 	pp = ptr;
 	if (len > 0) {
 		/* Skip over the GZIP header */
@@ -586,7 +628,7 @@ ved_objiterate(void *priv, int flush, const void *ptr, ssize_t len)
 		if (dl > 0) {
 			if (dl > len)
 				dl = len;
-			if (ved_bytes(foo->req, foo->preq, VDP_NULL, pp, dl))
+			if (ved_bytes(req, foo->ecx, act, pp, dl))
 				return(-1);
 			foo->ll += dl;
 			len -= dl;
@@ -597,7 +639,7 @@ ved_objiterate(void *priv, int flush, const void *ptr, ssize_t len)
 		/* Remove the "LAST" bit */
 		foo->dbits[0] = *pp;
 		foo->dbits[0] &= ~(1U << (foo->last & 7));
-		if (ved_bytes(foo->req, foo->preq, VDP_NULL, foo->dbits, 1))
+		if (ved_bytes(req, foo->ecx, act, foo->dbits, 1))
 			return (-1);
 		foo->ll++;
 		len--;
@@ -609,7 +651,7 @@ ved_objiterate(void *priv, int flush, const void *ptr, ssize_t len)
 		if (dl > 0) {
 			if (dl > len)
 				dl = len;
-			if (ved_bytes(foo->req, foo->preq, VDP_NULL, pp, dl))
+			if (ved_bytes(req, foo->ecx, act, pp, dl))
 				return (-1);
 			foo->ll += dl;
 			len -= dl;
@@ -671,8 +713,7 @@ ved_objiterate(void *priv, int flush, const void *ptr, ssize_t len)
 		default:
 			WRONG("compiler must be broken");
 		}
-		if (ved_bytes(foo->req, foo->preq,
-		    VDP_NULL, foo->dbits + 1, foo->lpad))
+		if (ved_bytes(req, foo->ecx, act, foo->dbits + 1, foo->lpad))
 			return (-1);
 	}
 	if (len > 0) {
@@ -696,95 +737,60 @@ ved_objiterate(void *priv, int flush, const void *ptr, ssize_t len)
 	return (0);
 }
 
-static void
-ved_stripgzip(struct req *req, const struct boc *boc)
+static int v_matchproto_(vdp_fini_f)
+ved_gzgz_fini(struct req *req, void **priv)
 {
-	ssize_t l;
-	const char *p;
 	uint32_t icrc;
 	uint32_t ilen;
-	uint8_t *dbits;
-	struct ecx *ecx;
-	struct ved_foo foo;
+	struct ved_foo *foo;
 
-	CHECK_OBJ_NOTNULL(req, REQ_MAGIC);
-	CHECK_OBJ_NOTNULL(req->objcore, OBJCORE_MAGIC);
-	CAST_OBJ_NOTNULL(ecx, req->transport_priv, ECX_MAGIC);
+	CAST_OBJ_NOTNULL(foo, *priv, VED_FOO_MAGIC);
+	*priv = NULL;
 
-	INIT_OBJ(&foo, VED_FOO_MAGIC);
-	foo.req = req;
-	foo.preq = ecx->preq;
-	memset(foo.tailbuf, 0xdd, sizeof foo.tailbuf);
+	(void)ved_bytes(req, foo->ecx, VDP_FLUSH, NULL, 0);
 
-	/* OA_GZIPBITS is not valid until BOS_FINISHED */
-	if (boc != NULL)
-		ObjWaitState(req->objcore, BOS_FINISHED);
-	if (req->objcore->flags & OC_F_FAILED) {
-		/* No way of signalling errors in the middle of
-		   the ESI body. Omit this ESI fragment. */
-		return;
-	}
+	icrc = vle32dec(foo->tailbuf);
+	ilen = vle32dec(foo->tailbuf + 4);
+	foo->ecx->crc = crc32_combine(foo->ecx->crc, icrc, ilen);
+	foo->ecx->l_crc += ilen;
 
-	AN(ObjCheckFlag(req->wrk, req->objcore, OF_GZIPED));
-
-	/*
-	 * This is the interesting case: Deliver all the deflate
-	 * blocks, stripping the "LAST" bit of the last one and
-	 * padding it, as necessary, to a byte boundary.
-	 */
-
-	p = ObjGetAttr(req->wrk, req->objcore, OA_GZIPBITS, &l);
-	AN(p);
-	assert(l == 32);
-	foo.start = vbe64dec(p);
-	foo.last = vbe64dec(p + 8);
-	foo.stop = vbe64dec(p + 16);
-	foo.olen = ObjGetLen(req->wrk, req->objcore);
-	assert(foo.start > 0 && foo.start < foo.olen * 8);
-	assert(foo.last > 0 && foo.last < foo.olen * 8);
-	assert(foo.stop > 0 && foo.stop < foo.olen * 8);
-	assert(foo.last >= foo.start);
-	assert(foo.last < foo.stop);
-
-	/* The start bit must be byte aligned. */
-	AZ(foo.start & 7);
-
-	dbits = WS_Alloc(req->ws, 8);
-	AN(dbits);
-	foo.dbits = dbits;
-	(void)ObjIterate(req->wrk, req->objcore, &foo, ved_objiterate, 0);
-	/* XXX: error check ?? */
-	(void)ved_bytes(req, foo.preq, VDP_FLUSH, NULL, 0);
-
-	icrc = vle32dec(foo.tailbuf);
-	ilen = vle32dec(foo.tailbuf + 4);
-
-	ecx->crc = crc32_combine(ecx->crc, icrc, ilen);
-	ecx->l_crc += ilen;
+	return (0);
 }
 
-/*--------------------------------------------------------------------*/
+static const struct vdp ved_gzgz = {
+	.name =         "VZZ",
+	.init =         ved_gzgz_init,
+	.bytes =        ved_gzgz_bytes,
+	.fini =         ved_gzgz_fini,
+};
 
-static int v_matchproto_(vdp_bytes)
+/*--------------------------------------------------------------------
+ * Straight through without processing.
+ */
+
+static int v_matchproto_(vdp_fini_f)
+ved_vdp_fini(struct req *req, void **priv)
+{
+	(void)req;
+	*priv = NULL;
+	return (0);
+}
+
+static int v_matchproto_(vdp_bytes_f)
 ved_vdp_bytes(struct req *req, enum vdp_action act, void **priv,
     const void *ptr, ssize_t len)
 {
-	struct req *preq;
+	struct ecx *ecx;
 
 	CHECK_OBJ_NOTNULL(req, REQ_MAGIC);
-	if (act == VDP_INIT)
-		return (0);
-	if (act == VDP_FINI) {
-		*priv = NULL;
-		return (0);
-	}
-	CAST_OBJ_NOTNULL(preq, *priv, REQ_MAGIC);
-	return (ved_bytes(req, preq, act, ptr, len));
+	CAST_OBJ_NOTNULL(ecx, *priv, ECX_MAGIC);
+	return (ved_bytes(req, ecx, act, ptr, len));
 }
 
 static const struct vdp ved_ved = {
 	.name =		"VED",
-	.func =		ved_vdp_bytes,
+	.bytes =	ved_vdp_bytes,
+	.fini =		ved_vdp_fini,
 };
 
 /*--------------------------------------------------------------------*/
@@ -794,6 +800,7 @@ ved_deliver(struct req *req, struct boc *boc, int wantbody)
 {
 	int i;
 	struct ecx *ecx;
+	struct ved_foo foo[1];
 
 	CHECK_OBJ_NOTNULL(req, REQ_MAGIC);
 	CHECK_OBJ_ORNULL(boc, BOC_MAGIC);
@@ -807,17 +814,32 @@ ved_deliver(struct req *req, struct boc *boc, int wantbody)
 	if (boc == NULL && ObjGetLen(req->wrk, req->objcore) == 0)
 		return;
 
-	req->res_mode |= RES_ESI_CHILD;
 	i = ObjCheckFlag(req->wrk, req->objcore, OF_GZIPED);
 	if (ecx->isgzip && i && !(req->res_mode & RES_ESI)) {
-		ved_stripgzip(req, boc);
+		/* A gzip'ed include which is not ESI processed */
+
+		/* OA_GZIPBITS are not valid until BOS_FINISHED */
+		if (boc != NULL)
+			ObjWaitState(req->objcore, BOS_FINISHED);
+
+		if (req->objcore->flags & OC_F_FAILED) {
+			/* No way of signalling errors in the middle of
+			   the ESI body. Omit this ESI fragment. */
+			return;
+		}
+
+		INIT_OBJ(foo, VED_FOO_MAGIC);
+		foo->ecx = ecx;
+		XXXAZ(VDP_Push(req, &ved_gzgz, foo));
+
+	} else if (ecx->isgzip && !i) {
+		/* Non-Gzip'ed include in gzip'ed parent */
+		XXXAZ(VDP_Push(req, &ved_pretend_gz, ecx));
 	} else {
-		if (ecx->isgzip && !i)
-			(void)VDP_push(req, &ved_vdp_pgz, ecx, 1);
-		else
-			(void)VDP_push(req, &ved_ved, ecx->preq, 1);
-		(void)VDP_DeliverObj(req);
-		(void)VDP_bytes(req, VDP_FLUSH, NULL, 0);
+		/* Anything else goes straight through */
+		XXXAZ(VDP_Push(req, &ved_ved, ecx));
 	}
+	(void)VDP_DeliverObj(req);
+	(void)VDP_bytes(req, VDP_FLUSH, NULL, 0);
 	VDP_close(req);
 }
