@@ -34,38 +34,37 @@
 
 #include <stdlib.h>
 
-#include "cache.h"
+#include "cache_varnishd.h"
+#include "cache_objhead.h"
 
 #include "binary_heap.h"
-#include "hash/hash_slinger.h"
 #include "vtim.h"
 
 struct exp_priv {
 	unsigned			magic;
 #define EXP_PRIV_MAGIC			0x9db22482
+	/* shared */
 	struct lock			mtx;
-
-	struct worker			*wrk;
-	struct vsl_log			vsl;
-
 	VSTAILQ_HEAD(,objcore)		inbox;
-	struct binheap			*heap;
 	pthread_cond_t			condvar;
 
-	pthread_rwlock_t		cb_rwl;
+	/* owned by exp thread */
+	struct worker			*wrk;
+	struct vsl_log			vsl;
+	struct binheap			*heap;
 };
 
 static struct exp_priv *exphdl;
 
 /*--------------------------------------------------------------------
- * Calculate an objects effective ttl time, taking req.ttl into account
+ * Calculate an object's effective ttl time, taking req.ttl into account
  * if it is available.
  */
 
-double
+vtim_real
 EXP_Ttl(const struct req *req, const struct objcore *oc)
 {
-	double r;
+	vtim_dur r;
 
 	CHECK_OBJ_NOTNULL(oc, OBJCORE_MAGIC);
 
@@ -75,6 +74,24 @@ EXP_Ttl(const struct req *req, const struct objcore *oc)
 	return (oc->t_origin + r);
 }
 
+/*--------------------------------------------------------------------
+ * Calculate an object's effective ttl+grace time, taking req.grace into
+ * account if it is available.
+ */
+
+vtim_real
+EXP_Ttl_grace(const struct req *req, const struct objcore *oc)
+{
+	vtim_dur g;
+
+	CHECK_OBJ_NOTNULL(oc, OBJCORE_MAGIC);
+
+	g = oc->grace;
+	if (req != NULL && req->d_grace >= 0. && req->d_grace < g)
+		g = req->d_grace;
+	return (EXP_Ttl(req, oc) + g);
+}
+
 /*--------------------------------------------------------------------
  * Post an objcore to the exp_thread's inbox.
  */
@@ -143,9 +160,10 @@ EXP_Insert(struct worker *wrk, struct objcore *oc)
  */
 
 void
-EXP_Rearm(struct objcore *oc, double now, double ttl, double grace, double keep)
+EXP_Rearm(struct objcore *oc, vtim_real now,
+    vtim_dur ttl, vtim_dur grace, vtim_dur keep)
 {
-	double when;
+	vtim_real when;
 
 	CHECK_OBJ_NOTNULL(oc, OBJCORE_MAGIC);
 	assert(oc->refcnt > 0);
@@ -162,7 +180,7 @@ EXP_Rearm(struct objcore *oc, double now, double ttl, double grace, double keep)
 
 	when = EXP_WHEN(oc);
 
-	VSL(SLT_ExpKill, 0, "EXP_Rearm p=%p E=%.9f e=%.9f f=0x%x", oc,
+	VSL(SLT_ExpKill, 0, "EXP_Rearm p=%p E=%.6f e=%.6f f=0x%x", oc,
 	    oc->timer_when, when, oc->flags);
 
 	if (when < oc->t_origin || when < oc->timer_when)
@@ -181,7 +199,7 @@ exp_inbox(struct exp_priv *ep, struct objcore *oc, unsigned flags)
 	CHECK_OBJ_NOTNULL(oc, OBJCORE_MAGIC);
 	assert(oc->refcnt > 0);
 
-	VSLb(&ep->vsl, SLT_ExpKill, "EXP_Inbox flg=%x p=%p e=%.9f f=0x%x",
+	VSLb(&ep->vsl, SLT_ExpKill, "EXP_Inbox flg=%x p=%p e=%.6f f=0x%x",
 	    flags, oc, oc->timer_when, oc->flags);
 
 	if (flags & OC_EF_REMOVE) {
@@ -202,7 +220,7 @@ exp_inbox(struct exp_priv *ep, struct objcore *oc, unsigned flags)
 		ObjSendEvent(ep->wrk, oc, OEV_TTLCHG);
 	}
 
-	VSLb(&ep->vsl, SLT_ExpKill, "EXP_When p=%p e=%.9f f=0x%x", oc,
+	VSLb(&ep->vsl, SLT_ExpKill, "EXP_When p=%p e=%.6f f=0x%x", oc,
 	    oc->timer_when, flags);
 
 	/*
@@ -228,8 +246,8 @@ exp_inbox(struct exp_priv *ep, struct objcore *oc, unsigned flags)
  * Expire stuff from the binheap
  */
 
-static double
-exp_expire(struct exp_priv *ep, double now)
+static vtim_real
+exp_expire(struct exp_priv *ep, vtim_real now)
 {
 	struct objcore *oc;
 
@@ -238,7 +256,7 @@ exp_expire(struct exp_priv *ep, double now)
 	oc = binheap_root(ep->heap);
 	if (oc == NULL)
 		return (now + 355./113.);
-	VSLb(&ep->vsl, SLT_ExpKill, "EXP_expire p=%p e=%.9f f=0x%x", oc,
+	VSLb(&ep->vsl, SLT_ExpKill, "EXP_expire p=%p e=%.6f f=0x%x", oc,
 	    oc->timer_when - now, oc->flags);
 
 	CHECK_OBJ_NOTNULL(oc, OBJCORE_MAGIC);
@@ -280,7 +298,7 @@ exp_expire(struct exp_priv *ep, double now)
  * object expires, accounting also for graceability, it is killed.
  */
 
-static int __match_proto__(binheap_cmp_t)
+static int v_matchproto_(binheap_cmp_t)
 object_cmp(void *priv, const void *a, const void *b)
 {
 	const struct objcore *aa, *bb;
@@ -291,7 +309,7 @@ object_cmp(void *priv, const void *a, const void *b)
 	return (aa->timer_when < bb->timer_when);
 }
 
-static void __match_proto__(binheap_update_t)
+static void v_matchproto_(binheap_update_t)
 object_update(void *priv, void *p, unsigned u)
 {
 	struct objcore *oc;
@@ -301,11 +319,11 @@ object_update(void *priv, void *p, unsigned u)
 	oc->timer_idx = u;
 }
 
-static void * __match_proto__(bgthread_t)
+static void * v_matchproto_(bgthread_t)
 exp_thread(struct worker *wrk, void *priv)
 {
 	struct objcore *oc;
-	double t = 0, tnext = 0;
+	vtim_real t = 0, tnext = 0;
 	struct exp_priv *ep;
 	unsigned flags = 0;
 
@@ -360,7 +378,6 @@ EXP_Init(void)
 	Lck_New(&ep->mtx, lck_exp);
 	AZ(pthread_cond_init(&ep->condvar, NULL));
 	VSTAILQ_INIT(&ep->inbox);
-	AZ(pthread_rwlock_init(&ep->cb_rwl, NULL));
 	exphdl = ep;
-	WRK_BgThread(&pt, "cache-timeout", exp_thread, ep);
+	WRK_BgThread(&pt, "cache-exp", exp_thread, ep);
 }
