@@ -48,7 +48,7 @@ vbf_allocobj(struct busyobj *bo, unsigned l)
 {
 	struct objcore *oc;
 	const struct stevedore *stv;
-	double lifetime;
+	vtim_dur lifetime;
 
 	CHECK_OBJ_NOTNULL(bo, BUSYOBJ_MAGIC);
 	oc = bo->fetch_objcore;
@@ -126,7 +126,7 @@ vbf_beresp2obj(struct busyobj *bo)
 	l += l2;
 
 	if (bo->uncacheable)
-		bo->fetch_objcore->flags |= OC_F_PASS;
+		bo->fetch_objcore->flags |= OC_F_HFM;
 
 	if (!vbf_allocobj(bo, l)) {
 		if (vary != NULL)
@@ -195,7 +195,7 @@ vbf_stp_mkbereq(struct worker *wrk, struct busyobj *bo)
 	if (bo->stale_oc != NULL &&
 	    ObjCheckFlag(bo->wrk, bo->stale_oc, OF_IMSCAND) &&
 	    (bo->stale_oc->boc != NULL || ObjGetLen(wrk, bo->stale_oc) != 0)) {
-		AZ(bo->stale_oc->flags & (OC_F_PASS|OC_F_PRIVATE));
+		AZ(bo->stale_oc->flags & (OC_F_HFM|OC_F_PRIVATE));
 		q = HTTP_GetHdrPack(bo->wrk, bo->stale_oc, H_Last_Modified);
 		if (q != NULL)
 			http_PrintfHeader(bo->bereq0,
@@ -208,7 +208,7 @@ vbf_stp_mkbereq(struct worker *wrk, struct busyobj *bo)
 
 	HTTP_Setup(bo->bereq, bo->ws, bo->vsl, SLT_BereqMethod);
 	bo->ws_bo = WS_Snapshot(bo->ws);
-	HTTP_Copy(bo->bereq, bo->bereq0);
+	HTTP_Clone(bo->bereq, bo->bereq0);
 
 	if (bo->req->req_body_status == REQ_BODY_NONE) {
 		bo->req = NULL;
@@ -258,7 +258,7 @@ static enum fetch_step
 vbf_stp_startfetch(struct worker *wrk, struct busyobj *bo)
 {
 	int i;
-	double now;
+	vtim_real now;
 
 	CHECK_OBJ_NOTNULL(wrk, WORKER_MAGIC);
 	CHECK_OBJ_NOTNULL(bo, BUSYOBJ_MAGIC);
@@ -355,12 +355,13 @@ vbf_stp_startfetch(struct worker *wrk, struct busyobj *bo)
 	if (http_IsStatus(bo->beresp, 304)) {
 		if (bo->stale_oc != NULL &&
 		    ObjCheckFlag(bo->wrk, bo->stale_oc, OF_IMSCAND)) {
-			AZ(bo->stale_oc->flags & (OC_F_PASS|OC_F_PRIVATE));
-			if (ObjCheckFlag(bo->wrk, bo->stale_oc, OF_CHGGZIP)) {
+			AZ(bo->stale_oc->flags & (OC_F_HFM|OC_F_PRIVATE));
+			if (ObjCheckFlag(bo->wrk, bo->stale_oc, OF_CHGCE)) {
 				/*
-				 * If we changed the gzip status of the object
-				 * the stored Content_Encoding controls we
-				 * must weaken any new ETag we get.
+				 * If a VFP changed C-E in the stored
+				 * object, then don't overwrite C-E from
+				 * the IMS fetch, and we must weaken any
+				 * new ETag we get.
 				 */
 				http_Unset(bo->beresp, H_Content_Encoding);
 				RFC2616_Weaken_Etag(bo->beresp);
@@ -418,7 +419,7 @@ vbf_stp_startfetch(struct worker *wrk, struct busyobj *bo)
 		wrk->handling = VCL_RET_DELIVER;
 	}
 	if (bo->do_pass || bo->uncacheable)
-		bo->fetch_objcore->flags |= OC_F_PASS;
+		bo->fetch_objcore->flags |= OC_F_HFM;
 
 	assert(wrk->handling == VCL_RET_DELIVER);
 
@@ -454,7 +455,7 @@ vbf_stp_fetchbody(struct worker *wrk, struct busyobj *bo)
 			 * We don't fail the fetch, in order for HitMiss
 			 * objects to be created.
 			 */
-			AN(vfc->oc->flags & OC_F_PASS);
+			AN(vfc->oc->flags & OC_F_HFM);
 			VSLb(wrk->vsl, SLT_Debug,
 			    "Fetch: Pass delivery abandoned");
 			bo->htc->doclose = SC_RX_BODY;
@@ -499,101 +500,9 @@ vbf_stp_fetchbody(struct worker *wrk, struct busyobj *bo)
 	return (F_STP_FETCHEND);
 }
 
-/*--------------------------------------------------------------------
- */
-
-static void
-vbf_default_filter_list(const struct busyobj *bo, struct vsb *vsb)
-{
-	const char *p;
-	int do_gzip = bo->do_gzip;
-	int do_gunzip = bo->do_gunzip;
-	int is_gzip = 0, is_gunzip = 0;
-
-	/*
-	 * The VCL variables beresp.do_g[un]zip tells us how we want the
-	 * object processed before it is stored.
-	 *
-	 * The backend Content-Encoding header tells us what we are going
-	 * to receive, which we classify in the following three classes:
-	 *
-	 *	"Content-Encoding: gzip"	--> object is gzip'ed.
-	 *	no Content-Encoding		--> object is not gzip'ed.
-	 *	anything else			--> do nothing wrt gzip
-	 */
-
-	/* No body -> done */
-	if (bo->htc->body_status == BS_NONE || bo->htc->content_length == 0)
-		return;
-
-	if (!cache_param->http_gzip_support)
-		do_gzip = do_gunzip = 0;
-
-	if (http_GetHdr(bo->beresp, H_Content_Encoding, &p))
-		is_gzip = !strcasecmp(p, "gzip");
-	else
-		is_gunzip = 1;
-
-	/* We won't gunzip unless it is gzip'ed */
-	if (do_gunzip && !is_gzip)
-		do_gunzip = 0;
-
-	/* We wont gzip unless if it already is gzip'ed */
-	if (do_gzip && !is_gunzip)
-		do_gzip = 0;
-
-	if (do_gunzip || (is_gzip && bo->do_esi))
-		VSB_cat(vsb, " gunzip");
-
-	if (bo->do_esi && (do_gzip || (is_gzip && !do_gunzip))) {
-		VSB_cat(vsb, " esi_gzip");
-		return;
-	}
-
-	if (bo->do_esi) {
-		VSB_cat(vsb, " esi");
-		return;
-	}
-
-	if (do_gzip)
-		VSB_cat(vsb, " gzip");
-
-	if (is_gzip && !do_gunzip)
-		VSB_cat(vsb, " testgunzip");
-}
-
-const char *
-VBF_Get_Filter_List(struct busyobj *bo)
-{
-	unsigned u;
-	struct vsb vsb[1];
-
-	u = WS_Reserve(bo->ws, 0);
-	if (u == 0) {
-		WS_Release(bo->ws, 0);
-		WS_MarkOverflow(bo->ws);
-		return (NULL);
-	}
-	AN(VSB_new(vsb, bo->ws->f, u, VSB_FIXEDLEN));
-	vbf_default_filter_list(bo, vsb);
-	if (VSB_finish(vsb)) {
-		WS_Release(bo->ws, 0);
-		WS_MarkOverflow(bo->ws);
-		return (NULL);
-	}
-	if (VSB_len(vsb)) {
-		WS_Release(bo->ws, VSB_len(vsb) + 1);
-		return (VSB_data(vsb) + 1);
-	}
-	WS_Release(bo->ws, 0);
-	return ("");
-}
-
 static enum fetch_step
 vbf_stp_fetch(struct worker *wrk, struct busyobj *bo)
 {
-	const char *p;
-
 	CHECK_OBJ_NOTNULL(wrk, WORKER_MAGIC);
 	CHECK_OBJ_NOTNULL(bo, BUSYOBJ_MAGIC);
 	CHECK_OBJ_NOTNULL(bo->fetch_objcore, OBJCORE_MAGIC);
@@ -642,10 +551,10 @@ vbf_stp_fetch(struct worker *wrk, struct busyobj *bo)
 		ObjSetFlag(bo->wrk, bo->fetch_objcore, OF_##U, 1);
 #include "tbl/obj_attr.h"
 
-	if (!(bo->fetch_objcore->flags & OC_F_PASS) &&
+	if (!(bo->fetch_objcore->flags & OC_F_HFM) &&
 	    http_IsStatus(bo->beresp, 200) && (
-	      http_GetHdr(bo->beresp, H_Last_Modified, &p) ||
-	      http_GetHdr(bo->beresp, H_ETag, &p)))
+	      http_GetHdr(bo->beresp, H_Last_Modified, NULL) ||
+	      http_GetHdr(bo->beresp, H_ETag, NULL)))
 		ObjSetFlag(bo->wrk, bo->fetch_objcore, OF_IMSCAND, 1);
 
 	assert(bo->fetch_objcore->boc->refcount >= 1);
@@ -701,16 +610,16 @@ vbf_stp_fetchend(struct worker *wrk, struct busyobj *bo)
 /*--------------------------------------------------------------------
  */
 
-static int
-vbf_objiterator(void *priv, int flush, const void *ptr, ssize_t len)
+static int v_matchproto_(objiterate_f)
+vbf_objiterator(void *priv, unsigned flush, const void *ptr, ssize_t len)
 {
 	struct busyobj *bo;
 	ssize_t l;
 	const uint8_t *ps = ptr;
 	uint8_t *pd;
 
-	(void)flush;
 	CAST_OBJ_NOTNULL(bo, priv, BUSYOBJ_MAGIC);
+	(void)flush;
 
 	while (len > 0) {
 		l = len;
@@ -740,7 +649,8 @@ vbf_stp_condfetch(struct worker *wrk, struct busyobj *bo)
 		    OA_ESIDATA));
 
 	AZ(ObjCopyAttr(bo->wrk, bo->fetch_objcore, bo->stale_oc, OA_FLAGS));
-	ObjSetFlag(bo->wrk, bo->fetch_objcore, OF_IMSCAND, 0);
+	if (bo->fetch_objcore->flags & OC_F_HFM)
+		ObjSetFlag(bo->wrk, bo->fetch_objcore, OF_IMSCAND, 0);
 	AZ(ObjCopyAttr(bo->wrk, bo->fetch_objcore, bo->stale_oc, OA_GZIPBITS));
 
 	if (bo->do_stream) {
@@ -770,7 +680,7 @@ static enum fetch_step
 vbf_stp_error(struct worker *wrk, struct busyobj *bo)
 {
 	ssize_t l, ll, o;
-	double now;
+	vtim_real now;
 	uint8_t *ptr;
 	struct vsb *synth_body;
 
@@ -987,6 +897,7 @@ VBF_Fetch(struct worker *wrk, struct req *req, struct objcore *oc,
 
 	bo = VBO_GetBusyObj(wrk, req);
 	CHECK_OBJ_NOTNULL(bo, BUSYOBJ_MAGIC);
+	AN(bo->vcl);
 
 	boc = HSH_RefBoc(oc);
 	CHECK_OBJ_NOTNULL(boc, BOC_MAGIC);
@@ -1008,6 +919,7 @@ VBF_Fetch(struct worker *wrk, struct req *req, struct objcore *oc,
 	}
 
 	VSLb(bo->vsl, SLT_Begin, "bereq %u %s", VXID(req->vsl->wid), how);
+	VSLb(bo->vsl, SLT_VCL_use, "%s", VCL_Name(bo->vcl));
 	VSLb(req->vsl, SLT_Link, "bereq %u %s", VXID(bo->vsl->wid), how);
 
 	THR_SetBusyobj(bo);
@@ -1015,8 +927,6 @@ VBF_Fetch(struct worker *wrk, struct req *req, struct objcore *oc,
 	bo->sp = req->sp;
 	SES_Ref(bo->sp);
 
-	AN(bo->vcl);
-
 	oc->boc->vary = req->vary_b;
 	req->vary_b = NULL;
 
