@@ -76,7 +76,17 @@ static struct objhead *private_oh;
 static void hsh_rush1(const struct worker *, struct objhead *,
     struct rush *, int);
 static void hsh_rush2(struct worker *, struct rush *);
-static int HSH_DerefObjHead(struct worker *wrk, struct objhead **poh);
+static int hsh_deref_objhead(struct worker *wrk, struct objhead **poh);
+static int hsh_deref_objhead_unlock(struct worker *wrk, struct objhead **poh);
+
+/*---------------------------------------------------------------------*/
+
+#define VCF_RETURN(x) const struct vcf_return VCF_##x[1] = { \
+	{ .name = #x, } \
+};
+
+VCF_RETURNS()
+#undef VCF_RETURN
 
 /*---------------------------------------------------------------------*/
 
@@ -337,17 +347,18 @@ hsh_insert_busyobj(struct worker *wrk, struct objhead *oh)
  */
 
 enum lookup_e
-HSH_Lookup(struct req *req, struct objcore **ocp, struct objcore **bocp,
-    int always_insert)
+HSH_Lookup(struct req *req, struct objcore **ocp, struct objcore **bocp)
 {
 	struct worker *wrk;
 	struct objhead *oh;
 	struct objcore *oc;
 	struct objcore *exp_oc;
-	double exp_t_origin;
+	const struct vcf_return *vr;
+	vtim_real exp_t_origin;
 	int busy_found;
-	enum lookup_e retval;
 	const uint8_t *vary;
+	unsigned xid = 0;
+	float dttl = 0.0;
 
 	AN(ocp);
 	*ocp = NULL;
@@ -358,6 +369,7 @@ HSH_Lookup(struct req *req, struct objcore **ocp, struct objcore **bocp,
 	wrk = req->wrk;
 	CHECK_OBJ_NOTNULL(wrk, WORKER_MAGIC);
 	CHECK_OBJ_NOTNULL(req->http, HTTP_MAGIC);
+	CHECK_OBJ_ORNULL(req->vcf, VCF_MAGIC);
 	AN(hash);
 
 	hsh_prealloc(wrk);
@@ -381,7 +393,7 @@ HSH_Lookup(struct req *req, struct objcore **ocp, struct objcore **bocp,
 	CHECK_OBJ_NOTNULL(oh, OBJHEAD_MAGIC);
 	Lck_AssertHeld(&oh->mtx);
 
-	if (always_insert) {
+	if (req->hash_always_miss) {
 		/* XXX: should we do predictive Vary in this case ? */
 		/* Insert new objcore in objecthead and release mutex */
 		*bocp = hsh_insert_busyobj(wrk, oh);
@@ -435,101 +447,111 @@ HSH_Lookup(struct req *req, struct objcore **ocp, struct objcore **bocp,
 				continue;
 		}
 
+		if (req->vcf != NULL) {
+			vr = req->vcf->func(req, &oc, &exp_oc, 0);
+			if (vr == VCF_CONTINUE)
+				continue;
+			if (vr == VCF_MISS) {
+				oc = NULL;
+				break;
+			}
+			if (vr == VCF_HIT)
+				break;
+			assert(vr == VCF_DEFAULT);
+		}
+
 		if (EXP_Ttl(req, oc) > req->t_req) {
-			/* If still valid, use it */
 			assert(oh->refcnt > 1);
 			assert(oc->objhead == oh);
-			if (oc->flags & OC_F_HFP) {
-				wrk->stats->cache_hitpass++;
-				VSLb(req->vsl, SLT_HitPass, "%u %.6f",
-				    ObjGetXID(wrk, oc), EXP_Dttl(req, oc));
-				oc = NULL;
-				retval = HSH_MISS;
-			} else if (oc->flags & OC_F_PASS) {
-				wrk->stats->cache_hitmiss++;
-				VSLb(req->vsl, SLT_HitMiss, "%u %.6f",
-				    ObjGetXID(wrk, oc), EXP_Dttl(req, oc));
-				*bocp = hsh_insert_busyobj(wrk, oh);
-				oc->refcnt++;
-				retval = HSH_MISS;
-			} else {
-				oc->refcnt++;
-				if (oc->hits < LONG_MAX)
-					oc->hits++;
-				retval = HSH_HIT;
-			}
-			Lck_Unlock(&oh->mtx);
-			*ocp = oc;
-			if (*bocp == NULL)
-				assert(HSH_DerefObjHead(wrk, &oh));
-			return (retval);
+			break;
 		}
+
 		if (EXP_Ttl(NULL, oc) < req->t_req && /* ignore req.ttl */
 		    oc->t_origin > exp_t_origin) {
 			/* record the newest object */
 			exp_oc = oc;
 			exp_t_origin = oc->t_origin;
+			assert(oh->refcnt > 1);
+			assert(exp_oc->objhead == oh);
 		}
 	}
 
-	if (exp_oc != NULL && exp_oc->flags & OC_F_PASS) {
+	if (req->vcf != NULL)
+		(void)req->vcf->func(req, &oc, &exp_oc, 1);
+
+	if (oc != NULL && oc->flags & OC_F_HFP) {
+		xid = ObjGetXID(wrk, oc);
+		dttl = EXP_Dttl(req, oc);
+		AN(hsh_deref_objhead_unlock(wrk, &oh));
+		wrk->stats->cache_hitpass++;
+		VSLb(req->vsl, SLT_HitPass, "%u %.6f", xid, dttl);
+		return (HSH_HITPASS);
+	}
+
+	if (oc != NULL) {
+		*ocp = oc;
+		oc->refcnt++;
+		if (oc->flags & OC_F_HFM) {
+			xid = ObjGetXID(wrk, oc);
+			dttl = EXP_Dttl(req, oc);
+			*bocp = hsh_insert_busyobj(wrk, oh);
+			Lck_Unlock(&oh->mtx);
+			wrk->stats->cache_hitmiss++;
+			VSLb(req->vsl, SLT_HitMiss, "%u %.6f", xid, dttl);
+			return (HSH_HITMISS);
+		}
+		oc->hits++;
+		AN(hsh_deref_objhead_unlock(wrk, &oh));
+		return (HSH_HIT);
+	}
+
+	if (exp_oc != NULL && exp_oc->flags & OC_F_HFM) {
+		/*
+		 * expired HFM ("grace/keep HFM")
+		 *
+		 * XXX should HFM objects actually have grace/keep ?
+		 * XXX also:  why isn't *ocp = exp_oc ?
+		 */
+		xid = ObjGetXID(wrk, exp_oc);
+		dttl = EXP_Dttl(req, exp_oc);
+		*bocp = hsh_insert_busyobj(wrk, oh);
+		Lck_Unlock(&oh->mtx);
 		wrk->stats->cache_hitmiss++;
-		VSLb(req->vsl, SLT_HitMiss, "%u %.6f", ObjGetXID(wrk, exp_oc),
-		    EXP_Dttl(req, exp_oc));
-		exp_oc = NULL;
-		busy_found = 0;
+		VSLb(req->vsl, SLT_HitMiss, "%u %.6f", xid, dttl);
+		return (HSH_HITMISS);
 	}
 
 	if (!busy_found) {
-		/* Insert objcore in objecthead */
 		*bocp = hsh_insert_busyobj(wrk, oh);
 
 		if (exp_oc != NULL) {
-			assert(oh->refcnt > 1);
-			assert(exp_oc->objhead == oh);
 			exp_oc->refcnt++;
-			Lck_Unlock(&oh->mtx);
 			*ocp = exp_oc;
-
-			if (EXP_Ttl_grace(req, exp_oc) < req->t_req) {
-				retval = HSH_MISS;
-			} else {
-				if (exp_oc->hits < LONG_MAX)
-					exp_oc->hits++;
-				retval = HSH_EXPBUSY;
+			if (EXP_Ttl_grace(req, exp_oc) >= req->t_req) {
+				exp_oc->hits++;
+				Lck_Unlock(&oh->mtx);
+				return (HSH_GRACE);
 			}
-		} else {
-			Lck_Unlock(&oh->mtx);
-			retval = HSH_MISS;
 		}
-
-		return (retval);
+		Lck_Unlock(&oh->mtx);
+		return (HSH_MISS);
 	}
 
 	AN(busy_found);
 	if (exp_oc != NULL && EXP_Ttl_grace(req, exp_oc) >= req->t_req) {
 		/* we do not wait on the busy object if in grace */
-		assert(oh->refcnt > 1);
-		assert(exp_oc->objhead == oh);
 		exp_oc->refcnt++;
-		Lck_Unlock(&oh->mtx);
 		*ocp = exp_oc;
-
-		assert(HSH_DerefObjHead(wrk, &oh));
-		if (exp_oc->hits < LONG_MAX)
-			exp_oc->hits++;
-		return (HSH_EXP);
+		exp_oc->hits++;
+		AN(hsh_deref_objhead_unlock(wrk, &oh));
+		return (HSH_GRACE);
 	}
 
 	/* There are one or more busy objects, wait for them */
+	VTAILQ_INSERT_TAIL(&oh->waitinglist, req, w_list);
 
 	AZ(req->hash_ignore_busy);
 
-	VTAILQ_INSERT_TAIL(&oh->waitinglist, req, w_list);
-	if (DO_DEBUG(DBG_WAITINGLIST))
-		VSLb(req->vsl, SLT_Debug, "on waiting list <%p>", oh);
-
-	wrk->stats->busy_sleep++;
 	/*
 	 * The objhead reference transfers to the sess, we get it
 	 * back when the sess comes off the waiting list and
@@ -538,7 +560,13 @@ HSH_Lookup(struct req *req, struct objcore **ocp, struct objcore **bocp,
 	req->hash_objhead = oh;
 	req->wrk = NULL;
 	req->waitinglist = 1;
+
+	if (DO_DEBUG(DBG_WAITINGLIST))
+		VSLb(req->vsl, SLT_Debug, "on waiting list <%p>", oh);
+
 	Lck_Unlock(&oh->mtx);
+
+	wrk->stats->busy_sleep++;
 	return (HSH_BUSY);
 }
 
@@ -613,8 +641,8 @@ hsh_rush2(struct worker *wrk, struct rush *r)
  */
 
 unsigned
-HSH_Purge(struct worker *wrk, struct objhead *oh, double ttl_now, double ttl,
-double grace, double keep)
+HSH_Purge(struct worker *wrk, struct objhead *oh, vtim_real ttl_now,
+    vtim_dur ttl, vtim_dur grace, vtim_dur keep)
 {
 	struct objcore *oc, **ocp;
 	unsigned spc, ospc, nobj, n, n_tot = 0;
@@ -773,8 +801,10 @@ HSH_Unbusy(struct worker *wrk, struct objcore *oc)
 	VTAILQ_REMOVE(&oh->objcs, oc, hsh_list);
 	VTAILQ_INSERT_HEAD(&oh->objcs, oc, hsh_list);
 	oc->flags &= ~OC_F_BUSY;
-	if (!VTAILQ_EMPTY(&oh->waitinglist))
+	if (!VTAILQ_EMPTY(&oh->waitinglist)) {
+		assert(oh->refcnt > 1);
 		hsh_rush1(wrk, oh, &rush, HSH_RUSH_POLICY);
+	}
 	Lck_Unlock(&oh->mtx);
 	if (!(oc->flags & OC_F_PRIVATE))
 		EXP_Insert(wrk, oc);
@@ -929,8 +959,10 @@ HSH_DerefObjCore(struct worker *wrk, struct objcore **ocp, int rushmax)
 	r = --oc->refcnt;
 	if (!r)
 		VTAILQ_REMOVE(&oh->objcs, oc, hsh_list);
-	if (!VTAILQ_EMPTY(&oh->waitinglist))
+	if (!VTAILQ_EMPTY(&oh->waitinglist)) {
+		assert(oh->refcnt > 1);
 		hsh_rush1(wrk, oh, &rush, rushmax);
+	}
 	Lck_Unlock(&oh->mtx);
 	hsh_rush2(wrk, &rush);
 	if (r != 0)
@@ -947,52 +979,45 @@ HSH_DerefObjCore(struct worker *wrk, struct objcore **ocp, int rushmax)
 
 	/* Drop our ref on the objhead */
 	assert(oh->refcnt > 0);
-	(void)HSH_DerefObjHead(wrk, &oh);
+	(void)hsh_deref_objhead(wrk, &oh);
 	return (0);
 }
 
 static int
-HSH_DerefObjHead(struct worker *wrk, struct objhead **poh)
+hsh_deref_objhead_unlock(struct worker *wrk, struct objhead **poh)
 {
 	struct objhead *oh;
-	struct rush rush;
-	int r;
 
 	CHECK_OBJ_NOTNULL(wrk, WORKER_MAGIC);
 	TAKE_OBJ_NOTNULL(oh, poh, OBJHEAD_MAGIC);
-	INIT_OBJ(&rush, RUSH_MAGIC);
+
+	Lck_AssertHeld(&oh->mtx);
 
 	if (oh == private_oh) {
 		assert(VTAILQ_EMPTY(&oh->waitinglist));
-		Lck_Lock(&oh->mtx);
 		assert(oh->refcnt > 1);
 		oh->refcnt--;
 		Lck_Unlock(&oh->mtx);
-		return(1);
+		return (1);
 	}
 
-	/*
-	 * Make absolutely certain that we do not let the final ref
-	 * disappear until the waitinglist is empty.
-	 * This is necessary because the req's on the waiting list do
-	 * not hold any ref on the objhead of their own, and we cannot
-	 * just make the hold the same ref's as objcore, that would
-	 * confuse hashers.
-	 */
-	Lck_Lock(&oh->mtx);
-	while (oh->refcnt == 1 && !VTAILQ_EMPTY(&oh->waitinglist)) {
-		hsh_rush1(wrk, oh, &rush, HSH_RUSH_ALL);
-		Lck_Unlock(&oh->mtx);
-		hsh_rush2(wrk, &rush);
-		Lck_Lock(&oh->mtx);
-	}
-	Lck_Unlock(&oh->mtx);
+	if (oh->refcnt == 1)
+		assert(VTAILQ_EMPTY(&oh->waitinglist));
 
 	assert(oh->refcnt > 0);
-	r = hash->deref(oh);
-	if (!r)
-		HSH_DeleteObjHead(wrk, oh);
-	return (r);
+	return (hash->deref(wrk, oh));
+}
+
+static int
+hsh_deref_objhead(struct worker *wrk, struct objhead **poh)
+{
+	struct objhead *oh;
+
+	CHECK_OBJ_NOTNULL(wrk, WORKER_MAGIC);
+	TAKE_OBJ_NOTNULL(oh, poh, OBJHEAD_MAGIC);
+
+	Lck_Lock(&oh->mtx);
+	return (hsh_deref_objhead_unlock(wrk, &oh));
 }
 
 void
