@@ -143,7 +143,7 @@ HTTP_estimate(unsigned nhttp)
 {
 
 	/* XXX: We trust the structs to size-aligned as necessary */
-	return (PRNDUP(sizeof (struct http) + sizeof(txt) * nhttp + nhttp));
+	return (PRNDUP(sizeof(struct http) + sizeof(txt) * nhttp + nhttp));
 }
 
 struct http *
@@ -190,19 +190,39 @@ http_Teardown(struct http *hp)
 	memset(hp->hdf, 0, sizeof *hp->hdf * hp->shd);
 }
 
-/*--------------------------------------------------------------------*/
+/*--------------------------------------------------------------------
+ * Duplicate the http content into another http
+ * We cannot just memcpy the struct because the hd & hdf are private
+ * storage to the struct http.
+ */
 
 void
-HTTP_Copy(struct http *to, const struct http * const fm)
+HTTP_Dup(struct http *to, const struct http * const fm)
 {
 
 	assert(fm->nhd <= to->shd);
-	memcpy(&to->nhd, &fm->nhd, sizeof *to - offsetof(struct http, nhd));
 	memcpy(to->hd, fm->hd, fm->nhd * sizeof *to->hd);
 	memcpy(to->hdf, fm->hdf, fm->nhd * sizeof *to->hdf);
+	to->nhd = fm->nhd;
+	to->logtag = fm->logtag;
+	to->status = fm->status;
 	to->protover = fm->protover;
 }
 
+
+/*--------------------------------------------------------------------
+ * Clone the entire http structure, including vsl & ws
+ */
+
+void
+HTTP_Clone(struct http *to, const struct http * const fm)
+{
+
+	HTTP_Dup(to, fm);
+	to->vsl = fm->vsl;
+	to->ws = fm->ws;
+}
+
 /*--------------------------------------------------------------------*/
 
 void
@@ -212,7 +232,8 @@ http_Proto(struct http *to)
 
 	fm = to->hd[HTTP_HDR_PROTO].b;
 
-	if ((fm[0] == 'H' || fm[0] == 'h') &&
+	if (fm != NULL &&
+	    (fm[0] == 'H' || fm[0] == 'h') &&
 	    (fm[1] == 'T' || fm[1] == 't') &&
 	    (fm[2] == 'T' || fm[2] == 't') &&
 	    (fm[3] == 'P' || fm[3] == 'p') &&
@@ -969,7 +990,7 @@ HTTP_GetStatusPack(struct worker *wrk, struct objcore *oc)
 	ptr = ObjGetAttr(wrk, oc, OA_HEADERS, NULL);
 	AN(ptr);
 
-	return(vbe16dec(ptr + 2));
+	return (vbe16dec(ptr + 2));
 }
 
 /*--------------------------------------------------------------------*/
@@ -1228,7 +1249,7 @@ http_PrintfHeader(struct http *to, const char *fmt, ...)
 }
 
 void
-http_TimeHeader(struct http *to, const char *fmt, double now)
+http_TimeHeader(struct http *to, const char *fmt, vtim_real now)
 {
 	char *p;
 
