@@ -30,8 +30,7 @@
 
 #include "config.h"
 
-#include <stddef.h>
-#include "cache.h"
+#include "cache_varnishd.h"
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -45,7 +44,7 @@ struct memitem {
 #define MEMITEM_MAGIC			0x42e55401
 	unsigned			size;
 	VTAILQ_ENTRY(memitem)		list;
-	double				touched;
+	vtim_real			touched;	// XXX -> mono?
 };
 
 VTAILQ_HEAD(memhead_s, memitem);
@@ -60,10 +59,11 @@ struct mempool {
 	volatile struct poolparam	*param;
 	volatile unsigned		*cur_size;
 	uint64_t			live;
+	struct vsc_seg			*vsc_seg;
 	struct VSC_mempool		*vsc;
 	unsigned			n_pool;
 	pthread_t			thread;
-	double				t_now;
+	vtim_real			t_now;	// XXX -> mono?
 	int				self_destruct;
 };
 
@@ -99,8 +99,8 @@ mpl_guard(void *priv)
 {
 	struct mempool *mpl;
 	struct memitem *mi = NULL;
-	double __state_variable__(mpl_slp);
-	double last = 0;
+	vtim_dur v_statevariable_(mpl_slp);
+	vtim_real last = 0;
 
 	CAST_OBJ_NOTNULL(mpl, priv, MEMPOOL_MAGIC);
 	THR_SetName(mpl->name);
@@ -161,7 +161,7 @@ mpl_guard(void *priv)
 				FREE_OBJ(mi);
 				mi = NULL;
 			}
-			VSC_mempool_Destroy(&mpl->vsc);
+			VSC_mempool_Destroy(&mpl->vsc_seg);
 			Lck_Unlock(&mpl->mtx);
 			Lck_Delete(&mpl->mtx);
 			FREE_OBJ(mpl);
@@ -239,7 +239,7 @@ MPL_New(const char *name,
 	VTAILQ_INIT(&mpl->surplus);
 	Lck_New(&mpl->mtx, lck_mempool);
 	/* XXX: prealloc min_pool */
-	mpl->vsc = VSC_mempool_New(mpl->name + 4);
+	mpl->vsc = VSC_mempool_New(NULL, &mpl->vsc_seg, mpl->name + 4);
 	AN(mpl->vsc);
 	AZ(pthread_create(&mpl->thread, NULL, mpl_guard, mpl));
 	AZ(pthread_detach(mpl->thread));
@@ -305,7 +305,7 @@ MPL_Get(struct mempool *mpl, unsigned *size)
 
 	CHECK_OBJ_NOTNULL(mi, MEMITEM_MAGIC);
 	/* Throw away sizeof info for FlexeLint: */
-	return ((void*)(uintptr_t)(mi+1));
+	return ((void *)(uintptr_t)(mi + 1));
 }
 
 void
