@@ -318,7 +318,7 @@ ObjGetLen(struct worker *wrk, struct objcore *oc)
 	CHECK_OBJ_NOTNULL(wrk, WORKER_MAGIC);
 
 	AZ(ObjGetU64(wrk, oc, OA_LEN, &len));
-	return(len);
+	return (len);
 }
 
 /*====================================================================
@@ -481,7 +481,7 @@ ObjSetAttr(struct worker *wrk, struct objcore *oc, enum obj_attr attr,
  */
 
 void
-ObjTouch(struct worker *wrk, struct objcore *oc, double now)
+ObjTouch(struct worker *wrk, struct objcore *oc, vtim_real now)
 {
 	const struct obj_methods *om = obj_getmethods(oc);
 
