@@ -30,23 +30,21 @@
 #include "config.h"
 
 #include <execinfo.h>
-#include <errno.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <signal.h>
 
-#include "vtim.h"
-
-#include "cache.h"
+#include "cache_varnishd.h"
 #include "cache_transport.h"
 
 #include "cache_filter.h"
 #include "common/heritage.h"
+#include "waiter/waiter.h"
 
-#include "vrt.h"
-#include "cache_director.h"
 #include "storage/storage.h"
 #include "vcli_serve.h"
+#include "vtim.h"
+#include "vcs.h"
 
 /*
  * The panic string is constructed in memory, then copied to the
@@ -61,6 +59,7 @@ static struct vsb pan_vsb_storage, *pan_vsb;
 static pthread_mutex_t panicstr_mtx;
 
 static void pan_sess(struct vsb *, const struct sess *);
+static void pan_req(struct vsb *, const struct req *);
 
 /*--------------------------------------------------------------------*/
 
@@ -84,7 +83,7 @@ reqbody_status_2str(enum req_body_state_e e)
 #define REQ_BODY(U) case REQ_BODY_##U: return("R_BODY_" #U);
 #include "tbl/req_body.h"
 	default:
-		return("?");
+		return ("?");
 	}
 }
 
@@ -107,18 +106,18 @@ const char *
 sess_close_2str(enum sess_close sc, int want_desc)
 {
 	switch (sc) {
-	case SC_NULL:		return(want_desc ? "(null)": "NULL");
+	case SC_NULL:		return (want_desc ? "(null)" : "NULL");
 #define SESS_CLOSE(nm, s, err, desc)			\
 	case SC_##nm: return(want_desc ? desc : #nm);
 #include "tbl/sess_close.h"
 
-	default:		return(want_desc ? "(invalid)" : "INVALID");
+	default:		return (want_desc ? "(invalid)" : "INVALID");
 	}
 }
 
 /*--------------------------------------------------------------------*/
 
-#define N_ALREADY 64
+#define N_ALREADY 256
 static const void *already_list[N_ALREADY];
 static int already_idx;
 
@@ -317,8 +316,6 @@ pan_wrk(struct vsb *vsb, const struct worker *wrk)
 		return;
 	VSB_indent(vsb, 2);
 	PAN_CheckMagic(vsb, wrk, WORKER_MAGIC);
-	VSB_printf(vsb, "stack = {0x%jx -> 0x%jx},\n",
-	    (uintmax_t)wrk->stack_start, (uintmax_t)wrk->stack_end);
 	pan_ws(vsb, wrk->aws);
 
 	m = wrk->cur_method;
@@ -357,9 +354,68 @@ pan_wrk(struct vsb *vsb, const struct worker *wrk)
 }
 
 static void
-pan_busyobj(struct vsb *vsb, const struct busyobj *bo)
+pan_vfp(struct vsb *vsb, const struct vfp_ctx *vfc)
 {
 	struct vfp_entry *vfe;
+
+	VSB_printf(vsb, "vfc = %p {\n", vfc);
+	VSB_indent(vsb, 2);
+	PAN_CheckMagic(vsb, vfc, VFP_CTX_MAGIC);
+	VSB_printf(vsb, "failed = %d,\n", vfc->failed);
+	VSB_printf(vsb, "req = %p,\n", vfc->req);
+	VSB_printf(vsb, "resp = %p,\n", vfc->resp);
+	VSB_printf(vsb, "wrk = %p,\n", vfc->wrk);
+	VSB_printf(vsb, "oc = %p,\n", vfc->oc);
+
+	if (!VTAILQ_EMPTY(&vfc->vfp)) {
+		VSB_printf(vsb, "filters = {\n");
+		VSB_indent(vsb, 2);
+		VTAILQ_FOREACH(vfe, &vfc->vfp, list) {
+			VSB_printf(vsb, "%s = %p {\n", vfe->vfp->name, vfe);
+			VSB_indent(vsb, 2);
+			VSB_printf(vsb, "priv1 = %p,\n", vfe->priv1);
+			VSB_printf(vsb, "priv2 = %zd,\n", vfe->priv2);
+			VSB_printf(vsb, "closed = %d\n", vfe->closed);
+			VSB_indent(vsb, -2);
+			VSB_printf(vsb, "},\n");
+		}
+		VSB_indent(vsb, -2);
+		VSB_printf(vsb, "},\n");
+	}
+
+	VSB_printf(vsb, "obj_flags = 0x%x,\n", vfc->obj_flags);
+	VSB_indent(vsb, -2);
+	VSB_printf(vsb, "},\n");
+}
+
+static void
+pan_vdp(struct vsb *vsb, const struct vdp_ctx *vdc)
+{
+	struct vdp_entry *vde;
+
+	VSB_printf(vsb, "vdc = %p {\n", vdc);
+	VSB_indent(vsb, 2);
+	PAN_CheckMagic(vsb, vdc, VDP_CTX_MAGIC);
+	VSB_printf(vsb, "nxt = %p,\n", vdc->nxt);
+	VSB_printf(vsb, "retval = %d,\n", vdc->retval);
+
+	if (!VTAILQ_EMPTY(&vdc->vdp)) {
+		VSB_printf(vsb, "filters = {\n");
+		VSB_indent(vsb, 2);
+		VTAILQ_FOREACH(vde, &vdc->vdp, list)
+			VSB_printf(vsb, "%s = %p { priv = %p }\n",
+			    vde->vdp->name, vde, vde->priv);
+		VSB_indent(vsb, -2);
+		VSB_printf(vsb, "},\n");
+	}
+
+	VSB_indent(vsb, -2);
+	VSB_printf(vsb, "},\n");
+}
+
+static void
+pan_busyobj(struct vsb *vsb, const struct busyobj *bo)
+{
 	const char *p;
 
 	VSB_printf(vsb, "busyobj = %p {\n", bo);
@@ -367,9 +423,39 @@ pan_busyobj(struct vsb *vsb, const struct busyobj *bo)
 		return;
 	VSB_indent(vsb, 2);
 	PAN_CheckMagic(vsb, bo, BUSYOBJ_MAGIC);
+	VSB_printf(vsb, "end = %p,\n", bo->end);
+	VSB_printf(vsb, "retries = %d,\n", bo->retries);
+
+	if (bo->req != NULL)
+		pan_req(vsb, bo->req);
+	if (bo->sp != NULL)
+		pan_sess(vsb, bo->sp);
+	if (bo->wrk != NULL)
+		pan_wrk(vsb, bo->wrk);
+
+	if (bo->vfc != NULL)
+		pan_vfp(vsb, bo->vfc);
+	if (bo->filter_list != NULL)
+		VSB_printf(vsb, "filter_list = \"%s\",\n", bo->filter_list);
+
 	pan_ws(vsb, bo->ws);
-	VSB_printf(vsb, "retries = %d, ", bo->retries);
-	VSB_printf(vsb, "failed = %d, ", bo->vfc->failed);
+	VSB_printf(vsb, "ws_bo = %p,\n", (void *)bo->ws_bo);
+
+	// bereq0 left out
+	if (bo->bereq != NULL && bo->bereq->ws != NULL)
+		pan_http(vsb, "bereq", bo->bereq);
+	if (bo->beresp != NULL && bo->beresp->ws != NULL)
+		pan_http(vsb, "beresp", bo->beresp);
+	if (bo->stale_oc)
+		pan_objcore(vsb, "stale_oc", bo->stale_oc);
+	if (bo->fetch_objcore)
+		pan_objcore(vsb, "fetch", bo->fetch_objcore);
+
+	if (VALID_OBJ(bo->htc, HTTP_CONN_MAGIC))
+		pan_htc(vsb, bo->htc);
+
+	// fetch_task left out
+
 	VSB_printf(vsb, "flags = {");
 	p = "";
 /*lint -save -esym(438,p) -e539 */
@@ -379,32 +465,14 @@ pan_busyobj(struct vsb *vsb, const struct busyobj *bo)
 /*lint -restore */
 	VSB_printf(vsb, "},\n");
 
-	if (VALID_OBJ(bo->htc, HTTP_CONN_MAGIC))
-		pan_htc(vsb, bo->htc);
-
-	if (!VTAILQ_EMPTY(&bo->vfc->vfp)) {
-		VSB_printf(vsb, "filters =");
-		VTAILQ_FOREACH(vfe, &bo->vfc->vfp, list)
-			VSB_printf(vsb, " %s=%d",
-			    vfe->vfp->name, (int)vfe->closed);
-		VSB_printf(vsb, "\n");
-	}
+	// timeouts/timers/acct/storage left out
 
 	VDI_Panic(bo->director_req, vsb, "director_req");
 	if (bo->director_resp == bo->director_req)
 		VSB_printf(vsb, "director_resp = director_req,\n");
 	else
 		VDI_Panic(bo->director_resp, vsb, "director_resp");
-	if (bo->bereq != NULL && bo->bereq->ws != NULL)
-		pan_http(vsb, "bereq", bo->bereq);
-	if (bo->beresp != NULL && bo->beresp->ws != NULL)
-		pan_http(vsb, "beresp", bo->beresp);
-	if (bo->fetch_objcore)
-		pan_objcore(vsb, "fetch", bo->fetch_objcore);
-	if (bo->stale_oc)
-		pan_objcore(vsb, "ims", bo->stale_oc);
-	VCL_Panic(vsb, bo->vcl);
-	VMOD_Panic(vsb);
+	VCL_Panic(vsb, "vcl", bo->vcl);
 	VSB_indent(vsb, -2);
 	VSB_printf(vsb, "},\n");
 }
@@ -467,9 +535,11 @@ pan_req(struct vsb *vsb, const struct req *req)
 	pan_http(vsb, "req", req->http);
 	if (req->resp->ws != NULL)
 		pan_http(vsb, "resp", req->resp);
+	if (req->vdc != NULL)
+		pan_vdp(vsb, req->vdc);
 
-	VCL_Panic(vsb, req->vcl);
-	VMOD_Panic(vsb);
+	VCL_Panic(vsb, "vcl", req->vcl);
+	VCL_Panic(vsb, "vcl0", req->vcl0);
 
 	if (req->body_oc != NULL)
 		pan_objcore(vsb, "BODY", req->body_oc);
@@ -483,6 +553,14 @@ pan_req(struct vsb *vsb, const struct req *req)
 	VSB_indent(vsb, -2);
 	VSB_printf(vsb, "},\n");
 
+	pan_privs(vsb, req->privs);
+
+	VSB_printf(vsb, "topreq = {\n");
+	VSB_indent(vsb, 2);
+	pan_req(vsb, req->topreq);
+	VSB_indent(vsb, -2);
+	VSB_printf(vsb, "},\n");
+
 	VSB_indent(vsb, -2);
 	VSB_printf(vsb, "},\n");
 }
@@ -506,6 +584,7 @@ pan_sess(struct vsb *vsb, const struct sess *sp)
 	    sp->fd, VXID(sp->vxid));
 	VSB_printf(vsb, "t_open = %f,\n", sp->t_open);
 	VSB_printf(vsb, "t_idle = %f,\n", sp->t_idle);
+	pan_ws(vsb, sp->ws);
 	VSB_printf(vsb, "transport = %s",
 	    xp == NULL ? "<none>" : xp->name);
 	if (xp != NULL && xp->sess_panic != NULL) {
@@ -518,9 +597,11 @@ pan_sess(struct vsb *vsb, const struct sess *sp)
 	VSB_printf(vsb, "\n");
 	ci = SES_Get_String_Attr(sp, SA_CLIENT_IP);
 	cp = SES_Get_String_Attr(sp, SA_CLIENT_PORT);
-	VSB_printf(vsb, "client = %s %s,\n", ci, cp);
-
-	pan_privs(vsb, sp->privs);
+	if (VALID_OBJ(sp->listen_sock, LISTEN_SOCK_MAGIC))
+		VSB_printf(vsb, "client = %s %s %s,\n", ci, cp,
+			   sp->listen_sock->endpoint);
+	else
+		VSB_printf(vsb, "client = %s %s <unknown>\n", ci, cp);
 
 	VSB_indent(vsb, -2);
 	VSB_printf(vsb, "},\n");
@@ -569,6 +650,33 @@ pan_backtrace(struct vsb *vsb)
 	VSB_indent(vsb, -2);
 }
 
+#ifdef HAVE_PTHREAD_GETATTR_NP
+static void
+pan_threadattr(struct vsb *vsb)
+{
+	pthread_attr_t attr[1];
+	size_t sz;
+	void *addr;
+
+	if (pthread_getattr_np(pthread_self(), attr) != 0)
+		return;
+
+	VSB_cat(vsb, "pthread.attr = {\n");
+	VSB_indent(vsb, 2);
+
+	if (pthread_attr_getguardsize(attr, &sz) == 0)
+		VSB_printf(vsb, "guard = %zu,\n", sz);
+	if (pthread_attr_getstack(attr, &addr, &sz) == 0) {
+		VSB_printf(vsb, "stack_bottom = %p,\n", addr);
+		VSB_printf(vsb, "stack_top = %p,\n", (char *)addr + sz);
+		VSB_printf(vsb, "stack_size = %zu,\n", sz);
+	}
+	VSB_indent(vsb, -2);
+	VSB_cat(vsb, "}\n");
+	(void) pthread_attr_destroy(attr);
+}
+#endif
+
 /*--------------------------------------------------------------------*/
 
 static void __attribute__((__noreturn__))
@@ -628,19 +736,23 @@ pan_ic(const char *func, const char *file, int line, const char *cond,
 	VSB_printf(pan_vsb, "version = %s, vrt api = %u.%u\n",
 	    VCS_version, VRT_MAJOR_VERSION, VRT_MINOR_VERSION);
 	VSB_printf(pan_vsb, "ident = %s,%s\n",
-	    VSB_data(vident) + 1, Waiter_GetName());
+	    heritage.ident, Waiter_GetName());
 	VSB_printf(pan_vsb, "now = %f (mono), %f (real)\n",
 	    VTIM_mono(), VTIM_real());
 
 	pan_backtrace(pan_vsb);
 
 	if (err)
-		VSB_printf(pan_vsb, "errno = %d (%s)\n", err, strerror(err));
+		VSB_printf(pan_vsb, "errno = %d (%s)\n", err, vstrerror(err));
 
 	q = THR_GetName();
 	if (q != NULL)
 		VSB_printf(pan_vsb, "thread = (%s)\n", q);
 
+#ifdef HAVE_PTHREAD_GETATTR_NP
+	pan_threadattr(pan_vsb);
+#endif
+
 	if (!FEATURE(FEATURE_SHORT_PANIC)) {
 		req = THR_GetRequest();
 		VSB_cat(pan_vsb, "thr.");
@@ -652,6 +764,7 @@ pan_ic(const char *func, const char *file, int line, const char *cond,
 		pan_busyobj(pan_vsb, bo);
 		if (bo != NULL)
 			VSL_Flush(bo->vsl, 0);
+		VMOD_Panic(pan_vsb);
 	} else {
 		VSB_printf(pan_vsb,
 		    "Feature short panic supressed details.\n");
@@ -667,7 +780,7 @@ pan_ic(const char *func, const char *file, int line, const char *cond,
 
 /*--------------------------------------------------------------------*/
 
-static void __match_proto__(cli_func_t)
+static void v_matchproto_(cli_func_t)
 ccf_panic(struct cli *cli, const char * const *av, void *priv)
 {
 
