@@ -29,8 +29,8 @@
 
 #include "config.h"
 
-#include "cache/cache.h"
-#include "cache/cache_filter.h"
+#include "cache_varnishd.h"
+#include "cache_filter.h"
 
 #include "vct.h"
 
@@ -44,7 +44,20 @@ struct vrg_priv {
 	ssize_t			range_off;
 };
 
-static int __match_proto__(vdp_bytes)
+static int v_matchproto_(vdp_fini_f)
+vrg_range_fini(struct req *req, void **priv)
+{
+	struct vrg_priv *vrg_priv;
+
+	CHECK_OBJ_NOTNULL(req, REQ_MAGIC);
+	CAST_OBJ_NOTNULL(vrg_priv, *priv, VRG_PRIV_MAGIC);
+	if (vrg_priv->range_off < vrg_priv->range_high)
+		Req_Fail(req, SC_RANGE_SHORT);
+	*priv = NULL;	/* struct on ws, no need to free */
+	return (0);
+}
+
+static int v_matchproto_(vdp_bytes_f)
 vrg_range_bytes(struct req *req, enum vdp_action act, void **priv,
     const void *ptr, ssize_t len)
 {
@@ -54,15 +67,8 @@ vrg_range_bytes(struct req *req, enum vdp_action act, void **priv,
 	struct vrg_priv *vrg_priv;
 
 	CHECK_OBJ_NOTNULL(req, REQ_MAGIC);
-	if (act == VDP_INIT)
-		return (0);
+	AN(priv);
 	CAST_OBJ_NOTNULL(vrg_priv, *priv, VRG_PRIV_MAGIC);
-	if (act == VDP_FINI) {
-		if (vrg_priv->range_off < vrg_priv->range_high)
-			Req_Fail(req, SC_RANGE_SHORT);
-		*priv = NULL;	/* struct on ws, no need to free */
-		return (0);
-	}
 
 	l = vrg_priv->range_low - vrg_priv->range_off;
 	if (l > 0) {
@@ -87,7 +93,7 @@ vrg_range_bytes(struct req *req, enum vdp_action act, void **priv,
 /*--------------------------------------------------------------------*/
 
 static const char *
-vrg_dorange(struct req *req, const char *r)
+vrg_dorange(struct req *req, const char *r, void **priv)
 {
 	ssize_t low, high, has_low, has_high, t;
 	struct vrg_priv *vrg_priv;
@@ -170,34 +176,39 @@ vrg_dorange(struct req *req, const char *r)
 	vrg_priv->range_off = 0;
 	vrg_priv->range_low = low;
 	vrg_priv->range_high = high + 1;
-	VDP_push(req, vrg_range_bytes, vrg_priv, 1, "RNG");
+	*priv = vrg_priv;
 	http_PutResponse(req->resp, "HTTP/1.1", 206, NULL);
 	return (NULL);
 }
 
-void
-VRG_dorange(struct req *req, const char *r)
+static int v_matchproto_(vdp_init_f)
+vrg_range_init(struct req *req, void **priv)
 {
+	const char *r;
 	const char *err;
 
-	CHECK_OBJ_NOTNULL(req, REQ_MAGIC);
-	CHECK_OBJ_NOTNULL(req->objcore, OBJCORE_MAGIC);
-	assert(http_IsStatus(req->resp, 200));
+	assert(http_GetHdr(req->http, H_Range, &r));
+	err = vrg_dorange(req, r, priv);
+	if (err == NULL)
+		return (*priv == NULL ? 1 : 0);
 
-	/* We must snapshot the length if we're streaming from the backend */
-
-	err = vrg_dorange(req, r);
-	if (err != NULL) {
-		VSLb(req->vsl, SLT_Debug, "RANGE_FAIL %s", err);
-		if (req->resp_len >= 0)
-			http_PrintfHeader(req->resp,
-			    "Content-Range: bytes */%jd",
-			    (intmax_t)req->resp_len);
-		http_PutResponse(req->resp, "HTTP/1.1", 416, NULL);
-		/*
-		 * XXX: We ought to produce a body explaining things.
-		 * XXX: That really calls for us to hit vcl_synth{}
-		 */
-		req->resp_len = 0;
-	}
+	VSLb(req->vsl, SLT_Debug, "RANGE_FAIL %s", err);
+	if (req->resp_len >= 0)
+		http_PrintfHeader(req->resp,
+		    "Content-Range: bytes */%jd",
+		    (intmax_t)req->resp_len);
+	http_PutResponse(req->resp, "HTTP/1.1", 416, NULL);
+	/*
+	 * XXX: We ought to produce a body explaining things.
+	 * XXX: That really calls for us to hit vcl_synth{}
+	 */
+	req->resp_len = 0;
+	return (1);
 }
+
+const struct vdp VDP_range = {
+	.name =		"range",
+	.init =		vrg_range_init,
+	.bytes =	vrg_range_bytes,
+	.fini =		vrg_range_fini,
+};
