@@ -43,8 +43,8 @@
 
 #include "vtim.h"
 
-void
-Req_AcctLogCharge(struct VSC_main *ds, struct req *req)
+static void
+req_AcctLogCharge(struct VSC_main_wrk *ds, struct req *req)
 {
 	struct acct_req *a;
 
@@ -63,12 +63,12 @@ Req_AcctLogCharge(struct VSC_main *ds, struct req *req)
 		    (uintmax_t)(a->resp_hdrbytes + a->resp_bodybytes));
 	}
 
-	/* Charge to main byte counters (except for ESI subrequests) */
-#define ACCT(foo)			\
-	if (req->esi_level == 0)	\
-		ds->s_##foo += a->foo;	\
-	a->foo = 0;
+	AN(req->topreq);
+	if (IS_TOPREQ(req)) {
+#define ACCT(foo) ds->s_##foo += a->foo;
 #include "tbl/acct_fields_req.h"
+	}
+	memset(a, 0, sizeof *a);
 }
 
 /*--------------------------------------------------------------------
@@ -93,7 +93,6 @@ Req_New(const struct worker *wrk, struct sess *sp)
 	AN(req);
 	req->magic = REQ_MAGIC;
 	req->sp = sp;
-	req->top = req;	// esi overrides
 
 	e = (char*)req + sz;
 	p = (char*)(req + 1);
@@ -143,12 +142,12 @@ Req_New(const struct worker *wrk, struct sess *sp)
 
 	WS_Init(req->ws, "req", p, e - p);
 
-	req->req_bodybytes = 0;
-
 	req->t_first = NAN;
 	req->t_prev = NAN;
 	req->t_req = NAN;
 
+	req->topreq = req;
+
 	return (req);
 }
 
@@ -166,8 +165,7 @@ Req_Release(struct req *req)
 #include "tbl/acct_fields_req.h"
 
 	AZ(req->vcl);
-	if (req->vsl->wid)
-		VSL_End(req->vsl);
+	AZ(req->vsl->wid);
 	sp = req->sp;
 	CHECK_OBJ_NOTNULL(sp, SESS_MAGIC);
 	pp = sp->pool;
@@ -176,6 +174,7 @@ Req_Release(struct req *req)
 	MPL_AssertSane(req);
 	VSL_Flush(req->vsl, 0);
 	req->sp = NULL;
+	req->topreq = NULL;
 	MPL_Free(pp->mpl_req, req);
 }
 
@@ -190,7 +189,9 @@ Req_Rollback(struct req *req)
 {
 	VCL_TaskLeave(req->vcl, req->privs);
 	VCL_TaskEnter(req->vcl, req->privs);
-	HTTP_Copy(req->http, req->http0);
+	HTTP_Clone(req->http, req->http0);
+	if (WS_Overflowed(req->ws))
+		req->wrk->stats->ws_client_overflow++;
 	WS_Reset(req->ws, req->ws_req);
 }
 
@@ -205,28 +206,22 @@ Req_Cleanup(struct sess *sp, struct worker *wrk, struct req *req)
 	CHECK_OBJ_NOTNULL(sp, SESS_MAGIC);
 	CHECK_OBJ_NOTNULL(wrk, WORKER_MAGIC);
 	CHECK_OBJ_NOTNULL(req, REQ_MAGIC);
+	CHECK_OBJ_NOTNULL(req->topreq, REQ_MAGIC);
 	assert(sp == req->sp);
+	AZ(req->vcl0);
 
 	req->director_hint = NULL;
 	req->restarts = 0;
 
-	AZ(req->esi_level);
 	AZ(req->privs->magic);
-	assert(req->top == req);
 
-	if (req->vcl != NULL) {
-		if (wrk->vcl != NULL)
-			VCL_Rel(&wrk->vcl);
-		wrk->vcl = req->vcl;
-		req->vcl = NULL;
-	}
+	if (req->vcl != NULL)
+		VCL_Recache(wrk, &req->vcl);
 
 	/* Charge and log byte counters */
-	if (req->vsl->wid) {
-		Req_AcctLogCharge(wrk->stats, req);
+	req_AcctLogCharge(wrk->stats, req);
+	if (req->vsl->wid)
 		VSL_End(req->vsl);
-	}
-	req->req_bodybytes = 0;
 
 	if (!isnan(req->t_prev) && req->t_prev > 0. && req->t_prev > sp->t_idle)
 		sp->t_idle = req->t_prev;
@@ -240,8 +235,12 @@ Req_Cleanup(struct sess *sp, struct worker *wrk, struct req *req)
 
 	req->hash_always_miss = 0;
 	req->hash_ignore_busy = 0;
+	req->esi_level = 0;
 	req->is_hit = 0;
 
+	if (WS_Overflowed(req->ws))
+		wrk->stats->ws_client_overflow++;
+
 	WS_Reset(req->ws, 0);
 }
 
