@@ -31,9 +31,10 @@
 
 #include <stdlib.h>
 
-#include "cache.h"
+#include "cache_varnishd.h"
 
 #include "vtim.h"
+#include "vct.h"
 
 /*--------------------------------------------------------------------
  * TTL and Age calculation in Varnish
@@ -61,12 +62,27 @@
  *
  */
 
+static inline int
+rfc2616_time(const char *p)
+{
+	char *ep;
+	int val;
+	if (*p == '-')
+		return (0);
+	val = strtoul(p, &ep, 10);
+	for (; *ep != '\0' && vct_issp(*ep); ep++)
+		continue;
+	if (*ep == '\0' || *ep == ',')
+		return (val);
+	return (0);
+}
+
 void
-RFC2616_Ttl(struct busyobj *bo, double now, double *t_origin,
+RFC2616_Ttl(struct busyobj *bo, vtim_real now, vtim_real *t_origin,
     float *ttl, float *grace, float *keep)
 {
 	unsigned max_age, age;
-	double h_date, h_expires;
+	vtim_real h_date, h_expires;
 	const char *p;
 	const struct http *hp;
 
@@ -140,12 +156,7 @@ RFC2616_Ttl(struct busyobj *bo, double now, double *t_origin,
 		if ((http_GetHdrField(hp, H_Cache_Control, "s-maxage", &p) ||
 		    http_GetHdrField(hp, H_Cache_Control, "max-age", &p)) &&
 		    p != NULL) {
-
-			if (*p == '-')
-				max_age = 0;
-			else
-				max_age = strtoul(p, NULL, 0);
-
+			max_age = rfc2616_time(p);
 			*ttl = max_age;
 			break;
 		}
@@ -190,17 +201,14 @@ RFC2616_Ttl(struct busyobj *bo, double now, double *t_origin,
 	 */
 	if (*ttl >= 0 && http_GetHdrField(hp, H_Cache_Control,
 	    "stale-while-revalidate", &p) && p != NULL) {
-
-		if (*p == '-')
-			*grace = 0;
-		else
-			*grace = strtoul(p, NULL, 0);
+		*grace = rfc2616_time(p);
 	}
 
 	VSLb(bo->vsl, SLT_TTL,
-	    "RFC %.0f %.0f %.0f %.0f %.0f %.0f %.0f %u",
+	    "RFC %.0f %.0f %.0f %.0f %.0f %.0f %.0f %u %s",
 	    *ttl, *grace, *keep, now,
-	    *t_origin, h_date, h_expires, max_age);
+	    *t_origin, h_date, h_expires, max_age,
+	    bo->uncacheable ? "uncacheable" : "cacheable");
 }
 
 /*--------------------------------------------------------------------
@@ -257,7 +265,10 @@ int
 RFC2616_Do_Cond(const struct req *req)
 {
 	const char *p, *e;
-	double ims, lm;
+	vtim_real ims, lm;
+
+	if (!http_IsStatus(req->resp, 200))
+		return (0);
 
 	/*
 	 * We MUST ignore If-Modified-Since if we have an If-None-Match
@@ -325,3 +336,68 @@ RFC2616_Vary_AE(struct http *hp)
 		http_SetHeader(hp, "Vary: Accept-Encoding");
 	}
 }
+
+/*--------------------------------------------------------------------*/
+
+void
+RFC2616_Response_Body(const struct worker *wrk, const struct busyobj *bo)
+{
+
+	CHECK_OBJ_NOTNULL(wrk, WORKER_MAGIC);
+	CHECK_OBJ_NOTNULL(bo, BUSYOBJ_MAGIC);
+
+	/*
+	 * Figure out how the fetch is supposed to happen, before the
+	 * headers are adultered by VCL
+	 */
+	if (!strcasecmp(http_GetMethod(bo->bereq), "head")) {
+		/*
+		 * A HEAD request can never have a body in the reply,
+		 * no matter what the headers might say.
+		 * [RFC7231 4.3.2 p25]
+		 */
+		wrk->stats->fetch_head++;
+		bo->htc->body_status = BS_NONE;
+	} else if (http_GetStatus(bo->beresp) <= 199) {
+		/*
+		 * 1xx responses never have a body.
+		 * [RFC7230 3.3.2 p31]
+		 * ... but we should never see them.
+		 */
+		wrk->stats->fetch_1xx++;
+		bo->htc->body_status = BS_ERROR;
+	} else if (http_IsStatus(bo->beresp, 204)) {
+		/*
+		 * 204 is "No Content", obviously don't expect a body.
+		 * [RFC7230 3.3.1 p29 and 3.3.2 p31]
+		 */
+		wrk->stats->fetch_204++;
+		if ((http_GetHdr(bo->beresp, H_Content_Length, NULL) &&
+		    bo->htc->content_length != 0) ||
+		    http_GetHdr(bo->beresp, H_Transfer_Encoding, NULL))
+			bo->htc->body_status = BS_ERROR;
+		else
+			bo->htc->body_status = BS_NONE;
+	} else if (http_IsStatus(bo->beresp, 304)) {
+		/*
+		 * 304 is "Not Modified" it has no body.
+		 * [RFC7230 3.3 p28]
+		 */
+		wrk->stats->fetch_304++;
+		bo->htc->body_status = BS_NONE;
+	} else if (bo->htc->body_status == BS_CHUNKED) {
+		wrk->stats->fetch_chunked++;
+	} else if (bo->htc->body_status == BS_LENGTH) {
+		assert(bo->htc->content_length > 0);
+		wrk->stats->fetch_length++;
+	} else if (bo->htc->body_status == BS_EOF) {
+		wrk->stats->fetch_eof++;
+	} else if (bo->htc->body_status == BS_ERROR) {
+		wrk->stats->fetch_bad++;
+	} else if (bo->htc->body_status == BS_NONE) {
+		wrk->stats->fetch_none++;
+	} else {
+		WRONG("wrong bodystatus");
+	}
+}
+
