@@ -82,6 +82,10 @@ VRT_acl_match(VRT_CTX, VCL_ACL acl, VCL_IP ip)
 
 	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);
 	CHECK_OBJ_NOTNULL(acl, VRT_ACL_MAGIC);
+	if (ip == NULL) {
+		VRT_fail(ctx, "Cannot match a null IP address");
+		return (0);
+	}
 	assert(VSA_Sane(ip));
 	return (acl->match(ctx, ip));
 }
@@ -150,7 +154,7 @@ VRT_GetHdr(VRT_CTX, const struct gethdr_s *hs)
 	if (hs->where == HDR_OBJ) {
 		CHECK_OBJ_NOTNULL(ctx->req, REQ_MAGIC);
 		CHECK_OBJ_NOTNULL(ctx->req->objcore, OBJCORE_MAGIC);
-		return(HTTP_GetHdrPack(ctx->req->wrk, ctx->req->objcore,
+		return (HTTP_GetHdrPack(ctx->req->wrk, ctx->req->objcore,
 		    hs->what));
 	}
 	hp = VRT_selecthttp(ctx, hs->where);
@@ -358,7 +362,7 @@ VRT_Strands(char *d, size_t dl, VCL_STRANDS s)
 		if (s->p[i] != NULL && *s->p[i] != '\0') {
 			x = strlen(s->p[i]);
 			if (b + x >= e)
-				return(NULL);
+				return (NULL);
 			memcpy(b, s->p[i], x);
 			b += x;
 		}
@@ -487,11 +491,11 @@ VRT_handling(VRT_CTX, unsigned hand)
 {
 
 	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);
-	if (ctx->handling == NULL)
-		return;
+	assert(hand != VCL_RET_FAIL);
+	AN(ctx->handling);
+	AZ(*ctx->handling);
 	assert(hand > 0);
 	assert(hand < VCL_RET_MAX);
-	// XXX:NOTYET assert(*ctx->handling == 0);
 	*ctx->handling = hand;
 }
 
@@ -503,16 +507,21 @@ VRT_fail(VRT_CTX, const char *fmt, ...)
 	va_list ap;
 
 	assert(ctx->vsl != NULL || ctx->msg != NULL);
+	AN(ctx->handling);
+	if (*ctx->handling == VCL_RET_FAIL)
+		return;
+	AZ(*ctx->handling);
+	AN(fmt);
 	AZ(strchr(fmt, '\n'));
 	va_start(ap, fmt);
-	if (ctx->vsl != NULL)
+	if (ctx->vsl != NULL) {
 		VSLbv(ctx->vsl, SLT_VCL_Error, fmt, ap);
-	else {
+	} else {
 		VSB_vprintf(ctx->msg, fmt, ap);
 		VSB_putc(ctx->msg, '\n');
 	}
 	va_end(ap);
-	VRT_handling(ctx, VCL_RET_FAIL);
+	*ctx->handling = VCL_RET_FAIL;
 }
 
 /*--------------------------------------------------------------------
@@ -635,7 +644,7 @@ VRT_Rollback(VRT_CTX, VCL_HTTP hp)
 		// -> VBO_Rollback ?
 		VCL_TaskLeave(ctx->bo->vcl, ctx->bo->privs);
 		VCL_TaskEnter(ctx->bo->vcl, ctx->bo->privs);
-		HTTP_Copy(ctx->bo->bereq, ctx->bo->bereq0);
+		HTTP_Clone(ctx->bo->bereq, ctx->bo->bereq0);
 		WS_Reset(ctx->bo->bereq->ws, ctx->bo->ws_bo);
 		WS_Reset(ctx->bo->ws, ctx->bo->ws_bo);
 	} else
@@ -659,7 +668,7 @@ VRT_synth_page(VRT_CTX, const char *str, ...)
 			p = "(null)";
 		if (VSB_cat(vsb, p)) {
 			VRT_fail(ctx, "synthetic(): %s",
-				 strerror(VSB_error(vsb)));
+				 vstrerror(VSB_error(vsb)));
 			break;
 		}
 		p = va_arg(ap, const char *);
@@ -669,26 +678,6 @@ VRT_synth_page(VRT_CTX, const char *str, ...)
 
 /*--------------------------------------------------------------------*/
 
-static struct VSC_main *
-vrt_stats(VRT_CTX)
-{
-	struct worker *wrk;
-
-	if (ctx->req) {
-		CHECK_OBJ_NOTNULL(ctx->req, REQ_MAGIC);
-		wrk = ctx->req->wrk;
-	} else if (ctx->bo) {
-		CHECK_OBJ_NOTNULL(ctx->bo, BUSYOBJ_MAGIC);
-		wrk = ctx->bo->wrk;
-	} else {
-		// XXX
-		return (VSC_C_main);
-	}
-
-	CHECK_OBJ_NOTNULL(wrk, WORKER_MAGIC);
-	return (wrk->stats);
-}
-
 VCL_VOID
 VRT_ban_string(VRT_CTX, VCL_STRING str)
 {
@@ -744,7 +733,7 @@ VRT_ban_string(VRT_CTX, VCL_STRING str)
 			break;
 		}
 		if (av[++i] == NULL) {
-			err = BAN_Commit(bp, vrt_stats(ctx));
+			err = BAN_Commit(bp);
 			if (err == NULL)
 				bp = NULL;
 			else
@@ -788,9 +777,8 @@ VRT_purge(VRT_CTX, VCL_DURATION ttl, VCL_DURATION grace, VCL_DURATION keep)
 	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);
 
 	if ((ctx->method & (VCL_MET_HIT|VCL_MET_MISS)) == 0) {
-		VSLb(ctx->vsl, SLT_VCL_Error,
+		VRT_fail(ctx,
 		    "purge can only happen in vcl_hit{} or vcl_miss{}");
-		VRT_handling(ctx, VCL_RET_FAIL);
 		return (0);
 	}
 
@@ -836,7 +824,7 @@ int
 VRT_strcmp(const char *s1, const char *s2)
 {
 	if (s1 == NULL || s2 == NULL)
-		return(1);
+		return (1);
 	return (strcmp(s1, s2));
 }
 
@@ -851,24 +839,33 @@ VCL_BOOL
 VRT_ipcmp(VCL_IP sua1, VCL_IP sua2)
 {
 	if (sua1 == NULL || sua2 == NULL)
-		return(1);
+		return (1);
 	return (VSA_Compare_IP(sua1, sua2));
 }
 
+/*
+ * the pointer passed as src must have at least VCL_TASK lifetime
+ */
 VCL_BLOB
-VRT_blob(VRT_CTX, const char *err, const void *src, size_t len)
+VRT_blob(VRT_CTX, const char *err, const void *src, size_t len, unsigned type)
 {
-	struct vmod_priv *p;
-	void *d;
+	struct vrt_blob *p;
 
 	p = (void *)WS_Alloc(ctx->ws, sizeof *p);
-	d = WS_Copy(ctx->ws, src, len);
-	if (p == NULL || d == NULL) {
+	if (p == NULL) {
 		VRT_fail(ctx, "Workspace overflow (%s)", err);
 		return (NULL);
 	}
-	memset(p, 0, sizeof *p);
+
+	p->type = type;
 	p->len = len;
-	p->priv = d;
+	p->blob = src;
+
 	return (p);
 }
+
+int
+VRT_VSA_GetPtr(const struct suckaddr *sua, const unsigned char ** dst)
+{
+	return (VSA_GetPtr(sua, dst));
+}
