@@ -32,11 +32,11 @@
 
 #include <stdio.h>
 
-#include "cache.h"
+#include "cache_varnishd.h"
 #include "common/heritage.h"
 
-#include "cache_director.h"
-#include "vrt.h"
+#include "vcl.h"
+
 #include "vrt_obj.h"
 
 static char vrt_hostname[255] = "";
@@ -67,7 +67,7 @@ vrt_do_string(VRT_CTX, struct http *hp, int fld,
 }
 
 #define VRT_HDR_L(obj, hdr, fld)					\
-void									\
+VCL_VOID								\
 VRT_l_##obj##_##hdr(VRT_CTX, const char *p, ...)			\
 {									\
 	va_list ap;							\
@@ -79,7 +79,7 @@ VRT_l_##obj##_##hdr(VRT_CTX, const char *p, ...)			\
 }
 
 #define VRT_HDR_R(obj, hdr, fld)					\
-const char *								\
+VCL_STRING								\
 VRT_r_##obj##_##hdr(VRT_CTX)						\
 {									\
 	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);				\
@@ -92,24 +92,27 @@ VRT_r_##obj##_##hdr(VRT_CTX)						\
 	VRT_HDR_R(obj, hdr, fld)
 
 #define VRT_STATUS_L(obj)						\
-void									\
-VRT_l_##obj##_status(VRT_CTX, long num)					\
+VCL_VOID								\
+VRT_l_##obj##_status(VRT_CTX, VCL_INT num)				\
 {									\
 									\
 	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);				\
 	CHECK_OBJ_NOTNULL(ctx->http_##obj, HTTP_MAGIC);			\
 	if (num < 0)							\
-		VRT_fail(ctx, "%s.status (%ld) is negative", #obj, num); \
+		VRT_fail(ctx, "%s.status (%jd) is negative",		\
+		    #obj, (intmax_t)num);				\
 	else if (num > 65535)						\
-		VRT_fail(ctx, "%s.status (%ld) > 65535", #obj, num);	\
+		VRT_fail(ctx, "%s.status (%jd) > 65535",		\
+		    #obj, (intmax_t)num);				\
 	else if ((num % 1000) < 100)					\
-		VRT_fail(ctx, "illegal %s.status (%ld) (..0##)", #obj, num); \
+		VRT_fail(ctx, "illegal %s.status (%jd) (..0##)",	\
+		    #obj, (intmax_t)num);				\
 	else								\
 		http_SetStatus(ctx->http_##obj, (uint16_t)num);		\
 }
 
 #define VRT_STATUS_R(obj)						\
-long									\
+VCL_INT									\
 VRT_r_##obj##_status(VRT_CTX)						\
 {									\
 									\
@@ -143,7 +146,7 @@ VRT_STATUS_R(beresp)
  * Pulling things out of the packed object->http
  */
 
-long
+VCL_INT
 VRT_r_obj_status(VRT_CTX)
 {
 	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);
@@ -153,7 +156,7 @@ VRT_r_obj_status(VRT_CTX)
 	return (HTTP_GetStatusPack(ctx->req->wrk, ctx->req->objcore));
 }
 
-const char *
+VCL_STRING
 VRT_r_obj_proto(VRT_CTX)
 {
 	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);
@@ -164,7 +167,7 @@ VRT_r_obj_proto(VRT_CTX)
 	    H__Proto));
 }
 
-const char *
+VCL_STRING
 VRT_r_obj_reason(VRT_CTX)
 {
 	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);
@@ -182,7 +185,7 @@ VRT_r_obj_reason(VRT_CTX)
 #define VBERESPW0(field)
 #define VBERESPW1(field)						\
 void									\
-VRT_l_beresp_##field(VRT_CTX, unsigned a)				\
+VRT_l_beresp_##field(VRT_CTX, VCL_BOOL a)				\
 {									\
 	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);				\
 	CHECK_OBJ_NOTNULL(ctx->bo, BUSYOBJ_MAGIC);			\
@@ -191,7 +194,7 @@ VRT_l_beresp_##field(VRT_CTX, unsigned a)				\
 
 #define VBERESPR0(field)
 #define VBERESPR1(field)						\
-unsigned								\
+VCL_BOOL								\
 VRT_r_beresp_##field(VRT_CTX)						\
 {									\
 	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);				\
@@ -206,7 +209,7 @@ VRT_r_beresp_##field(VRT_CTX)						\
 
 /*--------------------------------------------------------------------*/
 
-unsigned
+VCL_BOOL
 VRT_r_bereq_is_bgfetch(VRT_CTX)
 {
 	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);
@@ -214,7 +217,7 @@ VRT_r_bereq_is_bgfetch(VRT_CTX)
 	return (ctx->bo->is_bgfetch);
 }
 
-unsigned
+VCL_BOOL
 VRT_r_bereq_uncacheable(VRT_CTX)
 {
 	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);
@@ -222,11 +225,14 @@ VRT_r_bereq_uncacheable(VRT_CTX)
 	return (ctx->bo->do_pass);
 }
 
-void
-VRT_l_beresp_uncacheable(VRT_CTX, unsigned a)
+VCL_VOID
+VRT_l_beresp_uncacheable(VRT_CTX, VCL_BOOL a)
 {
+	struct objcore *oc;
+
 	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);
 	CHECK_OBJ_NOTNULL(ctx->bo, BUSYOBJ_MAGIC);
+	CHECK_OBJ_NOTNULL(ctx->bo->fetch_objcore, OBJCORE_MAGIC);
 
 	if (ctx->bo->uncacheable && !a) {
 		VSLb(ctx->vsl, SLT_VCL_Error,
@@ -234,9 +240,14 @@ VRT_l_beresp_uncacheable(VRT_CTX, unsigned a)
 	} else if (a) {
 		ctx->bo->uncacheable = 1;
 	}
+	oc = ctx->bo->fetch_objcore;
+
+	VSLb(ctx->vsl, SLT_TTL, "VCL %.0f %.0f %.0f %.0f %s",	\
+	    oc->ttl, oc->grace, oc->keep, oc->t_origin,		\
+	    ctx->bo->uncacheable ? "uncacheable" : "cacheable");\
 }
 
-unsigned
+VCL_BOOL
 VRT_r_beresp_uncacheable(VRT_CTX)
 {
 	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);
@@ -246,10 +257,9 @@ VRT_r_beresp_uncacheable(VRT_CTX)
 
 /*--------------------------------------------------------------------*/
 
-const char *
+VCL_STRING
 VRT_r_client_identity(VRT_CTX)
 {
-
 	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);
 	CHECK_OBJ_NOTNULL(ctx->req, REQ_MAGIC);
 	if (ctx->req->client_identity != NULL)
@@ -257,7 +267,7 @@ VRT_r_client_identity(VRT_CTX)
 	return (SES_Get_String_Attr(ctx->req->sp, SA_CLIENT_IP));
 }
 
-void
+VCL_VOID
 VRT_l_client_identity(VRT_CTX, const char *str, ...)
 {
 	va_list ap;
@@ -279,8 +289,8 @@ VRT_l_client_identity(VRT_CTX, const char *str, ...)
 /*--------------------------------------------------------------------*/
 
 #define BEREQ_TIMEOUT(which)					\
-void								\
-VRT_l_bereq_##which(VRT_CTX, double num)			\
+VCL_VOID							\
+VRT_l_bereq_##which(VRT_CTX, VCL_DURATION num)			\
 {								\
 								\
 	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);			\
@@ -288,7 +298,7 @@ VRT_l_bereq_##which(VRT_CTX, double num)			\
 	ctx->bo->which = (num > 0.0 ? num : 0.0);		\
 }								\
 								\
-double								\
+VCL_DURATION							\
 VRT_r_bereq_##which(VRT_CTX)					\
 {								\
 								\
@@ -303,7 +313,7 @@ BEREQ_TIMEOUT(between_bytes_timeout)
 
 /*--------------------------------------------------------------------*/
 
-const char *
+VCL_STRING
 VRT_r_beresp_backend_name(VRT_CTX)
 {
 
@@ -326,43 +336,7 @@ VRT_r_beresp_backend_ip(VRT_CTX)
 
 	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);
 	CHECK_OBJ_NOTNULL(ctx->bo, BUSYOBJ_MAGIC);
-	return (VDI_GetIP(ctx->bo->wrk, ctx->bo));
-}
-
-/*--------------------------------------------------------------------*/
-
-const char *
-VRT_r_beresp_storage_hint(VRT_CTX)
-{
-	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);
-	CHECK_OBJ_NOTNULL(ctx->bo, BUSYOBJ_MAGIC);
-	if (ctx->bo->storage_hint != NULL)
-		return (ctx->bo->storage_hint);
-	else
-		return (NULL);
-}
-
-void
-VRT_l_beresp_storage_hint(VRT_CTX, const char *str, ...)
-{
-	const struct stevedore *stv;
-	va_list ap;
-	const char *b;
-
-	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);
-	CHECK_OBJ_NOTNULL(ctx->bo, BUSYOBJ_MAGIC);
-	va_start(ap, str);
-	b = VRT_String(ctx->bo->ws, NULL, str, ap);	// XXX: ctx->ws ?
-	va_end(ap);
-	if (b == NULL) {
-		VSLb(ctx->vsl, SLT_LostHeader, "storage_hint");
-		WS_MarkOverflow(ctx->bo->beresp->ws);
-		return;
-	}
-	ctx->bo->storage_hint = b;
-	stv = STV_find(b);
-	if (stv != NULL)
-		ctx->bo->storage = stv;
+	return (VDI_GetIP(ctx->bo));
 }
 
 /*--------------------------------------------------------------------*/
@@ -375,7 +349,7 @@ VRT_r_req_storage(VRT_CTX)
 	return (ctx->req->storage);
 }
 
-void
+VCL_VOID
 VRT_l_req_storage(VRT_CTX, VCL_STEVEDORE stv)
 {
 	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);
@@ -393,7 +367,7 @@ VRT_r_beresp_storage(VRT_CTX)
 	return (ctx->bo->storage);
 }
 
-void
+VCL_VOID
 VRT_l_beresp_storage(VRT_CTX, VCL_STEVEDORE stv)
 {
 	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);
@@ -401,11 +375,72 @@ VRT_l_beresp_storage(VRT_CTX, VCL_STEVEDORE stv)
 	ctx->bo->storage = stv;
 }
 
+/*--------------------------------------------------------------------
+ * VCL <= 4.0 ONLY
+ */
+
+#include "storage/storage.h"
+
+VCL_STRING
+VRT_r_beresp_storage_hint(VRT_CTX)
+{
+	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);
+	CHECK_OBJ_NOTNULL(ctx->bo, BUSYOBJ_MAGIC);
+	if (ctx->bo->storage == NULL)
+		return (NULL);
+	CHECK_OBJ_NOTNULL(ctx->bo->storage, STEVEDORE_MAGIC);
+	return (ctx->bo->storage->vclname);
+}
+
+VCL_VOID
+VRT_l_beresp_storage_hint(VRT_CTX, const char *str, ...)
+{
+	const char *p;
+	va_list ap;
+	uintptr_t sn;
+	VCL_STEVEDORE stv;
+
+	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);
+	CHECK_OBJ_NOTNULL(ctx->bo, BUSYOBJ_MAGIC);
+
+	sn = WS_Snapshot(ctx->ws);
+	va_start(ap, str);
+	p = VRT_String(ctx->ws, NULL, str, ap);
+	va_end(ap);
+
+	if (p == NULL) {
+		VSLb(ctx->vsl, SLT_LostHeader, "storage_hint");
+		WS_Reset(ctx->ws, sn);
+		WS_MarkOverflow(ctx->ws);
+		return;
+	}
+
+	stv = VRT_stevedore(p);
+	if (stv != NULL)
+		ctx->bo->storage = stv;
+
+	WS_Reset(ctx->ws, sn);
+}
+
 /*--------------------------------------------------------------------*/
 
-#define REQ_VAR_L(nm, elem, type,extra)					\
+VCL_STEVEDORE
+VRT_r_obj_storage(VRT_CTX)
+{
+	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);
+	CHECK_OBJ_NOTNULL(ctx->req, REQ_MAGIC);
+	CHECK_OBJ_NOTNULL(ctx->req->objcore, OBJCORE_MAGIC);
+	AN(ctx->req->objcore->stobj);
+	CHECK_OBJ_NOTNULL(ctx->req->objcore->stobj->stevedore,
+	    STEVEDORE_MAGIC);
+	return (ctx->req->objcore->stobj->stevedore);
+}
+
+/*--------------------------------------------------------------------*/
+
+#define REQ_VAR_L(nm, elem, type, extra)				\
 									\
-void									\
+VCL_VOID								\
 VRT_l_req_##nm(VRT_CTX, type arg)					\
 {									\
 	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);				\
@@ -424,15 +459,17 @@ VRT_r_req_##nm(VRT_CTX)							\
 	return (ctx->req->elem);					\
 }
 
-REQ_VAR_L(backend_hint, director_hint, const struct director *,)
-REQ_VAR_R(backend_hint, director_hint, const struct director *)
-REQ_VAR_L(ttl, d_ttl, double, if (!(arg>0.0)) arg = 0;)
-REQ_VAR_R(ttl, d_ttl, double)
+REQ_VAR_L(backend_hint, director_hint, VCL_BACKEND,)
+REQ_VAR_R(backend_hint, director_hint, VCL_BACKEND)
+REQ_VAR_L(ttl, d_ttl, VCL_DURATION, if (!(arg>0.0)) arg = 0;)
+REQ_VAR_R(ttl, d_ttl, VCL_DURATION)
+REQ_VAR_L(grace, d_grace, VCL_DURATION, if (!(arg>0.0)) arg = 0;)
+REQ_VAR_R(grace, d_grace, VCL_DURATION)
 
 /*--------------------------------------------------------------------*/
 
-void
-VRT_l_bereq_backend(VRT_CTX, const struct director *be)
+VCL_VOID
+VRT_l_bereq_backend(VRT_CTX, VCL_BACKEND be)
 {
 
 	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);
@@ -440,7 +477,7 @@ VRT_l_bereq_backend(VRT_CTX, const struct director *be)
 	ctx->bo->director_req = be;
 }
 
-const struct director *
+VCL_BACKEND
 VRT_r_bereq_backend(VRT_CTX)
 {
 
@@ -449,7 +486,7 @@ VRT_r_bereq_backend(VRT_CTX)
 	return (ctx->bo->director_req);
 }
 
-const struct director *
+VCL_BACKEND
 VRT_r_beresp_backend(VRT_CTX)
 {
 
@@ -460,45 +497,48 @@ VRT_r_beresp_backend(VRT_CTX)
 
 /*--------------------------------------------------------------------*/
 
-void
-VRT_l_bereq_body(VRT_CTX, const char *p, ...)
+VCL_VOID
+VRT_u_bereq_body(VRT_CTX)
 {
 	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);
 	CHECK_OBJ_NOTNULL(ctx->bo, BUSYOBJ_MAGIC);
-	assert(p == vrt_magic_string_unset);
 	if (ctx->bo->req != NULL) {
-		CHECK_OBJ_NOTNULL(ctx->bo->req, REQ_MAGIC);
+		CHECK_OBJ(ctx->bo->req, REQ_MAGIC);
 		ctx->bo->req = NULL;
+		ObjSetState(ctx->bo->wrk,
+		    ctx->bo->fetch_objcore, BOS_REQ_DONE);
 		http_Unset(ctx->bo->bereq, H_Content_Length);
 	}
 }
 
 /*--------------------------------------------------------------------*/
 
-void
-VRT_l_req_esi(VRT_CTX, unsigned process_esi)
+VCL_VOID
+VRT_l_req_esi(VRT_CTX, VCL_BOOL process_esi)
 {
 
 	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);
 	CHECK_OBJ_NOTNULL(ctx->req, REQ_MAGIC);
+	assert(ctx->syntax <= 40);
 	/*
 	 * Only allow you to turn of esi in the main request
 	 * else everything gets confused
 	 */
-	if (ctx->req->esi_level == 0)
+	if (IS_TOPREQ(ctx->req))
 		ctx->req->disable_esi = !process_esi;
 }
 
-unsigned
+VCL_BOOL
 VRT_r_req_esi(VRT_CTX)
 {
 
 	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);
 	CHECK_OBJ_NOTNULL(ctx->req, REQ_MAGIC);
+	assert(ctx->syntax <= 40);
 	return (!ctx->req->disable_esi);
 }
 
-long
+VCL_INT
 VRT_r_req_esi_level(VRT_CTX)
 {
 
@@ -509,7 +549,7 @@ VRT_r_req_esi_level(VRT_CTX)
 
 /*--------------------------------------------------------------------*/
 
-unsigned
+VCL_BOOL
 VRT_r_req_can_gzip(VRT_CTX)
 {
 
@@ -520,7 +560,7 @@ VRT_r_req_can_gzip(VRT_CTX)
 
 /*--------------------------------------------------------------------*/
 
-long
+VCL_INT
 VRT_r_req_restarts(VRT_CTX)
 {
 
@@ -529,7 +569,7 @@ VRT_r_req_restarts(VRT_CTX)
 	return (ctx->req->restarts);
 }
 
-long
+VCL_INT
 VRT_r_bereq_retries(VRT_CTX)
 {
 
@@ -544,14 +584,28 @@ VRT_r_bereq_retries(VRT_CTX)
  *	ttl is relative to t_origin
  *	grace&keep are relative to ttl
  * In VCL:
- *	ttl is relative to now
+ *	ttl is relative to "ttl_now", which is t_req on the client
+ *	side, except in vcl_deliver, where it is ctx->now. On the
+ *	fetch side "ttl_now" is ctx->now (which is bo->t_prev).
  *	grace&keep are relative to ttl
  */
 
+static double
+ttl_now(VRT_CTX)
+{
+	if (ctx->bo) {
+		return (ctx->now);
+	} else {
+		CHECK_OBJ(ctx->req, REQ_MAGIC);
+		return (ctx->method == VCL_MET_DELIVER
+		    ? ctx->now : ctx->req->t_req);
+	}
+}
+
 #define VRT_DO_EXP_L(which, oc, fld, offset)			\
 								\
-void								\
-VRT_l_##which##_##fld(VRT_CTX, double a)			\
+VCL_VOID							\
+VRT_l_##which##_##fld(VRT_CTX, VCL_DURATION a)			\
 {								\
 								\
 	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);			\
@@ -559,13 +613,14 @@ VRT_l_##which##_##fld(VRT_CTX, double a)			\
 	if (a < 0.0)						\
 		a = 0.0;					\
 	oc->fld = a;						\
-	VSLb(ctx->vsl, SLT_TTL, "VCL %.0f %.0f %.0f %.0f",	\
-	    oc->ttl, oc->grace, oc->keep, oc->t_origin);	\
+	VSLb(ctx->vsl, SLT_TTL, "VCL %.0f %.0f %.0f %.0f %s",	\
+	    oc->ttl, oc->grace, oc->keep, oc->t_origin,		\
+	    ctx->bo->uncacheable ? "uncacheable" : "cacheable");\
 }
 
 #define VRT_DO_EXP_R(which, oc, fld, offset)			\
 								\
-double								\
+VCL_DURATION							\
 VRT_r_##which##_##fld(VRT_CTX)					\
 {								\
 	double d;						\
@@ -578,60 +633,85 @@ VRT_r_##which##_##fld(VRT_CTX)					\
 	return (d);						\
 }
 
+/*lint -save -e835 */	// Zero right hand arg to '-'
+
 VRT_DO_EXP_R(obj, ctx->req->objcore, ttl,
-    ctx->now - ctx->req->objcore->t_origin)
+    ttl_now(ctx) - ctx->req->objcore->t_origin)
 VRT_DO_EXP_R(obj, ctx->req->objcore, grace, 0)
 VRT_DO_EXP_R(obj, ctx->req->objcore, keep, 0)
-
 VRT_DO_EXP_L(beresp, ctx->bo->fetch_objcore, ttl,
-    ctx->now - ctx->bo->fetch_objcore->t_origin)
+    ttl_now(ctx) - ctx->bo->fetch_objcore->t_origin)
 VRT_DO_EXP_R(beresp, ctx->bo->fetch_objcore, ttl,
-    ctx->now - ctx->bo->fetch_objcore->t_origin)
+    ttl_now(ctx) - ctx->bo->fetch_objcore->t_origin)
+
 VRT_DO_EXP_L(beresp, ctx->bo->fetch_objcore, grace, 0)
 VRT_DO_EXP_R(beresp, ctx->bo->fetch_objcore, grace, 0)
 VRT_DO_EXP_L(beresp, ctx->bo->fetch_objcore, keep, 0)
 VRT_DO_EXP_R(beresp, ctx->bo->fetch_objcore, keep, 0)
 
+/*lint -restore */
+
 /*--------------------------------------------------------------------
  */
 
 #define VRT_DO_AGE_R(which, oc)					\
 								\
-double								\
+VCL_DURATION							\
 VRT_r_##which##_##age(VRT_CTX)					\
 {								\
 								\
 	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);			\
-	return (ctx->now - oc->t_origin);			\
+	return (ttl_now(ctx) - oc->t_origin);			\
 }
 
 VRT_DO_AGE_R(obj, ctx->req->objcore)
 VRT_DO_AGE_R(beresp, ctx->bo->fetch_objcore)
 
 /*--------------------------------------------------------------------
- * [be]req.xid
+ * [[be]req|sess].xid
  */
 
-const char *
+VCL_STRING
 VRT_r_req_xid(VRT_CTX)
 {
 
 	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);
 	CHECK_OBJ_NOTNULL(ctx->req, REQ_MAGIC);
+	CHECK_OBJ_NOTNULL(ctx->req->http, HTTP_MAGIC);
+	AN(ctx->req->vsl);
 
 	return (WS_Printf(ctx->req->http->ws, "%u",
 	    VXID(ctx->req->vsl->wid)));
 }
 
-const char *
+VCL_STRING
 VRT_r_bereq_xid(VRT_CTX)
 {
 
 	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);
 	CHECK_OBJ_NOTNULL(ctx->bo, BUSYOBJ_MAGIC);
+	AN(ctx->bo->vsl);
 
-	return (WS_Printf(ctx->bo->bereq->ws, "%u",
-	    VXID(ctx->bo->vsl->wid)));
+	return (WS_Printf(ctx->ws, "%u", VXID(ctx->bo->vsl->wid)));
+}
+
+VCL_STRING
+VRT_r_sess_xid(VRT_CTX)
+{
+	struct sess *sp;
+
+	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);
+
+	if (ctx->req) {
+		CHECK_OBJ(ctx->req, REQ_MAGIC);
+		sp = ctx->req->sp;
+	} else {
+		CHECK_OBJ_NOTNULL(ctx->bo, BUSYOBJ_MAGIC);
+		sp = ctx->bo->sp;
+	}
+
+	CHECK_OBJ_NOTNULL(sp, SESS_MAGIC);
+	return (WS_Printf(ctx->ws, "%u", VXID(sp->vxid)));
 }
 
 /*--------------------------------------------------------------------
@@ -640,8 +720,8 @@ VRT_r_bereq_xid(VRT_CTX)
 
 #define VREQW0(field)
 #define VREQW1(field)							\
-void									\
-VRT_l_req_##field(VRT_CTX, unsigned a)					\
+VCL_VOID								\
+VRT_l_req_##field(VRT_CTX, VCL_BOOL a)					\
 {									\
 	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);				\
 	CHECK_OBJ_NOTNULL(ctx->req, REQ_MAGIC);				\
@@ -650,7 +730,7 @@ VRT_l_req_##field(VRT_CTX, unsigned a)					\
 
 #define VREQR0(field)
 #define VREQR1(field)							\
-unsigned								\
+VCL_BOOL								\
 VRT_r_req_##field(VRT_CTX)						\
 {									\
 	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);				\
@@ -683,9 +763,37 @@ GIP(client)
 GIP(server)
 #undef GIP
 
+/*--------------------------------------------------------------------
+ * local.[endpoint|socket]
+ */
+
+#define LOC(var,fld)						\
+VCL_STRING							\
+VRT_r_local_##var(VRT_CTX)					\
+{								\
+	struct sess *sp;					\
+								\
+	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);			\
+	if (ctx->req) {						\
+		CHECK_OBJ(ctx->req, REQ_MAGIC);			\
+		sp = ctx->req->sp;				\
+	} else {						\
+		CHECK_OBJ_NOTNULL(ctx->bo, BUSYOBJ_MAGIC);	\
+		sp = ctx->bo->sp;				\
+	}							\
+								\
+	CHECK_OBJ_NOTNULL(sp->listen_sock, LISTEN_SOCK_MAGIC);	\
+	AN(sp->listen_sock->fld);				\
+	return (sp->listen_sock->fld);				\
+}
+
+LOC(endpoint, endpoint)
+LOC(socket, name)
+#undef LOC
+
 /*--------------------------------------------------------------------*/
 
-const char*
+VCL_STRING
 VRT_r_server_identity(VRT_CTX)
 {
 
@@ -696,7 +804,7 @@ VRT_r_server_identity(VRT_CTX)
 		return ("varnishd");
 }
 
-const char*
+VCL_STRING
 VRT_r_server_hostname(VRT_CTX)
 {
 
@@ -708,28 +816,30 @@ VRT_r_server_hostname(VRT_CTX)
 
 /*--------------------------------------------------------------------*/
 
-long
+VCL_INT
 VRT_r_obj_hits(VRT_CTX)
 {
 	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);
 	CHECK_OBJ_NOTNULL(ctx->req, REQ_MAGIC);
 	CHECK_OBJ_NOTNULL(ctx->req->objcore, OBJCORE_MAGIC);
+	if (ctx->method == VCL_MET_HIT)
+		return (ctx->req->objcore->hits);
 	return (ctx->req->is_hit ? ctx->req->objcore->hits : 0);
 }
 
-unsigned
+VCL_BOOL
 VRT_r_obj_uncacheable(VRT_CTX)
 {
 
 	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);
 	CHECK_OBJ_NOTNULL(ctx->req, REQ_MAGIC);
 	CHECK_OBJ_NOTNULL(ctx->req->objcore, OBJCORE_MAGIC);
-	return (ctx->req->objcore->flags & OC_F_PASS ? 1 : 0);
+	return (ctx->req->objcore->flags & OC_F_HFM ? 1 : 0);
 }
 
 /*--------------------------------------------------------------------*/
 
-unsigned
+VCL_BOOL
 VRT_r_resp_is_streaming(VRT_CTX)
 {
 	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);
@@ -742,8 +852,34 @@ VRT_r_resp_is_streaming(VRT_CTX)
 
 /*--------------------------------------------------------------------*/
 
+VCL_VOID
+VRT_l_resp_do_esi(VRT_CTX, VCL_BOOL process_esi)
+{
+
+	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);
+	CHECK_OBJ_NOTNULL(ctx->req, REQ_MAGIC);
+	assert(ctx->syntax >= 41);
+	/*
+	 * Only allow you to turn of esi in the main request
+	 * else everything gets confused
+	 */
+	ctx->req->disable_esi = !process_esi;
+}
+
+VCL_BOOL
+VRT_r_resp_do_esi(VRT_CTX)
+{
+
+	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);
+	CHECK_OBJ_NOTNULL(ctx->req, REQ_MAGIC);
+	assert(ctx->syntax >= 41);
+	return (!ctx->req->disable_esi);
+}
+
+/*--------------------------------------------------------------------*/
+
 #define VRT_BODY_L(which)					\
-void								\
+VCL_VOID							\
 VRT_l_##which##_body(VRT_CTX, const char *str, ...)		\
 {								\
 	va_list ap;						\
@@ -767,31 +903,16 @@ VRT_BODY_L(resp)
 
 /*--------------------------------------------------------------------*/
 
-static struct vmod_priv *
-vrt_do_blob(VRT_CTX, const char *err, const uint8_t *src, size_t len)
-{
-	struct vmod_priv *p;
-	void *d;
-
-	p = (void *)WS_Alloc(ctx->ws, sizeof *p);
-	d = WS_Copy(ctx->ws, src, len);
-	if (p == NULL || d == NULL) {
-		VRT_fail(ctx, "Workspace overflow (%s)", err);
-		return (NULL);
-	}
-	memset(p, 0, sizeof *p);
-	p->len = len;
-	p->priv = d;
-	return (p);
-}
+			/* digest */
+#define BLOB_HASH_TYPE 0x00d16357
 
 VCL_BLOB
 VRT_r_req_hash(VRT_CTX)
 {
 	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);
 	CHECK_OBJ_NOTNULL(ctx->req, REQ_MAGIC);
-	return (vrt_do_blob(ctx, "req.hash", ctx->req->digest,
-	    DIGEST_LEN));
+	return (VRT_blob(ctx, "req.hash", ctx->req->digest, DIGEST_LEN,
+	    BLOB_HASH_TYPE));
 }
 
 VCL_BLOB
@@ -799,14 +920,14 @@ VRT_r_bereq_hash(VRT_CTX)
 {
 	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);
 	CHECK_OBJ_NOTNULL(ctx->bo, BUSYOBJ_MAGIC);
-	return (vrt_do_blob(ctx, "bereq.hash", ctx->bo->digest,
-	    DIGEST_LEN));
+	return (VRT_blob(ctx, "bereq.hash", ctx->bo->digest, DIGEST_LEN,
+	    BLOB_HASH_TYPE));
 }
 
 /*--------------------------------------------------------------------*/
 
 #define HTTP_VAR(x)						\
-struct http *							\
+VCL_HTTP							\
 VRT_r_##x(VRT_CTX)						\
 {								\
 	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);			\
