@@ -31,7 +31,7 @@
 
 #include "config.h"
 
-#include "cache.h"
+#include "cache_varnishd.h"
 
 #include <dlfcn.h>
 #include <stdio.h>
@@ -39,7 +39,6 @@
 
 #include "vcli_serve.h"
 #include "vmod_abi.h"
-#include "vrt.h"
 
 /*--------------------------------------------------------------------
  * Modules stuff
@@ -54,6 +53,7 @@ struct vmod {
 	int			ref;
 
 	char			*nm;
+	unsigned		nbr;
 	char			*path;
 	char			*backup;
 	void			*hdl;
@@ -78,8 +78,8 @@ vmod_abi_mismatch(const struct vmod_data *d)
 }
 
 int
-VRT_Vmod_Init(VRT_CTX, struct vmod **hdl, void *ptr, int len, const char *nm,
-    const char *path, const char *file_id, const char *backup)
+VRT_Vmod_Init(VRT_CTX, struct vmod **hdl, unsigned nbr, void *ptr, int len,
+    const char *nm, const char *path, const char *file_id, const char *backup)
 {
 	struct vmod *v;
 	const struct vmod_data *d;
@@ -94,7 +94,8 @@ VRT_Vmod_Init(VRT_CTX, struct vmod **hdl, void *ptr, int len, const char *nm,
 
 	dlhdl = dlopen(backup, RTLD_NOW | RTLD_LOCAL);
 	if (dlhdl == NULL) {
-		VSB_printf(ctx->msg, "Loading vmod %s from %s:\n", nm, backup);
+		VSB_printf(ctx->msg, "Loading vmod %s from %s (%s):\n",
+		    nm, backup, path);
 		VSB_printf(ctx->msg, "dlopen() failed: %s\n", dlerror());
 		return (1);
 	}
@@ -114,8 +115,8 @@ VRT_Vmod_Init(VRT_CTX, struct vmod **hdl, void *ptr, int len, const char *nm,
 		if (d == NULL ||
 		    d->file_id == NULL ||
 		    strcmp(d->file_id, file_id)) {
-			VSB_printf(ctx->msg,
-			    "Loading vmod %s from %s:\n", nm, path);
+			VSB_printf(ctx->msg, "Loading vmod %s from %s (%s):\n",
+			    nm, backup, path);
 			VSB_printf(ctx->msg,
 			    "This is no longer the same file seen by"
 			    " the VCL-compiler.\n");
@@ -129,15 +130,16 @@ VRT_Vmod_Init(VRT_CTX, struct vmod **hdl, void *ptr, int len, const char *nm,
 		    d->func == NULL ||
 		    d->func_len <= 0 ||
 		    d->proto == NULL ||
-		    d->spec == NULL) {
-			VSB_printf(ctx->msg,
-			    "Loading VMOD %s from %s:\n", nm, path);
+		    d->json == NULL) {
+			VSB_printf(ctx->msg, "Loading vmod %s from %s (%s):\n",
+			    nm, backup, path);
 			VSB_printf(ctx->msg, "VMOD data is mangled.\n");
 			(void)dlclose(v->hdl);
 			FREE_OBJ(v);
 			return (1);
 		}
 
+		v->nbr = nbr;
 		v->funclen = d->func_len;
 		v->funcs = d->func;
 		v->abi = d->abi;
@@ -160,7 +162,7 @@ VRT_Vmod_Init(VRT_CTX, struct vmod **hdl, void *ptr, int len, const char *nm,
 }
 
 void
-VRT_Vmod_Fini(struct vmod **hdl)
+VRT_Vmod_Unload(VRT_CTX, struct vmod **hdl)
 {
 	struct vmod *v;
 
@@ -168,6 +170,9 @@ VRT_Vmod_Fini(struct vmod **hdl)
 
 	TAKE_OBJ_NOTNULL(v, hdl, VMOD_MAGIC);
 
+	VCL_TaskLeave(ctx->vcl, cli_task_privs);
+	VCL_TaskEnter(ctx->vcl, cli_task_privs);
+
 #ifndef DONT_DLCLOSE_VMODS
 	/*
 	 * atexit(3) handlers are not called during dlclose(3).  We don't
@@ -202,7 +207,7 @@ VMOD_Panic(struct vsb *vsb)
 
 /*---------------------------------------------------------------------*/
 
-static void __match_proto__(cli_func_t)
+static void v_matchproto_(cli_func_t)
 ccf_debug_vmod(struct cli *cli, const char * const *av, void *priv)
 {
 	struct vmod *v;
