@@ -52,18 +52,20 @@
 
 #include "config.h"
 
-#include <errno.h>
 #include <stdlib.h>
 
 #include "cache_varnishd.h"
 #include "cache_pool.h"
 
+#include "vcli_serve.h"
 #include "vtim.h"
 
 #include "hash/hash_slinger.h"
 
 static void Pool_Work_Thread(struct pool *pp, struct worker *wrk);
 
+static uintmax_t reqpoolfail;
+
 /*--------------------------------------------------------------------
  * Create and start a back-ground thread which as its own worker and
  * session data structures;
@@ -82,7 +84,7 @@ wrk_bgthread(void *arg)
 {
 	struct bgthread *bt;
 	struct worker wrk;
-	struct VSC_main ds;
+	struct VSC_main_wrk ds;
 
 	CAST_OBJ_NOTNULL(bt, arg, BGTHREAD_MAGIC);
 	THR_SetName(bt->name);
@@ -119,7 +121,7 @@ WRK_Thread(struct pool *qp, size_t stacksize, unsigned thread_workspace)
 {
 	// child_signal_handler stack overflow check uses struct worker addr
 	struct worker *w, ww;
-	struct VSC_main ds;
+	struct VSC_main_wrk ds;
 	unsigned char ws[thread_workspace];
 
 	AN(qp);
@@ -154,11 +156,11 @@ WRK_Thread(struct pool *qp, size_t stacksize, unsigned thread_workspace)
  */
 
 static void
-pool_addstat(struct VSC_main *dst, struct VSC_main *src)
+pool_addstat(struct VSC_main_wrk *dst, struct VSC_main_wrk *src)
 {
 
 	dst->summs++;
-	VSC_main_Summ(dst, src);
+	VSC_main_Summ_wrk_wrk(dst, src);
 	memset(src, 0, sizeof *src);
 }
 
@@ -261,6 +263,17 @@ Pool_Task(struct pool *pp, struct pool_task *task, enum task_prio prio)
 	AN(task->func);
 	assert(prio < TASK_QUEUE_END);
 
+	if (prio == TASK_QUEUE_REQ && reqpoolfail) {
+		retval = reqpoolfail & 1;
+		reqpoolfail >>= 1;
+		if (retval) {
+			VSL(SLT_Debug, 0,
+			    "Failing due to reqpoolfail (next= 0x%jx)",
+			    reqpoolfail);
+			return (retval);
+		}
+	}
+
 	Lck_Lock(&pp->mtx);
 
 	/* The common case first:  Take an idle thread, do it. */
@@ -321,18 +334,18 @@ Pool_Work_Thread(struct pool *pp, struct worker *wrk)
 {
 	struct pool_task *tp = NULL;
 	struct pool_task tpx, tps;
+	vtim_real tmo;
 	int i, prio_lim;
 
 	CHECK_OBJ_NOTNULL(pp, POOL_MAGIC);
 	wrk->pool = pp;
 	while (1) {
-		Lck_Lock(&pp->mtx);
-
 		CHECK_OBJ_NOTNULL(wrk, WORKER_MAGIC);
 
 		WS_Reset(wrk->aws, 0);
 		AZ(wrk->vsl);
 
+		Lck_Lock(&pp->mtx);
 		if (pp->nidle < pool_reserve())
 			prio_lim = TASK_QUEUE_RESERVE + 1;
 		else
@@ -371,8 +384,13 @@ Pool_Work_Thread(struct pool *pp, struct worker *wrk)
 			pp->nidle++;
 			do {
 				// see signaling_note at the top for explanation
-				i = Lck_CondWait(&wrk->cond, &pp->mtx,
-				    wrk->vcl == NULL ?  0 : wrk->lastused+60.);
+				if (wrk->vcl == NULL)
+					tmo = 0;
+				else if (DO_DEBUG(DBG_VTC_MODE))
+					tmo =  wrk->lastused+1.;
+				else
+					tmo =  wrk->lastused+60.;
+				i = Lck_CondWait(&wrk->cond, &pp->mtx, tmo);
 				if (i == ETIMEDOUT)
 					VCL_Rel(&wrk->vcl);
 			} while (wrk->task.func == NULL);
@@ -395,6 +413,8 @@ Pool_Work_Thread(struct pool *pp, struct worker *wrk)
 			tp = &tpx;
 		} while (tp->func != NULL);
 
+		if (WS_Overflowed(wrk->aws))
+			wrk->stats->ws_thread_overflow++;
 		/* cleanup for next task */
 		wrk->seen_methods = 0;
 	}
@@ -446,7 +466,7 @@ pool_breed(struct pool *qp)
 
 	if (pthread_create(&tp, &tp_attr, pool_thread, pi)) {
 		VSL(SLT_Debug, 0, "Create worker thread failed %d %s",
-		    errno, strerror(errno));
+		    errno, vstrerror(errno));
 		Lck_Lock(&pool_mtx);
 		VSC_C_main->threads_failed++;
 		Lck_Unlock(&pool_mtx);
@@ -603,3 +623,40 @@ pool_herder(void *priv)
 	}
 	return (NULL);
 }
+
+/*--------------------------------------------------------------------
+ * Debugging aids
+ */
+
+static void v_matchproto_(cli_func_t)
+debug_reqpoolfail(struct cli *cli, const char * const *av, void *priv)
+{
+	uintmax_t u = 1;
+	const char *p;
+
+	(void)priv;
+	(void)cli;
+	reqpoolfail = 0;
+	for (p = av[2]; *p != '\0'; p++) {
+		if (*p == 'F' || *p == 'f')
+			reqpoolfail |= u;
+		u <<= 1;
+	}
+}
+
+static struct cli_proto debug_cmds[] = {
+	{ CLICMD_DEBUG_REQPOOLFAIL,		"d", debug_reqpoolfail },
+	{ NULL }
+};
+
+/*--------------------------------------------------------------------
+ *
+ */
+
+void
+WRK_Init(void)
+{
+
+	CLI_AddFuncs(debug_cmds);
+}
+
