@@ -35,7 +35,7 @@
 
 /*--------------------------------------------------------------------*/
 
-static int v_matchproto_(vdp_bytes)
+static int v_matchproto_(vdp_bytes_f)
 v1d_bytes(struct req *req, enum vdp_action act, void **priv,
     const void *ptr, ssize_t len)
 {
@@ -43,8 +43,6 @@ v1d_bytes(struct req *req, enum vdp_action act, void **priv,
 
 	CHECK_OBJ_NOTNULL(req, REQ_MAGIC);
 	(void)priv;
-	if (act == VDP_INIT || act == VDP_FINI)
-		return (0);
 
 	AZ(req->vdc->nxt);		/* always at the bottom of the pile */
 
@@ -59,7 +57,7 @@ v1d_bytes(struct req *req, enum vdp_action act, void **priv,
 
 static const struct vdp v1d_vdp = {
 	.name =		"V1B",
-	.func =		v1d_bytes,
+	.bytes =	v1d_bytes,
 };
 
 static void
@@ -75,6 +73,7 @@ v1d_error(struct req *req, const char *msg)
 	VSLb(req->vsl, SLT_RespStatus, "500");
 	VSLb(req->vsl, SLT_RespReason, "Internal Server Error");
 
+	req->wrk->stats->client_resp_500++;
 	(void)write(req->sp->fd, r_500, sizeof r_500 - 1);
 	req->doclose = SC_TX_EOF;
 }
@@ -85,7 +84,7 @@ v1d_error(struct req *req, const char *msg)
 void v_matchproto_(vtr_deliver_f)
 V1D_Deliver(struct req *req, struct boc *boc, int sendbody)
 {
-	int err;
+	int err = 0, chunked = 0;
 	unsigned u;
 	uint64_t hdrbytes, bytes;
 
@@ -93,18 +92,6 @@ V1D_Deliver(struct req *req, struct boc *boc, int sendbody)
 	CHECK_OBJ_ORNULL(boc, BOC_MAGIC);
 	CHECK_OBJ_NOTNULL(req->objcore, OBJCORE_MAGIC);
 
-	if (sendbody) {
-		if (http_GetHdr(req->resp, H_Content_Length, NULL))
-			req->res_mode |= RES_LEN;
-		else if (req->http->protover == 11) {
-			req->res_mode |= RES_CHUNKED;
-			http_SetHeader(req->resp, "Transfer-Encoding: chunked");
-		} else {
-			req->res_mode |= RES_EOF;
-			req->doclose = SC_TX_EOF;
-		}
-	}
-
 	if (!req->doclose && http_HdrIs(req->resp, H_Connection, "close")) {
 		req->doclose = SC_RESP_CLOSE;
 	} else if (req->doclose) {
@@ -115,6 +102,23 @@ V1D_Deliver(struct req *req, struct boc *boc, int sendbody)
 	} else if (!http_GetHdr(req->resp, H_Connection, NULL))
 		http_SetHeader(req->resp, "Connection: keep-alive");
 
+	if (sendbody) {
+		if (!http_GetHdr(req->resp, H_Content_Length, NULL)) {
+			if (req->http->protover == 11) {
+				chunked = 1;
+				http_SetHeader(req->resp,
+				    "Transfer-Encoding: chunked");
+			} else {
+				req->doclose = SC_TX_EOF;
+			}
+		}
+		if (VDP_Push(req, &v1d_vdp, NULL)) {
+			v1d_error(req, "workspace_thread overflow");
+			AZ(req->wrk->v1l);
+			return;
+		}
+	}
+
 	if (WS_Overflowed(req->ws)) {
 		v1d_error(req, "workspace_client overflow");
 		return;
@@ -125,18 +129,9 @@ V1D_Deliver(struct req *req, struct boc *boc, int sendbody)
 		return;
 	}
 
-	if (req->resp_len == 0)
-		sendbody = 0;
-
-	if (sendbody && VDP_push(req, &v1d_vdp, NULL, 1)) {
-		v1d_error(req, "workspace_thread overflow");
-		AZ(req->wrk->v1l);
-		return;
-	}
-
 	AZ(req->wrk->v1l);
 	V1L_Open(req->wrk, req->wrk->aws,
-		 &req->sp->fd, req->vsl, req->t_prev, 0);
+		 &req->sp->fd, req->vsl, req->t_prev, cache_param->http1_iovs);
 
 	if (WS_Overflowed(req->wrk->aws)) {
 		v1d_error(req, "workspace_thread overflow");
@@ -146,49 +141,18 @@ V1D_Deliver(struct req *req, struct boc *boc, int sendbody)
 
 	hdrbytes = HTTP1_Write(req->wrk, req->resp, HTTP1_Resp);
 
-	if (DO_DEBUG(DBG_FLUSH_HEAD))
-		(void)V1L_Flush(req->wrk);
-
-	if (!sendbody || req->res_mode & RES_ESI) {
-		if (V1L_Close(req->wrk, &bytes) && req->sp->fd >= 0) {
-			Req_Fail(req, SC_REM_CLOSE);
-			sendbody = 0;
-		}
-
-		/* Charge bytes sent as reported from V1L_Close. Only
-		 * header-bytes have been attempted sent. */
-		req->acct.resp_hdrbytes += bytes;
-		hdrbytes = 0;
+	if (sendbody) {
+		if (DO_DEBUG(DBG_FLUSH_HEAD))
+			(void)V1L_Flush(req->wrk);
+		if (chunked)
+			V1L_Chunked(req->wrk);
+		err = VDP_DeliverObj(req);
+		if (!err && chunked)
+			V1L_EndChunk(req->wrk);
 	}
 
-	if (!sendbody) {
-		AZ(req->wrk->v1l);
-		VDP_close(req);
-		return;
-	}
-
-	AN(sendbody);
-	if (req->res_mode & RES_ESI) {
-		AZ(req->wrk->v1l);
-
-		V1L_Open(req->wrk, req->wrk->aws,
-			 &req->sp->fd, req->vsl, req->t_prev,
-			 cache_param->esi_iovs);
-
-		if (WS_Overflowed(req->wrk->aws)) {
-			v1d_error(req, "workspace_thread overflow");
-			AZ(req->wrk->v1l);
-			return;
-		}
-	}
-
-	if (req->res_mode & RES_CHUNKED)
-		V1L_Chunked(req->wrk);
-	err = VDP_DeliverObj(req);
-	if (!err && (req->res_mode & RES_CHUNKED))
-		V1L_EndChunk(req->wrk);
-
 	u = V1L_Close(req->wrk, &bytes);
+	AZ(req->wrk->v1l);
 
 	/* Bytes accounting */
 	if (bytes < hdrbytes)
@@ -200,6 +164,5 @@ V1D_Deliver(struct req *req, struct boc *boc, int sendbody)
 
 	if ((u || err) && req->sp->fd >= 0)
 		Req_Fail(req, SC_REM_CLOSE);
-	AZ(req->wrk->v1l);
 	VDP_close(req);
 }
