@@ -32,7 +32,6 @@
 #include "cache/cache_varnishd.h"
 #include "cache/cache_filter.h"
 
-#include <errno.h>
 #include <stdio.h>
 #include <stdlib.h>
 
@@ -46,7 +45,7 @@
  */
 
 static int v_matchproto_(objiterate_f)
-vbf_iter_req_body(void *priv, int flush, const void *ptr, ssize_t l)
+vbf_iter_req_body(void *priv, unsigned flush, const void *ptr, ssize_t l)
 {
 	struct busyobj *bo;
 
@@ -98,7 +97,7 @@ V1F_SendReq(struct worker *wrk, struct busyobj *bo, uint64_t *ctr_hdrbytes,
 
 	VSLb(bo->vsl, SLT_BackendStart, "%s %s", abuf, pbuf);
 
-	(void)VTCP_blocking(*htc->rfd);	/* XXX: we should timeout instead */
+	VTCP_blocking(*htc->rfd);	/* XXX: we should timeout instead */
 	V1L_Open(wrk, wrk->aws, htc->rfd, bo->vsl, bo->t_prev, 0);
 	hdrbytes = HTTP1_Write(wrk, hp, HTTP1_Req);
 
@@ -112,10 +111,19 @@ V1F_SendReq(struct worker *wrk, struct busyobj *bo, uint64_t *ctr_hdrbytes,
 		i = VRB_Iterate(bo->req, vbf_iter_req_body, bo);
 
 		if (bo->req->req_body_status == REQ_BODY_FAIL) {
+			/*
+			 * XXX: (#2332) We should test to see if the backend
+			 * XXX: sent us some headers explaining why.
+			 * XXX: This is hard because of the mistaken API split
+			 * XXX: between cache_backend.c and V1F, and therefore
+			 * XXX: Parked in this comment, pending renovation of
+			 * XXX: the VDI/backend-protocol API to allow non-H1
+			 * XXX: backends.
+			 */
 			assert(i < 0);
 			VSLb(bo->vsl, SLT_FetchError,
 			    "req.body read error: %d (%s)",
-			    errno, strerror(errno));
+			    errno, vstrerror(errno));
 			bo->req->doclose = SC_RX_BODY;
 		}
 		if (do_chunked)
@@ -134,7 +142,7 @@ V1F_SendReq(struct worker *wrk, struct busyobj *bo, uint64_t *ctr_hdrbytes,
 
 	if (j != 0 || i < 0) {
 		VSLb(bo->vsl, SLT_FetchError, "backend write error: %d (%s)",
-		    errno, strerror(errno));
+		    errno, vstrerror(errno));
 		VSLb_ts_busyobj(bo, "Bereq", W_TIM_real(wrk));
 		htc->doclose = SC_TX_ERROR;
 		return (-1);
@@ -170,7 +178,7 @@ V1F_FetchRespHdr(struct busyobj *bo)
 
 	t = VTIM_real() + htc->first_byte_timeout;
 	hs = HTC_RxStuff(htc, HTTP1_Complete, NULL, NULL,
-	    t, t + htc->between_bytes_timeout, cache_param->http_resp_size);
+	    t, NAN, htc->between_bytes_timeout, cache_param->http_resp_size);
 	if (hs != HTC_S_COMPLETE) {
 		bo->acct.beresp_hdrbytes +=
 		    htc->rxbuf_e - htc->rxbuf_b;
@@ -191,6 +199,10 @@ V1F_FetchRespHdr(struct busyobj *bo)
 			VSLb(bo->vsl, SLT_FetchError, "overflow");
 			htc->doclose = SC_RX_OVERFLOW;
 			break;
+		case HTC_S_IDLE:
+			VSLb(bo->vsl, SLT_FetchError, "first byte timeout");
+			htc->doclose = SC_RX_TIMEOUT;
+			break;
 		default:
 			VSLb(bo->vsl, SLT_FetchError, "HTC %s (%d)",
 			     HTC_Status(hs), hs);
