@@ -30,14 +30,18 @@
  * We try to use writev() if possible in order to minimize number of
  * syscalls made and packets sent.  It also just might allow the worker
  * thread to complete the request without holding stuff locked.
+ *
+ * XXX: chunked header (generated in Flush) and Tail (EndChunk)
+ *      are not accounted by means of the size_t returned. Obvious ideas:
+ *	- add size_t return value to Flush and EndChunk
+ *	- base accounting on (struct v1l).cnt
  */
 
 #include "config.h"
 
 #include <sys/uio.h>
-#include "cache/cache.h"
+#include "cache/cache_varnishd.h"
 
-#include <errno.h>
 #include <stdio.h>
 
 #include "cache_http1.h"
@@ -64,11 +68,13 @@ struct v1l {
 };
 
 /*--------------------------------------------------------------------
+ * for niov == 0, reserve the ws for max number of iovs
+ * otherwise, up to niov
  */
 
 void
-V1L_Reserve(struct worker *wrk, struct ws *ws, int *fd, struct vsl_log *vsl,
-    double t0)
+V1L_Open(struct worker *wrk, struct ws *ws, int *fd, struct vsl_log *vsl,
+    double t0, unsigned niov)
 {
 	struct v1l *v1l;
 	unsigned u;
@@ -79,7 +85,12 @@ V1L_Reserve(struct worker *wrk, struct ws *ws, int *fd, struct vsl_log *vsl,
 
 	if (WS_Overflowed(ws))
 		return;
+
+	if (niov != 0)
+		assert(niov >= 3);
+
 	res = WS_Snapshot(ws);
+
 	v1l = WS_Alloc(ws, sizeof *v1l);
 	if (v1l == NULL)
 		return;
@@ -97,30 +108,35 @@ V1L_Reserve(struct worker *wrk, struct ws *ws, int *fd, struct vsl_log *vsl,
 	}
 	if (u > IOV_MAX)
 		u = IOV_MAX;
+	if (niov != 0 && u > niov)
+		u = niov;
 	v1l->iov = (void*)ws->f;
 	v1l->siov = u;
 	v1l->ciov = u;
-	v1l->werr = 0;
-	v1l->liov = 0;
-	v1l->niov = 0;
 	v1l->wfd = fd;
 	v1l->t0 = t0;
 	v1l->vsl = vsl;
 	wrk->v1l = v1l;
+
+	if (niov != 0)
+		WS_Release(ws, u * sizeof(struct iovec));
 }
 
 unsigned
-V1L_FlushRelease(struct worker *wrk)
+V1L_Close(struct worker *wrk, uint64_t *cnt)
 {
 	struct v1l *v1l;
 	unsigned u;
 
 	CHECK_OBJ_NOTNULL(wrk, WORKER_MAGIC);
+	AN(cnt);
 	u = V1L_Flush(wrk);
 	v1l = wrk->v1l;
 	wrk->v1l = NULL;
 	CHECK_OBJ_NOTNULL(v1l, V1L_MAGIC);
-	WS_Release(v1l->ws, 0);
+	*cnt = v1l->cnt;
+	if (v1l->ws->r)
+		WS_Release(v1l->ws, 0);
 	WS_Reset(v1l->ws, v1l->res);
 	return (u);
 }
@@ -190,7 +206,7 @@ V1L_Flush(const struct worker *wrk)
 			 * was sent.
 			 *
 			 * XXX: Add a "minimum sent data per timeout
-			 * counter to prevent slowlaris attacks
+			 * counter to prevent slowloris attacks
 			*/
 
 			if (VTIM_real() - v1l->t0 > cache_param->send_timeout) {
@@ -215,7 +231,7 @@ V1L_Flush(const struct worker *wrk)
 			v1l->werr++;
 			VSLb(v1l->vsl, SLT_Debug,
 			    "Write error, retval = %zd, len = %zd, errno = %s",
-			    i, v1l->liov, strerror(errno));
+			    i, v1l->liov, vstrerror(errno));
 		}
 	}
 	v1l->liov = 0;
