@@ -34,8 +34,6 @@
 #include <sys/types.h>
 #include <sys/wait.h>
 
-#include <errno.h>
-#include <fcntl.h>
 #include <poll.h>
 #include <signal.h>
 #include <stdarg.h>
@@ -117,12 +115,9 @@ mgt_panic_clear(void)
 	VSB_destroy(&child_panic);
 }
 
-static void v_matchproto_(cli_func_t)
-mch_cli_panic_show(struct cli *cli, const char * const *av, void *priv)
+static void
+cli_panic_show(struct cli *cli, const char * const *av, int json)
 {
-	(void)av;
-	(void)priv;
-
 	if (!child_panic) {
 		VCLI_SetResult(cli, CLIS_CANT);
 		VCLI_Out(cli,
@@ -130,7 +125,29 @@ mch_cli_panic_show(struct cli *cli, const char * const *av, void *priv)
 		return;
 	}
 
-	VCLI_Out(cli, "%s\n", VSB_data(child_panic));
+	if (!json) {
+		VCLI_Out(cli, "%s\n", VSB_data(child_panic));
+		return;
+	}
+
+	VCLI_JSON_begin(cli, 2, av);
+	VCLI_Out(cli, ",\n");
+	VCLI_JSON_str(cli, VSB_data(child_panic));
+	VCLI_JSON_end(cli);
+}
+
+static void v_matchproto_(cli_func_t)
+mch_cli_panic_show(struct cli *cli, const char * const *av, void *priv)
+{
+	(void)priv;
+	cli_panic_show(cli, av, 0);
+}
+
+static void v_matchproto_(cli_func_t)
+mch_cli_panic_show_json(struct cli *cli, const char * const *av, void *priv)
+{
+	(void)priv;
+	cli_panic_show(cli, av, 1);
 }
 
 static void v_matchproto_(cli_func_t)
@@ -163,12 +180,16 @@ mch_cli_panic_clear(struct cli *cli, const char * const *av, void *priv)
  *
  * This is likely to a bit on the low side, as libc and other libraries
  * has a tendency to cache file descriptors (syslog, resolver, etc.)
- * so we add a margin of 100 fds.
+ * so we add a margin of 10 fds.
+ *
+ * For added safety, we check that we see no file descriptor open for
+ * another margin above the limit for which we close by design
  */
 
 static int		mgt_max_fd;
 
-#define CLOSE_FD_UP_TO	(mgt_max_fd + 100)
+#define CLOSE_FD_UP_TO	(mgt_max_fd + 10)
+#define CHECK_FD_UP_TO	(CLOSE_FD_UP_TO + 10)
 
 void
 MCH_TrackHighFd(int fd)
@@ -192,6 +213,9 @@ MCH_Fd_Inherit(int fd, const char *what)
 {
 
 	assert(fd >= 0);
+	// XXX why?
+	if (fd > 0)
+		MCH_TrackHighFd(fd);
 	if (fd_map == NULL)
 		fd_map = vbit_new(128);
 	AN(fd_map);
@@ -325,12 +349,16 @@ mgt_launch_child(struct cli *cli)
 		 */
 		closelog();
 
-		for (i = STDERR_FILENO + 1; i < CLOSE_FD_UP_TO; i++) {
+		for (i = STDERR_FILENO + 1; i <= CLOSE_FD_UP_TO; i++) {
 			if (vbit_test(fd_map, i))
 				continue;
 			if (close(i) == 0)
 				VFIL_null_fd(i);
 		}
+		for (i = CLOSE_FD_UP_TO + 1; i <= CHECK_FD_UP_TO; i++) {
+			assert(close(i) == -1);
+			assert(errno == EBADF);
+		}
 
 		mgt_ProcTitle("Child");
 
@@ -588,7 +616,7 @@ MCH_Cli_Fail(void)
 		    " killed it.", (intmax_t)child_pid);
 	else
 		MGT_Complain(C_ERR, "Failed to kill child with PID %jd: %s",
-		    (intmax_t)child_pid, strerror(errno));
+		    (intmax_t)child_pid, vstrerror(errno));
 }
 
 /*=====================================================================
@@ -620,7 +648,7 @@ MCH_Start_Child(void)
 	mgt_launch_child(NULL);
 	if (child_state != CH_RUNNING)
 		return (2);
-	return(0);
+	return (0);
 }
 
 /*====================================================================
@@ -679,11 +707,23 @@ mch_cli_server_status(struct cli *cli, const char * const *av, void *priv)
 	VCLI_Out(cli, "Child in state %s", ch_state[child_state]);
 }
 
+static void v_matchproto_(cli_func_t)
+mch_cli_server_status_json(struct cli *cli, const char * const *av, void *priv)
+{
+	(void)priv;
+	VCLI_JSON_begin(cli, 2, av);
+	VCLI_Out(cli, ", ");
+	VCLI_JSON_str(cli, ch_state[child_state]);
+	VCLI_JSON_end(cli);
+}
+
 static struct cli_proto cli_mch[] = {
-	{ CLICMD_SERVER_STATUS,		"", mch_cli_server_status },
+	{ CLICMD_SERVER_STATUS,		"", mch_cli_server_status,
+	  mch_cli_server_status_json },
 	{ CLICMD_SERVER_START,		"", mch_cli_server_start },
 	{ CLICMD_SERVER_STOP,		"", mch_cli_server_stop },
-	{ CLICMD_PANIC_SHOW,		"", mch_cli_panic_show },
+	{ CLICMD_PANIC_SHOW,		"", mch_cli_panic_show,
+	  mch_cli_panic_show_json },
 	{ CLICMD_PANIC_CLEAR,		"", mch_cli_panic_clear },
 	{ NULL }
 };
