@@ -30,7 +30,6 @@
 
 #include "config.h"
 
-#include <errno.h>
 #include <fcntl.h>
 #include <grp.h>
 #include <pwd.h>
@@ -41,6 +40,7 @@
 #include <sys/stat.h>
 
 #include "mgt/mgt.h"
+#include "common/heritage.h"
 
 #ifdef __linux__
 #include <sys/prctl.h>
@@ -119,7 +119,7 @@ vju_getccgid(const char *arg)
 /**********************************************************************
  */
 
-static int __match_proto__(jail_init_f)
+static int v_matchproto_(jail_init_f)
 vju_init(char **args)
 {
 	if (args == NULL) {
@@ -183,7 +183,7 @@ vju_init(char **args)
 	return (0);
 }
 
-static void __match_proto__(jail_master_f)
+static void v_matchproto_(jail_master_f)
 vju_master(enum jail_master_e jme)
 {
 	if (jme == JAIL_MASTER_LOW) {
@@ -195,7 +195,7 @@ vju_master(enum jail_master_e jme)
 	}
 }
 
-static void __match_proto__(jail_subproc_f)
+static void v_matchproto_(jail_subproc_f)
 vju_subproc(enum jail_subproc_e jse)
 {
 	int i;
@@ -238,14 +238,26 @@ vju_subproc(enum jail_subproc_e jse)
 #endif
 }
 
-static int __match_proto__(jail_make_dir_f)
-vju_make_vcldir(const char *dname)
+static int v_matchproto_(jail_make_dir_f)
+vju_make_subdir(const char *dname, const char *what, struct vsb *vsb)
 {
+	int e;
+
+	AN(dname);
+	AN(what);
 	AZ(seteuid(0));
 
 	if (mkdir(dname, 0755) < 0 && errno != EEXIST) {
-		MGT_Complain(C_ERR, "Cannot create VCL directory '%s': %s",
-		    dname, strerror(errno));
+		e = errno;
+		if (vsb != NULL) {
+			VSB_printf(vsb,
+			    "Cannot create %s directory '%s': %s\n",
+			    what, dname, vstrerror(e));
+		} else {
+			MGT_Complain(C_ERR,
+			    "Cannot create %s directory '%s': %s",
+			    what, dname, vstrerror(e));
+		}
 		return (1);
 	}
 	AZ(chown(dname, vju_uid, vju_gid));
@@ -253,8 +265,26 @@ vju_make_vcldir(const char *dname)
 	return (0);
 }
 
+static int v_matchproto_(jail_make_dir_f)
+vju_make_workdir(const char *dname, const char *what, struct vsb *vsb)
+{
 
-static void __match_proto__(jail_fixfd_f)
+	AN(dname);
+	AZ(what);
+	AZ(vsb);
+	AZ(seteuid(0));
+
+	if (mkdir(dname, 0755) < 0 && errno != EEXIST) {
+		MGT_Complain(C_ERR, "Cannot create working directory '%s': %s",
+		    dname, vstrerror(errno));
+		return (1);
+	}
+	AZ(chown(dname, -1, vju_gid));
+	AZ(seteuid(vju_uid));
+	return (0);
+}
+
+static void v_matchproto_(jail_fixfd_f)
 vju_fixfd(int fd, enum jail_fixfd_e what)
 {
 	/* Called under JAIL_MASTER_FILE */
@@ -282,7 +312,8 @@ const struct jail_tech jail_tech_unix = {
 	.name =		"unix",
 	.init =		vju_init,
 	.master =	vju_master,
-	.make_vcldir =	vju_make_vcldir,
+	.make_subdir =	vju_make_subdir,
+	.make_workdir =	vju_make_workdir,
 	.fixfd =	vju_fixfd,
 	.subproc =	vju_subproc,
 };
