@@ -43,6 +43,8 @@
 #include "vav.h"
 #include "vnum.h"
 
+const char * const JSON_FMT = (const char *)&JSON_FMT;
+
 /*--------------------------------------------------------------------
  * Generic handling of double typed parameters
  */
@@ -53,7 +55,7 @@ tweak_generic_double(struct vsb *vsb, volatile double *dest,
 {
 	volatile double u, minv = 0, maxv = 0;
 
-	if (arg != NULL) {
+	if (arg != NULL && arg != JSON_FMT) {
 		if (min != NULL) {
 			minv = VNUM(min);
 			if (isnan(minv)) {
@@ -123,7 +125,7 @@ tweak_bool(struct vsb *vsb, const struct parspec *par, const char *arg)
 	volatile unsigned *dest;
 
 	dest = par->priv;
-	if (arg != NULL) {
+	if (arg != NULL && arg != JSON_FMT) {
 		if (!strcasecmp(arg, "off"))
 			*dest = 0;
 		else if (!strcasecmp(arg, "disable"))
@@ -144,6 +146,8 @@ tweak_bool(struct vsb *vsb, const struct parspec *par, const char *arg)
 			VSB_printf(vsb, "use \"on\" or \"off\"\n");
 			return (-1);
 		}
+	} else if (arg == JSON_FMT) {
+		VSB_printf(vsb, "%s", *dest ? "true" : "false");
 	} else {
 		VSB_printf(vsb, "%s", *dest ? "on" : "off");
 	}
@@ -159,7 +163,7 @@ tweak_generic_uint(struct vsb *vsb, volatile unsigned *dest, const char *arg,
 	unsigned u, minv = 0, maxv = 0;
 	char *p;
 
-	if (arg != NULL) {
+	if (arg != NULL && arg != JSON_FMT) {
 		if (min != NULL) {
 			p = NULL;
 			minv = strtoul(min, &p, 0);
@@ -195,7 +199,7 @@ tweak_generic_uint(struct vsb *vsb, volatile unsigned *dest, const char *arg,
 			return (-1);
 		}
 		*dest = u;
-	} else if (*dest == UINT_MAX) {
+	} else if (*dest == UINT_MAX && arg != JSON_FMT) {
 		VSB_printf(vsb, "unlimited");
 	} else {
 		VSB_printf(vsb, "%u", *dest);
@@ -246,7 +250,7 @@ tweak_generic_bytes(struct vsb *vsb, volatile ssize_t *dest, const char *arg,
 	uintmax_t r, rmin = 0, rmax = 0;
 	const char *p;
 
-	if (arg != NULL) {
+	if (arg != NULL && arg != JSON_FMT) {
 		if (min != NULL) {
 			p = VNUM_2bytes(min, &rmin, 0);
 			if (p != NULL) {
@@ -285,6 +289,8 @@ tweak_generic_bytes(struct vsb *vsb, volatile ssize_t *dest, const char *arg,
 			return (-1);
 		}
 		*dest = r;
+	} else if (arg == JSON_FMT) {
+		VSB_printf(vsb, "%zd", *dest);
 	} else {
 		fmt_bytes(vsb, *dest);
 	}
@@ -334,7 +340,6 @@ tweak_vsl_buffer(struct vsb *vsb, const struct parspec *par, const char *arg)
 		return (-1);
 	*d1 = dest;
 	MCF_ParamConf(MCF_MAXIMUM, "vsl_reclen", "%u", *d1 - 12);
-	MCF_ParamConf(MCF_MAXIMUM, "shm_reclen", "%u", *d1 - 12);
 	return (0);
 }
 
@@ -364,6 +369,8 @@ tweak_string(struct vsb *vsb, const struct parspec *par, const char *arg)
 	/* XXX should have tweak_generic_string */
 	if (arg == NULL) {
 		VSB_quote(vsb, *p, -1, 0);
+	} else if (arg == JSON_FMT) {
+		VSB_quote(vsb, *p, -1, VSB_QUOTE_JSON|VSB_QUOTE_CSTR);
 	} else {
 		REPLACE(*p, arg);
 	}
@@ -380,7 +387,15 @@ tweak_poolparam(struct vsb *vsb, const struct parspec *par, const char *arg)
 	int retval = 0;
 
 	pp = par->priv;
-	if (arg == NULL) {
+	if (arg == JSON_FMT) {
+		VSB_printf(vsb, "{\n");
+		VSB_indent(vsb, 8);
+		VSB_printf(vsb, "\"min_pool\": %u,\n", pp->min_pool);
+		VSB_printf(vsb, "\"max_pool\": %u,\n", pp->max_pool);
+		VSB_printf(vsb, "\"max_age\": %g\n", pp->max_age);
+		VSB_indent(vsb, -4);
+		VSB_printf(vsb, "}");
+	} else if (arg == NULL) {
 		VSB_printf(vsb, "%u,%u,%g",
 		    pp->min_pool, pp->max_pool, pp->max_age);
 	} else {
