@@ -31,7 +31,6 @@
 
 #include "config.h"
 
-#include <errno.h>
 #include <fcntl.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -48,10 +47,12 @@
 #include "vev.h"
 #include "vtim.h"
 
-static const char * const VCL_STATE_COLD = "cold";
-static const char * const VCL_STATE_WARM = "warm";
-static const char * const VCL_STATE_AUTO = "auto";
+#define VCL_STATE(sym, str)					\
+	static const char * const VCL_STATE_ ## sym = str;
+#include "tbl/vcl_states.h"
+
 static const char * const VCL_STATE_LABEL = "label";
+
 static int vcl_count;
 
 struct vclprog;
@@ -107,6 +108,18 @@ static int mgt_vcl_setstate(struct cli *, struct vclprog *, const char *);
 
 /*--------------------------------------------------------------------*/
 
+static const char *
+mcf_vcl_parse_state(const char *s)
+{
+	if (s == NULL || *s == '\0')
+		return (NULL);
+#define VCL_STATE(sym, str)					\
+	if (!strcmp(s, VCL_STATE_ ## sym))			\
+		return (VCL_STATE_ ## sym);
+#include "tbl/vcl_states.h"
+	return (NULL);
+}
+
 static struct vclprog *
 mcf_vcl_byname(const char *name)
 {
@@ -172,7 +185,7 @@ mcf_find_no_vcl(struct cli *cli, const char *name)
 static int
 mcf_is_label(const struct vclprog *vp)
 {
-	return (!strcmp(vp->state, VCL_STATE_LABEL));
+	return (vp->state == VCL_STATE_LABEL);
 }
 
 /*--------------------------------------------------------------------*/
@@ -257,7 +270,8 @@ mgt_vcl_del(struct vclprog *vp)
 	if (vp->state != VCL_STATE_LABEL)
 		vcl_count--;
 	if (vp->fname != NULL) {
-		AZ(unlink(vp->fname));
+		if (!MGT_DO_DEBUG(DBG_VCL_KEEP))
+			AZ(unlink(vp->fname));
 		p = strrchr(vp->fname, '/');
 		AN(p);
 		*p = '\0';
@@ -318,13 +332,13 @@ mgt_vcl_cache_vmod(const char *nm, const char *fm, const char *to)
 		return (0);
 	if (fo < 0) {
 		fprintf(stderr, "Creating copy of vmod %s: %s\n",
-		    nm, strerror(errno));
+		    nm, vstrerror(errno));
 		return (1);
 	}
 	fi = open(fm, O_RDONLY);
 	if (fi < 0) {
 		fprintf(stderr, "Opening vmod %s from %s: %s\n",
-		    nm, fm, strerror(errno));
+		    nm, fm, vstrerror(errno));
 		AZ(unlink(to));
 		closefd(&fo);
 		return (1);
@@ -335,7 +349,7 @@ mgt_vcl_cache_vmod(const char *nm, const char *fm, const char *to)
 			break;
 		if (sz < 0 || sz != write(fo, buf, sz)) {
 			fprintf(stderr, "Copying vmod %s: %s\n",
-			    nm, strerror(errno));
+			    nm, vstrerror(errno));
 			AZ(unlink(to));
 			ret = 1;
 			break;
@@ -344,7 +358,7 @@ mgt_vcl_cache_vmod(const char *nm, const char *fm, const char *to)
 	closefd(&fi);
 	AZ(fchmod(fo, 0444));
 	closefd(&fo);
-	return(ret);
+	return (ret);
 }
 
 void
@@ -480,13 +494,10 @@ mgt_new_vcl(struct cli *cli, const char *vclname, const char *vclsrc,
 
 	if (state == NULL)
 		state = VCL_STATE_AUTO;
-	else if (!strcmp(state, VCL_STATE_AUTO))
-		state = VCL_STATE_AUTO;
-	else if (!strcmp(state, VCL_STATE_COLD))
-		state = VCL_STATE_COLD;
-	else if (!strcmp(state, VCL_STATE_WARM))
-		state = VCL_STATE_WARM;
-	else {
+	else
+		state = mcf_vcl_parse_state(state);
+
+	if (state == NULL) {
 		VCLI_Out(cli, "State must be one of auto, cold or warm.");
 		VCLI_SetResult(cli, CLIS_PARAM);
 		return;
@@ -526,7 +537,7 @@ mgt_new_vcl(struct cli *cli, const char *vclname, const char *vclsrc,
 	}
 	free(p);
 
-	if (vp->warm && !strcmp(vp->state, "auto"))
+	if (vp->warm && vp->state == VCL_STATE_AUTO)
 		vp->go_cold = VTIM_mono();
 }
 
@@ -645,6 +656,7 @@ mcf_vcl_load(struct cli *cli, const char * const *av, void *priv)
 static void v_matchproto_(cli_func_t)
 mcf_vcl_state(struct cli *cli, const char * const *av, void *priv)
 {
+	const char *state;
 	struct vclprog *vp;
 
 	(void)priv;
@@ -657,22 +669,30 @@ mcf_vcl_state(struct cli *cli, const char * const *av, void *priv)
 		VCLI_SetResult(cli, CLIS_PARAM);
 		return;
 	}
+
+	state = mcf_vcl_parse_state(av[3]);
+	if (state == NULL) {
+		VCLI_Out(cli, "State must be one of auto, cold or warm.");
+		VCLI_SetResult(cli, CLIS_PARAM);
+		return;
+	}
+
 	if (!VTAILQ_EMPTY(&vp->dto)) {
-		AN(strcmp(vp->state, "cold"));
-		if (!strcmp(av[3], "cold")) {
+		assert(vp->state != VCL_STATE_COLD);
+		if (state == VCL_STATE_COLD) {
 			VCLI_Out(cli, "A labeled VCL cannot be set cold");
 			VCLI_SetResult(cli, CLIS_CANT);
 			return;
 		}
 	}
 
-	if (!strcmp(vp->state, av[3]))
+	if (vp->state == state)
 		return;
 
-	if (!strcmp(av[3], VCL_STATE_AUTO)) {
+	if (state == VCL_STATE_AUTO) {
 		vp->state = VCL_STATE_AUTO;
 		(void)mgt_vcl_setstate(cli, vp, VCL_STATE_AUTO);
-	} else if (!strcmp(av[3], VCL_STATE_COLD)) {
+	} else if (state == VCL_STATE_COLD) {
 		if (vp == active_vcl) {
 			VCLI_Out(cli, "Cannot set the active VCL cold.");
 			VCLI_SetResult(cli, CLIS_CANT);
@@ -680,7 +700,7 @@ mcf_vcl_state(struct cli *cli, const char * const *av, void *priv)
 		}
 		vp->state = VCL_STATE_AUTO;
 		(void)mgt_vcl_setstate(cli, vp, VCL_STATE_COLD);
-	} else if (!strcmp(av[3], VCL_STATE_WARM)) {
+	} else if (state == VCL_STATE_WARM) {
 		if (mgt_vcl_setstate(cli, vp, VCL_STATE_WARM) == 0)
 			vp->state = VCL_STATE_WARM;
 	} else {
@@ -764,9 +784,9 @@ mcf_vcl_discard(struct cli *cli, const char * const *av, void *priv)
 	else
 		(void)mgt_vcl_setstate(cli, vp, VCL_STATE_COLD);
 	if (MCH_Running()) {
-		/* XXX If this fails the child is crashing, figure that later */
 		assert(vp->state != VCL_STATE_WARM);
-		(void)mgt_cli_askchild(&status, &p, "vcl.discard %s\n", av[2]);
+		if (mgt_cli_askchild(&status, &p, "vcl.discard %s\n", av[2]))
+			assert(status == CLIS_OK || status == CLIS_COMMS);
 		free(p);
 	}
 	mgt_vcl_del(vp);
@@ -779,11 +799,13 @@ mcf_vcl_list(struct cli *cli, const char * const *av, void *priv)
 	char *p;
 	struct vclprog *vp;
 	struct vcldep *vd;
+	struct vsb *vsb;
 
 	/* NB: Shall generate same output as vcl_cli_list() */
 
 	(void)av;
 	(void)priv;
+
 	if (MCH_Running()) {
 		if (!mgt_cli_askchild(&status, &p, "vcl.list\n")) {
 			VCLI_SetResult(cli, status);
@@ -791,26 +813,84 @@ mcf_vcl_list(struct cli *cli, const char * const *av, void *priv)
 		}
 		free(p);
 	} else {
+		vsb = VSB_new_auto();
+		AN(vsb);
+
 		VTAILQ_FOREACH(vp, &vclhead, list) {
-			VCLI_Out(cli, "%-10s %5s",
-			    vp == active_vcl ? "active" : "available",
-			    vp->state);
-			VCLI_Out(cli, "/%-8s", vp->warm ?
+			VSB_printf(vsb, "%s",
+			    vp == active_vcl ? "active" : "available");
+			VSB_printf(vsb, "\t%s\t%s", vp->state, vp->warm ?
 			    VCL_STATE_WARM : VCL_STATE_COLD);
-			VCLI_Out(cli, " %6s %s", "-", vp->name);
+			VSB_printf(vsb, "\t%6s\t%s", "-", vp->name);
 			if (mcf_is_label(vp)) {
 				vd = VTAILQ_FIRST(&vp->dfrom);
 				AN(vd);
-				VCLI_Out(cli, " -> %s", vd->to->name);
+				VSB_printf(vsb, "\t->\t%s", vd->to->name);
 				if (vp->nto > 0)
-					VCLI_Out(cli, " (%d return(vcl)%s)",
+					VSB_printf(vsb, " (%d return(vcl)%s)",
 					    vp->nto, vp->nto > 1 ? "'s" : "");
 			} else if (vp->nto > 0) {
-				VCLI_Out(cli, " (%d label%s)",
+				VSB_printf(vsb, "\t<-\t(%d label%s)",
 				    vp->nto, vp->nto > 1 ? "s" : "");
 			}
-			VCLI_Out(cli, "\n");
+			VSB_printf(vsb, "\n");
 		}
+		VCLI_VTE(cli, &vsb, 80);
+	}
+}
+
+static void v_matchproto_(cli_func_t)
+mcf_vcl_list_json(struct cli *cli, const char * const *av, void *priv)
+{
+	unsigned status;
+	char *p;
+	struct vclprog *vp;
+	struct vcldep *vd;
+
+	/* NB: Shall generate same output as vcl_cli_list() */
+
+	(void)priv;
+	if (MCH_Running()) {
+		if (!mgt_cli_askchild(&status, &p, "vcl.list -j\n")) {
+			VCLI_SetResult(cli, status);
+			VCLI_Out(cli, "%s", p);
+		}
+		free(p);
+	} else {
+		VCLI_JSON_begin(cli, 2, av);
+		VCLI_Out(cli, ",\n");
+		VTAILQ_FOREACH(vp, &vclhead, list) {
+			VCLI_Out(cli, "{\n");
+			VSB_indent(cli->sb, 2);
+			VCLI_Out(cli, "\"status\": \"%s\",\n",
+			    vp == active_vcl ? "active" : "available");
+			VCLI_Out(cli, "\"state\": \"%s\",\n", vp->state);
+			VCLI_Out(cli, "\"temperature\": \"%s\",\n",
+			    vp->warm ? VCL_STATE_WARM : VCL_STATE_COLD);
+			VCLI_Out(cli, "\"name\": \"%s\"", vp->name);
+			if (mcf_is_label(vp)) {
+				vd = VTAILQ_FIRST(&vp->dfrom);
+				AN(vd);
+				VCLI_Out(cli, ",\n");
+				VCLI_Out(cli, "\"label\": {\n");
+				VSB_indent(cli->sb, 2);
+				VCLI_Out(cli, "\"name\": \"%s\"", vd->to->name);
+				if (vp->nto > 0)
+					VCLI_Out(cli, ",\n\"refs\": %d",
+						 vp->nto);
+				VSB_indent(cli->sb, -2);
+				VCLI_Out(cli, "\n");
+				VCLI_Out(cli, "}");
+			} else if (vp->nto > 0) {
+				VCLI_Out(cli, ",\n");
+				VCLI_Out(cli, "\"labels\": %d", vp->nto);
+			}
+			VSB_indent(cli->sb, -2);
+			VCLI_Out(cli, "\n}");
+			if (VTAILQ_NEXT(vp, list) != NULL)
+				VCLI_Out(cli, ",\n");
+		}
+		VCLI_JSON_end(cli);
 	}
 }
 
@@ -843,6 +923,13 @@ mcf_vcl_label(struct cli *cli, const char * const *av, void *priv)
 			VCLI_Out(cli, "%s is not a label", vpl->name);
 			return;
 		}
+		if (!VTAILQ_EMPTY(&vpl->dfrom) &&
+		    VTAILQ_FIRST(&vpl->dfrom)->to == vpt) {
+			VCLI_SetResult(cli, CLIS_PARAM);
+			VCLI_Out(cli, "VCL '%s' already has label '%s'",
+			    vpt->name, vpl->name);
+			return;
+		}
 		if (!VTAILQ_EMPTY(&vpt->dfrom) &&
 		    !VTAILQ_EMPTY(&vpl->dto)) {
 			VCLI_SetResult(cli, CLIS_PARAM);
@@ -907,7 +994,7 @@ static struct cli_proto cli_vcl[] = {
 	{ CLICMD_VCL_USE,		"", mcf_vcl_use },
 	{ CLICMD_VCL_STATE,		"", mcf_vcl_state },
 	{ CLICMD_VCL_DISCARD,		"", mcf_vcl_discard },
-	{ CLICMD_VCL_LIST,		"", mcf_vcl_list },
+	{ CLICMD_VCL_LIST,		"", mcf_vcl_list, mcf_vcl_list_json },
 	{ CLICMD_VCL_LABEL,		"", mcf_vcl_label },
 	{ NULL }
 };
