@@ -40,7 +40,6 @@
 
 #include <sys/mman.h>
 
-#include <errno.h>
 #include <stdio.h>
 #include <stdlib.h>
 
@@ -205,7 +204,7 @@ smp_mgt_init(struct stevedore *parent, int ac, char * const *av)
 
 	if (sc->base == MAP_FAILED)
 		ARGV_ERR("(-spersistent) failed to mmap (%s)\n",
-		    strerror(errno));
+		    vstrerror(errno));
 	if (target != NULL && sc->base != target)
 		fprintf(stderr, "WARNING: Persistent silo lost to ASLR %s\n",
 		    sc->filename);
