@@ -65,7 +65,7 @@ struct storage {
 typedef void storage_init_f(struct stevedore *, int ac, char * const *av);
 typedef void storage_open_f(struct stevedore *);
 typedef int storage_allocobj_f(struct worker *, const struct stevedore *,
-    struct objcore *, unsigned ltot, int really);
+    struct objcore *, unsigned);
 typedef void storage_close_f(const struct stevedore *, int pass);
 typedef int storage_baninfo_f(const struct stevedore *, enum baninfo event,
     const uint8_t *ban, unsigned len);
@@ -82,7 +82,6 @@ typedef void sml_free_f(struct storage *);
 #define VRTSTVVAR(nm,vt,ct,def) \
     typedef ct stv_var_##nm(const struct stevedore *);
 #include "tbl/vrt_stv_var.h"
-#undef VRTSTVVAR
 
 /*--------------------------------------------------------------------*/
 
@@ -115,21 +114,23 @@ struct stevedore {
 
 #define VRTSTVVAR(nm, vtype, ctype, dval) stv_var_##nm *var_##nm;
 #include "tbl/vrt_stv_var.h"
-#undef VRTSTVVAR
 
 	/* private fields for the stevedore */
 	void			*priv;
 
 	VTAILQ_ENTRY(stevedore)	list;
-	char			ident[16];	/* XXX: match VSM_chunk.ident */
-	char			*vclname;
+	const char		*ident;
+	const char		*vclname;
 };
 
-VTAILQ_HEAD(stevedore_head, stevedore);
-
-extern struct stevedore_head stv_stevedores;
 extern struct stevedore *stv_transient;
 
+/*--------------------------------------------------------------------*/
+
+#define STV_Foreach(arg) for (arg = NULL; STV__iter(&arg);)
+
+int STV__iter(struct stevedore ** const );
+
 /*--------------------------------------------------------------------*/
 int STV_GetFile(const char *fn, int *fdp, const char **fnp, const char *ctx);
 uintmax_t STV_FileSize(int fd, const char *size, unsigned *granularity,
@@ -138,15 +139,13 @@ uintmax_t STV_FileSize(int fd, const char *size, unsigned *granularity,
 /*--------------------------------------------------------------------*/
 struct lru *LRU_Alloc(void);
 void LRU_Free(struct lru **);
-void LRU_Add(struct objcore *, double now);
+void LRU_Add(struct objcore *, vtim_real now);
 void LRU_Remove(struct objcore *);
 int LRU_NukeOne(struct worker *, struct lru *);
-void LRU_Touch(struct worker *, struct objcore *, double now);
+void LRU_Touch(struct worker *, struct objcore *, vtim_real now);
 
 /*--------------------------------------------------------------------*/
+extern const struct stevedore smu_stevedore;
 extern const struct stevedore sma_stevedore;
 extern const struct stevedore smf_stevedore;
 extern const struct stevedore smp_stevedore;
-#ifdef HAVE_LIBUMEM
-extern const struct stevedore smu_stevedore;
-#endif
