@@ -33,12 +33,13 @@
 
 #if defined(HAVE_KQUEUE)
 
-#include "cache/cache.h"
+#include "cache/cache_varnishd.h"
 
 #include <sys/event.h>
 
 #include <stdlib.h>
 
+#include "waiter/waiter.h"
 #include "waiter/waiter_priv.h"
 #include "vtim.h"
 
@@ -130,12 +131,12 @@ vwk_thread(void *priv)
 	closefd(&vwk->pipe[0]);
 	closefd(&vwk->pipe[1]);
 	closefd(&vwk->kq);
-	return(NULL);
+	return (NULL);
 }
 
 /*--------------------------------------------------------------------*/
 
-static int __match_proto__(waiter_enter_f)
+static int v_matchproto_(waiter_enter_f)
 vwk_enter(void *priv, struct waited *wp)
 {
 	struct vwk *vwk;
@@ -158,7 +159,7 @@ vwk_enter(void *priv, struct waited *wp)
 
 /*--------------------------------------------------------------------*/
 
-static void __match_proto__(waiter_init_f)
+static void v_matchproto_(waiter_init_f)
 vwk_init(struct waiter *w)
 {
 	struct vwk *vwk;
@@ -184,7 +185,7 @@ vwk_init(struct waiter *w)
  * fail somehow.
  */
 
-static void __match_proto__(waiter_fini_f)
+static void v_matchproto_(waiter_fini_f)
 vwk_fini(struct waiter *w)
 {
 	struct vwk *vwk;
