@@ -68,13 +68,13 @@
 
 #include <sys/time.h>
 
-#include <errno.h>
 #include <port.h>
 #include <stdio.h>
 #include <stdlib.h>
 
-#include "cache/cache.h"
+#include "cache/cache_varnishd.h"
 
+#include "waiter/waiter.h"
 #include "waiter/waiter_priv.h"
 #include "waiter/mgt_waiter.h"
 #include "vtim.h"
@@ -211,7 +211,7 @@ vws_thread(void *priv)
 		for (ei = 0; ei < nevents; ei++)
 			vws_port_ev(vws, w, &ev[ei], now);
 	}
-	return NULL;
+	return (NULL);
 }
 
 /*--------------------------------------------------------------------*/
@@ -232,7 +232,7 @@ vws_enter(void *priv, struct waited *wp)
 
 /*--------------------------------------------------------------------*/
 
-static void __match_proto__(waiter_init_f)
+static void v_matchproto_(waiter_init_f)
 vws_init(struct waiter *w)
 {
 	struct vws *vws;
@@ -249,7 +249,7 @@ vws_init(struct waiter *w)
 
 /*--------------------------------------------------------------------*/
 
-static void __match_proto__(waiter_fini_f)
+static void v_matchproto_(waiter_fini_f)
 vws_fini(struct waiter *w)
 {
 	struct vws *vws;
