@@ -33,7 +33,6 @@
 
 #include "config.h"
 
-#include <errno.h>
 #include <limits.h>
 #include <math.h>
 #include <pthread.h>
@@ -372,14 +371,6 @@ accumulate(struct VSL_data *vsl, struct VSL_transaction * const pt[],
 	return (0);
 }
 
-static int v_matchproto_(VUT_cb_f)
-sighup(struct VUT *v)
-{
-	assert(v == vut);
-	quit = 1;
-	return (1);
-}
-
 static void * v_matchproto_(pthread_t)
 do_curses(void *arg)
 {
@@ -393,7 +384,7 @@ do_curses(void *arg)
 	intrflush(stdscr, FALSE);
 	curs_set(0);
 	erase();
-	while (!quit) {
+	while (!quit && !vut->last_sighup) {
 		AZ(pthread_mutex_lock(&mtx));
 		update();
 		AZ(pthread_mutex_unlock(&mtx));
@@ -473,18 +464,6 @@ do_curses(void *arg)
 
 /*--------------------------------------------------------------------*/
 
-static void v_noreturn_
-usage(int status)
-{
-	const char **opt;
-
-	fprintf(stderr, "Usage: %s <options>\n\n", vut->progname);
-	fprintf(stderr, "Options:\n");
-	for (opt = vopt_spec.vopt_usage; *opt != NULL; opt += 2)
-		fprintf(stderr, " %-25s %s\n", *opt, *(opt + 1));
-	exit(status);
-}
-
 static void v_noreturn_
 profile_error(const char *s)
 {
@@ -493,14 +472,6 @@ profile_error(const char *s)
 	exit(1);
 }
 
-static void
-vut_sighandler(int sig)
-{
-
-	if (vut != NULL)
-		VUT_Signaled(vut, sig);
-}
-
 int
 main(int argc, char **argv)
 {
@@ -519,7 +490,7 @@ main(int argc, char **argv)
 		switch (i) {
 		case 'h':
 			/* Usage help */
-			usage(0);
+			VUT_Usage(vut, &vopt_spec, 0);
 		case 'p':
 			ms_delay = lround(1e3 * strtod(optarg, NULL));
 			if (ms_delay <= 0)
@@ -602,12 +573,12 @@ main(int argc, char **argv)
 			break;
 		default:
 			if (!VUT_Arg(vut, i, optarg))
-				usage(1);
+				VUT_Usage(vut, &vopt_spec, 1);
 		}
 	}
 
 	if (optind != argc)
-		usage(1);
+		VUT_Usage(vut, &vopt_spec, 1);
 
 	/* Check for valid grouping mode */
 	assert(vut->g_arg < VSL_g__MAX);
@@ -651,7 +622,6 @@ main(int argc, char **argv)
 
 	log_ten = log(10.0);
 
-	VUT_Signal(vut_sighandler);
 	VUT_Setup(vut);
 	if (vut->vsm)
 		ident = VSM_Dup(vut->vsm, "Arg", "-i");
@@ -660,8 +630,7 @@ main(int argc, char **argv)
 	AZ(pthread_create(&thr, NULL, do_curses, NULL));
 	vut->dispatch_f = accumulate;
 	vut->dispatch_priv = NULL;
-	vut->sighup_f = sighup;
-	VUT_Main(vut);
+	(void)VUT_Main(vut);
 	end_of_file = 1;
 	AZ(pthread_join(thr, NULL));
 	VUT_Fini(&vut);
