@@ -39,13 +39,10 @@
 
 #include "config.h"
 
-#define _WITH_GETLINE
-
 #include <stdlib.h>
 #include <stdio.h>
 #include <unistd.h>
 #include <string.h>
-#include <errno.h>
 #include <signal.h>
 #include <stdarg.h>
 #include <inttypes.h>
@@ -57,11 +54,12 @@
 #define VOPT_DEFINITION
 #define VOPT_INC "varnishncsa_options.h"
 
-#include "base64.h"
+#include "vdef.h"
+
+#include "b64.h"
 #include "vapi/vsl.h"
 #include "vapi/voptget.h"
 #include "vas.h"
-#include "vdef.h"
 #include "vsb.h"
 #include "vut.h"
 #include "vqueue.h"
@@ -161,18 +159,6 @@ static struct ctx {
 	int32_t			vxid;
 } CTX;
 
-static void __attribute__((__noreturn__))
-usage(int status)
-{
-	const char **opt;
-
-	fprintf(stderr, "Usage: %s <options>\n\n", vut->progname);
-	fprintf(stderr, "Options:\n");
-	for (opt = vopt_spec.vopt_usage; *opt != NULL; opt += 2)
-		fprintf(stderr, " %-25s %s\n", *opt, *(opt + 1));
-	exit(status);
-}
-
 static void
 openout(int append)
 {
@@ -184,20 +170,20 @@ openout(int append)
 		    strerror(errno));
 }
 
-static int __match_proto__(VUT_cb_f)
+static int v_matchproto_(VUT_cb_f)
 rotateout(struct VUT *v)
 {
 
 	assert(v == vut);
 	AN(CTX.w_arg);
 	AN(CTX.fo);
-	fclose(CTX.fo);
+	(void)fclose(CTX.fo);
 	openout(1);
 	AN(CTX.fo);
 	return (0);
 }
 
-static int __match_proto__(VUT_cb_f)
+static int v_matchproto_(VUT_cb_f)
 flushout(struct VUT *v)
 {
 
@@ -208,65 +194,20 @@ flushout(struct VUT *v)
 	return (0);
 }
 
-static int
-vsb_esc_cat(struct vsb *sb, const char *b, const char *e)
-{
-	AN(b);
-
-	for (; b < e; b++) {
-		if (isspace(*b)) {
-			switch (*b) {
-			case '\n':
-				VSB_cat(sb, "\\n");
-				break;
-			case '\t':
-				VSB_cat(sb, "\\t");
-				break;
-			case '\f':
-				VSB_cat(sb, "\\f");
-				break;
-			case '\r':
-				VSB_cat(sb, "\\r");
-				break;
-			case '\v':
-				VSB_cat(sb, "\\v");
-				break;
-			default:
-				VSB_putc(sb, *b);
-				break;
-			}
-		} else if (isprint(*b)) {
-			switch (*b) {
-			case '"':
-				VSB_cat(sb, "\\\"");
-				break;
-			case '\\':
-				VSB_cat(sb, "\\\\");
-				break;
-			default:
-				VSB_putc(sb, *b);
-				break;
-			}
-		} else
-			VSB_printf(sb, "\\x%02hhx", *b);
-	}
-
-	return (VSB_error(sb));
-}
-
 static inline int
 vsb_fcat(struct vsb *vsb, const struct fragment *f, const char *dflt)
 {
 	if (f->gen == CTX.gen) {
 		assert(f->b <= f->e);
-		return (vsb_esc_cat(vsb, f->b, f->e));
-	}
-	if (dflt)
-		return (vsb_esc_cat(vsb, dflt, dflt + strlen(dflt)));
-	return (-1);
+		VSB_quote(vsb, f->b, f->e - f->b, VSB_QUOTE_ESCHEX);
+	} else if (dflt)
+		VSB_quote(vsb, dflt, -1, VSB_QUOTE_ESCHEX);
+	else
+		return (-1);
+	return (0);
 }
 
-static int __match_proto__(format_f)
+static int v_matchproto_(format_f)
 format_string(const struct format *format)
 {
 
@@ -276,7 +217,7 @@ format_string(const struct format *format)
 	return (1);
 }
 
-static int __match_proto__(format_f)
+static int v_matchproto_(format_f)
 format_strptr(const struct format *format)
 {
 
@@ -287,7 +228,7 @@ format_strptr(const struct format *format)
 	return (1);
 }
 
-static int __match_proto__(format_f)
+static int v_matchproto_(format_f)
 format_int32(const struct format *format)
 {
 
@@ -296,7 +237,7 @@ format_int32(const struct format *format)
 	return (1);
 }
 
-static int __match_proto__(format_f)
+static int v_matchproto_(format_f)
 format_fragment(const struct format *format)
 {
 
@@ -306,15 +247,14 @@ format_fragment(const struct format *format)
 	if (format->frag->gen != CTX.gen) {
 		if (format->string == NULL)
 			return (-1);
-		AZ(vsb_esc_cat(CTX.vsb, format->string,
-		    format->string + strlen(format->string)));
+		VSB_quote(CTX.vsb, format->string, -1, VSB_QUOTE_ESCHEX);
 		return (0);
 	}
 	AZ(vsb_fcat(CTX.vsb, format->frag, NULL));
 	return (1);
 }
 
-static int __match_proto__(format_f)
+static int v_matchproto_(format_f)
 format_time(const struct format *format)
 {
 	double t_start, t_end;
@@ -353,9 +293,9 @@ format_time(const struct format *format)
 		break;
 	case 't':
 		AN(format->time_fmt);
-		t = t_start;
-		localtime_r(&t, &tm);
-		strftime(buf, sizeof buf, format->time_fmt, &tm);
+		t = (long)floor(t_start);
+		(void)localtime_r(&t, &tm);
+		AN(strftime(buf, sizeof buf, format->time_fmt, &tm));
 		AZ(VSB_cat(CTX.vsb, buf));
 		break;
 	case 'T':
@@ -368,7 +308,7 @@ format_time(const struct format *format)
 	return (1);
 }
 
-static int __match_proto__(format_f)
+static int v_matchproto_(format_f)
 format_requestline(const struct format *format)
 {
 
@@ -388,7 +328,7 @@ format_requestline(const struct format *format)
 	return (1);
 }
 
-static int __match_proto__(format_f)
+static int v_matchproto_(format_f)
 format_auth(const struct format *format)
 {
 	char buf[128];
@@ -399,14 +339,13 @@ format_auth(const struct format *format)
 	    CTX.frag[F_auth].e)) {
 		if (format->string == NULL)
 			return (-1);
-		AZ(vsb_esc_cat(CTX.vsb, format->string,
-		    format->string + strlen(format->string)));
+		VSB_quote(CTX.vsb, format->string, -1, VSB_QUOTE_ESCHEX);
 		return (0);
 	}
 	q = strchr(buf, ':');
 	if (q != NULL)
 		*q = '\0';
-	AZ(vsb_esc_cat(CTX.vsb, buf, buf + strlen(buf)));
+	VSB_quote(CTX.vsb, buf, -1, VSB_QUOTE_ESCHEX);
 	return (1);
 }
 
@@ -420,6 +359,7 @@ print(void)
 	VTAILQ_FOREACH(f, &CTX.format, list) {
 		CHECK_OBJ_NOTNULL(f, FORMAT_MAGIC);
 		i = (f->func)(f);
+		AZ(VSB_error(CTX.vsb));
 		if (r > i)
 			r = i;
 	}
@@ -569,6 +509,9 @@ addf_vsl(enum VSL_tag_e tag, long i, const char *prefix)
 
 	ALLOC_OBJ(w, VSL_WATCH_MAGIC);
 	AN(w);
+	if (VSL_tagflags[tag])
+		VUT_Error(vut, 1, "Tag %s can contain control characters",
+		    VSL_tags[tag]);
 	w->tag = tag;
 	assert(i <= INT_MAX);
 	w->idx = i;
@@ -666,7 +609,7 @@ parse_x_format(char *buf)
 			VUT_Error(vut, 1, "Unknown log tag: %s", buf);
 		assert(slt >= 0);
 
-		addf_vsl(slt, lval, r);
+		addf_vsl((enum VSL_tag_e)slt, lval, r);
 		return;
 	}
 	VUT_Error(vut, 1, "Unknown formatting extension: %s", buf);
@@ -688,13 +631,12 @@ parse_format(const char *format)
 	for (p = format; *p != '\0'; p++) {
 
 		/* Allow the most essential escape sequences in format */
-		if (*p == '\\') {
-			p++;
-			if (*p == 't')
+		if (*p == '\\' && p[1] != '\0') {
+			if (*++p == 't')
 				AZ(VSB_putc(vsb, '\t'));
 			else if (*p == 'n')
 				AZ(VSB_putc(vsb, '\n'));
-			else if (*p != '\0')
+			else
 				AZ(VSB_putc(vsb, *p));
 			continue;
 		}
@@ -833,33 +775,31 @@ frag_fields(int force, const char *b, const char *e, ...)
 	AN(e);
 	va_start(ap, e);
 
-	field = va_arg(ap, int);
-	frag = va_arg(ap, struct fragment *);
-	for (n = 1, q = b; q < e; n++) {
-		/* caller must sort the fields, or this loop will not work: */
-		assert(field >= n);
-		AN(frag);
-
-		p = q;
-		/* Skip WS */
-		while (p < e && isspace(*p))
-			p++;
-		q = p;
-		/* Skip non-WS */
-		while (q < e && !isspace(*q))
-			q++;
-
-		if (field == n) {
-			if (frag->gen != CTX.gen || force) {
-				/* We only grab the same matching field once */
-				frag->gen = CTX.gen;
-				frag->b = p;
-				frag->e = q;
-			}
-			field = va_arg(ap, int);
-			if (field == 0)
-				break;
-			frag = va_arg(ap, struct fragment *);
+	n = 0;
+	while (1) {
+		field = va_arg(ap, int);
+		frag = va_arg(ap, struct fragment *);
+		if (field == 0) {
+			AZ(frag);
+			break;
+		}
+		p = q = NULL;
+		while (n < field) {
+			while (b < e && isspace(*b))
+				b++;
+			p = b;
+			while (b < e && !isspace(*b))
+				b++;
+			q = b;
+			n++;
+		}
+		assert(p != NULL && q != NULL);
+		assert(p < e && q > p);
+		if (frag->gen != CTX.gen || force) {
+			/* We only grab the same matching field once */
+			frag->gen = CTX.gen;
+			frag->b = p;
+			frag->e = q;
 		}
 	}
 	va_end(ap);
@@ -922,12 +862,12 @@ process_vsl(const struct vsl_watch_head *head, enum VSL_tag_e tag,
 	}
 }
 
-static int __match_proto__(VSLQ_dispatch_f)
+static int v_matchproto_(VSLQ_dispatch_f)
 dispatch_f(struct VSL_data *vsl, struct VSL_transaction * const pt[],
     void *priv)
 {
 	struct VSL_transaction *t;
-	unsigned tag;
+	enum VSL_tag_e tag;
 	const char *b, *e, *p;
 	struct watch *w;
 	int i, skip, be_mark;
@@ -960,6 +900,9 @@ dispatch_f(struct VSL_data *vsl, struct VSL_transaction * const pt[],
 		skip = 0;
 		while (skip == 0 && 1 == VSL_Next(t->c)) {
 			tag = VSL_TAG(t->c->rec.ptr);
+			if (VSL_tagflags[tag])
+				continue;
+
 			b = VSL_CDATA(t->c->rec.ptr);
 			e = b + VSL_LEN(t->c->rec.ptr);
 			while (e > b && e[-1] == '\0')
@@ -1107,20 +1050,6 @@ dispatch_f(struct VSL_data *vsl, struct VSL_transaction * const pt[],
 	return (0);
 }
 
-static int __match_proto__(VUT_cb_f)
-sighup(struct VUT *v)
-{
-	assert(v == vut);
-	return (1);
-}
-
-static void
-vut_sighandler(int sig)
-{
-	CHECK_OBJ_NOTNULL(vut, VUT_MAGIC);
-	VUT_Signaled(vut, sig);
-}
-
 static char *
 read_format(const char *formatfile)
 {
@@ -1139,11 +1068,10 @@ read_format(const char *formatfile)
 		free(fmt);
 		if (feof(fmtfile))
 			VUT_Error(vut, 1, "Empty format file");
-		else
-			VUT_Error(vut, 1, "Can't read format from file (%s)",
-			    strerror(errno));
+		VUT_Error(vut, 1, "Can't read format from file (%s)",
+		    strerror(errno));
 	}
-	fclose(fmtfile);
+	AZ(fclose(fmtfile));
 	if (fmt[fmtlen - 1] == '\n')
 		fmt[fmtlen - 1] = '\0';
 	return (fmt);
@@ -1167,6 +1095,8 @@ main(int argc, char * const *argv)
 	AN(CTX.vsb);
 	VB64_init();
 
+	tzset();		// We use localtime_r(3)
+
 	while ((opt = getopt(argc, argv, vopt_spec.vopt_optstring)) != -1) {
 		switch (opt) {
 		case 'a':
@@ -1182,22 +1112,21 @@ main(int argc, char * const *argv)
 			CTX.c_opt = 1;
 			break;
 		case 'F':
-			/* Format string */
 			if (format != NULL)
-				free(format);
-			format = strdup(optarg);
+				VUT_Error(vut, 1, "Format already set");
+			REPLACE(format, optarg);
 			AN(format);
 			break;
 		case 'f':
-			/* Format string from file */
 			if (format != NULL)
-				free(format);
+				VUT_Error(vut, 1, "Format already set");
+			/* Format string from file */
 			format = read_format(optarg);
 			AN(format);
 			break;
 		case 'h':
 			/* Usage help */
-			usage(0);
+			VUT_Usage(vut, &vopt_spec, 0);
 			break;
 		case 'w':
 			/* Write to file */
@@ -1205,7 +1134,7 @@ main(int argc, char * const *argv)
 			break;
 		default:
 			if (!VUT_Arg(vut, opt, optarg))
-				usage(1);
+				VUT_Usage(vut, &vopt_spec, 1);
 		}
 	}
 	/* default is client mode: */
@@ -1213,7 +1142,7 @@ main(int argc, char * const *argv)
 		CTX.c_opt = 1;
 
 	if (optind != argc)
-		usage(1);
+		VUT_Usage(vut, &vopt_spec, 1);
 
 	if (vut->D_opt && !CTX.w_arg)
 		VUT_Error(vut, 1, "Missing -w option");
@@ -1226,13 +1155,11 @@ main(int argc, char * const *argv)
 
 	/* Prepare output format */
 	parse_format(format);
-	free(format);
-	format = NULL;
+	REPLACE(format, NULL);
 
 	/* Setup output */
 	vut->dispatch_f = dispatch_f;
 	vut->dispatch_priv = NULL;
-	vut->sighup_f = sighup;
 	if (CTX.w_arg) {
 		openout(CTX.a_opt);
 		AN(CTX.fo);
@@ -1242,9 +1169,8 @@ main(int argc, char * const *argv)
 		CTX.fo = stdout;
 	vut->idle_f = flushout;
 
-	VUT_Signal(vut_sighandler);
 	VUT_Setup(vut);
-	VUT_Main(vut);
+	(void)VUT_Main(vut);
 	VUT_Fini(&vut);
 
 	exit(0);
