@@ -55,7 +55,7 @@ static struct VUT *vut;
 
 /*--------------------------------------------------------------------*/
 
-static int __match_proto__(VSC_iter_f)
+static int v_matchproto_(VSC_iter_f)
 do_xml_cb(void *priv, const struct VSC_point * const pt)
 {
 	uint64_t val;
@@ -93,7 +93,7 @@ do_xml(struct vsm *vsm, struct vsc *vsc)
 
 /*--------------------------------------------------------------------*/
 
-static int __match_proto__(VSC_iter_f)
+static int v_matchproto_(VSC_iter_f)
 do_json_cb(void *priv, const struct VSC_point * const pt)
 {
 	uint64_t val;
@@ -152,7 +152,7 @@ struct once_priv {
 	int pad;
 };
 
-static int __match_proto__(VSC_iter_f)
+static int v_matchproto_(VSC_iter_f)
 do_once_cb_first(void *priv, const struct VSC_point * const pt)
 {
 	struct once_priv *op;
@@ -169,7 +169,7 @@ do_once_cb_first(void *priv, const struct VSC_point * const pt)
 	return (1);
 }
 
-static int __match_proto__(VSC_iter_f)
+static int v_matchproto_(VSC_iter_f)
 do_once_cb(void *priv, const struct VSC_point * const pt)
 {
 	struct once_priv *op;
@@ -199,18 +199,23 @@ do_once_cb(void *priv, const struct VSC_point * const pt)
 static void
 do_once(struct vsm *vsm, struct vsc *vsc)
 {
+	struct vsc *vsconce = VSC_New();
 	struct once_priv op;
 
+	AN(vsconce);
+	AN(VSC_Arg(vsconce, 'f', "MAIN.uptime"));
+
 	memset(&op, 0, sizeof op);
 	op.pad = 18;
 
-	(void)VSC_Iter(vsc, vsm, do_once_cb_first, &op);
+	(void)VSC_Iter(vsconce, vsm, do_once_cb_first, &op);
+	VSC_Destroy(&vsconce, vsm);
 	(void)VSC_Iter(vsc, vsm, do_once_cb, &op);
 }
 
 /*--------------------------------------------------------------------*/
 
-static int __match_proto__(VSC_iter_f)
+static int v_matchproto_(VSC_iter_f)
 do_list_cb(void *priv, const struct VSC_point * const pt)
 {
 	int i;
@@ -240,7 +245,7 @@ list_fields(struct vsm *vsm, struct vsc *vsc)
 
 /*--------------------------------------------------------------------*/
 
-static void __attribute__((__noreturn__))
+static void v_noreturn_
 usage(int status)
 {
 	const char **opt;
@@ -259,6 +264,7 @@ main(int argc, char * const *argv)
 	int once = 0, xml = 0, json = 0, f_list = 0, curses = 0;
 	signed char opt;
 	int i;
+	int has_f = 0;
 	struct vsc *vsc;
 
 	vut = VUT_InitProg(argc, argv, &vopt_spec);
@@ -276,6 +282,7 @@ main(int argc, char * const *argv)
 		case 'h':
 			/* Usage help */
 			usage(0);
+			break;
 		case 'l':
 			f_list = 1;
 			break;
@@ -287,6 +294,7 @@ main(int argc, char * const *argv)
 			break;
 		case 'f':
 			AN(VSC_Arg(vsc, opt, optarg));
+			has_f = 1;
 			break;
 		case 'V':
 			AN(VUT_Arg(vut, opt, optarg));
@@ -309,8 +317,15 @@ main(int argc, char * const *argv)
 	if (VSM_Attach(vd, STDERR_FILENO))
 		VUT_Error(vut, 1, "%s", VSM_Error(vd));
 
-	if (curses)
-		do_curses(vd, vsc, 1.0);
+	if (curses) {
+		if (has_f) {
+			AN(VSC_Arg(vsc, 'f', "MGT.uptime"));
+			AN(VSC_Arg(vsc, 'f', "MAIN.uptime"));
+			AN(VSC_Arg(vsc, 'f', "MAIN.cache_hit"));
+			AN(VSC_Arg(vsc, 'f', "MAIN.cache_miss"));
+		}
+		do_curses(vd, vsc);
+	}
 	else if (xml)
 		do_xml(vd, vsc);
 	else if (json)
