@@ -28,7 +28,6 @@
 
 #include "config.h"
 
-#include <errno.h>
 #include <poll.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -147,7 +146,7 @@ barrier_sock_thread(void *priv)
 		    b->name, err, strerror(errno), errno);
 	}
 	assert(sock > 0);
-	(void)VTCP_nonblocking(sock);
+	VTCP_nonblocking(sock);
 	VTCP_myname(sock, abuf, sizeof abuf, pbuf, sizeof pbuf);
 
 	macro_def(vl, b->name, "addr", "%s", abuf);
@@ -348,8 +347,7 @@ barrier_sync(struct barrier *b, struct vtclog *vl)
 
 /* SECTION: barrier barrier
  *
- * NOTE: this can be used from the top-level as well as from client and server
- * specifications.
+ * NOTE: This command is available everywhere commands are given.
  *
  * Barriers allows you to synchronize different threads to make sure events
  * occur in the right order. It's even possible to use them in VCL.
