@@ -29,17 +29,20 @@
 #include "config.h"
 
 #include <sys/socket.h>
+#include <sys/un.h>
 
-#include <errno.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#include <poll.h>
+#include <unistd.h>
 
 #include "vtc.h"
 
 #include "vsa.h"
 #include "vss.h"
 #include "vtcp.h"
+#include "vus.h"
 
 struct client {
 	unsigned		magic;
@@ -56,6 +59,7 @@ struct client {
 	int			proxy_version;
 
 	unsigned		repeat;
+	unsigned		keepalive;
 
 	unsigned		running;
 	pthread_t		tp;
@@ -68,7 +72,7 @@ static VTAILQ_HEAD(, client)	clients =
  * Send the proxy header
  */
 
-static int __match_proto__(vss_resolved_f)
+static int v_matchproto_(vss_resolved_f)
 proxy_cb(void *priv, const struct suckaddr *sa)
 {
 	struct suckaddr **addr = priv;
@@ -105,6 +109,86 @@ client_proxy(struct vtclog *vl, int fd, int version, const char *spec)
 	free(sas);
 }
 
+/**********************************************************************
+ * Socket connect.
+ */
+
+static int
+client_tcp_connect(struct vtclog *vl, const char *addr, double tmo,
+		   const char **errp)
+{
+	int fd;
+	char mabuf[32], mpbuf[32];
+
+	fd = VTCP_open(addr, NULL, tmo, errp);
+	if (fd < 0)
+		return (fd);
+	VTCP_myname(fd, mabuf, sizeof mabuf, mpbuf, sizeof mpbuf);
+	vtc_log(vl, 3, "connected fd %d from %s %s to %s", fd, mabuf, mpbuf,
+		addr);
+	return (fd);
+}
+
+/* cf. VTCP_Open() */
+static int v_matchproto_(vus_resolved_f)
+uds_open(void *priv, const struct sockaddr_un *uds)
+{
+	double *p;
+	int s, i, tmo;
+	struct pollfd fds[1];
+	socklen_t sl = sizeof(*uds);
+
+	AN(priv);
+	AN(uds);
+	p = priv;
+	assert(*p > 0.);
+	tmo = (int)(*p * 1e3);
+
+	s = socket(uds->sun_family, SOCK_STREAM, 0);
+	if (s < 0)
+		return (s);
+
+	VTCP_nonblocking(s);
+	i = connect(s, (const void*)uds, sl);
+	if (i == 0)
+		return(s);
+	if (errno != EINPROGRESS) {
+		closefd(&s);
+		return (-1);
+	}
+
+	fds[0].fd = s;
+	fds[0].events = POLLWRNORM;
+	fds[0].revents = 0;
+	i = poll(fds, 1, tmo);
+
+	if (i == 0) {
+		closefd(&s);
+		errno = ETIMEDOUT;
+		return (-1);
+	}
+
+	return (VTCP_connected(s));
+}
+
+static int
+client_uds_connect(struct vtclog *vl, const char *path, double tmo,
+		   const char **errp)
+{
+	int fd;
+
+	assert(tmo >= 0);
+
+	errno = 0;
+	fd = VUS_resolver(path, uds_open, &tmo, errp);
+	if (fd < 0) {
+		*errp = strerror(errno);
+		return (fd);
+	}
+	vtc_log(vl, 3, "connected fd %d to %s", fd, path);
+	return (fd);
+}
+
 /**********************************************************************
  * Client thread
  */
@@ -117,45 +201,49 @@ client_thread(void *priv)
 	int fd;
 	unsigned u;
 	struct vsb *vsb;
-	char *p;
-	char mabuf[32], mpbuf[32];
 	const char *err;
 
 	CAST_OBJ_NOTNULL(c, priv, CLIENT_MAGIC);
 	AN(*c->connect);
 
 	vl = vtc_logopen(c->name);
+	pthread_cleanup_push(vtc_logclose, vl);
 
-	p = strdup(c->connect);
-	AN(p);
-	vsb = macro_expand(vl, p);
+	vsb = macro_expand(vl, c->connect);
 	AN(vsb);
 
 	if (c->repeat == 0)
 		c->repeat = 1;
 	if (c->repeat != 1)
-		vtc_log(vl, 2, "Started (%u iterations)", c->repeat);
+		vtc_log(vl, 2, "Started (%u iterations%s)", c->repeat,
+			c->keepalive ? " using keepalive" : "");
 	for (u = 0; u < c->repeat; u++) {
-		vtc_log(vl, 3, "Connect to %s", VSB_data(vsb));
-		fd = VTCP_open(VSB_data(vsb), NULL, 10., &err);
+		char *addr = VSB_data(vsb);
+
+		vtc_log(vl, 3, "Connect to %s", addr);
+		if (*addr == '/')
+			fd = client_uds_connect(vl, addr, 10., &err);
+		else
+			fd = client_tcp_connect(vl, VSB_data(vsb), 10., &err);
 		if (fd < 0)
 			vtc_fatal(c->vl, "Failed to open %s: %s",
 			    VSB_data(vsb), err);
-		assert(fd >= 0);
 		/* VTCP_blocking does its own checks, trust it */
-		(void)VTCP_blocking(fd);
-		VTCP_myname(fd, mabuf, sizeof mabuf, mpbuf, sizeof mpbuf);
-		vtc_log(vl, 3, "connected fd %d from %s %s to %s",
-		    fd, mabuf, mpbuf, VSB_data(vsb));
+		VTCP_blocking(fd);
 		if (c->proxy_spec != NULL)
 			client_proxy(vl, fd, c->proxy_version, c->proxy_spec);
-		fd = http_process(vl, c->spec, fd, NULL);
+		if (! c->keepalive)
+			fd = http_process(vl, c->spec, fd, NULL, addr);
+		else
+			while (fd >= 0 && u++ < c->repeat)
+				fd = http_process(vl, c->spec, fd, NULL, addr);
 		vtc_log(vl, 3, "closing fd %d", fd);
 		VTCP_close(&fd);
 	}
 	vtc_log(vl, 2, "Ending");
 	VSB_destroy(&vsb);
-	free(p);
+	pthread_cleanup_pop(0);
+	vtc_logclose(vl);
 	return (NULL);
 }
 
@@ -310,6 +398,10 @@ cmd_client(CMD_ARGS)
 			av++;
 			continue;
 		}
+		if (!strcmp(*av, "-keepalive")) {
+			c->keepalive = 1;
+			continue;
+		}
 		if (!strcmp(*av, "-start")) {
 			client_start(c);
 			continue;
