@@ -30,7 +30,6 @@
 
 #include <sys/socket.h>
 
-#include <errno.h>
 #include <math.h>
 #include <poll.h>
 #include <stdio.h>
@@ -45,6 +44,7 @@
 #include "vfil.h"
 #include "vgz.h"
 #include "vnum.h"
+#include "vrnd.h"
 #include "vtcp.h"
 #include "hpack.h"
 
@@ -75,9 +75,9 @@ extern const struct cmds http_cmds[];
  *
  * Be careful though, servers will by default listen to the 127.0.0.1 IP and
  * will pick a random port, and publish 3 macros: sNAME_addr, sNAME_port and
- * sNAME_sock, but only once they are started. For varnishtest to
- * create the vcl with the correct values, the server must be started when you
- * use -vcl+backend.
+ * sNAME_sock, but only once they are started.
+ * For 'varnish -vcl+backend' to create the vcl with the correct values, the
+ * server must be started first.
  *
  * SECTION: client-server.args Arguments
  *
@@ -94,16 +94,23 @@ extern const struct cmds http_cmds[];
  * \-repeat NUMBER
  *        Instead of processing the specification only once, do it NUMBER times.
  *
+ * \-keepalive
+ *        For repeat, do not open new connections but rather run all
+ *        iterations in the same connection
+ *
  * \-break (server only)
  *        Stop the server.
  *
  * \-listen STRING (server only)
  *        Dictate the listening socket for the server. STRING is of the form
- *        "IP PORT".
+ *        "IP PORT", or "/PATH/TO/SOCKET" for a Unix domain socket. In the
+ *        latter case, the path must begin with '/', and the server must be
+ *        able to create it.
  *
  * \-connect STRING (client only)
  *        Indicate the server to connect to. STRING is also of the form
- *        "IP PORT".
+ *        "IP PORT", or "/PATH/TO/SOCKET". As with "server -listen", a
+ *        Unix domain socket is recognized when STRING begins with a '/'.
  *
  * \-dispatch (server only, s0 only)
  *        Normally, to keep things simple, server threads only handle one
@@ -186,7 +193,7 @@ synth_body(const char *len, int rnd)
 				k = '!';
 			l = k;
 		} else if (rnd) {
-			b[j] = (random() % 95) + ' ';
+			b[j] = (VRND_RandomTestable() % 95) + ' ';
 		} else {
 			b[j] = (char)l;
 			if (++l == '~')
@@ -277,6 +284,7 @@ http_count_header(char * const *hh, const char *hdr)
  *
  *         - remote.ip
  *         - remote.port
+ *         - remote.path
  *         - req.method
  *         - req.url
  *         - req.proto
@@ -297,31 +305,33 @@ cmd_var_resolve(struct http *hp, char *spec)
 {
 	char **hh, *hdr;
 	if (!strcmp(spec, "remote.ip"))
-		return(hp->rem_ip);
+		return (hp->rem_ip);
 	if (!strcmp(spec, "remote.port"))
-		return(hp->rem_port);
+		return (hp->rem_port);
+	if (!strcmp(spec, "remote.path"))
+		return (hp->rem_path);
 	if (!strcmp(spec, "req.method"))
-		return(hp->req[0]);
+		return (hp->req[0]);
 	if (!strcmp(spec, "req.url"))
-		return(hp->req[1]);
+		return (hp->req[1]);
 	if (!strcmp(spec, "req.proto"))
-		return(hp->req[2]);
+		return (hp->req[2]);
 	if (!strcmp(spec, "resp.proto"))
-		return(hp->resp[0]);
+		return (hp->resp[0]);
 	if (!strcmp(spec, "resp.status"))
-		return(hp->resp[1]);
+		return (hp->resp[1]);
 	if (!strcmp(spec, "resp.reason"))
-		return(hp->resp[2]);
+		return (hp->resp[2]);
 	if (!strcmp(spec, "resp.chunklen"))
-		return(hp->chunklen);
+		return (hp->chunklen);
 	if (!strcmp(spec, "req.bodylen"))
-		return(hp->bodylen);
+		return (hp->bodylen);
 	if (!strcmp(spec, "req.body"))
-		return(hp->body != NULL ? hp->body : spec);
+		return (hp->body != NULL ? hp->body : spec);
 	if (!strcmp(spec, "resp.bodylen"))
-		return(hp->bodylen);
+		return (hp->bodylen);
 	if (!strcmp(spec, "resp.body"))
-		return(hp->body != NULL ? hp->body : spec);
+		return (hp->body != NULL ? hp->body : spec);
 	if (!strncmp(spec, "req.http.", 9)) {
 		hh = hp->req;
 		hdr = spec + 9;
@@ -409,7 +419,11 @@ http_splitheader(struct http *hp, int req)
 	}
 
 	n = 0;
-	p = hp->rxbuf;
+	p = hp->rx_b;
+	if (*p == '\0') {
+		vtc_log(hp->vl, 4, "No headers");
+		return;
+	}
 
 	/* REQ/PROTO */
 	while (vct_islws(*p))
@@ -417,20 +431,20 @@ http_splitheader(struct http *hp, int req)
 	hh[n++] = p;
 	while (!vct_islws(*p))
 		p++;
-	AZ(vct_iscrlf(p));
+	AZ(vct_iscrlf(p, hp->rx_e));
 	*p++ = '\0';
 
 	/* URL/STATUS */
 	while (vct_issp(*p))		/* XXX: H space only */
 		p++;
-	AZ(vct_iscrlf(p));
+	AZ(vct_iscrlf(p, hp->rx_e));
 	hh[n++] = p;
 	while (!vct_islws(*p))
 		p++;
-	if (vct_iscrlf(p)) {
+	if (vct_iscrlf(p, hp->rx_e)) {
 		hh[n++] = NULL;
 		q = p;
-		p += vct_skipcrlf(p);
+		p = vct_skipcrlf(p, hp->rx_e);
 		*q = '\0';
 	} else {
 		*p++ = '\0';
@@ -438,26 +452,26 @@ http_splitheader(struct http *hp, int req)
 		while (vct_issp(*p))		/* XXX: H space only */
 			p++;
 		hh[n++] = p;
-		while (!vct_iscrlf(p))
+		while (!vct_iscrlf(p, hp->rx_e))
 			p++;
 		q = p;
-		p += vct_skipcrlf(p);
+		p = vct_skipcrlf(p, hp->rx_e);
 		*q = '\0';
 	}
 	assert(n == 3);
 
 	while (*p != '\0') {
 		assert(n < MAX_HDR);
-		if (vct_iscrlf(p))
+		if (vct_iscrlf(p, hp->rx_e))
 			break;
 		hh[n++] = p++;
-		while (*p != '\0' && !vct_iscrlf(p))
+		while (*p != '\0' && !vct_iscrlf(p, hp->rx_e))
 			p++;
 		q = p;
-		p += vct_skipcrlf(p);
+		p = vct_skipcrlf(p, hp->rx_e);
 		*q = '\0';
 	}
-	p += vct_skipcrlf(p);
+	p = vct_skipcrlf(p, hp->rx_e);
 	assert(*p == '\0');
 
 	for (n = 0; n < 3 || hh[n] != NULL; n++) {
@@ -497,8 +511,8 @@ http_rxchar(struct http *hp, int n, int eof)
 			continue;
 		}
 		assert(i > 0);
-		assert(hp->prxbuf + n < hp->nrxbuf);
-		i = read(hp->fd, hp->rxbuf + hp->prxbuf, n);
+		assert(hp->rx_p + n < hp->rx_e);
+		i = read(hp->fd, hp->rx_p, n);
 		if (!(pfd[0].revents & POLLIN))
 			vtc_log(hp->vl, 4,
 			    "HTTP rx poll (fd:%d revents: %x n=%d, i=%d)",
@@ -517,8 +531,8 @@ http_rxchar(struct http *hp, int n, int eof)
 			    hp->fd, strerror(errno));
 			return (-1);
 		}
-		hp->prxbuf += i;
-		hp->rxbuf[hp->prxbuf] = '\0';
+		hp->rx_p += i;
+		*hp->rx_p = '\0';
 		n -= i;
 	}
 	return (1);
@@ -527,93 +541,98 @@ http_rxchar(struct http *hp, int n, int eof)
 static int
 http_rxchunk(struct http *hp)
 {
-	char *q;
-	int l, i;
+	char *q, *old;
+	int i;
 
-	l = hp->prxbuf;
+	old = hp->rx_p;
 	do {
 		if (http_rxchar(hp, 1, 0) < 0)
 			return (-1);
-	} while (hp->rxbuf[hp->prxbuf - 1] != '\n');
-	vtc_dump(hp->vl, 4, "len", hp->rxbuf + l, -1);
-	i = strtoul(hp->rxbuf + l, &q, 16);
+	} while (hp->rx_p[-1] != '\n');
+	vtc_dump(hp->vl, 4, "len", old, -1);
+	i = strtoul(old, &q, 16);
 	bprintf(hp->chunklen, "%d", i);
-	if ((q == hp->rxbuf + l) ||
-		(*q != '\0' && !vct_islws(*q))) {
-		vtc_log(hp->vl, hp->fatal, "chunked fail %02x @ %td",
-		    *q, q - (hp->rxbuf + l));
+	if ((q == old) || (q == hp->rx_p) || (*q != '\0' && !vct_islws(*q))) {
+		vtc_log(hp->vl, hp->fatal, "Chunklen fail (%02x @ %td)",
+		    (*q & 0xff), q - old);
 		return (-1);
 	}
-	assert(q != hp->rxbuf + l);
 	assert(*q == '\0' || vct_islws(*q));
-	hp->prxbuf = l;
+	hp->rx_p = old;
 	if (i > 0) {
 		if (http_rxchar(hp, i, 0) < 0)
 			return (-1);
-		vtc_dump(hp->vl, 4, "chunk", hp->rxbuf + l, i);
+		vtc_dump(hp->vl, 4, "chunk", old, i);
 	}
-	l = hp->prxbuf;
+	old = hp->rx_p;
 	if (http_rxchar(hp, 2, 0) < 0)
 		return (-1);
-	if (!vct_iscrlf(hp->rxbuf + l)) {
-		vtc_log(hp->vl, hp->fatal,
-		    "Wrong chunk tail[0] = %02x",
-		    hp->rxbuf[l] & 0xff);
+	if (!vct_iscrlf(old, hp->rx_e)) {
+		vtc_log(hp->vl, hp->fatal, "Chunklen without CRLF");
 		return (-1);
 	}
-	if (!vct_iscrlf(hp->rxbuf + l + 1)) {
-		vtc_log(hp->vl, hp->fatal,
-		    "Wrong chunk tail[1] = %02x",
-		    hp->rxbuf[l + 1] & 0xff);
-		return (-1);
-	}
-	hp->prxbuf = l;
-	hp->rxbuf[l] = '\0';
+	hp->rx_p = old;
+	*hp->rx_p = '\0';
 	return (i);
 }
 
 /**********************************************************************
  * Swallow a HTTP message body
+ *
+ * max: 0 is all
  */
 
 static void
-http_swallow_body(struct http *hp, char * const *hh, int body)
+http_swallow_body(struct http *hp, char * const *hh, int body, int max)
 {
-	char *p;
+	const char *p, *q;
 	int i, l, ll;
 
-	ll = 0;
+	l = hp->rx_p - hp->body;
+
 	p = http_find_header(hh, "transfer-encoding");
+	q = http_find_header(hh, "content-length");
 	if (p != NULL && !strcasecmp(p, "chunked")) {
-		while (http_rxchunk(hp) > 0)
-			continue;
-		vtc_dump(hp->vl, 4, "body", hp->body, ll);
-		ll = hp->rxbuf + hp->prxbuf - hp->body;
-		hp->bodyl = ll;
-		bprintf(hp->bodylen, "%d", ll);
-		return;
-	}
-	p = http_find_header(hh, "content-length");
-	if (p != NULL) {
-		l = strtoul(p, NULL, 10);
-		if (http_rxchar(hp, l, 0) < 0)
+		if (q != NULL) {
+			vtc_log(hp->vl, hp->fatal, "Both C-E: Chunked and C-L");
 			return;
-		vtc_dump(hp->vl, 4, "body", hp->body, l);
-		hp->bodyl = l;
-		bprintf(hp->bodylen, "%d", l);
-		return;
-	}
-	if (body) {
+		}
+		ll = 0;
+		while (http_rxchunk(hp) > 0) {
+			ll = (hp->rx_p - hp->body) - l;
+			if (max && ll >= max)
+				break;
+		}
+		p = "chunked";
+	} else if (q != NULL) {
+		ll = strtoul(q, NULL, 10);
+		if (max && ll > l + max)
+			ll = max;
+		else
+			ll -= l;
+		i = http_rxchar(hp, ll, 0);
+		if (i < 0)
+			return;
+		p = "c-l";
+	} else if (body) {
+		ll = 0;
 		do  {
 			i = http_rxchar(hp, 1, 1);
 			if (i < 0)
 				return;
 			ll += i;
+			if (max && ll >= max)
+				break;
 		} while (i > 0);
-		vtc_dump(hp->vl, 4, "rxeof", hp->body, ll);
+		p = "eof";
+	} else {
+		p = "none";
+		ll = l = 0;
 	}
-	hp->bodyl = ll;
-	bprintf(hp->bodylen, "%d", ll);
+	vtc_dump(hp->vl, 4, p, hp->body + l, ll);
+	l += ll;
+	hp->bodyl = l;
+	bprintf(hp->bodylen, "%d", l);
 }
 
 /**********************************************************************
@@ -623,38 +642,44 @@ http_swallow_body(struct http *hp, char * const *hh, int body)
 static void
 http_rxhdr(struct http *hp)
 {
-	int i;
+	int i, s = 0;
 	char *p;
+	ssize_t l;
 
 	CHECK_OBJ_NOTNULL(hp, HTTP_MAGIC);
-	hp->prxbuf = 0;
+	hp->rx_p = hp->rx_b;
+	*hp->rx_p = '\0';
 	hp->body = NULL;
 	bprintf(hp->bodylen, "%s", "<undef>");
 	while (1) {
-		(void)http_rxchar(hp, 1, 0);
-		p = hp->rxbuf + hp->prxbuf - 1;
-		for (i = 0; p > hp->rxbuf; p--) {
-			if (*p != '\n')
-				break;
-			if (p - 1 > hp->rxbuf && p[-1] == '\r')
-				p--;
-			if (++i == 2)
-				break;
-		}
-		if (i == 2)
+		p = hp->rx_p;
+		i = http_rxchar(hp, 1, 1);
+		if (i < 1)
 			break;
+		if (s == 0 && *p == '\r')
+			s = 1;
+		else if ((s == 0 || s == 1) && *p == '\n')
+			s = 2;
+		else if (s == 2 && *p == '\r')
+			s = 3;
+		else if ((s == 2 || s == 3) && *p == '\n')
+			break;
+		else
+			s = 0;
 	}
-	vtc_dump(hp->vl, 4, "rxhdr", hp->rxbuf, -1);
-	vtc_log(hp->vl, 4, "rxhdrlen = %zd", strlen(hp->rxbuf));
-	hp->body = hp->rxbuf + hp->prxbuf;
+	l = hp->rx_p - hp->rx_b;
+	vtc_dump(hp->vl, 4, "rxhdr", hp->rx_b, l);
+	vtc_log(hp->vl, 4, "rxhdrlen = %zd", l);
+	hp->body = hp->rx_p;
 }
 
 /* SECTION: client-server.spec.rxresp
  *
  * rxresp [-no_obj] (client only)
- *         Receive and parse a response's headers and body. If -no_obj is present, only get
- *         the headers.
+ *         Receive and parse a response's headers and body. If -no_obj is
+ *         present, only get the headers.
  */
+
 static void
 cmd_http_rxresp(CMD_ARGS)
 {
@@ -681,10 +706,14 @@ cmd_http_rxresp(CMD_ARGS)
 		    "Multiple Content-Length headers.\n");
 	if (!has_obj)
 		return;
-	else if (!strcmp(hp->resp[1], "200"))
-		http_swallow_body(hp, hp->resp, 1);
+	if (!hp->resp[0] || !hp->resp[1])
+		return;
+	if (hp->head_method)
+		return;
+	if (!strcmp(hp->resp[1], "200"))
+		http_swallow_body(hp, hp->resp, 1, 0);
 	else
-		http_swallow_body(hp, hp->resp, 0);
+		http_swallow_body(hp, hp->resp, 0, 0);
 	vtc_log(hp->vl, 4, "bodylen = %s", hp->bodylen);
 }
 
@@ -693,6 +722,7 @@ cmd_http_rxresp(CMD_ARGS)
  * rxresphdrs (client only)
  *         Receive and parse a response's headers.
  */
+
 static void
 cmd_http_rxresphdrs(CMD_ARGS)
 {
@@ -714,7 +744,6 @@ cmd_http_rxresphdrs(CMD_ARGS)
 		    "Multiple Content-Length headers.\n");
 }
 
-
 /**********************************************************************
  * Ungzip rx'ed body
  */
@@ -760,6 +789,7 @@ cmd_http_gunzip(CMD_ARGS)
 	vtc_log(hp->vl, 3, "new bodylen %u", hp->bodyl);
 	vtc_dump(hp->vl, 4, "body", hp->body, hp->bodyl);
 	bprintf(hp->bodylen, "%u", hp->bodyl);
+#ifdef VGZ_EXTENSIONS
 	vtc_log(hp->vl, 4, "startbit = %ju %ju/%ju",
 	    (uintmax_t)vz.start_bit,
 	    (uintmax_t)vz.start_bit >> 3, (uintmax_t)vz.start_bit & 7);
@@ -769,6 +799,7 @@ cmd_http_gunzip(CMD_ARGS)
 	vtc_log(hp->vl, 4, "stopbit = %ju %ju/%ju",
 	    (uintmax_t)vz.stop_bit,
 	    (uintmax_t)vz.stop_bit >> 3, (uintmax_t)vz.stop_bit & 7);
+#endif
 	if (i != Z_STREAM_END)
 		vtc_log(hp->vl, hp->fatal,
 		    "Gunzip error = %d (%s) in:%jd out:%jd",
@@ -784,8 +815,11 @@ cmd_http_gunzip(CMD_ARGS)
 static void
 gzip_body(const struct http *hp, const char *txt, char **body, int *bodylen)
 {
-	int l, i;
+	int l;
 	z_stream vz;
+#ifdef VGZ_EXTENSIONS
+	int i;
+#endif
 
 	memset(&vz, 0, sizeof vz);
 
@@ -802,12 +836,13 @@ gzip_body(const struct http *hp, const char *txt, char **body, int *bodylen)
 	assert(Z_OK == deflateInit2(&vz,
 	    hp->gziplevel, Z_DEFLATED, 31, 9, Z_DEFAULT_STRATEGY));
 	assert(Z_STREAM_END == deflate(&vz, Z_FINISH));
+	*bodylen = vz.total_out;
+#ifdef VGZ_EXTENSIONS
 	i = vz.stop_bit & 7;
 	if (hp->gzipresidual >= 0 && hp->gzipresidual != i)
 		vtc_log(hp->vl, hp->fatal,
 		    "Wrong gzip residual got %d wanted %d",
 		    i, hp->gzipresidual);
-	*bodylen = vz.total_out;
 	vtc_log(hp->vl, 4, "startbit = %ju %ju/%ju",
 	    (uintmax_t)vz.start_bit,
 	    (uintmax_t)vz.start_bit >> 3, (uintmax_t)vz.start_bit & 7);
@@ -818,6 +853,7 @@ gzip_body(const struct http *hp, const char *txt, char **body, int *bodylen)
 	    (uintmax_t)vz.stop_bit,
 	    (uintmax_t)vz.stop_bit >> 3, (uintmax_t)vz.stop_bit & 7);
 	assert(Z_OK == deflateEnd(&vz));
+#endif
 }
 
 /**********************************************************************
@@ -826,11 +862,12 @@ gzip_body(const struct http *hp, const char *txt, char **body, int *bodylen)
 
 static char* const *
 http_tx_parse_args(char * const *av, struct vtclog *vl, struct http *hp,
-    char* body)
+    char *body, unsigned nohost)
 {
 	int bodylen = 0;
 	char *b, *c;
-	char *nullbody = NULL;
+	char *nullbody;
+	char *m;
 	int nolen = 0;
 	int l;
 
@@ -840,7 +877,11 @@ http_tx_parse_args(char * const *av, struct vtclog *vl, struct http *hp,
 	for (; *av != NULL; av++) {
 		if (!strcmp(*av, "-nolen")) {
 			nolen = 1;
+		} else if (!strcmp(*av, "-nohost")) {
+			nohost = 1;
 		} else if (!strcmp(*av, "-hdr")) {
+			if (!strncasecmp(av[1], "Host:", 5))
+				nohost = 1;
 			VSB_printf(hp->vsb, "%s%s", av[1], nl);
 			av++;
 		} else if (!strcmp(*av, "-hdrlen")) {
@@ -873,6 +914,7 @@ http_tx_parse_args(char * const *av, struct vtclog *vl, struct http *hp,
 			}
 		} else if (!strcmp(*av, "-bodylen")) {
 			assert(body == nullbody);
+			free(body);
 			body = synth_body(av[1], 0);
 			bodylen = strlen(body);
 			av++;
@@ -884,13 +926,16 @@ http_tx_parse_args(char * const *av, struct vtclog *vl, struct http *hp,
 			av++;
 		} else if (!strcmp(*av, "-gziplen")) {
 			assert(body == nullbody);
+			free(body);
 			b = synth_body(av[1], 1);
 			gzip_body(hp, b, &body, &bodylen);
+			free(b);
 			VSB_printf(hp->vsb, "Content-Encoding: gzip%s", nl);
 			// vtc_hexdump(hp->vl, 4, "gzip", (void*)body, bodylen);
 			av++;
 		} else if (!strcmp(*av, "-gzipbody")) {
 			assert(body == nullbody);
+			free(body);
 			gzip_body(hp, av[1], &body, &bodylen);
 			VSB_printf(hp->vsb, "Content-Encoding: gzip%s", nl);
 			// vtc_hexdump(hp->vl, 4, "gzip", (void*)body, bodylen);
@@ -898,6 +943,12 @@ http_tx_parse_args(char * const *av, struct vtclog *vl, struct http *hp,
 		} else
 			break;
 	}
+	if (!nohost) {
+		m = macro_get("localhost", NULL);
+		AN(m);
+		VSB_printf(hp->vsb, "Host: %s%s", m, nl);
+		free(m);
+	}
 	if (body != NULL && !nolen)
 		VSB_printf(hp->vsb, "Content-Length: %d%s", bodylen, nl);
 	VSB_cat(hp->vsb, nl);
@@ -938,8 +989,11 @@ http_tx_parse_args(char * const *av, struct vtclog *vl, struct http *hp,
  *         These three switches can appear in any order but must come before the
  *         following ones.
  *
+ *         \-nohost
+ *                 Don't include a Host header in the request.
+ *
  *         \-nolen
- *                 Don't include a Content-Length header in the response.
+ *                 Don't include a Content-Length header.
  *
  *         \-hdr STRING
  *                 Add STRING as a header, it must follow this format:
@@ -1013,7 +1067,7 @@ cmd_http_txresp(CMD_ARGS)
 	/* send a "Content-Length: 0" header unless something else happens */
 	REPLACE(body, "");
 
-	av = http_tx_parse_args(av, vl, hp, body);
+	av = http_tx_parse_args(av, vl, hp, body, 1);
 	if (*av != NULL)
 		vtc_fatal(hp->vl, "Unknown http txresp spec: %s\n", *av);
 
@@ -1086,7 +1140,7 @@ cmd_http_rxreq(CMD_ARGS)
 	http_splitheader(hp, 1);
 	if (http_count_header(hp->req, "Content-Length") > 1)
 		vtc_fatal(vl, "Multiple Content-Length headers.\n");
-	http_swallow_body(hp, hp->req, 0);
+	http_swallow_body(hp, hp->req, 0, 0);
 	vtc_log(vl, 4, "bodylen = %s", hp->bodylen);
 }
 
@@ -1135,20 +1189,23 @@ cmd_http_rxreqbody(CMD_ARGS)
 
 	for (; *av != NULL; av++)
 		vtc_fatal(hp->vl, "Unknown http rxreq spec: %s\n", *av);
-	http_swallow_body(hp, hp->req, 0);
+	http_swallow_body(hp, hp->req, 0, 0);
 	vtc_log(hp->vl, 4, "bodylen = %s", hp->bodylen);
 }
 
 /* SECTION: client-server.spec.rxrespbody
  *
  * rxrespbody (client only)
- *         Receive a response's body.
+ *         Receive (part of) a response's body.
+ *
+ * -max : max length of this receive, 0 for all
  */
 
 static void
 cmd_http_rxrespbody(CMD_ARGS)
 {
 	struct http *hp;
+	int max = 0;
 
 	(void)cmd;
 	(void)vl;
@@ -1158,8 +1215,14 @@ cmd_http_rxrespbody(CMD_ARGS)
 	av++;
 
 	for (; *av != NULL; av++)
-		vtc_fatal(hp->vl, "Unknown http rxrespbody spec: %s\n", *av);
-	http_swallow_body(hp, hp->resp, 0);
+		if (!strcmp(*av, "-max")) {
+			max = atoi(av[1]);
+			av++;
+		} else
+			vtc_fatal(hp->vl,
+			    "Unknown http rxrespbody spec: %s\n", *av);
+
+	http_swallow_body(hp, hp->resp, 1, max);
 	vtc_log(hp->vl, 4, "bodylen = %s", hp->bodylen);
 }
 
@@ -1182,7 +1245,7 @@ cmd_http_rxchunk(CMD_ARGS)
 
 	i = http_rxchunk(hp);
 	if (i == 0) {
-		ll = hp->rxbuf + hp->prxbuf - hp->body;
+		ll = hp->rx_p - hp->body;
 		hp->bodyl = ll;
 		bprintf(hp->bodylen, "%d", ll);
 		vtc_log(hp->vl, 4, "bodylen = %s", hp->bodylen);
@@ -1201,6 +1264,7 @@ cmd_http_txreq(CMD_ARGS)
 	const char *url = "/";
 	const char *proto = "HTTP/1.1";
 	const char *up = NULL;
+	unsigned nohost;
 
 	(void)cmd;
 	(void)vl;
@@ -1220,6 +1284,7 @@ cmd_http_txreq(CMD_ARGS)
 			av++;
 		} else if (!strcmp(*av, "-req")) {
 			req = av[1];
+			hp->head_method = !strcasecmp(av[1], "HEAD") ;
 			av++;
 		} else if (!hp->sfd && !strcmp(*av, "-up")) {
 			up = av[1];
@@ -1234,7 +1299,8 @@ cmd_http_txreq(CMD_ARGS)
 				"Upgrade: h2c%s"
 				"HTTP2-Settings: %s%s", nl, nl, up, nl);
 
-	av = http_tx_parse_args(av, vl, hp, NULL);
+	nohost = strcasecmp(proto, "HTTP/1.1") != 0;
+	av = http_tx_parse_args(av, vl, hp, NULL, nohost);
 	if (*av != NULL)
 		vtc_fatal(hp->vl, "Unknown http txreq spec: %s\n", *av);
 	http_write(hp, 4, "txreq");
@@ -1417,7 +1483,7 @@ cmd_http_chunked(CMD_ARGS)
 /* SECTION: client-server.spec.chunkedlen
  *
  * chunkedlen NUMBER
- *         Do as ``chunked`` except that varnishtest will generate the string
+ *         Do as ``chunked`` except that the string will be generated
  *         for you, with a length of NUMBER characters.
  */
 
@@ -1503,14 +1569,14 @@ cmd_http_expect_close(CMD_ARGS)
 		stop_h2(hp);
 	while (1) {
 		fds[0].fd = hp->fd;
-		fds[0].events = POLLIN | POLLERR;
+		fds[0].events = POLLIN;
 		fds[0].revents = 0;
 		i = poll(fds, 1, hp->timeout);
 		if (i < 0 && errno == EINTR)
 			continue;
 		if (i == 0)
 			vtc_log(vl, hp->fatal, "Expected close: timeout");
-		if (i != 1 || !(fds[0].revents & (POLLIN|POLLERR)))
+		if (i != 1 || !(fds[0].revents & (POLLIN|POLLERR|POLLHUP)))
 			vtc_log(vl, hp->fatal,
 			    "Expected close: poll = %d, revents = 0x%x",
 			    i, fds[0].revents);
@@ -1624,17 +1690,6 @@ cmd_http_fatal(CMD_ARGS)
 
 #define cmd_http_non_fatal cmd_http_fatal
 
-/* SECTION: client-server.spec.delay
- *
- * delay
- *	Same as for the top-level delay.
- *
- * SECTION: client-server.spec.barrier
- *
- * barrier
- *	Same as for the top-level barrier
- */
-
 static const char PREFACE[24] = {
 	0x50, 0x52, 0x49, 0x20, 0x2a, 0x20, 0x48, 0x54,
 	0x54, 0x50, 0x2f, 0x32, 0x2e, 0x30, 0x0d, 0x0a,
@@ -1686,10 +1741,10 @@ cmd_http_rxpri(CMD_ARGS)
 	CAST_OBJ_NOTNULL(hp, priv, HTTP_MAGIC);
 	ONLY_SERVER(hp, av);
 
-	hp->prxbuf = 0;
+	hp->rx_p = hp->rx_b;
 	if (!http_rxchar(hp, sizeof(PREFACE), 0))
 		vtc_fatal(vl, "Couldn't retrieve connection preface");
-	if (memcmp(hp->rxbuf, PREFACE, sizeof(PREFACE)))
+	if (memcmp(hp->rx_b, PREFACE, sizeof(PREFACE)))
 		vtc_fatal(vl, "Received invalid preface\n");
 	start_h2(hp);
 	AN(hp->h2);
@@ -1721,7 +1776,7 @@ cmd_http_settings(CMD_ARGS)
 			if (*p != '\0')
 				vtc_fatal(hp->vl, "-dectbl takes an integer as "
 				    "argument (found %s)", av[1]);
-			HPK_ResizeTbl(hp->decctx, n);
+			assert(HPK_ResizeTbl(hp->decctx, n) != hpk_err);
 			av++;
 		} else
 			vtc_fatal(vl, "Unknown settings spec: %s\n", *av);
@@ -1731,7 +1786,7 @@ cmd_http_settings(CMD_ARGS)
 static void
 cmd_http_stream(CMD_ARGS)
 {
-	struct http *hp = (struct http *)priv;
+	struct http *hp;
 	CAST_OBJ_NOTNULL(hp, priv, HTTP_MAGIC);
 	if (!hp->h2) {
 		vtc_log(hp->vl, 4, "Not in H/2 mode, do what's needed");
@@ -1780,7 +1835,6 @@ cmd_http_write_body(CMD_ARGS)
  */
 
 const struct cmds http_cmds[] = {
-#define CMD(n) { #n, cmd_##n },
 #define CMD_HTTP(n) { #n, cmd_http_##n },
 	/* session */
 	CMD_HTTP(accept)
@@ -1829,18 +1883,28 @@ const struct cmds http_cmds[] = {
 	CMD_HTTP(expect)
 	CMD_HTTP(expect_close)
 	CMD_HTTP(expect_pattern)
-
-	/* general purpose */
-	CMD(barrier)
-	CMD(delay)
-	CMD(shell)
 #undef CMD_HTTP
-#undef CMD
 	{ NULL, NULL }
 };
 
+static void
+http_process_cleanup(void *arg)
+{
+	struct http *hp = arg;
+
+	if (hp->h2)
+		stop_h2(hp);
+	VSB_destroy(&hp->vsb);
+	free(hp->rx_b);
+	free(hp->rem_ip);
+	free(hp->rem_port);
+	free(hp->rem_path);
+	FREE_OBJ(hp);
+}
+
 int
-http_process(struct vtclog *vl, const char *spec, int sock, int *sfd)
+http_process(struct vtclog *vl, const char *spec, int sock, int *sfd,
+	     const char *addr)
 {
 	struct http *hp;
 	int retval;
@@ -1852,8 +1916,11 @@ http_process(struct vtclog *vl, const char *spec, int sock, int *sfd)
 	hp->timeout = vtc_maxdur * 1000 / 2;
 
 	hp->nrxbuf = 2048*1024;
-	hp->rxbuf = malloc(hp->nrxbuf);		/* XXX */
-	AN(hp->rxbuf);
+	hp->rx_b = malloc(hp->nrxbuf);
+	AN(hp->rx_b);
+	hp->rx_e = hp->rx_b + hp->nrxbuf;
+	hp->rx_p = hp->rx_b;
+	*hp->rx_p = '\0';
 
 	hp->vsb = VSB_new_auto();
 	AN(hp->vsb);
@@ -1870,17 +1937,20 @@ http_process(struct vtclog *vl, const char *spec, int sock, int *sfd)
 	hp->gziplevel = 0;
 	hp->gzipresidual = -1;
 
-	VTCP_hisname(sock,
-	    hp->rem_ip, VTCP_ADDRBUFSIZE, hp->rem_port, VTCP_PORTBUFSIZE);
+	if (*addr != '/') {
+		VTCP_hisname(sock, hp->rem_ip, VTCP_ADDRBUFSIZE, hp->rem_port,
+			     VTCP_PORTBUFSIZE);
+		hp->rem_path = NULL;
+	} else {
+		strcpy(hp->rem_ip, "0.0.0.0");
+		strcpy(hp->rem_port, "0");
+		hp->rem_path = strdup(addr);
+	}
+	pthread_cleanup_push(http_process_cleanup, hp);
 	parse_string(spec, http_cmds, hp, vl);
-	if (hp->h2)
-		stop_h2(hp);
 	retval = hp->fd;
-	VSB_destroy(&hp->vsb);
-	free(hp->rxbuf);
-	free(hp->rem_ip);
-	free(hp->rem_port);
-	free(hp);
+	pthread_cleanup_pop(0);
+	http_process_cleanup(hp);
 	return (retval);
 }
 
@@ -1925,7 +1995,7 @@ xxx(void)
 		*ibuf = 0;
 		for (j = 0; j < 7; j++) {
 			snprintf(strchr(ibuf, 0), 5, "%x",
-			    (unsigned)random() & 0xffff);
+			    (unsigned)VRND_RandomTestable() & 0xffff);
 			vz.next_in = TRUST_ME(ibuf);
 			vz.avail_in = strlen(ibuf);
 			vz.next_out = TRUST_ME(obuf);
