@@ -31,7 +31,6 @@
 #include <sys/types.h>
 #include <sys/socket.h>
 
-#include <errno.h>
 #include <math.h>
 #include <poll.h>
 #include <stdio.h>
@@ -77,6 +76,7 @@ enum h2_type {
 	TYPE_MAX
 };
 
+//lint -save -e849	Same enum value
 enum {
 	ACK = 0x1,
 	END_STREAM = 0x1,
@@ -84,6 +84,7 @@ enum {
 	END_HEADERS = 0x4,
 	PRIORITY = 0x20,
 };
+//lint -restore
 
 struct stream {
 	unsigned		magic;
@@ -103,7 +104,7 @@ struct stream {
 	VTAILQ_HEAD(, frame)   fq;
 
 	char			*body;
-	int			bodylen;
+	long			bodylen;
 	struct hpk_hdr		req[MAX_HDR];
 	struct hpk_hdr		resp[MAX_HDR];
 
@@ -124,14 +125,14 @@ clean_headers(struct hpk_hdr *h)
 	}
 }
 
-#define ONLY_CLIENT(hp, av)						\
+#define ONLY_H2_CLIENT(hp, av)						\
 	do {								\
 		if (hp->sfd != NULL)					\
 			vtc_fatal(hp->vl,				\
 			    "\"%s\" only possible in client", av[0]);	\
 	} while (0)
 
-#define ONLY_SERVER(hp, av)						\
+#define ONLY_H2_SERVER(hp, av)						\
 	do {								\
 		if (hp->sfd == NULL)					\
 			vtc_fatal(hp->vl,				\
@@ -252,10 +253,7 @@ readFrameHeader(struct frame *f, const char *buf)
 
 	f->flags = (unsigned char)buf[4];
 
-	f->stid  = (0xff & (unsigned char)buf[5]) << 24;
-	f->stid += (0xff & (unsigned char)buf[6]) << 16;
-	f->stid += (0xff & (unsigned char)buf[7]) <<  8;
-	f->stid += (0xff & (unsigned char)buf[8]);
+	f->stid  = vbe32dec(buf+5);
 }
 
 static void
@@ -271,10 +269,7 @@ writeFrameHeader(char *buf, const struct frame *f)
 
 	buf[4] = f->flags;
 
-	buf[5] = (f->stid >> 24) & 0xff;
-	buf[6] = (f->stid >> 16) & 0xff;
-	buf[7] = (f->stid >>  8) & 0xff;
-	buf[8] = (f->stid      ) & 0xff;
+	vbe32enc(buf + 5, f->stid);
 }
 
 #define INIT_FRAME(f, ty, sz, id, fl) \
@@ -299,8 +294,7 @@ clean_frame(struct frame **f)
 	if ((*f)->type == TYPE_GOAWAY)
 		free((*f)->md.goaway.debug);
 	free((*f)->data);
-	free(*f);
-	*f = NULL;
+	FREE_OBJ(*f);
 }
 
 static void
@@ -381,6 +375,7 @@ parse_data(struct stream *s, struct frame *f)
 	struct http *hp;
 	uint32_t size = f->size;
 	char *data = f->data;
+
 	CHECK_OBJ_NOTNULL(f, FRAME_MAGIC);
 	CHECK_OBJ_NOTNULL(s, STREAM_MAGIC);
 	CAST_OBJ_NOTNULL(hp, s->hp, HTTP_MAGIC);;
@@ -411,12 +406,7 @@ parse_data(struct stream *s, struct frame *f)
 		return;
 	}
 
-	if (s->body) {
-		s->body = realloc(s->body, s->bodylen + size + 1L);
-	} else {
-		AZ(s->bodylen);
-		s->body = malloc(size + 1L);
-	}
+	s->body = realloc(s->body, s->bodylen + size + 1L);
 	AN(s->body);
 	memcpy(s->body + s->bodylen, data, size);
 	s->bodylen += size;
@@ -791,7 +781,7 @@ receive_frame(void *priv)
 					else
 						hdrs = s->resp;
 				}
-				hdrs[0].t = 0;
+				hdrs[0].t = hpk_unset;
 				AZ(vsb);
 				vsb = VSB_new_auto();
 				/*FALLTHROUGH*/
@@ -840,8 +830,8 @@ receive_frame(void *priv)
 		continue;
 	}
 	AZ(pthread_mutex_unlock(&hp->mtx));
-	if (!vtc_error)
-		AZ(vsb);
+	if (vsb != NULL)
+		VSB_destroy(&vsb);
 	return (NULL);
 }
 
@@ -879,7 +869,7 @@ do { \
 
 #define RETURN_BUFFED(val) \
 do { \
-	snprintf(buf, 20, "%d", val); \
+	snprintf(buf, 20, "%ld", (long)val); \
 	return (buf); \
 } while (0)
 
@@ -891,7 +881,7 @@ find_header(const struct hpk_hdr *h, const char *k)
 	int kl = strlen(k);
 	while (h->t) {
 		if (kl == h->key.len  && !strncasecmp(h->key.ptr, k, kl))
-			return h->value.ptr;
+			return (h->value.ptr);
 		h++;
 	}
 	return (NULL);
@@ -931,7 +921,7 @@ cmd_var_resolve(const struct stream *s, const char *spec, char *buf)
 		CHECK_LAST_FRAME(PING);
 		return (f->md.ping.data);
 	}
-	else if (!strcmp(spec, "ping.ack")) {
+	if (!strcmp(spec, "ping.ack")) {
 		CHECK_LAST_FRAME(PING);
 		snprintf(buf, 20, (f->flags & ACK) ? "true" : "false");
 		return (buf);
@@ -940,7 +930,7 @@ cmd_var_resolve(const struct stream *s, const char *spec, char *buf)
 	 * winup.size
 	 *	The size of the upgrade given by the WINDOW_UPDATE frame.
 	 */
-	else if (!strcmp(spec, "winup.size")) {
+	if (!strcmp(spec, "winup.size")) {
 		CHECK_LAST_FRAME(WINDOW_UPDATE);
 		RETURN_BUFFED(f->md.winup_size);
 	}
@@ -954,16 +944,16 @@ cmd_var_resolve(const struct stream *s, const char *spec, char *buf)
 	 * prio.weight
 	 *	The dependency weight.
 	 */
-	else if (!strcmp(spec, "prio.stream")) {
+	if (!strcmp(spec, "prio.stream")) {
 		CHECK_LAST_FRAME(PRIORITY);
 		RETURN_BUFFED(f->md.prio.stream);
 	}
-	else if (!strcmp(spec, "prio.exclusive")) {
+	if (!strcmp(spec, "prio.exclusive")) {
 		CHECK_LAST_FRAME(PRIORITY);
 		snprintf(buf, 20, f->md.prio.exclusive ? "true" : "false");
 		return (buf);
 	}
-	else if (!strcmp(spec, "prio.weight")) {
+	if (!strcmp(spec, "prio.weight")) {
 		CHECK_LAST_FRAME(PRIORITY);
 		RETURN_BUFFED(f->md.prio.weight);
 	}
@@ -971,7 +961,7 @@ cmd_var_resolve(const struct stream *s, const char *spec, char *buf)
 	 * rst.err
 	 *	The error code (as integer) of the RESET_STREAM frame.
 	 */
-	else if (!strcmp(spec, "rst.err")) {
+	if (!strcmp(spec, "rst.err")) {
 		CHECK_LAST_FRAME(RST_STREAM);
 		RETURN_BUFFED(f->md.rst_err);
 	}
@@ -999,14 +989,14 @@ cmd_var_resolve(const struct stream *s, const char *spec, char *buf)
 	 * settings.hdrsize
 	 *	Value of MAX_HEADER_LIST_SIZE if set, <undef> otherwise.
 	 */
-	else if (!strncmp(spec, "settings.", 9)) {
+	if (!strncmp(spec, "settings.", 9)) {
 		CHECK_LAST_FRAME(SETTINGS);
 		spec += 9;
 		if (!strcmp(spec, "ack")) {
 			snprintf(buf, 20, (f->flags & ACK) ? "true" : "false");
 			return (buf);
 		}
-		else if (!strcmp(spec, "push")) {
+		if (!strcmp(spec, "push")) {
 			if (isnan(f->md.settings[2]))
 				return (NULL);
 			else if (f->md.settings[2] == 1)
@@ -1015,17 +1005,17 @@ cmd_var_resolve(const struct stream *s, const char *spec, char *buf)
 				snprintf(buf, 20, "false");
 			return (buf);
 		}
-		else if (!strcmp(spec, "hdrtbl"))     { RETURN_SETTINGS(1); }
-		else if (!strcmp(spec, "maxstreams")) { RETURN_SETTINGS(3); }
-		else if (!strcmp(spec, "winsize"))    { RETURN_SETTINGS(4); }
-		else if (!strcmp(spec, "framesize"))  { RETURN_SETTINGS(5); }
-		else if (!strcmp(spec, "hdrsize"))    { RETURN_SETTINGS(6); }
+		if (!strcmp(spec, "hdrtbl"))     { RETURN_SETTINGS(1); }
+		if (!strcmp(spec, "maxstreams")) { RETURN_SETTINGS(3); }
+		if (!strcmp(spec, "winsize"))    { RETURN_SETTINGS(4); }
+		if (!strcmp(spec, "framesize"))  { RETURN_SETTINGS(5); }
+		if (!strcmp(spec, "hdrsize"))    { RETURN_SETTINGS(6); }
 	}
 	/* SECTION: stream.spec.zexpect.push PUSH_PROMISE specific
 	 * push.id
 	 *	The id of the promised stream.
 	 */
-	else if (!strcmp(spec, "push.id")) {
+	if (!strcmp(spec, "push.id")) {
 		CHECK_LAST_FRAME(PUSH_PROMISE);
 		RETURN_BUFFED(f->md.promised);
 	}
@@ -1039,7 +1029,7 @@ cmd_var_resolve(const struct stream *s, const char *spec, char *buf)
 	 * goaway.debug
 	 *	Debug data, if any.
 	 */
-	else if (!strncmp(spec, "goaway.", 7)) {
+	if (!strncmp(spec, "goaway.", 7)) {
 		spec += 7;
 		CHECK_LAST_FRAME(GOAWAY);
 
@@ -1067,7 +1057,7 @@ cmd_var_resolve(const struct stream *s, const char *spec, char *buf)
 	 * frame.padding (for DATA, HEADERS, PUSH_PROMISE frames)
 	 *	Number of padded bytes.
 	 */
-	else if (!strncmp(spec, "frame.", 6)) {
+	if (!strncmp(spec, "frame.", 6)) {
 		spec += 6;
 		if (!f)
 			vtc_fatal(s->hp->vl, "No frame received yet.");
@@ -1096,19 +1086,19 @@ cmd_var_resolve(const struct stream *s, const char *spec, char *buf)
 	 * stream.dependency
 	 *	Id of the stream this one depends on.
 	 */
-	else if (!strcmp(spec, "stream.window")) {
+	if (!strcmp(spec, "stream.window")) {
 		snprintf(buf, 20, "%jd",
 		    (intmax_t)(s->id ? s->ws : s->hp->ws));
 		return (buf);
 	}
-	else if (!strcmp(spec, "stream.weight")) {
+	if (!strcmp(spec, "stream.weight")) {
 		if (s->id) {
 			snprintf(buf, 20, "%d", s->weight);
 			return (buf);
 		} else
 			return (NULL);
 	}
-	else if (!strcmp(spec, "stream.dependency")) {
+	if (!strcmp(spec, "stream.dependency")) {
 		if (s->id) {
 			snprintf(buf, 20, "%d", s->dependency);
 			return (buf);
@@ -1133,7 +1123,7 @@ cmd_var_resolve(const struct stream *s, const char *spec, char *buf)
 	 *	Value of the header at index INT of the decoding/encoding
 	 *	table.
 	 */
-	else if (!strncmp(spec, "tbl.dec", 7) || !strncmp(spec, "tbl.enc", 7)) {
+	if (!strncmp(spec, "tbl.dec", 7) || !strncmp(spec, "tbl.enc", 7)) {
 		if (spec[4] == 'd')
 			ctx = s->hp->decctx;
 		else
@@ -1186,7 +1176,7 @@ cmd_var_resolve(const struct stream *s, const char *spec, char *buf)
 	 * req.scheme / resp.scheme
 	 *	:method pseudo-header's value.
 	 */
-	else if (!strncmp(spec, "req.", 4) || !strncmp(spec, "resp.", 5)) {
+	if (!strncmp(spec, "req.", 4) || !strncmp(spec, "resp.", 5)) {
 		if (spec[2] == 'q') {
 			h = s->req;
 			spec += 4;
@@ -1214,17 +1204,15 @@ cmd_var_resolve(const struct stream *s, const char *spec, char *buf)
 			return (NULL);
 	}
 #define H2_ERROR(U,v,sc,t) \
-	else if (!strcmp(spec, #U)) { return (#v); }
+	if (!strcmp(spec, #U)) { return (#v); }
 #include "tbl/h2_error.h"
-	else
-		return (spec);
-	return (NULL);
+	return (spec);
 }
 
 /* SECTION: stream.spec.frame_sendhex sendhex
  *
  * Push bytes directly on the wire. sendhex takes exactly one argument: a string
- * describing the bytes, in hex notation, will possible whitespaces between
+ * describing the bytes, in hex notation, with possible whitespaces between
  * them. Here's an example::
  *
  *	sendhex "00 00 08 00 0900	8d"
@@ -1251,19 +1239,19 @@ cmd_sendhex(CMD_ARGS)
 	VSB_destroy(&vsb);
 }
 
-#define ENC(hdr, k, v)			\
-{					\
-	AN(k);				\
-	hdr.key.ptr = strdup(k);	\
-	AN(hdr.key.ptr);		\
-	hdr.key.len = strlen(k);	\
-	AN(v);				\
-	hdr.value.ptr = strdup(v);	\
-	AN(hdr.value.ptr);		\
-	hdr.value.len = strlen(v);	\
-	(void)HPK_EncHdr(iter, &hdr);	\
-	free(hdr.key.ptr);		\
-	free(hdr.value.ptr);		\
+#define ENC(hdr, k, v)					\
+{							\
+	AN(k);						\
+	hdr.key.ptr = strdup(k);			\
+	AN(hdr.key.ptr);				\
+	hdr.key.len = strlen(k);			\
+	AN(v);						\
+	hdr.value.ptr = strdup(v);			\
+	AN(hdr.value.ptr);				\
+	hdr.value.len = strlen(v);			\
+	assert(HPK_EncHdr(iter, &hdr) != hpk_err);	\
+	free(hdr.key.ptr);				\
+	free(hdr.value.ptr);				\
 }
 
 #define STR_ENC(av, field, str)						       \
@@ -1380,7 +1368,7 @@ cmd_tx11obj(CMD_ARGS)
 	int scheme_done = 1;
 	uint32_t stid = 0, pstid;
 	uint32_t weight = 16;
-	int exclusive = 0;
+	uint32_t exclusive = 0;
 	char *buf;
 	struct hpk_iter *iter;
 	struct frame f;
@@ -1397,19 +1385,19 @@ cmd_tx11obj(CMD_ARGS)
 	AN(buf);
 
 	if (!strcmp(cmd_str, "txreq")) {
-		ONLY_CLIENT(s->hp, av);
+		ONLY_H2_CLIENT(s->hp, av);
 		f.type = TYPE_HEADERS;
 		f.flags |= END_STREAM;
 		method_done = 0;
 		path_done = 0;
 		scheme_done = 0;
 	} else if (!strcmp(cmd_str, "txresp")) {
-		ONLY_SERVER(s->hp, av);
+		ONLY_H2_SERVER(s->hp, av);
 		f.type = TYPE_HEADERS;
 		f.flags |= END_STREAM;
 		status_done = 0;
 	} else if (!strcmp(cmd_str, "txpush")) {
-		ONLY_SERVER(s->hp, av);
+		ONLY_H2_SERVER(s->hp, av);
 		f.type = TYPE_PUSH_PROMISE;
 		method_done = 0;
 		path_done = 0;
@@ -1465,7 +1453,7 @@ cmd_tx11obj(CMD_ARGS)
 		else if (AV_IS("-idxHdr")) {
 			hdr.t = hpk_idx;
 			STRTOU32_CHECK(hdr.i, av, p, vl, "-idxHdr", 0);
-			HPK_EncHdr(iter, &hdr);
+			assert(HPK_EncHdr(iter, &hdr) != hpk_err);
 		}
 		else if (AV_IS("-litIdxHdr")) {
 			av++;
@@ -1482,7 +1470,7 @@ cmd_tx11obj(CMD_ARGS)
 			hdr.key.ptr = NULL;
 			hdr.key.len = 0;
 			STR_ENC(av, value,   "third -litHdr");
-			HPK_EncHdr(iter, &hdr);
+			assert(HPK_EncHdr(iter, &hdr) != hpk_err);
 		}
 		else if (AV_IS("-litHdr")) {
 			av++;
@@ -1495,7 +1483,7 @@ cmd_tx11obj(CMD_ARGS)
 
 			STR_ENC(av, key,   "second -litHdr");
 			STR_ENC(av, value, "fourth -litHdr");
-			HPK_EncHdr(iter, &hdr);
+			assert(HPK_EncHdr(iter, &hdr) != hpk_err);
 		}
 		else if (AV_IS("-nostrend")) {
 			f.flags &= ~END_STREAM;
@@ -1865,7 +1853,7 @@ cmd_txsettings(CMD_ARGS)
 		}
 		else if (!strcmp(*av, "-hdrtbl")) {
 			PUT_KV(av, vl, hdrtbl, val, 0x1);
-			HPK_ResizeTbl(s->hp->decctx, val);
+			assert(HPK_ResizeTbl(s->hp->decctx, val) != hpk_err);
 		}
 		else if (!strcmp(*av, "-maxstreams"))
 			PUT_KV(av, vl, maxstreams, val, 0x3);
@@ -2250,9 +2238,9 @@ cmd_rxmsg(CMD_ARGS)
 	CAST_OBJ_NOTNULL(s, priv, STREAM_MAGIC);
 
 	if (!strcmp(av[0], "rxreq"))
-		ONLY_SERVER(s->hp, av);
+		ONLY_H2_SERVER(s->hp, av);
 	else
-		ONLY_CLIENT(s->hp, av);
+		ONLY_H2_CLIENT(s->hp, av);
 
 	f = rxstuff(s);
 	if (!f)
@@ -2271,7 +2259,7 @@ cmd_rxmsg(CMD_ARGS)
 		CHKFRAME(f->type, TYPE_CONTINUATION, rcv, *av);
 	}
 
-	while (!end_stream && (f = rxstuff(s))) {
+	while (!end_stream && (f = rxstuff(s)) != NULL) {
 		rcv++;
 		CHKFRAME(f->type, TYPE_DATA, rcv, *av);
 		end_stream = f->flags & END_STREAM;
@@ -2394,7 +2382,8 @@ cmd_rxframe(CMD_ARGS)
 	(void)vl;
 	(void)av;
 	CAST_OBJ_NOTNULL(s, priv, STREAM_MAGIC);
-	rxstuff(s);
+	if (rxstuff(s) == NULL)
+		vtc_fatal(s->hp->vl, "No frame received");
 }
 
 static void
@@ -2455,7 +2444,6 @@ cmd_write_body(CMD_ARGS)
  * client or a server.
  */
 static const struct cmds stream_cmds[] = {
-#define CMD(n) { #n, cmd_##n },
 #define CMD_STREAM(n) { #n, cmd_##n },
 	/* spec */
 	CMD_STREAM(expect)
@@ -2485,14 +2473,8 @@ static const struct cmds stream_cmds[] = {
 	CMD_STREAM(txsettings)
 	CMD_STREAM(txwinup)
 	CMD_STREAM(write_body)
-
-	/* general purpose */
-	CMD(barrier)
-	CMD(delay)
-	CMD(shell)
 	{ NULL, NULL }
 #undef CMD_STREAM
-#undef CMD
 };
 
 static void *
@@ -2553,6 +2535,7 @@ static void
 stream_delete(struct stream *s)
 {
 	CHECK_OBJ_NOTNULL(s, STREAM_MAGIC);
+	free(s->body);
 	free(s->spec);
 	free(s->name);
 	FREE_OBJ(s);
@@ -2707,25 +2690,27 @@ void
 b64_settings(const struct http *hp, const char *s)
 {
 	uint16_t i;
-	uint64_t v;
+	uint64_t v, vv;
 	const char *buf;
 	int shift;
+
 	while (*s) {
 		v = 0;
 		for (shift = 42; shift >= 0; shift -= 6) {
 			if (*s >= 'A' && *s <= 'Z')
-				v |= (uint64_t)(*s - 'A') << shift;
+				vv = (*s - 'A');
 			else if (*s >= 'a' && *s <= 'z')
-				v |= (uint64_t)(*s - 'a' + 26) << shift;
+				vv = (*s - 'a') + 26;
 			else if (*s >= '0' && *s <= '9')
-				v |= (uint64_t)(*s - '0' + 52) << shift;
+				vv = (*s - '0') + 52;
 			else if (*s == '-')
-				v |= (uint64_t)62 << shift;
+				vv = 62;
 			else if (*s == '_')
-				v |= (uint64_t)63 << shift;
+				vv = 63;
 			else
 				vtc_fatal(hp->vl,
 				    "Bad \"HTTP2-Settings\" header");
+			v |= vv << shift;
 			s++;
 		}
 		i = v >> 32;
@@ -2738,9 +2723,9 @@ b64_settings(const struct http *hp, const char *s)
 
 		if (v == 1) {
 			if (hp->sfd)
-				HPK_ResizeTbl(hp->encctx, v);
+				assert(HPK_ResizeTbl(hp->encctx, v) != hpk_err);
 			else
-				HPK_ResizeTbl(hp->decctx, v);
+				assert(HPK_ResizeTbl(hp->decctx, v) != hpk_err);
 		}
 
 		vtc_log(hp->vl, 4, "Upgrade: %s (%d): %ju",
