@@ -26,14 +26,13 @@
  * SUCH DAMAGE.
  */
 
+#ifdef VTEST_WITH_VTC_VARNISH
+
 #include "config.h"
 
 #include <sys/types.h>
-#include <sys/wait.h>
 #include <sys/socket.h>
-#include <sys/resource.h>
 
-#include <errno.h>
 #include <fcntl.h>
 #include <fnmatch.h>
 #include <inttypes.h>
@@ -49,6 +48,7 @@
 #include "vapi/vsl.h"
 #include "vapi/vsm.h"
 #include "vcli.h"
+#include "vjsn.h"
 #include "vre.h"
 #include "vsub.h"
 #include "vtcp.h"
@@ -65,11 +65,13 @@ struct varnish {
 	int			fds[4];
 	pid_t			pid;
 
-	float			syntax;
+	double			syntax;
 
 	pthread_t		tp;
 	pthread_t		tp_vsl;
 
+	int			expect_exit;
+
 	int			cli_fd;
 	int			vcl_nbr;
 	char			*workdir;
@@ -172,6 +174,11 @@ wait_running(const struct varnish *v)
 			vtc_fatal(v->vl,
 			    "Child stopped before running: %u %s", st, r);
 		if (!strcmp(r, "Child in state running")) {
+			st = varnish_ask_cli(v, "debug.listen_address", &r);
+			if (st != CLIS_OK)
+				vtc_fatal(v->vl,
+					  "CLI status command failed: %u %s",
+					  st, r);
 			free(r);
 			break;
 		}
@@ -191,7 +198,7 @@ vsl_catchup(const struct varnish *v)
 	int vsl_idle;
 
 	vsl_idle = v->vsl_idle;
-	while (!vtc_error && v->pid && vsl_idle == v->vsl_idle)
+	while (!vtc_error && vsl_idle == v->vsl_idle)
 		VTIM_sleep(0.1);
 }
 
@@ -207,6 +214,7 @@ varnishlog_thread(void *priv)
 	unsigned len;
 	const char *tagname, *data;
 	int type, i, opt;
+	struct vsb *vsb = NULL;
 
 	CAST_OBJ_NOTNULL(v, priv, VARNISH_MAGIC);
 
@@ -216,7 +224,7 @@ varnishlog_thread(void *priv)
 
 	c = NULL;
 	opt = 0;
-	while (v->pid || c != NULL) {
+	while (v->fds[1] > 0 || c != NULL) {	//lint !e845 bug in flint
 		if (c == NULL) {
 			if (vtc_error)
 				break;
@@ -251,8 +259,19 @@ varnishlog_thread(void *priv)
 			    'b' : '-';
 			data = VSL_CDATA(c->rec.ptr);
 			v->vsl_tag_count[tag]++;
-			vtc_log(v->vl, 4, "vsl| %10u %-15s %c %.*s",
-			    vxid, tagname, type, (int)len, data);
+			if (VSL_tagflags[tag] & SLT_F_BINARY) {
+				if (vsb == NULL)
+					vsb = VSB_new_auto();
+				VSB_clear(vsb);
+				VSB_quote(vsb, data, len, VSB_QUOTE_HEX);
+				AZ(VSB_finish(vsb));
+				/* +2 to skip "0x" */
+				vtc_log(v->vl, 4, "vsl| %10u %-15s %c [%s]",
+				    vxid, tagname, type, VSB_data(vsb) + 2);
+			} else {
+				vtc_log(v->vl, 4, "vsl| %10u %-15s %c %.*s",
+				    vxid, tagname, type, (int)len, data);
+			}
 		}
 		if (i == 0) {
 			/* Nothing to do but wait */
@@ -275,6 +294,8 @@ varnishlog_thread(void *priv)
 	if (c)
 		VSL_DeleteCursor(c);
 	VSL_Delete(vsl);
+	if (vsb != NULL)
+		VSB_destroy(&vsb);
 
 	return (NULL);
 }
@@ -296,13 +317,10 @@ varnish_new(const char *name)
 
 	REPLACE(v->jail, "");
 
-	v->syntax = 4.0;
-
 	v->vl = vtc_logopen(name);
 	AN(v->vl);
 
-	bprintf(buf, "${tmpdir}/%s", name);
-	vsb = macro_expand(v->vl, buf);
+	vsb = macro_expandf(v->vl, "${tmpdir}/%s", name);
 	AN(vsb);
 	v->workdir = strdup(VSB_data(vsb));
 	AN(v->workdir);
@@ -358,32 +376,9 @@ static void *
 varnish_thread(void *priv)
 {
 	struct varnish *v;
-	char buf[65536];
-	struct pollfd fds[1];
-	int i;
 
 	CAST_OBJ_NOTNULL(v, priv, VARNISH_MAGIC);
-	(void)VTCP_nonblocking(v->fds[0]);
-	while (1) {
-		memset(fds, 0, sizeof fds);
-		fds->fd = v->fds[0];
-		fds->events = POLLIN;
-		i = poll(fds, 1, 1000);
-		if (i == 0)
-			continue;
-		if (fds->revents & POLLIN) {
-			i = read(v->fds[0], buf, sizeof buf - 1);
-			if (i > 0) {
-				buf[i] = '\0';
-				vtc_dump(v->vl, 3, "debug", buf, -2);
-			}
-		}
-		if (fds->revents & (POLLERR|POLLHUP)) {
-			vtc_log(v->vl, 4, "STDOUT poll 0x%x", fds->revents);
-			break;
-		}
-	}
-	return (NULL);
+	return (vtc_record(v->vl, v->fds[0], NULL));
 }
 
 /**********************************************************************
@@ -419,8 +414,9 @@ varnish_launch(struct varnish *v)
 	if (vtc_witness)
 		VSB_cat(vsb, " -p debug=+witness");
 	if (leave_temp) {
-		VSB_cat(vsb, " -p debug=+vsm_keep");
+		VSB_cat(vsb, " -p debug=+vcl_keep");
 		VSB_cat(vsb, " -p debug=+vmod_so_keep");
+		VSB_cat(vsb, " -p debug=+vsm_keep");
 	}
 	VSB_printf(vsb, " -l 2m");
 	VSB_printf(vsb, " -p auto_restart=off");
@@ -617,8 +613,6 @@ static void
 varnish_cleanup(struct varnish *v)
 {
 	void *p;
-	int status, r;
-	struct rusage ru;
 
 	/* Close the CLI connection */
 	closefd(&v->cli_fd);
@@ -630,26 +624,11 @@ varnish_cleanup(struct varnish *v)
 	AZ(pthread_join(v->tp, &p));
 	closefd(&v->fds[0]);
 
-	r = wait4(v->pid, &status, 0, &ru);
-	v->pid = 0;
-	vtc_log(v->vl, 2, "R %d Status: %04x (u %.6f s %.6f)", r, status,
-	    ru.ru_utime.tv_sec + 1e-6 * ru.ru_utime.tv_usec,
-	    ru.ru_stime.tv_sec + 1e-6 * ru.ru_stime.tv_usec
-	);
-
 	/* Pick up the VSL thread */
 	AZ(pthread_join(v->tp_vsl, &p));
 
-	if (WIFEXITED(status) && (WEXITSTATUS(status) == 0))
-		return;
-#ifdef WCOREDUMP
-	vtc_fatal(v->vl, "Bad exit code: %04x sig %x exit %x core %x",
-	    status, WTERMSIG(status), WEXITSTATUS(status),
-	    WCOREDUMP(status));
-#else
-	vtc_fatal(v->vl, "Bad exit code: %04x sig %x exit %x",
-	    status, WTERMSIG(status), WEXITSTATUS(status));
-#endif
+	vtc_wait4(v->vl, v->pid, v->expect_exit, 0, 0);
+	v->pid = 0;
 }
 
 /**********************************************************************
@@ -679,6 +658,34 @@ varnish_wait(struct varnish *v)
 }
 
 
+/**********************************************************************
+ * Ask a CLI JSON question
+ */
+
+static void
+varnish_cli_json(struct varnish *v, const char *cli)
+{
+	enum VCLI_status_e u;
+	char *resp = NULL;
+	const char *errptr;
+	struct vjsn *vj;
+
+	if (v->cli_fd < 0)
+		varnish_launch(v);
+	if (vtc_error)
+		return;
+	u = varnish_ask_cli(v, cli, &resp);
+	vtc_log(v->vl, 2, "CLI %03u <%s>", u, cli);
+	if (u != CLIS_OK)
+		vtc_fatal(v->vl,
+		    "FAIL CLI response %u expected %u", u, CLIS_OK);
+	vj = vjsn_parse(resp, &errptr);
+	if (vj == NULL)
+		vtc_fatal(v->vl, "FAIL CLI, not good JSON: %s", errptr);
+	vjsn_delete(&vj);
+	free(resp);
+}
+
 /**********************************************************************
  * Ask a CLI question
  */
@@ -778,7 +785,7 @@ varnish_vclbackend(struct varnish *v, const char *vcl)
 
 	VSB_printf(vsb2, "vcl %.1f;\n", v->syntax);
 
-	cmd_server_genvcl(vsb2);
+	cmd_server_gen_vcl(vsb2);
 
 	AZ(VSB_finish(vsb2));
 
@@ -851,7 +858,7 @@ varnish_vsc(const struct varnish *v, const char *arg)
  */
 
 struct stat_priv {
-	char target_name[256];
+	char target_pattern[256];
 	uintmax_t val;
 	const struct varnish *v;
 };
@@ -862,10 +869,10 @@ do_expect_cb(void *priv, const struct VSC_point * const pt)
 	struct stat_priv *sp = priv;
 
 	if (pt == NULL)
-		return(0);
+		return (0);
 
-	if (strcmp(pt->name, sp->target_name))
-		return(0);
+	if (fnmatch(sp->target_pattern, pt->name, 0))
+		return (0);
 
 	AZ(strcmp(pt->ctype, "uint64_t"));
 	AN(pt->ptr);
@@ -897,9 +904,9 @@ varnish_expect(const struct varnish *v, char * const *av)
 	}
 	p = strrchr(r, '.');
 	if (p == NULL) {
-		bprintf(sp.target_name, "MAIN.%s", r);
+		bprintf(sp.target_pattern, "MAIN.%s", r);
 	} else {
-		bprintf(sp.target_name, "%s", r);
+		bprintf(sp.target_pattern, "%s", r);
 	}
 
 	sp.val = 0;
@@ -952,6 +959,7 @@ varnish_expect(const struct varnish *v, char * const *av)
 		    av[0], (uintmax_t)sp.val, av[1], av[2], (uintmax_t)ref);
 	}
 }
+
 /* SECTION: varnish varnish
  *
  * Define and interact with varnish instances.
@@ -965,7 +973,13 @@ varnish_expect(const struct varnish *v, char * const *av)
  * process in the background, waiting for the ``-start`` switch to actually
  * start the child.
  *
- * With:
+ * Types used in the description below:
+ *
+ * PATTERN
+ *         is a 'glob' style pattern (ie: fnmatch(3)) as used in shell filename
+ *         expansion.
+ *
+ * Arguments:
  *
  * vNAME
  *	   Identify the Varnish server with a string, it must starts with 'v'.
@@ -1003,7 +1017,7 @@ varnish_expect(const struct varnish *v, char * const *av)
  *         Stop the child process.
  *
  * \-syntax
- *         Set the VCL syntax level (default: 4.0)
+ *         Set the VCL syntax level for this command (default: 4.1)
  *
  * \-wait
  *         Wait for that instance to terminate.
@@ -1022,7 +1036,7 @@ varnish_expect(const struct varnish *v, char * const *av)
  * ``varnishadm``) with these additional switches::
  *
  *         varnish vNAME [-cli STRING] [-cliok STRING] [-clierr STRING]
- *                       [-expect STRING OP NUMBER]
+ *                       [-clijson STRING] [-expect STRING OP NUMBER]
  *
  * \-cli STRING|-cliok STRING|-clierr STATUS STRING|-cliexpect REGEXP STRING
  *         All four of these will send STRING to the CLI, the only difference
@@ -1030,17 +1044,21 @@ varnish_expect(const struct varnish *v, char * const *av)
  *         anything, -cliok expects 200, -clierr expects STATUS, and
  *         -cliexpect expects the REGEXP to match the returned response.
  *
- * \-expect STRING OP NUMBER
- *         Look into the VSM and make sure the counter identified by STRING has
- *         a correct value. OP can be ==, >, >=, <, <=. For example::
+ * \-clijson STRING
+ *	   Send STRING to the CLI, expect success (CLIS_OK/200) and check
+ *	   that the response is parsable JSON.
  *
- *                 varnish v1 -expect SMA.s1.g_space > 1000000
+ * \-expect PATTERN OP NUMBER
+ *         Look into the VSM and make sure the first VSC counter identified by
+ *         PATTERN has a correct value. OP can be ==, >, >=, <, <=. For
+ *         example::
+ *
+ *                 varnish v1 -expect SM?.s1.g_space > 1000000
+ * \-expectexit NUMBER
+ *	   Expect varnishd to exit(3) with this value
  *
  * \-vsc PATTERN
- *         Dump VSC counters matching PATTERN.  The PATTERN is a 'glob'
- *         style pattern (ie: fnmatch(3)) as used in shell filename expansion.
- *         To see all counters use pattern "*", to see all counters about
- *         requests use "*req*".
+ *         Dump VSC counters matching PATTERN.
  *
  * \-vsl_catchup
  *         Wait until the logging thread has idled to make sure that all
@@ -1076,6 +1094,7 @@ cmd_varnish(CMD_ARGS)
 	if (v == NULL)
 		v = varnish_new(av[0]);
 	av++;
+	v->syntax = 4.1;
 
 	for (; *av != NULL; av++) {
 		if (vtc_error)
@@ -1115,6 +1134,12 @@ cmd_varnish(CMD_ARGS)
 			av += 2;
 			continue;
 		}
+		if (!strcmp(*av, "-clijson")) {
+			AN(av[1]);
+			varnish_cli_json(v, av[1]);
+			av++;
+			continue;
+		}
 		if (!strcmp(*av, "-cliok")) {
 			AN(av[1]);
 			varnish_cli(v, av[1], (unsigned)CLIS_OK, NULL);
@@ -1144,6 +1169,11 @@ cmd_varnish(CMD_ARGS)
 			av += 2;
 			continue;
 		}
+		if (!strcmp(*av, "-expectexit")) {
+			v->expect_exit = strtoul(av[1], NULL, 0);
+			av++;
+			continue;
+		}
 		if (!strcmp(*av, "-jail")) {
 			AN(av[1]);
 			AZ(v->pid);
@@ -1209,3 +1239,5 @@ cmd_varnish(CMD_ARGS)
 		vtc_fatal(v->vl, "Unknown varnish argument: %s", *av);
 	}
 }
+
+#endif /* VTEST_WITH_VTC_VARNISH */
