@@ -34,7 +34,6 @@
 #include "config.h"
 
 #include <ctype.h>
-#include <errno.h>
 #include <pthread.h>
 #include <signal.h>
 #include <stdarg.h>
@@ -70,8 +69,8 @@ struct top {
 	char			*rec_buf;
 	int			clen;
 	unsigned		hash;
-	VRB_ENTRY(top)		e_order;
-	VRB_ENTRY(top)		e_key;
+	VRBT_ENTRY(top)		e_order;
+	VRBT_ENTRY(top)		e_key;
 	double			count;
 };
 
@@ -85,8 +84,8 @@ static const char *ident;
 
 static volatile sig_atomic_t quit = 0;
 
-static VRB_HEAD(t_order, top) h_order = VRB_INITIALIZER(&h_order);
-static VRB_HEAD(t_key, top) h_key = VRB_INITIALIZER(&h_key);
+static VRBT_HEAD(t_order, top) h_order = VRBT_INITIALIZER(&h_order);
+static VRBT_HEAD(t_key, top) h_key = VRBT_INITIALIZER(&h_key);
 
 static inline int
 cmp_key(const struct top *a, const struct top *b)
@@ -110,10 +109,10 @@ cmp_order(const struct top *a, const struct top *b)
 	return (cmp_key(a, b));
 }
 
-VRB_PROTOTYPE_STATIC(t_order, top, e_order, cmp_order)
-VRB_GENERATE_STATIC(t_order, top, e_order, cmp_order)
-VRB_PROTOTYPE_STATIC(t_key, top, e_key, cmp_key)
-VRB_GENERATE_STATIC(t_key, top, e_key, cmp_key)
+VRBT_PROTOTYPE_STATIC(t_order, top, e_order, cmp_order)
+VRBT_GENERATE_STATIC(t_order, top, e_order, cmp_order)
+VRBT_PROTOTYPE_STATIC(t_key, top, e_key, cmp_key)
+VRBT_GENERATE_STATIC(t_key, top, e_key, cmp_key)
 
 static int v_matchproto_(VSLQ_dispatch_f)
 accumulate(struct VSL_data *vsl, struct VSL_transaction * const pt[],
@@ -155,12 +154,12 @@ accumulate(struct VSL_data *vsl, struct VSL_transaction * const pt[],
 			t.rec_data = VSL_CDATA(tr->c->rec.ptr);
 
 			AZ(pthread_mutex_lock(&mtx));
-			tp = VRB_FIND(t_key, &h_key, &t);
+			tp = VRBT_FIND(t_key, &h_key, &t);
 			if (tp) {
-				VRB_REMOVE(t_order, &h_order, tp);
+				VRBT_REMOVE(t_order, &h_order, tp);
 				tp->count += 1.0;
 				/* Reinsert to rebalance */
-				VRB_INSERT(t_order, &h_order, tp);
+				VRBT_INSERT(t_order, &h_order, tp);
 			} else {
 				ntop++;
 				tp = calloc(1, sizeof *tp);
@@ -172,8 +171,8 @@ accumulate(struct VSL_data *vsl, struct VSL_transaction * const pt[],
 				tp->rec_buf = strdup(t.rec_data);
 				tp->rec_data = tp->rec_buf;
 				AN(tp->rec_data);
-				VRB_INSERT(t_key, &h_key, tp);
-				VRB_INSERT(t_order, &h_order, tp);
+				VRBT_INSERT(t_key, &h_key, tp);
+				VRBT_INSERT(t_order, &h_order, tp);
 			}
 			AZ(pthread_mutex_unlock(&mtx));
 
@@ -183,22 +182,6 @@ accumulate(struct VSL_data *vsl, struct VSL_transaction * const pt[],
 	return (0);
 }
 
-static int v_matchproto_(VUT_cb_f)
-sighup(struct VUT *v)
-{
-	assert(v == vut);
-	quit = 1;
-	return (1);
-}
-
-static void
-vut_sighandler(int sig)
-{
-
-	if (vut != NULL)
-		VUT_Signaled(vut, sig);
-}
-
 static void
 update(int p)
 {
@@ -226,8 +209,8 @@ update(int p)
 	else
 		AC(mvprintw(0, len - 1, "%s", q));
 	AC(mvprintw(0, 0, "list length %u", ntop));
-	for (tp = VRB_MIN(t_order, &h_order); tp != NULL; tp = tp2) {
-		tp2 = VRB_NEXT(t_order, &h_order, tp);
+	for (tp = VRBT_MIN(t_order, &h_order); tp != NULL; tp = tp2) {
+		tp2 = VRBT_NEXT(t_order, &h_order, tp);
 
 		if (++l < LINES) {
 			len = tp->clen;
@@ -243,8 +226,8 @@ update(int p)
 			continue;
 		tp->count += (1.0/3.0 - tp->count) / (double)n;
 		if (tp->count * 10 < t || l > LINES * 10) {
-			VRB_REMOVE(t_key, &h_key, tp);
-			VRB_REMOVE(t_order, &h_order, tp);
+			VRBT_REMOVE(t_key, &h_key, tp);
+			VRBT_REMOVE(t_order, &h_order, tp);
 			free(tp->rec_buf);
 			free(tp);
 			ntop--;
@@ -274,7 +257,7 @@ do_curses(void *arg)
 	(void)curs_set(0);
 	AC(erase());
 	timeout(1000);
-	while (!quit) {
+	while (!quit && !vut->last_sighup) {
 		AZ(pthread_mutex_lock(&mtx));
 		update(period);
 		AZ(pthread_mutex_unlock(&mtx));
@@ -316,27 +299,14 @@ static void
 dump(void)
 {
 	struct top *tp, *tp2;
-	for (tp = VRB_MIN(t_order, &h_order); tp != NULL; tp = tp2) {
-		tp2 = VRB_NEXT(t_order, &h_order, tp);
+	for (tp = VRBT_MIN(t_order, &h_order); tp != NULL; tp = tp2) {
+		tp2 = VRBT_NEXT(t_order, &h_order, tp);
 		printf("%9.2f %s %*.*s\n",
 			tp->count, VSL_tags[tp->tag],
 			tp->clen, tp->clen, tp->rec_data);
 	}
 }
 
-//lint -sem(usage, r_no)
-static void v_noreturn_
-usage(int status)
-{
-	const char **opt;
-
-	fprintf(stderr, "Usage: %s <options>\n\n", vut->progname);
-	fprintf(stderr, "Options:\n");
-	for (opt = vopt_spec.vopt_usage; *opt != NULL; opt +=2)
-		fprintf(stderr, " %-25s %s\n", *opt, *(opt + 1));
-	exit(status);
-}
-
 int
 main(int argc, char **argv)
 {
@@ -358,7 +328,7 @@ main(int argc, char **argv)
 			break;
 		case 'h':
 			/* Usage help */
-			usage(0);
+			VUT_Usage(vut, &vopt_spec, 0);
 		case 'p':
 			errno = 0;
 			e = NULL;
@@ -371,14 +341,13 @@ main(int argc, char **argv)
 			break;
 		default:
 			if (!VUT_Arg(vut, o, optarg))
-				usage(1);
+				VUT_Usage(vut, &vopt_spec, 1);
 		}
 	}
 
 	if (optind != argc)
-		usage(1);
+		VUT_Usage(vut, &vopt_spec, 1);
 
-	VUT_Signal(vut_sighandler);
 	VUT_Setup(vut);
 	if (vut->vsm)
 		ident = VSM_Dup(vut->vsm, "Arg", "-i");
@@ -386,7 +355,6 @@ main(int argc, char **argv)
 		ident = strdup("");
 	vut->dispatch_f = accumulate;
 	vut->dispatch_priv = NULL;
-	vut->sighup_f = sighup;
 	if (once) {
 		(void)VUT_Main(vut);
 		dump();
