@@ -30,6 +30,15 @@
 
 /*lint -save -e525 -e539 */
 
+#if defined(XYZZY)
+  #error "Temporary macro XYZZY already defined"
+#endif
+
+#if defined(HAVE_ACCEPT_FILTERS)
+  #define XYZZY MUST_RESTART
+#else
+  #define XYZZY NOT_IMPLEMENTED
+#endif
 PARAM(
 	/* name */	accept_filter,
 	/* typ */	bool,
@@ -37,12 +46,13 @@ PARAM(
 	/* max */	NULL,
 	/* default */	"on",
 	/* units */	"bool",
-	/* flags */	MUST_RESTART,
+	/* flags */	XYZZY,
 	/* s-text */
-	"Enable kernel accept-filters (if available in the kernel).",
+	"Enable kernel accept-filters.",
 	/* l-text */	NULL,
 	/* func */	NULL
 )
+#undef XYZZY
 
 PARAM(
 	/* name */	acceptor_sleep_decay,
@@ -126,6 +136,38 @@ PARAM(
 	/* func */	NULL
 )
 
+PARAM(
+	/* name */	ban_cutoff,
+	/* typ */	uint,
+	/* min */	"0",
+	/* max */	NULL,
+	/* default */	"0",
+	/* units */	"bans",
+	/* flags */	EXPERIMENTAL,
+	/* s-text */
+	"Expurge long tail content from the cache to keep the number of bans "
+	"below this value. 0 disables.\n\n"
+	"When this parameter is set to a non-zero value, the ban lurker "
+	"continues to work the ban list as usual top to bottom, but when it "
+	"reaches the ban_cutoff-th ban, it treats all objects as if they "
+	"matched a ban and expurges them from cache. As actively used objects "
+	"get tested against the ban list at request time and thus are likely "
+	"to be associated with bans near the top of the ban list, with "
+	"ban_cutoff, least recently accessed objects (the \"long tail\") are "
+	"removed.\n\n"
+	"This parameter is a safety net to avoid bad response times due to "
+	"bans being tested at lookup time. Setting a cutoff trades response "
+	"time for cache efficiency. The recommended value is proportional to "
+	"rate(bans_lurker_tests_tested) / n_objects while the ban lurker is "
+	"working, which is the number of bans the system can sustain. The "
+	"additional latency due to request ban testing is in the order of "
+	"ban_cutoff / rate(bans_lurker_tests_tested). For example, for "
+	"rate(bans_lurker_tests_tested) = 2M/s and a tolerable latency of "
+	"100ms, a good value for ban_cutoff may be 200K.",
+	/* l-text */	"",
+	/* func */	NULL
+)
+
 PARAM(
 	/* name */	ban_lurker_age,
 	/* typ */	timeout,
@@ -137,8 +179,10 @@ PARAM(
 	/* s-text */
 	"The ban lurker will ignore bans until they are this old.  "
 	"When a ban is added, the active traffic will be tested against it "
-	"as part of object lookup.  This parameter "
-	"holds the ban-lurker off, until the rush is over.",
+	"as part of object lookup.  Because many applications issue bans in "
+	"bursts, this parameter holds the ban-lurker off until the rush is "
+	"over.\n"
+	"This should be set to the approximate time which a ban-burst takes.",
 	/* l-text */	"",
 	/* func */	NULL
 )
@@ -176,6 +220,21 @@ PARAM(
 	/* func */	NULL
 )
 
+PARAM(
+	/* name */	ban_lurker_holdoff,
+	/* typ */	timeout,
+	/* min */	"0",
+	/* max */	NULL,
+	/* default */	"0.010",
+	/* units */	"seconds",
+	/* flags */	EXPERIMENTAL,
+	/* s-text */
+	"How long the ban lurker sleeps when giving way to lookup"
+	" due to lock contention.",
+	/* l-text */	"",
+	/* func */	NULL
+)
+
 PARAM(
 	/* name */	first_byte_timeout,
 	/* typ */	timeout,
@@ -227,18 +286,40 @@ PARAM(
 )
 
 PARAM(
-	/* name */	cli_buffer,
-	/* typ */	bytes_u,
-	/* min */	"4k",
+	/* name */	backend_local_error_holddown,
+	/* typ */	timeout,
+	/* min */	"0.000",
 	/* max */	NULL,
-	/* default */	"8k",
-	/* units */	"bytes",
-	/* flags */	0,
+	/* default */	"10.000",
+	/* units */	"seconds",
+	/* flags */	EXPERIMENTAL,
 	/* s-text */
-	"Size of buffer for CLI command input.\n"
-	"You may need to increase this if you have big VCL files and use "
-	"the vcl.inline CLI command.\n"
-	"NB: Must be specified with -p to have effect.",
+	"When connecting to backends, certain error codes "
+	"(EADDRNOTAVAIL, EACCESS, EPERM) signal a local resource shortage "
+	"or configuration issue for which retrying connection attempts "
+	"may worsen the situation due to the complexity of the operations "
+	"involved in the kernel.\n"
+	"This parameter prevents repeated connection attempts for the "
+	"configured duration.",
+	/* l-text */	"",
+	/* func */	NULL
+)
+
+PARAM(
+	/* name */	backend_remote_error_holddown,
+	/* typ */	timeout,
+	/* min */	"0.000",
+	/* max */	NULL,
+	/* default */	"0.250",
+	/* units */	"seconds",
+	/* flags */	EXPERIMENTAL,
+	/* s-text */
+	"When connecting to backends, certain error codes (ECONNREFUSED, "
+	"ENETUNREACH) signal fundamental connection issues such as the backend "
+	"not accepting connections or routing problems for which repeated "
+	"connection attempts are considered useless\n"
+	"This parameter prevents repeated connection attempts for the "
+	"configured duration.",
 	/* l-text */	"",
 	/* func */	NULL
 )
@@ -289,6 +370,21 @@ PARAM(
 	/* func */	NULL
 )
 
+PARAM(
+	/* name */	clock_step,
+	/* typ */	timeout,
+	/* min */	"0.000",
+	/* max */	NULL,
+	/* default */	"1.000",
+	/* units */	"seconds",
+	/* flags */	0,
+	/* s-text */
+	"How much observed clock step we are willing to accept before "
+	"we panic.",
+	/* l-text */	"",
+	/* func */	NULL
+)
+
 PARAM(
 	/* name */	connect_timeout,
 	/* typ */	timeout,
@@ -322,6 +418,8 @@ PARAM(
 )
 
 #if 0
+/* actual location mgt_param_bits.c*/
+/* see tbl/debug_bits.h */
 PARAM(
 	/* name */	debug,
 	/* typ */	debug,
@@ -399,7 +497,25 @@ PARAM(
 	/* func */	NULL
 )
 
+PARAM(
+	/* name */	http1_iovs,
+	/* typ */	uint,
+	/* min */	"5",
+	/* max */	"1024",		// XXX stringify IOV_MAX
+	/* default */	"64",
+	/* units */	"struct iovec (=16 bytes)",
+	/* flags */	WIZARD,
+	/* s-text */
+	"Number of io vectors to allocate for HTTP1 protocol transmission."
+	"  A HTTP1 header needs 7 + 2 per HTTP header field."
+	"  Allocated from workspace_thread.",
+	/* l-text */	"",
+	/* func */	NULL
+)
+
 #if 0
+/* actual location mgt_param_bits.c*/
+/* See tbl/feature_bits.h */
 PARAM(
 	/* name */	feature,
 	/* typ */	feature,
@@ -522,7 +638,11 @@ PARAM(
 	"\n"
 	"Clients that do not support gzip will have their Accept-Encoding "
 	"header removed. For more information on how gzip is implemented "
-	"please see the chapter on gzip in the Varnish reference.",
+	"please see the chapter on gzip in the Varnish reference.\n"
+	"\n"
+	"When gzip support is disabled the variables beresp.do_gzip and "
+	"beresp.do_gunzip have no effect in VCL.",
+	/* XXX: what about the effect on beresp.filters? */
 	/* l-text */	"",
 	/* func */	NULL
 )
@@ -619,14 +739,23 @@ PARAM(
 	/* s-text */
 	"Maximum number of bytes of HTTP backend response we will deal "
 	"with.  This is a limit on all bytes up to the double blank line "
-	"which ends the HTTP request.\n"
-	"The memory for the request is allocated from the backend workspace "
+	"which ends the HTTP response.\n"
+	"The memory for the response is allocated from the backend workspace "
 	"(param: workspace_backend) and this parameter limits how much "
-	"of that the request is allowed to take up.",
+	"of that the response is allowed to take up.",
 	/* l-text */	"",
 	/* func */	NULL
 )
 
+#if defined(XYZZY)
+  #error "Temporary macro XYZZY already defined"
+#endif
+
+#if defined(SO_SNDTIMEO_WORKS)
+  #define XYZZY DELAYED_EFFECT
+#else
+  #define XYZZY NOT_IMPLEMENTED
+#endif
 PARAM(
 	/* name */	idle_send_timeout,
 	/* typ */	timeout,
@@ -634,15 +763,17 @@ PARAM(
 	/* max */	NULL,
 	/* default */	"60.000",
 	/* units */	"seconds",
-	/* flags */	DELAYED_EFFECT,
+	/* flags */	XYZZY,
 	/* s-text */
-	"Time to wait with no data sent. If no data has been transmitted "
-	"in this many\n"
-	"seconds the session is closed.\n"
-	"See setsockopt(2) under SO_SNDTIMEO for more information.",
+	"Send timeout for individual pieces of data on client connections."
+	" May get extended if 'send_timeout' applies.\n\n"
+	"When this timeout is hit, the session is closed.\n\n"
+	"See the man page for `setsockopt(2)` under ``SO_SNDTIMEO`` for more"
+	" information.",
 	/* l-text */	"",
 	/* func */	NULL
 )
+#undef XYZZY
 
 PARAM(
 	/* name */	listen_depth,
@@ -699,9 +830,7 @@ PARAM(
 	/* units */	"restarts",
 	/* flags */	0,
 	/* s-text */
-	"Upper limit on how many times a request can restart.\n"
-	"Be aware that restarts are likely to cause a hit against the "
-	"backend, so don't increase thoughtlessly.",
+	"Upper limit on how many times a request can restart.",
 	/* l-text */	"",
 	/* func */	NULL
 )
@@ -736,6 +865,7 @@ PARAM(
 )
 
 #if 0
+/* actual location mgt_param_tbl.c */
 PARAM(
 	/* name */	pcre_match_limit,
 	/* typ */	uint,
@@ -751,6 +881,7 @@ PARAM(
 	/* func */	NULL
 )
 
+/* actual location mgt_param_tbl.c */
 PARAM(
 	/* name */	pcre_match_limit_recursion,
 	/* typ */	uint,
@@ -799,6 +930,7 @@ PARAM(
 )
 
 #if 0
+/* actual location mgt_param_tbl.c */
 PARAM(
 	/* name */	pool_req,
 	/* typ */	poolparam,
@@ -814,6 +946,7 @@ PARAM(
 	/* func */	NULL
 )
 
+/* actual location mgt_param_tbl.c */
 PARAM(
 	/* name */	pool_sess,
 	/* typ */	poolparam,
@@ -829,6 +962,7 @@ PARAM(
 	/* func */	NULL
 )
 
+/* actual location mgt_param_tbl.c */
 PARAM(
 	/* name */	pool_vbo,
 	/* typ */	poolparam,
@@ -877,6 +1011,15 @@ PARAM(
 	/* func */	NULL
 )
 
+#if defined(XYZZY)
+  #error "Temporary macro XYZZY already defined"
+#endif
+
+#if defined(SO_SNDTIMEO_WORKS)
+  #define XYZZY DELAYED_EFFECT
+#else
+  #define XYZZY NOT_IMPLEMENTED
+#endif
 PARAM(
 	/* name */	send_timeout,
 	/* typ */	timeout,
@@ -884,46 +1027,18 @@ PARAM(
 	/* max */	NULL,
 	/* default */	"600.000",
 	/* units */	"seconds",
-	/* flags */	DELAYED_EFFECT,
+	/* flags */	XYZZY,
 	/* s-text */
-	"Send timeout for client connections. If the HTTP response hasn't "
-	"been transmitted in this many\n"
-	"seconds the session is closed.\n"
-	"See setsockopt(2) under SO_SNDTIMEO for more information.",
-	/* l-text */	"",
-	/* func */	NULL
-)
-
-PARAM(
-	/* name */	session_max,
-	/* typ */	uint,
-	/* min */	"1000",
-	/* max */	NULL,
-	/* default */	"100000",
-	/* units */	"sessions",
-	/* flags */	0,
-	/* s-text */
-	"Maximum number of sessions we will allocate from one pool before "
-	"just dropping connections.\n"
-	"This is mostly an anti-DoS measure, and setting it plenty high "
-	"should not hurt, as long as you have the memory for it.",
-	/* l-text */	"",
-	/* func */	NULL
-)
-
-PARAM(
-	/* name */	shm_reclen,
-	/* typ */	vsl_reclen,
-	/* min */	"16b",
-	/* max */	NULL,
-	/* default */	"255b",
-	/* units */	"bytes",
-	/* flags */	0,
-	/* s-text */
-	"Old name for vsl_reclen, use that instead.",
+	"Total timeout for ordinary HTTP1 responses. Does not apply to some"
+	" internally generated errors and pipe mode.\n\n"
+	"When 'idle_send_timeout' is hit while sending an HTTP1 response, the"
+	" timeout is extended unless the total time already taken for sending"
+	" the response in its entirety exceeds this many seconds.\n\n"
+	"When this timeout is hit, the session is closed",
 	/* l-text */	"",
 	/* func */	NULL
 )
+#undef XYZZY
 
 PARAM(
 	/* name */	shortlived,
@@ -969,6 +1084,11 @@ PARAM(
 	/* func */	NULL
 )
 
+#if defined(HAVE_TCP_FASTOPEN)
+  #define XYZZY MUST_RESTART
+#else
+  #define XYZZY NOT_IMPLEMENTED
+#endif
 PARAM(
 	/* name */	tcp_fastopen,
 	/* typ */	bool,
@@ -976,24 +1096,30 @@ PARAM(
 	/* max */	NULL,
 	/* default */	"off",
 	/* units */	"bool",
-	/* flags */	MUST_RESTART,
+	/* flags */	XYZZY,
 	/* s-text */
-	"Enable TCP Fast Open extension (if available in the kernel).",
+	"Enable TCP Fast Open extension.",
 	/* l-text */	NULL,
 	/* func */	NULL
 )
+#undef XYZZY
 
-#if 0
+#if defined(HAVE_TCP_KEEP)
+  #define XYZZY	EXPERIMENTAL
+#else
+  #define XYZZY	NOT_IMPLEMENTED
+#endif
 PARAM(
 	/* name */	tcp_keepalive_intvl,
 	/* typ */	timeout,
-	/* min */	"1.000",
-	/* max */	"100.000",
-	/* default */	"5.000",
+	/* min */	"1",
+	/* max */	"100",
+	/* default */	"",
 	/* units */	"seconds",
-	/* flags */	EXPERIMENTAL,
+	/* flags */	XYZZY,
 	/* s-text */
-	"The number of seconds between TCP keep-alive probes.",
+	"The number of seconds between TCP keep-alive probes. "
+	"Ignored for Unix domain sockets.",
 	/* l-text */	"",
 	/* func */	NULL
 )
@@ -1003,13 +1129,13 @@ PARAM(
 	/* typ */	uint,
 	/* min */	"1",
 	/* max */	"100",
-	/* default */	"5",
+	/* default */	"",
 	/* units */	"probes",
-	/* flags */	EXPERIMENTAL,
+	/* flags */	XYZZY,
 	/* s-text */
 	"The maximum number of TCP keep-alive probes to send before giving "
 	"up and killing the connection if no response is obtained from the "
-	"other end.",
+	"other end. Ignored for Unix domain sockets.",
 	/* l-text */	"",
 	/* func */	NULL
 )
@@ -1017,18 +1143,22 @@ PARAM(
 PARAM(
 	/* name */	tcp_keepalive_time,
 	/* typ */	timeout,
-	/* min */	"1.000",
-	/* max */	"7200.000",
-	/* default */	"600.000",
+	/* min */	"1",
+	/* max */	"7200",
+	/* default */	"",
 	/* units */	"seconds",
-	/* flags */	EXPERIMENTAL,
+	/* flags */	XYZZY,
 	/* s-text */
 	"The number of seconds a connection needs to be idle before TCP "
-	"begins sending out keep-alive probes.",
+	"begins sending out keep-alive probes. "
+	"Ignored for Unix domain sockets.",
 	/* l-text */	"",
 	/* func */	NULL
 )
+#undef XYZZY
 
+#if 0
+/* actual location mgt_pool.c */
 PARAM(
 	/* name */	thread_pool_add_delay,
 	/* typ */	timeout,
@@ -1044,11 +1174,30 @@ PARAM(
 	"creating threads.\n"
 	"Set this to a few milliseconds if you see the 'threads_failed' "
 	"counter grow too much.\n"
-	"Setting this too high results in insuffient worker threads.",
+	"Setting this too high results in insufficient worker threads.",
 	/* l-text */	"",
 	/* func */	NULL
 )
 
+/* actual location mgt_pool.c */
+PARAM(
+	/* name */	thread_pool_watchdog,
+	/* typ */	timeout,
+	/* min */	"0.1",
+	/* max */	NULL,
+	/* default */	"60.000",
+	/* units */	"seconds",
+	/* flags */	EXPERIMENTAL,
+	/* s-text */
+	"Thread queue stuck watchdog.\n"
+	"\n"
+	"If no queued work have been released for this long,"
+	" the worker process panics itself.",
+	/* l-text */	"",
+	/* func */	NULL
+)
+
+/* actual location mgt_pool.c */
 PARAM(
 	/* name */	thread_pool_destroy_delay,
 	/* typ */	timeout,
@@ -1064,6 +1213,7 @@ PARAM(
 	/* func */	NULL
 )
 
+/* actual location mgt_pool.c */
 PARAM(
 	/* name */	thread_pool_fail_delay,
 	/* typ */	timeout,
@@ -1090,6 +1240,7 @@ PARAM(
 	/* func */	NULL
 )
 
+/* actual location mgt_pool.c */
 PARAM(
 	/* name */	thread_pool_max,
 	/* typ */	thread_pool_max,
@@ -1108,6 +1259,7 @@ PARAM(
 	/* func */	NULL
 )
 
+/* actual location mgt_pool.c */
 PARAM(
 	/* name */	thread_pool_min,
 	/* typ */	thread_pool_min,
@@ -1126,12 +1278,42 @@ PARAM(
 	/* func */	NULL
 )
 
+/* actual location mgt_pool.c */
+PARAM(
+	/* name */	thread_pool_reserve,
+	/* typ */	thread_pool_reserve,
+	/* min */	NULL,
+	/* max */	NULL,
+	/* default */	"0",
+	/* units */	"threads",
+	/* flags */	DELAYED_EFFECT| EXPERIMENTAL,
+	/* s-text */
+	"The number of worker threads reserved for vital tasks "
+	"in each pool.\n"
+	"\n"
+	"Tasks may require other tasks to complete (for example, "
+	"client requests may require backend requests). This reserve "
+	"is to ensure that such tasks still get to run even under high "
+	"load.\n"
+	"\n"
+	"Increasing the reserve may help setups with a high number of "
+	"backend requests at the expense of client performance. "
+	"Setting it too high will waste resources by keeping threads "
+	"unused.\n"
+	"\n"
+	"Default is 0 to auto-tune (currently 5% of thread_pool_min).\n"
+	"Minimum is 1 otherwise, maximum is 95% of thread_pool_min.",
+	/* l-text */	"",
+	/* func */	NULL
+)
+
+/* actual location mgt_pool.c */
 PARAM(
 	/* name */	thread_pool_stack,
 	/* typ */	bytes,
 	/* min */	"2k",
 	/* max */	NULL,
-	/* default */	"48k",
+	/* default */	"56k",
 	/* units */	"bytes",
 	/* flags */	EXPERIMENTAL,
 	/* s-text */
@@ -1142,6 +1324,7 @@ PARAM(
 	/* func */	NULL
 )
 
+/* actual location mgt_pool.c */
 PARAM(
 	/* name */	thread_pool_timeout,
 	/* typ */	timeout,
@@ -1159,6 +1342,7 @@ PARAM(
 	/* func */	NULL
 )
 
+/* actual location mgt_pool.c */
 PARAM(
 	/* name */	thread_pools,
 	/* typ */	uint,
@@ -1170,10 +1354,14 @@ PARAM(
 	/* s-text */
 	"Number of worker thread pools.\n"
 	"\n"
-	"Increasing number of worker pools decreases lock contention.\n"
+	"Increasing the number of worker pools decreases lock "
+	"contention. Each worker pool also has a thread accepting "
+	"new connections, so for very high rates of incoming new "
+	"connections on systems with many cores, increasing the "
+	"worker pools may be required.\n"
 	"\n"
 	"Too many pools waste CPU and RAM resources, and more than one "
-	"pool for each CPU is probably detrimal to performance.\n"
+	"pool for each CPU is most likely detrimental to performance.\n"
 	"\n"
 	"Can be increased on the fly, but decreases require a restart to "
 	"take effect.",
@@ -1181,6 +1369,7 @@ PARAM(
 	/* func */	NULL
 )
 
+/* actual location mgt_pool.c */
 PARAM(
 	/* name */	thread_queue_limit,
 	/* typ */	uint,
@@ -1190,7 +1379,7 @@ PARAM(
 	/* units */	NULL,
 	/* flags */	EXPERIMENTAL,
 	/* s-text */
-	"Permitted queue length per thread-pool.\n"
+	"Permitted request queue length per thread-pool.\n"
 	"\n"
 	"This sets the number of requests we will queue, waiting for an "
 	"available thread.  Above this limit sessions will be dropped "
@@ -1199,6 +1388,7 @@ PARAM(
 	/* func */	NULL
 )
 
+/* actual location mgt_pool.c */
 PARAM(
 	/* name */	thread_stats_rate,
 	/* typ */	uint,
@@ -1219,6 +1409,15 @@ PARAM(
 )
 #endif
 
+#if defined(XYZZY)
+  #error "Temporary macro XYZZY already defined"
+#endif
+
+#if defined(SO_RCVTIMEO_WORKS)
+  #define XYZZY 0
+#else
+  #define XYZZY NOT_IMPLEMENTED
+#endif
 PARAM(
 	/* name */	timeout_idle,
 	/* typ */	timeout,
@@ -1226,14 +1425,18 @@ PARAM(
 	/* max */	NULL,
 	/* default */	"5.000",
 	/* units */	"seconds",
-	/* flags */	0,
+	/* flags */	XYZZY,
 	/* s-text */
-	"Idle timeout for client connections.\n"
-	"A connection is considered idle, until we have received the full "
-	"request headers.",
+	"Idle timeout for client connections.\n\n"
+	"A connection is considered idle until we have received the full"
+	" request headers.\n\n"
+	"This parameter is particularly relevant for HTTP1 keepalive "
+	" connections which are closed unless the next request is received"
+	" before this timeout is reached.",
 	/* l-text */	"",
 	/* func */	NULL
 )
+#undef XYZZY
 
 PARAM(
 	/* name */	timeout_linger,
@@ -1256,6 +1459,7 @@ PARAM(
 )
 
 #if 0
+/* actual location mgt_param_tbl.c */
 PARAM(
 	/* name */	vcc_allow_inline_c,
 	/* typ */	bool,
@@ -1270,6 +1474,7 @@ PARAM(
 	/* func */	NULL
 )
 
+/* actual location mgt_param_tbl.c */
 PARAM(
 	/* name */	vcc_err_unref,
 	/* typ */	bool,
@@ -1284,6 +1489,7 @@ PARAM(
 	/* func */	NULL
 )
 
+/* actual location mgt_param_tbl.c */
 PARAM(
 	/* name */	vcc_unsafe_path,
 	/* typ */	bool,
@@ -1315,6 +1521,38 @@ PARAM(
 	/* func */	NULL
 )
 
+PARAM(
+	/* name */	max_vcl_handling,
+	/* typ */	uint,
+	/* min */	"0",
+	/* max */	"2",
+	/* default */	"1",
+	/* units */	NULL,
+	/* flags */	0,
+	/* s-text */
+	"Behaviour when attempting to exceed max_vcl loaded VCL.\n"
+	"\n*  0 - Ignore max_vcl parameter.\n"
+	"\n*  1 - Issue warning.\n"
+	"\n*  2 - Refuse loading VCLs.",
+	/* l-text */	"",
+	/* func */	NULL
+)
+
+PARAM(
+	/* name */	max_vcl,
+	/* typ */	uint,
+	/* min */	"0",
+	/* max */	NULL,
+	/* default */	"100",
+	/* units */	NULL,
+	/* flags */	0,
+	/* s-text */
+	"Threshold of loaded VCL programs.  (VCL labels are not counted.)"
+	"  Parameter max_vcl_handling determines behaviour.",
+	/* l-text */	"",
+	/* func */	NULL
+)
+
 PARAM(
 	/* name */	vsm_free_cooldown,
 	/* typ */	timeout,
@@ -1330,37 +1568,6 @@ PARAM(
 	/* func */	NULL
 )
 
-#if 0
-PARAM(
-	/* name */	vcl_dir,
-	/* typ */	string,
-	/* min */	NULL,
-	/* max */	NULL,
-	/* default */	/opt/varnish/etc/varnish,
-	/* units */	NULL,
-	/* flags */	0,
-	/* s-text */
-	"Directory from which relative VCL filenames (vcl.load and "
-	"include) are opened.",
-	/* l-text */	"",
-	/* func */	NULL
-)
-
-PARAM(
-	/* name */	vmod_dir,
-	/* typ */	string,
-	/* min */	NULL,
-	/* max */	NULL,
-	/* default */	/opt/varnish/lib/varnish/vmods,
-	/* units */	NULL,
-	/* flags */	0,
-	/* s-text */
-	"Directory where VCL modules are to be found.",
-	/* l-text */	"",
-	/* func */	NULL
-)
-#endif
-
 PARAM(
 	/* name */	vsl_buffer,
 	/* typ */	vsl_buffer,
@@ -1372,6 +1579,8 @@ PARAM(
 	/* s-text */
 	"Bytes of (req-/backend-)workspace dedicated to buffering VSL "
 	"records.\n"
+	"When this parameter is adjusted, most likely workspace_client "
+	"and workspace_backend will have to be adjusted by the same amount.\n\n"
 	"Setting this too high costs memory, setting it too low will cause "
 	"more VSL flushes and likely increase lock-contention on the VSL "
 	"mutex.\n\n"
@@ -1381,6 +1590,7 @@ PARAM(
 )
 
 #if 0
+/* actual location mgt_param_bits.c*/
 PARAM(
 	/* name */	vsl_mask,
 	/* typ */	vsl_mask,
@@ -1419,7 +1629,7 @@ PARAM(
 	/* name */	vsl_space,
 	/* typ */	bytes,
 	/* min */	"1M",
-	/* max */	NULL,
+	/* max */	"4G",
 	/* default */	"80M",
 	/* units */	"bytes",
 	/* flags */	MUST_RESTART,
@@ -1436,25 +1646,25 @@ PARAM(
 	/* name */	vsm_space,
 	/* typ */	bytes,
 	/* min */	"1M",
-	/* max */	NULL,
+	/* max */	"1G",
 	/* default */	"1M",
 	/* units */	"bytes",
-	/* flags */	MUST_RESTART,
+	/* flags */	0,
 	/* s-text */
-	"The amount of space to allocate for stats counters in the VSM "
-	"memory segment.  If you make this too small, some counters will "
-	"be invisible.  Making it too large just costs memory resources.",
+	"DEPRECATED: This parameter is ignored.\n"
+	"There is no global limit on amount of shared memory now.",
 	/* l-text */	"",
 	/* func */	NULL
 )
 
 #if 0
+/* see mgt_waiter.c */
 PARAM(
 	/* name */	waiter,
 	/* typ */	waiter,
 	/* min */	NULL,
 	/* max */	NULL,
-	/* default */	kqueue (possible values: kqueue, poll),
+	/* default */	"kqueue (possible values: kqueue, poll)",
 	/* units */	NULL,
 	/* flags */	MUST_RESTART| WIZARD,
 	/* s-text */
@@ -1488,8 +1698,12 @@ PARAM(
 	/* units */	"bytes",
 	/* flags */	DELAYED_EFFECT,
 	/* s-text */
-	"Bytes of HTTP protocol workspace for clients HTTP req/resp.  If "
-	"larger than 4k, use a multiple of 4k for VM efficiency.",
+	"Bytes of HTTP protocol workspace for clients HTTP req/resp.  Use a "
+	"multiple of 4k for VM efficiency.\n"
+	"For HTTP/2 compliance this must be at least 20k, in order to "
+	"receive fullsize (=16k) frames from the client.   That usually "
+	"happens only in POST/PUT bodies.  For other traffic-patterns "
+	"smaller values work just fine.",
 	/* l-text */	"",
 	/* func */	NULL
 )
@@ -1522,11 +1736,122 @@ PARAM(
 	"Bytes of auxiliary workspace per thread.\n"
 	"This workspace is used for certain temporary data structures "
 	"during the operation of a worker thread.\n"
-	"One use is for the io-vectors for writing requests and responses "
-	"to sockets, having too little space will result in more writev(2) "
-	"system calls, having too much just wastes the space.",
+	"One use is for the IO-vectors used during delivery. Setting "
+	"this parameter too low may increase the number of writev() "
+	"syscalls, setting it too high just wastes space.  ~0.1k + "
+	"UIO_MAXIOV * sizeof(struct iovec) (typically = ~16k for 64bit) "
+	"is considered the maximum sensible value under any known "
+	"circumstances (excluding exotic vmod use).",
 	/* l-text */	"",
 	/* func */	NULL
 )
 
+PARAM(
+	/* name */	h2_rx_window_low_water,
+	/* typ */	bytes_u,
+	/* min */	"65535",
+	/* max */	"1G",
+	/* default */	"10M",
+	/* units */	"bytes",
+	/* flags */	WIZARD,
+	/* s-text */
+	"HTTP2 Receive Window low water mark.\n"
+	"We try to keep the window at least this big\n"
+	"Only affects incoming request bodies (ie: POST, PUT etc.)",
+	/* l-text */	"",
+	/* func */	NULL
+)
+
+PARAM(
+	/* name */	h2_rx_window_increment,
+	/* typ */	bytes_u,
+	/* min */	"1M",
+	/* max */	"1G",
+	/* default */	"1M",
+	/* units */	"bytes",
+	/* flags */	WIZARD,
+	/* s-text */
+	"HTTP2 Receive Window Increments.\n"
+	"How big credits we send in WINDOW_UPDATE frames\n"
+	"Only affects incoming request bodies (ie: POST, PUT etc.)",
+	/* l-text */	"",
+	/* func */	NULL
+)
+
+PARAM(
+	/* name */      h2_header_table_size,
+	/* typ */       bytes_u,
+	/* min */       "0b",
+	/* max */       NULL,
+	/* default */   "4k",
+	/* units */     "bytes",
+	/* flags */     0,
+	/* s-text */
+	"HTTP2 header table size.\n"
+	"This is the size that will be used for the HPACK dynamic\n"
+	"decoding table.",
+	/* l-text */    "",
+	/* func */      NULL
+)
+
+PARAM(
+       /* name */      h2_max_concurrent_streams,
+       /* typ */       uint,
+       /* min */       "0",
+       /* max */       NULL,
+       /* default */   "100",
+       /* units */     "streams",
+       /* flags */     0,
+       /* s-text */
+       "HTTP2 Maximum number of concurrent streams.\n"
+       "This is the number of requests that can be active\n"
+       "at the same time for a single HTTP2 connection.",
+       /* l-text */    "",
+       /* func */      NULL
+)
+
+PARAM(
+	/* name */      h2_initial_window_size,
+	/* typ */       bytes_u,
+	/* min */       "0",
+	/* max */       "2147483647b",
+	/* default */   "65535b",
+	/* units */     "bytes",
+	/* flags */     0,
+	/* s-text */
+	"HTTP2 initial flow control window size.",
+	/* l-text */    "",
+	/* func */      NULL
+)
+
+PARAM(
+	/* name */      h2_max_frame_size,
+	/* typ */       bytes_u,
+	/* min */       "16k",
+	/* max */       "16777215b",
+	/* default */   "16k",
+	/* units */     "bytes",
+	/* flags */     0,
+	/* s-text */
+	"HTTP2 maximum per frame payload size we are willing to accept.",
+	/* l-text */    "",
+	/* func */      NULL
+)
+
+PARAM(
+	/* name */      h2_max_header_list_size,
+	/* typ */       bytes_u,
+	/* min */       "0b",
+	/* max */       NULL,
+	/* default */   "2147483647b",
+	/* units */     "bytes",
+	/* flags */     0,
+	/* s-text */
+	"HTTP2 maximum size of an uncompressed header list.",
+	/* l-text */    "",
+	/* func */      NULL
+)
+
+#undef PARAM
+
 /*lint -restore */
