@@ -35,8 +35,6 @@
 #ifndef VAPI_VSC_H_INCLUDED
 #define VAPI_VSC_H_INCLUDED
 
-#include "vapi/vsc_int.h"
-
 struct vsm;
 struct vsc;
 struct vsm_fantom;
@@ -137,6 +135,10 @@ int VSC_Iter(struct vsc *, struct vsm *, VSC_iter_f *, void *priv);
 	 *
 	 * The returned points are valid until the next call to VSC_Iter()
 	 *
+	 * Not safe for concurrent reads with the same vsc and vsm
+	 * handles.  For concurrency, initialize and attach separate
+	 * structs vsc and vsm.
+	 *
 	 * Arguments:
 	 *	    vd: The vsm context
 	 *	  func: The callback function
