@@ -30,9 +30,14 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
+ * Names of the form "v_[a-z_]*_" is reserved for this file.
+ *
+ * This file should always be the first non <...> include in a .c file.
  */
 
-#ifndef VDEF_H_INCLUDED
+#ifdef VDEF_H_INCLUDED
+#  error "vdef.h included multiple times"
+#endif
 #define VDEF_H_INCLUDED
 
 /* Safe printf into a fixed-size buffer */
@@ -59,23 +64,23 @@
 		*(fdp) = -1;			\
 	} while (0)
 
-#ifndef __GNUC_PREREQ
+#ifndef __GNUC_PREREQ__
 # if defined __GNUC__ && defined __GNUC_MINOR__
-#  define __GNUC_PREREQ(maj, min) \
+#  define __GNUC_PREREQ__(maj, min) \
 	(__GNUC__ > (maj) || (__GNUC__ == (maj) && __GNUC_MINOR__ >= (min)))
 # else
-#  define __GNUC_PREREQ(maj, min) 0
+#  define __GNUC_PREREQ__(maj, min) 0
 # endif
 #endif
 
-#ifdef __printflike
-#  define __v_printflike(f,a) __printflike(f,a)
-#elif __GNUC_PREREQ(2, 95) || defined(__INTEL_COMPILER)
-#  define __v_printflike(f,a) __attribute__((format(printf, f, a)))
+#if __GNUC_PREREQ__(2, 95) || defined(__INTEL_COMPILER)
+#  define v_printflike_(f,a) __attribute__((format(printf, f, a)))
 #else
-#  define __v_printflike(f,a)
+#  define v_printflike_(f,a)
 #endif
 
+#define v_noreturn_ __attribute__((__noreturn__))
+
 /*********************************************************************
  * Pointer alignment magic
  */
@@ -112,23 +117,27 @@
  * even if that means not const'ing a const'able argument.
  * The typedef should be specified as argument to the macro.
  */
-#define __match_proto__(xxx)		/*lint --e{818} */
+#define v_matchproto_(xxx)		/*lint --e{818} */
 
 /*
  * State variables may change value before we have considered the
  * previous value
  */
-#define __state_variable__(varname)	varname /*lint -esym(838,varname) */
+#define v_statevariable_(varname)	varname /*lint -esym(838,varname) */
 
 #ifdef __SUNPRO_C
-#define NEEDLESS(s)		{}
-#define __unused
+#  define NEEDLESS(s)		{}
 #else
-#define NEEDLESS(s)		s
+#  define NEEDLESS(s)		s
 #endif
 
-#ifndef STATIC_ASSERT
-#  define _Static_assert(a,b)
+#if __GNUC_PREREQ__(2, 7)
+#  define v_unused_ __attribute__((__unused__))
+#else
+#  define v_unused_
 #endif
 
-#endif /* VDEF_H_INCLUDED */
+/* VTIM API overhaul WIP */
+typedef double vtim_mono;
+typedef double vtim_real;
+typedef double vtim_dur;
