@@ -56,9 +56,9 @@ struct VUT {
 	struct VSL_data	*vsl;
 	struct vsm	*vsm;
 	struct VSLQ	*vslq;
-	int		sighup;
-	int		sigint;
-	int		sigusr1;
+
+	sig_atomic_t	last_sighup;
+	sig_atomic_t	last_sigusr1;
 
 	/* Callback functions */
 	VUT_cb_f	*idle_f;
@@ -69,10 +69,14 @@ struct VUT {
 };
 
 void VUT_Error(struct VUT *, int status, const char *fmt, ...)
-    __v_printflike(3, 4);
+    v_noreturn_ v_printflike_(3, 4);
 
 int VUT_Arg(struct VUT *, int opt, const char *arg);
 
+//lint -sem(VUT_Usage, r_no)
+void VUT_Usage(const struct VUT *, const struct vopt_spec *,
+    int status) v_noreturn_;
+
 #define VUT_InitProg(argc, argv, spec) VUT_Init(argv[0], argc, argv, spec)
 
 struct VUT * VUT_Init(const char *progname, int argc, char * const *argv,
