@@ -34,6 +34,8 @@
 #include <stdint.h>
 #include <string.h>
 
+#include "vdef.h"
+
 #include "vas.h"
 #include "vct.h"
 
@@ -52,7 +54,7 @@ const uint16_t vct_typtab[256] = {
 	[0x06]	=	VCT_CTL,
 	[0x07]	=	VCT_CTL,
 	[0x08]	=	VCT_CTL,
-	[0x09]	=	VCT_CTL | VCT_SP | VCT_SEPARATOR,
+	[0x09]	=	VCT_CTL | VCT_SP,
 	[0x0a]	=	VCT_CTL | VCT_CRLF,
 	[0x0b]	=	VCT_CTL | VCT_VT,
 	[0x0c]	=	VCT_CTL,
@@ -75,7 +77,7 @@ const uint16_t vct_typtab[256] = {
 	[0x1d]	=	VCT_CTL,
 	[0x1e]	=	VCT_CTL,
 	[0x1f]	=	VCT_CTL,
-	[0x20]  =	VCT_SP | VCT_SEPARATOR,
+	[0x20]  =	VCT_SP,
 	[0x21]  =	VCT_TCHAR,
 	[0x22]  =	VCT_SEPARATOR,
 	[0x23]  =	VCT_TCHAR,
