@@ -30,7 +30,6 @@
 #include "config.h"
 
 #include <time.h>
-#include <errno.h>
 #include <poll.h>
 #include <pthread.h>
 #include <signal.h>
@@ -354,7 +353,7 @@ VEV_Loop(struct vev_root *evb)
 /*--------------------------------------------------------------------*/
 
 static int
-vev_sched_timeout(struct vev_root *evb, struct vev *e, double t)
+vev_sched_timeout(struct vev_root *evb, struct vev *e, vtim_mono t)
 {
 	int i;
 
@@ -414,7 +413,7 @@ VEV_Once(struct vev_root *evb)
 	tmo = INFTIM;
 	e = binheap_root(evb->binheap);
 	if (e != NULL) {
-		CHECK_OBJ_NOTNULL(e, VEV_MAGIC);
+		CHECK_OBJ(e, VEV_MAGIC);
 		assert(e->__binheap_idx == BINHEAP_NOIDX + 1);
 		t = VTIM_mono();
 		if (e->__when <= t)
