@@ -32,7 +32,6 @@
 #include <time.h>
 #include <sys/types.h>
 #include <sys/stat.h>
-#include <errno.h>
 #include <fcntl.h>
 #include <limits.h>
 #include <stdio.h>
@@ -50,7 +49,7 @@
 #ifdef HAVE_SYS_VFS_H
 #  include <sys/vfs.h>
 #endif
-#if defined(__linux__) && defined(HAVE_FALLOCATE)
+#ifdef HAVE_FALLOCATE
 #  include <linux/magic.h>
 #endif
 
@@ -390,6 +389,8 @@ VFIL_searchpath(const struct vfil_path *vp, vfil_path_func_f *func, void *priv,
 		VSB_clear(vsb);
 		VSB_printf(vsb, "%s/%s", vd->dir, fni);
 		AZ(VSB_finish(vsb));
+		if (access(VSB_data(vsb), F_OK))
+			continue;
 		i = func(priv, VSB_data(vsb));
 		if (i <= 0) {
 			e = errno;
