@@ -30,7 +30,6 @@ __FBSDID("$FreeBSD: head/sys/kern/subr_vsb.c 222004 2011-05-17 06:36:32Z phk $")
 #include "config.h"
 
 #include <ctype.h>
-#include <errno.h>
 #include <stdarg.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -574,7 +573,7 @@ VSB_quote_pfx(struct vsb *s, const char *pfx, const void *v, int len, int how)
 			break;
 		case '\n':
 			if (how & VSB_QUOTE_CSTR) {
-				(void)VSB_printf(s, "\\n\"\n%s\t\"", pfx);
+				(void)VSB_printf(s, "\\n\"\n%s\"", pfx);
 			} else if (how & (VSB_QUOTE_NONL|VSB_QUOTE_UNSAFE)) {
 				(void)VSB_printf(s, "\n");
 				nl = 1;
@@ -588,12 +587,17 @@ VSB_quote_pfx(struct vsb *s, const char *pfx, const void *v, int len, int how)
 		case '\t':
 			(void)VSB_cat(s, "\\t");
 			break;
+		case '\v':
+			(void)VSB_cat(s, "\\v");
+			break;
 		default:
 			/* XXX: Implement VSB_QUOTE_JSON */
 			if (isgraph(*q))
 				(void)VSB_putc(s, *q);
+			else if (how & VSB_QUOTE_ESCHEX)
+				(void)VSB_printf(s, "\\x%02x", *q & 0xff);
 			else
-				(void)VSB_printf(s, "\\%o", *q & 0xff);
+				(void)VSB_printf(s, "\\%03o", *q & 0xff);
 			break;
 		}
 	}
