@@ -32,22 +32,17 @@
 #include <sys/types.h>
 #include <sys/socket.h>
 #include <sys/ioctl.h>
-#include <sys/time.h>
 #ifdef HAVE_SYS_FILIO_H
 #  include <sys/filio.h>
 #endif
 
 #include <netinet/in.h>
-#ifdef __linux
-#  include <netinet/tcp.h>
-#endif
+#include <netinet/tcp.h>
 
-#include <errno.h>
 #include <math.h>
 #include <netdb.h>
 #include <poll.h>
 #include <stdio.h>
-#include <stdlib.h>
 #include <string.h>
 #include <unistd.h>
 
@@ -56,6 +51,7 @@
 #include "vsa.h"
 #include "vss.h"
 #include "vtcp.h"
+#include "vtim.h"
 
 /*--------------------------------------------------------------------*/
 static void
@@ -217,9 +213,11 @@ VTCP_fastopen(int sock, int depth)
  * us to do two syscalls, one to get and one to set, the latter of
  * which mucks about a bit before it ends up calling ioctl(FIONBIO),
  * at least on FreeBSD.
+ * On Solaris ioctl(FIONBIO) can fail with connection related errnos,
+ * but as long as that is how they fail, we're fine.
  */
 
-int
+void
 VTCP_blocking(int sock)
 {
 	int i, j;
@@ -227,10 +225,9 @@ VTCP_blocking(int sock)
 	i = 0;
 	j = ioctl(sock, FIONBIO, &i);
 	VTCP_Assert(j);
-	return (j);
 }
 
-int
+void
 VTCP_nonblocking(int sock)
 {
 	int i, j;
@@ -238,7 +235,6 @@ VTCP_nonblocking(int sock)
 	i = 1;
 	j = ioctl(sock, FIONBIO, &i);
 	VTCP_Assert(j);
-	return (j);
 }
 
 /*--------------------------------------------------------------------
@@ -264,11 +260,11 @@ VTCP_connected(int s)
 	/* An error means no connection established */
 	errno = k;
 	if (k) {
-		AZ(close(s));
+		closefd(&s);
 		return (-1);
 	}
 
-	(void)VTCP_blocking(s);
+	VTCP_blocking(s);
 	return (s);
 }
 
@@ -279,6 +275,7 @@ VTCP_connect(const struct suckaddr *name, int msec)
 	struct pollfd fds[1];
 	const struct sockaddr *sa;
 	socklen_t sl;
+	int val;
 
 	if (name == NULL)
 		return (-1);
@@ -294,13 +291,16 @@ VTCP_connect(const struct suckaddr *name, int msec)
 
 	/* Set the socket non-blocking */
 	if (msec != 0)
-		(void)VTCP_nonblocking(s);
+		VTCP_nonblocking(s);
+
+	val = 1;
+	AZ(setsockopt(s, IPPROTO_TCP, TCP_NODELAY, &val, sizeof val));
 
 	i = connect(s, sa, sl);
 	if (i == 0)
 		return (s);
 	if (errno != EINPROGRESS) {
-		AZ(close(s));
+		closefd(&s);
 		return (-1);
 	}
 
@@ -321,7 +321,7 @@ VTCP_connect(const struct suckaddr *name, int msec)
 
 	if (i == 0) {
 		/* Timeout, close and give up */
-		AZ(close(s));
+		closefd(&s);
 		errno = ETIMEDOUT;
 		return (-1);
 	}
@@ -346,12 +346,10 @@ VTCP_close(int *s)
 }
 
 void
-VTCP_set_read_timeout(int s, double seconds)
+VTCP_set_read_timeout(int s, vtim_dur seconds)
 {
 #ifdef SO_RCVTIMEO_WORKS
-	struct timeval timeout;
-	timeout.tv_sec = (int)floor(seconds);
-	timeout.tv_usec = (int)(1e6 * (seconds - timeout.tv_sec));
+	struct timeval timeout = VTIM_timeval(seconds);
 	/*
 	 * Solaris bug (present at least in snv_151 and older): If this fails
 	 * with EINVAL, the socket is half-closed (SS_CANTSENDMORE) and the
@@ -369,16 +367,17 @@ VTCP_set_read_timeout(int s, double seconds)
 /*--------------------------------------------------------------------
  */
 
-static int __match_proto__(vss_resolved_f)
+static int v_matchproto_(vss_resolved_f)
 vtcp_open_callback(void *priv, const struct suckaddr *sa)
 {
+	/* XXX: vtim_dur? */
 	double *p = priv;
 
 	return (VTCP_connect(sa, (int)floor(*p * 1e3)));
 }
 
 int
-VTCP_open(const char *addr, const char *def_port, double timeout,
+VTCP_open(const char *addr, const char *def_port, vtim_dur timeout,
     const char **errp)
 {
 	int error;
@@ -428,7 +427,7 @@ VTCP_bind(const struct suckaddr *sa, const char **errp)
 		if (errp != NULL)
 			*errp = "setsockopt(SO_REUSEADDR, 1)";
 		e = errno;
-		AZ(close(sd));
+		closefd(&sd);
 		errno = e;
 		return (-1);
 	}
@@ -440,7 +439,7 @@ VTCP_bind(const struct suckaddr *sa, const char **errp)
 		if (errp != NULL)
 			*errp = "setsockopt(IPV6_V6ONLY, 1)";
 		e = errno;
-		AZ(close(sd));
+		closefd(&sd);
 		errno = e;
 		return (-1);
 	}
@@ -450,7 +449,7 @@ VTCP_bind(const struct suckaddr *sa, const char **errp)
 		if (errp != NULL)
 			*errp = "bind(2)";
 		e = errno;
-		AZ(close(sd));
+		closefd(&sd);
 		errno = e;
 		return (-1);
 	}
@@ -474,7 +473,7 @@ VTCP_listen(const struct suckaddr *sa, int depth, const char **errp)
 	if (sd >= 0)  {
 		if (listen(sd, depth) != 0) {
 			e = errno;
-			AZ(close(sd));
+			closefd(&sd);
 			errno = e;
 			if (errp != NULL)
 				*errp = "listen(2)";
@@ -491,7 +490,7 @@ struct helper {
 	const char	**errp;
 };
 
-static int __match_proto__(vss_resolved_f)
+static int v_matchproto_(vss_resolved_f)
 vtcp_lo_cb(void *priv, const struct suckaddr *sa)
 {
 	int sock;
@@ -519,7 +518,7 @@ VTCP_listen_on(const char *addr, const char *def_port, int depth,
 	sock = VSS_resolver(addr, def_port, vtcp_lo_cb, &h, errp);
 	if (*errp != NULL)
 		return (-1);
-	return(sock);
+	return (sock);
 }
 
 /*--------------------------------------------------------------------
@@ -567,7 +566,7 @@ VTCP_Check(int a)
 {
 	if (a == 0)
 		return (1);
-	if (errno == ECONNRESET || errno == ENOTCONN)
+	if (errno == ECONNRESET || errno == ENOTCONN || errno == EPIPE)
 		return (1);
 #if (defined (__SVR4) && defined (__sun)) || defined (__NetBSD__)
 	/*
@@ -577,6 +576,13 @@ VTCP_Check(int a)
 	 */
 	if (errno == EINVAL || errno == ETIMEDOUT)
 		return (1);
+#elif defined (__APPLE__)
+	/*
+	 * MacOS returns EINVAL if the other end unexpectedly reset
+	 * the connection.
+	 */
+	if (errno == EINVAL)
+		return (1);
 #endif
 	return (0);
 }
@@ -586,7 +592,7 @@ VTCP_Check(int a)
  */
 
 int
-VTCP_read(int fd, void *ptr, size_t len, double tmo)
+VTCP_read(int fd, void *ptr, size_t len, vtim_dur tmo)
 {
 	struct pollfd pfd[1];
 	int i, j;
