@@ -30,7 +30,6 @@
 
 #include "config.h"
 
-#include <errno.h>
 #include <stdarg.h>
 #include <stdint.h>
 #include <stdio.h>
@@ -40,7 +39,6 @@
 #include "miniobj.h"
 
 #include "vbm.h"
-#include "vmb.h"
 #include "vqueue.h"
 #include "vre.h"
 #include "vsb.h"
@@ -224,8 +222,39 @@ static const char * const VSL_transactions[VSL_t__MAX] = {
 			return (-5);			\
 	} while (0)
 
-int
-VSL_Print(const struct VSL_data *vsl, const struct VSL_cursor *c, void *fo)
+static int
+vsl_print_unsafe(FILE *fo, unsigned len, const char *data)
+{
+
+	VSL_PRINT(fo, "\"");
+	while (len-- > 0) {
+		if (*data >= ' ' && *data <= '~')
+			VSL_PRINT(fo, "%c", *data);
+		else
+			VSL_PRINT(fo, "%%%02x", (unsigned char)*data);
+		data++;
+	}
+	VSL_PRINT(fo, "\"\n");
+	return (0);
+}
+
+
+static int
+vsl_print_binary(FILE *fo, unsigned len, const char *data)
+{
+
+	VSL_PRINT(fo, "[");
+	while (len-- > 0) {
+		VSL_PRINT(fo, "%02x", (unsigned char)*data);
+		data++;
+	}
+	VSL_PRINT(fo, "]\n");
+	return (0);
+}
+
+static int
+vsl_print(const struct VSL_data *vsl, const struct VSL_cursor *c, void *fo,
+    int terse)
 {
 	enum VSL_tag_e tag;
 	uint32_t vxid;
@@ -245,57 +274,34 @@ VSL_Print(const struct VSL_data *vsl, const struct VSL_cursor *c, void *fo)
 	    'b' : '-';
 	data = VSL_CDATA(c->rec.ptr);
 
-	if (VSL_tagflags[tag] & SLT_F_BINARY) {
-		VSL_PRINT(fo, "%10u %-14s %c \"", vxid, VSL_tags[tag], type);
-		while (len-- > 0) {
-			if (len == 0 && tag == SLT_Debug && *data == '\0')
-				break;
-			if (*data >= ' ' && *data <= '~')
-				VSL_PRINT(fo, "%c", *data);
-			else
-				VSL_PRINT(fo, "%%%02x", (unsigned char)*data);
-			data++;
-		}
-		VSL_PRINT(fo, "\"\n");
-	} else
-		VSL_PRINT(fo, "%10u %-14s %c %.*s\n", vxid, VSL_tags[tag],
-		    type, (int)len, data);
+	if (!terse)
+		VSL_PRINT(fo, "%10u ", vxid);
+	VSL_PRINT(fo, "%-14s ", VSL_tags[tag]);
+	if (!terse)
+		VSL_PRINT(fo, "%c ", type);
+
+	if (VSL_tagflags[tag] & SLT_F_UNSAFE)
+		(void)vsl_print_unsafe(fo, len, data);
+	else if (VSL_tagflags[tag] & SLT_F_BINARY)
+		(void)vsl_print_binary(fo, len, data);
+	else
+		VSL_PRINT(fo, "%.*s\n", (int)len, data);
 
 	return (0);
 }
 
+int
+VSL_Print(const struct VSL_data *vsl, const struct VSL_cursor *c, void *fo)
+{
+
+	return (vsl_print(vsl, c, fo, 0));
+}
+
 int
 VSL_PrintTerse(const struct VSL_data *vsl, const struct VSL_cursor *c, void *fo)
 {
-	enum VSL_tag_e tag;
-	unsigned len;
-	const char *data;
 
-	CHECK_OBJ_NOTNULL(vsl, VSL_MAGIC);
-	if (c == NULL || c->rec.ptr == NULL)
-		return (0);
-	if (fo == NULL)
-		fo = stdout;
-	tag = VSL_TAG(c->rec.ptr);
-	len = VSL_LEN(c->rec.ptr);
-	data = VSL_CDATA(c->rec.ptr);
-
-	if (VSL_tagflags[tag] & SLT_F_BINARY) {
-		VSL_PRINT(fo, "%-14s \"", VSL_tags[tag]);
-		while (len-- > 0) {
-			if (len == 0 && tag == SLT_Debug && *data == '\0')
-				break;
-			if (*data >= ' ' && *data <= '~')
-				VSL_PRINT(fo, "%c", *data);
-			else
-				VSL_PRINT(fo, "%%%02x", (unsigned char)*data);
-			data++;
-		}
-		VSL_PRINT(fo, "\"\n");
-	} else
-		VSL_PRINT(fo, "%-14s %.*s\n", VSL_tags[tag], (int)len, data);
-
-	return (0);
+	return (vsl_print(vsl, c, fo, 1));
 }
 
 int
@@ -317,7 +323,7 @@ VSL_PrintAll(struct VSL_data *vsl, const struct VSL_cursor *c, void *fo)
 	}
 }
 
-int __match_proto__(VSLQ_dispatch_f)
+int v_matchproto_(VSLQ_dispatch_f)
 VSL_PrintTransactions(struct VSL_data *vsl, struct VSL_transaction * const pt[],
     void *fo)
 {
@@ -456,7 +462,7 @@ VSL_WriteAll(struct VSL_data *vsl, const struct VSL_cursor *c, void *fo)
 	}
 }
 
-int __match_proto__(VSLQ_dispatch_f)
+int v_matchproto_(VSLQ_dispatch_f)
 VSL_WriteTransactions(struct VSL_data *vsl, struct VSL_transaction * const pt[],
     void *fo)
 {
