@@ -28,14 +28,11 @@
  *
  */
 
+#include "config.h"
+
 #include <ctype.h>
 #include <math.h>
-#include <stdint.h>
 #include <stdio.h>
-#include <stdlib.h>
-#include <string.h>
-#include <strings.h>
-#include <unistd.h>
 
 #include "vdef.h"
 #include "vas.h"
@@ -59,20 +56,64 @@ struct vslq_query {
 	struct vex		*vex;
 };
 
-#define VSLQ_TEST_NUMOP(TYPE, PRE_LHS, OP, PRE_RHS)	\
-	switch (TYPE) {					\
-	case VEX_INT:					\
-		if (PRE_LHS##_int OP PRE_RHS##_int)	\
-			return (1);			\
-		return (0);				\
-	case VEX_FLOAT:					\
-		if (PRE_LHS##_float OP PRE_RHS##_float)	\
-			return (1);			\
-		return (0);				\
-	default:					\
-		WRONG("Wrong RHS type");		\
+#define VSLQ_TEST_NUMOP(TYPE, PRE_LHS, OP, PRE_RHS)		\
+	do {							\
+		switch (TYPE) {					\
+		case VEX_INT:					\
+			if (PRE_LHS##_int OP PRE_RHS##_int)	\
+				return (1);			\
+			return (0);				\
+		case VEX_FLOAT:					\
+			if (PRE_LHS##_float OP PRE_RHS##_float)	\
+				return (1);			\
+			return (0);				\
+		default:					\
+			WRONG("Wrong RHS type");		\
+		}						\
+	} while(0)						\
+
+static int
+vslq_test_vxid(const struct vex *vex, const struct VSL_transaction *trans)
+{
+	const struct vex_rhs *rhs;
+
+	AN(vex);
+	AN(trans);
+
+	rhs = vex->rhs;
+	CHECK_OBJ_NOTNULL(rhs, VEX_RHS_MAGIC);
+
+	/* Prepare */
+	switch (vex->tok) {
+	case T_EQ:		/* == */
+	case T_NEQ:		/* != */
+	case '<':
+	case '>':
+	case T_LEQ:		/* <= */
+	case T_GEQ:		/* >= */
+		if (rhs->type != VEX_INT)
+			WRONG("Wrong RHS type for vxid");
+		/* FALLTHROUGH */
+	default:
+		break;
 	}
 
+	/* Compare */
+	switch (vex->tok) {
+#define VXID_TEST_NUMOP(OP) return (trans->vxid OP rhs->val_int)
+	case T_EQ:	VXID_TEST_NUMOP(==);
+	case T_NEQ:	VXID_TEST_NUMOP(!=);
+	case '<':	VXID_TEST_NUMOP(<);
+	case '>':	VXID_TEST_NUMOP(>);
+	case T_LEQ:	VXID_TEST_NUMOP(<=);
+	case T_GEQ:	VXID_TEST_NUMOP(>=);
+	#undef VXID_TEST_NUMOP
+	default:	WRONG("Bad vxid expression token");
+	}
+
+	NEEDLESS(return (0));
+}
+
 static int
 vslq_test_rec(const struct vex *vex, const struct VSLC_ptr *rec)
 {
@@ -141,10 +182,9 @@ vslq_test_rec(const struct vex *vex, const struct VSLC_ptr *rec)
 		switch (rhs->type) {
 		case VEX_INT:
 			lhs_int = strtoll(b, &p, 0);
-			if (*p == '\0' || isspace(*p))
-				break;
-			/* Can't parse - no match */
-			return (0);
+			if (*p != '\0' && !isspace(*p))
+				return (0); /* Can't parse - no match */
+			break;
 		case VEX_FLOAT:
 			lhs_float = VNUMpfx(b, &q);
 			if (isnan(lhs_float))
@@ -213,8 +253,7 @@ vslq_test_rec(const struct vex *vex, const struct VSLC_ptr *rec)
 	default:
 		WRONG("Bad expression token");
 	}
-
-	return (0);
+	NEEDLESS(return (0));
 }
 
 static int
@@ -226,6 +265,17 @@ vslq_test(const struct vex *vex, struct VSL_transaction * const ptrans[])
 	CHECK_OBJ_NOTNULL(vex, VEX_MAGIC);
 	CHECK_OBJ_NOTNULL(vex->lhs, VEX_LHS_MAGIC);
 	AN(vex->lhs->tags);
+	assert(vex->lhs->vxid <= 1);
+
+	if (vex->lhs->vxid) {
+		AZ(vex->lhs->taglist);
+		for (t = ptrans[0]; t != NULL; t = *++ptrans)
+			if (vslq_test_vxid(vex, t))
+				return (1);
+		return (0);
+	}
+
+	AN(vex->lhs->taglist);
 
 	for (t = ptrans[0]; t != NULL; t = *++ptrans) {
 		if (vex->lhs->level >= 0) {
@@ -298,7 +348,7 @@ vslq_exec(const struct vex *vex, struct VSL_transaction * const ptrans[])
 	default:
 		return (vslq_test(vex, ptrans));
 	}
-	NEEDLESS_RETURN(0);
+	NEEDLESS(return 0);
 }
 
 struct vslq_query *
