@@ -33,7 +33,6 @@
 #include <sys/mman.h>
 #include <sys/stat.h>
 
-#include <errno.h>
 #include <fcntl.h>
 #include <float.h>
 #include <math.h>
@@ -56,6 +55,7 @@
 #include "vqueue.h"
 #include "vtim.h"
 
+#include "vapi/vsig.h"
 #include "vapi/vsm.h"
 
 #ifndef MAP_HASSEMAPHORE
@@ -677,7 +677,7 @@ VSM_Attach(struct vsm *vd, int progress)
 	}
 
 	AZ(vd->attached);
-	while (1) {
+	while (!VSIG_int && !VSIG_term) {
 		u = VSM_Status(vd);
 		VSM_ResetError(vd);
 		if (u & VSM_MGT_RUNNING) {
@@ -696,6 +696,7 @@ VSM_Attach(struct vsm *vd, int progress)
 			(void)write(progress, ".", 1);
 		VTIM_sleep(.25);
 	}
+	return (vsm_diag(vd, "Attach interrupted"));
 }
 
 /*--------------------------------------------------------------------*/
@@ -856,7 +857,7 @@ VSM_Unmap(struct vsm *vd, struct vsm_fantom *vf)
 	vf->b = NULL;
 	vf->e = NULL;
 	if (vg->refs > 0)
-		return(0);
+		return (0);
 
 	if (vg->cluster) {
 		assert(vg->s == NULL);
