@@ -37,7 +37,6 @@
 #include <stdlib.h>
 #include <unistd.h>
 #include <stdio.h>
-#include <errno.h>
 #include <string.h>
 #include <signal.h>
 #include <sys/stat.h> /* for MUSL */
@@ -55,6 +54,7 @@
 #include "vut.h"
 
 #include "vapi/voptget.h"
+#include "vapi/vsig.h"
 
 static int vut_synopsis(const struct vopt_spec *);
 static int vut_options(const struct vopt_spec *);
@@ -81,10 +81,7 @@ vut_vpf_remove(void)
 	AN(pfh);
 	AN(pfh_vut.P_arg);
 
-	if (VPF_Remove(pfh) != 0)
-		VUT_Error(&pfh_vut, 1, "Cannot remove pid file %s: %s",
-		    pfh_vut.P_arg, strerror(errno));
-
+	VPF_Remove(pfh);
 	free(pfh_vut.P_arg);
 	ZERO_OBJ(&pfh_vut, sizeof pfh_vut);
 	pfh = NULL;
@@ -110,32 +107,23 @@ vut_dispatch(struct VSL_data *vsl, struct VSL_transaction * const trans[],
 	return (i);
 }
 
-//lint -sem(vut_error, r_no)
-static void v_noreturn_ v_matchproto_(VUT_error_f)
-vut_error(struct VUT *vut, int status, const char *fmt, va_list ap)
-{
-
-	CHECK_OBJ_NOTNULL(vut, VUT_MAGIC);
-	AN(fmt);
-	vfprintf(stderr, fmt, ap);
-	fprintf(stderr, "\n");
-
-	exit(status);
-}
-
 void
 VUT_Error(struct VUT *vut, int status, const char *fmt, ...)
 {
 	va_list ap;
 
 	CHECK_OBJ_NOTNULL(vut, VUT_MAGIC);
-	AN(vut->error_f);
 	AN(status);
 
 	va_start(ap, fmt);
-	vut->error_f(vut, status, fmt, ap);
+	if (vut->error_f != NULL) {
+		vut->error_f(vut, status, fmt, ap);
+	} else {
+		vfprintf(stderr, fmt, ap);
+		fprintf(stderr, "\n");
+	}
 	va_end(ap);
-	exit(2);
+	exit(status);
 }
 
 int
@@ -216,6 +204,11 @@ VUT_Init(const char *progname, int argc, char * const *argv,
 	AN(argv);
 	AN(voc);
 
+	VSIG_Arm_hup();
+	VSIG_Arm_int();
+	VSIG_Arm_term();
+	VSIG_Arm_usr1();
+
 	ALLOC_OBJ(vut, VUT_MAGIC);
 	AN(vut);
 
@@ -231,7 +224,6 @@ VUT_Init(const char *progname, int argc, char * const *argv,
 	vut->progname = progname;
 	vut->g_arg = VSL_g_vxid;
 	vut->k_arg = -1;
-	vut->error_f = vut_error;
 	AZ(vut->vsl);
 	vut->vsl = VSL_New();
 	AN(vut->vsl);
@@ -254,9 +246,9 @@ VUT_Signaled(struct VUT *vut, int sig)
 {
 
 	CHECK_OBJ_NOTNULL(vut, VUT_MAGIC);
-	vut->sighup |= (int)(sig == SIGHUP);
-	vut->sigint |= (int)(sig == SIGINT || sig == SIGTERM);
-	vut->sigusr1 |= (int)(sig == SIGUSR1);
+#define VSIG_SIGNAL(UPPER, lower) \
+	VSIG_##lower += (int)(sig == SIG##UPPER);
+#include "tbl/vsig_list.h"
 }
 
 void
@@ -306,7 +298,8 @@ VUT_Setup(struct VUT *vut)
 			VUT_Error(vut, 1, "PID file already created");
 		pfh = VPF_Open(vut->P_arg, 0644, NULL);
 		if (pfh == NULL)
-			VUT_Error(vut, 1, "%s: %s", vut->P_arg, strerror(errno));
+			VUT_Error(vut, 1,
+			    "%s: %s", vut->P_arg, strerror(errno));
 	}
 
 	/* Daemon mode */
@@ -316,7 +309,7 @@ VUT_Setup(struct VUT *vut)
 	/* Write PID and setup exit handler */
 	if (vut->P_arg) {
 		AN(pfh);
-		AZ(VPF_Write(pfh));
+		VPF_Write(pfh);
 
 		/* NB: move ownership to a global pseudo-VUT. */
 		INIT_OBJ(&pfh_vut, VUT_MAGIC);
@@ -363,18 +356,21 @@ VUT_Main(struct VUT *vut)
 	CHECK_OBJ_NOTNULL(vut, VUT_MAGIC);
 	AN(vut->vslq);
 
-	while (!vut->sigint) {
-		if (vut->sighup && vut->sighup_f) {
+	while (!VSIG_int && !VSIG_term) {
+		if (VSIG_hup != vut->last_sighup) {
 			/* sighup callback */
-			vut->sighup = 0;
-			i = vut->sighup_f(vut);
+			vut->last_sighup = VSIG_hup;
+			if (vut->sighup_f != NULL)
+				i = vut->sighup_f(vut);
+			else
+				i = 1;
 			if (i)
 				break;
 		}
 
-		if (vut->sigusr1) {
+		if (VSIG_usr1 != vut->last_sigusr1) {
 			/* Flush and report any incomplete records */
-			vut->sigusr1 = 0;
+			vut->last_sigusr1 = VSIG_usr1;
 			(void)VSLQ_Flush(vut->vslq, vut_dispatch, vut);
 		}
 
@@ -422,7 +418,7 @@ VUT_Main(struct VUT *vut)
 
 		do
 			i = VSLQ_Dispatch(vut->vslq, vut_dispatch, vut);
-		while (i == vsl_more && !vut->sighup && !vut->sigusr1);
+		while (i == vsl_more && !VSIG_hup && !VSIG_usr1);
 
 		if (i == vsl_more)
 			continue;
@@ -461,6 +457,20 @@ VUT_Main(struct VUT *vut)
 
 /**********************************************************************/
 
+void v_noreturn_
+VUT_Usage(const struct VUT *vut, const struct vopt_spec *voc, int status)
+{
+	const char **opt;
+
+	fprintf(stderr, "Usage: %s <options>\n\n", vut->progname);
+	fprintf(stderr, "Options:\n");
+	for (opt = voc->vopt_usage; *opt != NULL; opt += 2)
+		fprintf(stderr, " %-25s %s\n", *opt, *(opt + 1));
+	exit(status);
+}
+
+/**********************************************************************/
+
 
 static void
 print_nobrackets(const char *s)
