@@ -52,7 +52,6 @@
 
 #include "config.h"
 
-#include <errno.h>
 #include <stdarg.h>
 #include <stdlib.h>
 #include <string.h>
@@ -234,9 +233,8 @@ EmitCoordinates(const struct vcc *tl, struct vsb *vsb)
 	VTAILQ_FOREACH(sp, &tl->sources, list) {
 		VSB_printf(vsb, "    /* ");
 		VSB_quote(vsb, sp->name, -1, VSB_QUOTE_CSTR);
-		VSB_printf(vsb, "*/\n");
-		VSB_printf(vsb, "\t");
-		VSB_quote(vsb, sp->b, sp->e - sp->b, VSB_QUOTE_CSTR);
+		VSB_printf(vsb, " */\n");
+		VSB_quote_pfx(vsb, "\t", sp->b, sp->e - sp->b, VSB_QUOTE_CSTR);
 		VSB_printf(vsb, ",\n");
 	}
 	VSB_printf(vsb, "};\n\n");
@@ -295,6 +293,7 @@ EmitInitFini(const struct vcc *tl)
 {
 	struct inifin *p, *q = NULL;
 	unsigned has_event = 0;
+	struct symbol *sy;
 
 	Fh(tl, 0, "\n");
 	Fh(tl, 0, "static unsigned vgc_inistep;\n");
@@ -320,6 +319,20 @@ EmitInitFini(const struct vcc *tl)
 			has_event = 1;
 	}
 
+	/* Handle failures from vcl_init */
+	Fc(tl, 0, "\n");
+	Fc(tl, 0, "\tif (*ctx->handling != VCL_RET_OK)\n");
+	Fc(tl, 0, "\t\treturn(1);\n");
+
+	VTAILQ_FOREACH(sy, &tl->sym_objects, sideways) {
+		Fc(tl, 0, "\tif (!%s) {\n", sy->rname);
+		Fc(tl, 0, "\t\t*ctx->handling = 0;\n");
+		Fc(tl, 0, "\t\tVRT_fail(ctx, "
+		    "\"Object %s not initialized\");\n" , sy->name);
+		Fc(tl, 0, "\t\treturn(1);\n");
+		Fc(tl, 0, "\t}\n");
+	}
+
 	Fc(tl, 0, "\treturn(0);\n");
 	Fc(tl, 0, "}\n");
 
@@ -580,6 +593,8 @@ vcc_CompileSource(struct vcc *tl, struct source *sp)
 	Fh(tl, 0, "/* ---===### VCC generated .h code ###===---*/\n");
 	Fc(tl, 0, "\n/* ---===### VCC generated .c code ###===---*/\n");
 
+	Fc(tl, 0, "\n#define END_ if (*ctx->handling) return\n");
+
 	vcc_Parse_Init(tl);
 
 	vcc_Expr_Init(tl);
@@ -647,7 +662,12 @@ vcc_CompileSource(struct vcc *tl, struct source *sp)
 
 	/* Tie vcl_init/fini in */
 	ifp = New_IniFin(tl);
-	VSB_printf(ifp->ini, "\tVGC_function_vcl_init(ctx);");
+	VSB_printf(ifp->ini, "\tVGC_function_vcl_init(ctx);\n");
+	/*
+	 * We do not return(1) if this fails, because the failure
+	 * could be half way into vcl_init{} so vcl_fini{} must
+	 * always be called, also on failure.
+	 */
 	VSB_printf(ifp->fin, "\t\tVGC_function_vcl_fini(ctx);");
 
 	/* Emit method functions */
@@ -744,6 +764,7 @@ VCC_New(void)
 	VTAILQ_INIT(&tl->tokens);
 	VTAILQ_INIT(&tl->sources);
 	VTAILQ_INIT(&tl->procs);
+	VTAILQ_INIT(&tl->sym_objects);
 
 	tl->nsources = 0;
 
