@@ -374,10 +374,10 @@ local int deflateStateCheck (strm)
 #ifdef GZIP
                                            s->status != GZIP_STATE &&
 #endif
-                                           s->status != EXTRA_STATE &&
-                                           s->status != NAME_STATE &&
-                                           s->status != COMMENT_STATE &&
-                                           s->status != HCRC_STATE &&
+                                           // s->status != EXTRA_STATE &&
+                                           // s->status != NAME_STATE &&
+                                           // s->status != COMMENT_STATE &&
+                                           // s->status != HCRC_STATE &&
                                            s->status != BUSY_STATE &&
                                            s->status != FINISH_STATE))
         return 1;
@@ -511,7 +511,7 @@ int ZEXPORT deflateResetKeep (strm)
         s->wrap == 2 ? crc32(0L, Z_NULL, 0) :
 #endif
         adler32(0L, Z_NULL, 0);
-    s->last_flush = Z_NO_FLUSH;
+    s->last_flush = -2;
 
     _tr_init(s);
 
@@ -606,12 +606,12 @@ int ZEXPORT deflateParams(strm, level, strategy)
     func = configuration_table[s->level].func;
 
     if ((strategy != s->strategy || func != configuration_table[level].func) &&
-        s->high_water) {
+        s->last_flush != -2) {
         /* Flush the last buffer: */
         int err = deflate(strm, Z_BLOCK);
         if (err == Z_STREAM_ERROR)
             return err;
-        if (strm->avail_out == 0)
+        if (strm->avail_in || (s->strstart - s->block_start) + s->lookahead)
             return Z_BUF_ERROR;
     }
     if (s->level != level) {
@@ -833,6 +833,8 @@ int ZEXPORT deflate (strm, flush)
 
     /* Write the header */
     if (s->status == INIT_STATE) {
+#ifdef NOVGZ
+	abort();
         /* zlib header */
         uInt header = (Z_DEFLATED + ((s->w_bits-8)<<4)) << 8;
         uInt level_flags;
@@ -865,6 +867,7 @@ int ZEXPORT deflate (strm, flush)
             s->last_flush = -1;
             return Z_OK;
         }
+#endif
     }
 #ifdef GZIP
     if (s->status == GZIP_STATE) {
@@ -893,6 +896,10 @@ int ZEXPORT deflate (strm, flush)
             }
         }
         else {
+		abort();
+	}
+    }
+#ifdef NOVGZ
             put_byte(s, (s->gzhead->text ? 1 : 0) +
                      (s->gzhead->hcrc ? 2 : 0) +
                      (s->gzhead->extra == Z_NULL ? 0 : 4) +
@@ -1010,6 +1017,7 @@ int ZEXPORT deflate (strm, flush)
             return Z_OK;
         }
     }
+#endif
 #endif
 
     if (strm->start_bit == 0)
