@@ -27,7 +27,6 @@
  */
 
 #include "config.h"
-#include <errno.h>
 
 #include "vdef.h"
 #include "vrt.h"
@@ -40,19 +39,19 @@
 size_t
 base64nopad_encode_l(size_t l)
 {
-	return base64_l(l) + 4;
+	return (base64_l(l) + 4);
 }
 
 size_t
 base64_encode_l(size_t l)
 {
-	return (((base64_l(l)) + 3) & ~3) + 1;
+	return ((((base64_l(l)) + 3) & ~3) + 1);
 }
 
 size_t
 base64_decode_l(size_t l)
 {
-	return ((l) * 3) >> 2;
+	return (((l) * 3) >> 2);
 }
 
 static inline int
@@ -63,19 +62,19 @@ decode(char *restrict *restrict dest, const char *restrict const buf,
 
 	if (n <= 1) {
 		errno = EINVAL;
-		return -1;
+		return (-1);
 	}
 	d = *dest;
 	for (int i = 0; i < n - 1; i++) {
 		if (d == buf + buflen) {
 			errno = ENOMEM;
-			return -1;
+			return (-1);
 		}
 		*d++ = (u >> 16) & 0xff;
 		u <<= 8;
 	}
 	*dest += d - *dest;
-	return 1;
+	return (1);
 }
 
 ssize_t
@@ -92,14 +91,14 @@ base64_encode(const enum encoding enc, const enum case_e kase,
 	AN(buf);
 	AN(alpha);
 	if (in == NULL || inlength == 0)
-		return 0;
+		return (0);
 
 	if ((enc == BASE64URLNOPAD &&
 	     buflen < base64nopad_encode_l(inlength)) ||
 	    (enc != BASE64URLNOPAD &&
 	     buflen < base64_encode_l(inlength))) {
 		errno = ENOMEM;
-		return -1;
+		return (-1);
 	}
 
 	while (end - in >= 3) {
@@ -127,7 +126,7 @@ base64_encode(const enum encoding enc, const enum case_e kase,
 		}
 	}
 	assert(p >= buf && p - buf <= buflen);
-	return p - buf;
+	return (p - buf);
 }
 
 ssize_t
@@ -140,6 +139,7 @@ base64_decode(const enum encoding dec, char *restrict const buf,
 	unsigned u = 0, term = 0;
 	int n = 0;
 	size_t len = SIZE_MAX;
+	const char *s;
 
 	AN(buf);
 	AN(alpha);
@@ -149,28 +149,28 @@ base64_decode(const enum encoding dec, char *restrict const buf,
 		len = inlen;
 
 	for (int i = 0; len > 0 && i < strings->n; i++) {
-		const char *s = strings->p[i];
+		s = strings->p[i];
 
 		if (s == NULL)
 			continue;
 		if (*s && term) {
 			errno = EINVAL;
-			return -1;
+			return (-1);
 		}
 		while (*s && len) {
 			while (n < 4) {
-				char b = alpha->i64[(unsigned) *s++];
+				char b = alpha->i64[(uint8_t) *s++];
 				u <<= 6;
 				if (b == ILL) {
 					errno = EINVAL;
-					return -1;
+					return (-1);
 				}
 				n++;
 				if (b == PAD) {
 					term++;
 					continue;
 				}
-				u |= (unsigned) b;
+				u |= (uint8_t) b;
 				if (--len == 0)
 					break;
 				if (!*s)
@@ -178,7 +178,7 @@ base64_decode(const enum encoding dec, char *restrict const buf,
 			}
 			if (n == 4) {
 				if (decode(&dest, buf, buflen, u, n-term) < 0)
-					return -1;
+					return (-1);
 				n = 0;
 			}
 		}
@@ -187,8 +187,8 @@ base64_decode(const enum encoding dec, char *restrict const buf,
 		if (!alpha->padding)
 			u <<= (6 * (4 - n));
 		if (decode(&dest, buf, buflen, u, n-term) < 0)
-			return -1;
+			return (-1);
 	}
 
-	return dest - buf;
+	return (dest - buf);
 }
