@@ -28,7 +28,6 @@
 
 #include "config.h"
 
-#include <errno.h>
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
@@ -57,6 +56,7 @@ static VCL_DURATION vcl_release_delay = 0.0;
 static pthread_mutex_t vsc_mtx = PTHREAD_MUTEX_INITIALIZER;
 static struct vsc_seg *vsc_seg = NULL;
 static struct VSC_debug *vsc = NULL;
+static int loads;
 
 /**********************************************************************/
 
@@ -95,16 +95,16 @@ xyzzy_author(VRT_CTX, VCL_ENUM person, VCL_ENUM someone)
 	(void)someone;
 
 	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);
-	if (person == xyzzy_enum_phk)
+	if (person == VENUM(phk))
 		return ("Poul-Henning");
 	assert(strcmp(person, "phk"));
-	if (person == xyzzy_enum_des)
+	if (person == VENUM(des))
 		return ("Dag-Erling");
 	assert(strcmp(person, "des"));
-	if (person == xyzzy_enum_kristian)
+	if (person == VENUM(kristian))
 		return ("Kristian");
 	assert(strcmp(person, "kristian"));
-	if (person == xyzzy_enum_mithrandir)
+	if (person == VENUM(mithrandir))
 		return ("Tollef");
 	assert(strcmp(person, "mithrandir"));
 	WRONG("Illegal VMOD enum");
@@ -148,7 +148,7 @@ xyzzy_test_priv_task(VRT_CTX, struct vmod_priv *priv, VCL_STRING s)
 		char *n = realloc(priv->priv,
 		    strlen(priv->priv) + strlen(s) + 2);
 		if (n == NULL)
-			return NULL;
+			return (NULL);
 		strcat(n, " ");
 		strcat(n, s);
 		priv->priv = n;
@@ -195,7 +195,7 @@ xyzzy_rot52(VRT_CTX, VCL_HTTP hp)
 }
 
 VCL_STRING v_matchproto_(td_debug_argtest)
-xyzzy_argtest(VRT_CTX, struct xyzzy_argtest_arg *arg)
+xyzzy_argtest(VRT_CTX, struct VARGS(argtest) *arg)
 {
 	char buf[100];
 
@@ -253,6 +253,14 @@ xyzzy_fail(VRT_CTX)
 	VRT_fail(ctx, "Forced failure");
 }
 
+VCL_BOOL v_matchproto_(td_debug_fail2)
+xyzzy_fail2(VRT_CTX)
+{
+
+	VRT_fail(ctx, "Forced failure");
+	return (1);
+}
+
 static void v_matchproto_(vmod_priv_free_f)
 priv_vcl_free(void *priv)
 {
@@ -277,6 +285,9 @@ event_load(VRT_CTX, struct vmod_priv *priv)
 	struct priv_vcl *priv_vcl;
 
 	AN(ctx->msg);
+
+	loads++;
+
 	if (cache_param->nuke_limit == 42) {
 		VSB_printf(ctx->msg, "nuke_limit is not the answer.");
 		return (-1);
@@ -363,19 +374,38 @@ event_cold(VRT_CTX, const struct vmod_priv *priv)
 	return (0);
 }
 
+static int
+event_discard(VRT_CTX, void *priv)
+{
+
+	(void)priv;
+
+	VRT_RemoveVFP(ctx, &xyzzy_rot13);
+
+	if (--loads)
+		return (0);
+
+	/*
+	 * The vsc and vsc_seg variables are not per-VCL, they are
+	 * the same in all VCL's which import the same binary version
+	 * of this VMOD, so we should only carry out cleanup on the
+	 * last discard event.
+	 */
+	if (vsc)
+		VSC_debug_Destroy(&vsc_seg);
+
+	return (0);
+}
+
 int v_matchproto_(vmod_event_f)
-event_function(VRT_CTX, struct vmod_priv *priv, enum vcl_event_e e)
+xyzzy_event_function(VRT_CTX, struct vmod_priv *priv, enum vcl_event_e e)
 {
 
 	switch (e) {
-	case VCL_EVENT_LOAD: return (event_load(ctx, priv));
-	case VCL_EVENT_WARM: return (event_warm(ctx, priv));
-	case VCL_EVENT_COLD: return (event_cold(ctx, priv));
-	case VCL_EVENT_DISCARD:
-		VRT_RemoveVFP(ctx, &xyzzy_rot13);
-		if (vsc)
-			VSC_debug_Destroy(&vsc_seg);
-		return (0);
+	case VCL_EVENT_LOAD:	return (event_load(ctx, priv));
+	case VCL_EVENT_WARM:	return (event_warm(ctx, priv));
+	case VCL_EVENT_COLD:	return (event_cold(ctx, priv));
+	case VCL_EVENT_DISCARD:	return (event_discard(ctx, priv));
 	default: return (0);
 	}
 }
@@ -552,3 +582,140 @@ xyzzy_sethdr(VRT_CTX, VCL_HEADER hs, VCL_STRANDS s)
 		}
 	}
 }
+
+VCL_DURATION
+xyzzy_priv_perf(VRT_CTX, VCL_INT size, VCL_INT rounds)
+{
+	vtim_mono t0, t1;
+	vtim_dur d;
+	struct vmod_priv *p;
+	VCL_INT s, r;
+	uintptr_t check = 0;
+
+	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);
+
+	for (s = 1; s <= size; s++) {
+		p = VRT_priv_task(ctx, (void *)(uintptr_t)s);
+		if (p == NULL) {
+			VRT_fail(ctx, "no priv task - out of ws?");
+			return (-1.0);
+		}
+		p->priv = NULL;
+	}
+
+	t0 = VTIM_mono();
+	for (r = 0; r < rounds; r++) {
+		for (s = 1; s <= size; s++) {
+			p = VRT_priv_task(ctx, (void *)(uintptr_t)s);
+			AN(p);
+			check += (uintptr_t)p->priv;
+			p->priv = (void *)(uintptr_t)(s * rounds + r);
+		}
+	}
+	t1 = VTIM_mono();
+
+	d = (t1 - t0) * 1e9 / ((double)size * (double)rounds);
+
+	VSLb(ctx->vsl, SLT_Debug,
+	     "perf size %jd rounds %jd time %.1fns check %jd",
+	     (intmax_t)size, (intmax_t)rounds, d, (uintmax_t)check);
+
+	return (d);
+}
+
+VCL_STRANDS
+xyzzy_return_strands(VRT_CTX, VCL_STRANDS strand)
+{
+
+	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);
+	return (strand);
+}
+
+/*---------------------------------------------------------------------*/
+
+static const struct vcf_return * v_matchproto_(vcf_func_f)
+xyzzy_catflap_simple(struct req *req, struct objcore **oc,
+    struct objcore **oc_exp, int state)
+{
+
+	CHECK_OBJ_NOTNULL(req, REQ_MAGIC);
+	CHECK_OBJ_NOTNULL(req->vcf, VCF_MAGIC);
+	assert(req->vcf->func == xyzzy_catflap_simple);
+
+	(void)oc;
+	(void)oc_exp;
+	if (state == 0) {
+		if (req->vcf->priv == VENUM(first))
+			return (VCF_HIT);
+		if (req->vcf->priv == VENUM(miss))
+			return (VCF_MISS);
+		WRONG("Shouldn't get here");
+	}
+	return (VCF_DEFAULT);
+}
+
+static const struct vcf_return * v_matchproto_(vcf_func_f)
+xyzzy_catflap_last(struct req *req, struct objcore **oc,
+    struct objcore **oc_exp, int state)
+{
+
+	CHECK_OBJ_NOTNULL(req, REQ_MAGIC);
+	CHECK_OBJ_NOTNULL(req->vcf, VCF_MAGIC);
+	assert(req->vcf->func == xyzzy_catflap_last);
+
+	(void)oc_exp;
+	if (state == 0) {
+		AN(oc);
+		CHECK_OBJ_NOTNULL(*oc, OBJCORE_MAGIC);
+		req->vcf->priv = *oc;
+		return (VCF_CONTINUE);
+	}
+	if (state == 1) {
+		AN(oc);
+		if (req->vcf->priv != NULL)
+			CAST_OBJ_NOTNULL(*oc, req->vcf->priv, OBJCORE_MAGIC);
+		return (VCF_CONTINUE);
+	}
+	return (VCF_DEFAULT);
+}
+
+VCL_VOID
+xyzzy_catflap(VRT_CTX, VCL_ENUM type)
+{
+	struct req *req;
+
+	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);
+	req = ctx->req;
+	CHECK_OBJ_NOTNULL(req, REQ_MAGIC);
+	XXXAZ(req->vcf);
+	req->vcf = WS_Alloc(req->ws, sizeof *req->vcf);
+	AN(req->vcf);
+	INIT_OBJ(req->vcf, VCF_MAGIC);
+	if (type == VENUM(first) || type == VENUM(miss)) {
+		req->vcf->func = xyzzy_catflap_simple;
+		req->vcf->priv = TRUST_ME(type);
+	} else if (type == VENUM(last)) {
+		req->vcf->func = xyzzy_catflap_last;
+	} else {
+		WRONG("Wrong VENUM");
+	}
+}
+
+VCL_BYTES
+xyzzy_stk(VRT_CTX)
+{
+	const VCL_BYTES max = 100 * 1024 * 1024;
+	const char *a, *b;
+	VCL_BYTES r;
+
+	a = TRUST_ME(&b);
+	b = TRUST_ME(ctx->req->wrk);
+	b += sizeof(*ctx->req->wrk);
+
+	if (b > a && (r = b - a) < max)
+		return (r);
+	if (a > b && (r = a - b) < max)
+		return (r);
+
+	return (0);
+}
