@@ -68,7 +68,6 @@ xyzzy_obj__fini(struct xyzzy_debug_obj **op)
 	AN(op);
 	AN(*op);
 	FREE_OBJ(*op);
-	*op = NULL;
 }
 
 VCL_VOID v_matchproto_()
@@ -152,3 +151,95 @@ xyzzy_obj_test_priv_top(VRT_CTX,
 	(void)o;
 	return (xyzzy_test_priv_top(ctx, priv, s));
 }
+
+/* ----------------------------------------------------------------------------
+ * obj_opt (optional arguments and privs)
+ */
+struct xyzzy_debug_obj_opt {
+	unsigned			magic;
+#define VMOD_DEBUG_OBJ_OPT_MAGIC	0xccbd9b78
+	char				*name;
+	struct VARGS(obj_opt_meth_opt)	args;
+	void				*freeptr;
+};
+
+VCL_VOID v_matchproto_()
+xyzzy_obj_opt__init(VRT_CTX,
+    struct xyzzy_debug_obj_opt **op, const char *vcl_name,
+    struct VARGS(obj_opt__init) *args)
+{
+	struct xyzzy_debug_obj_opt *o;
+
+	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);
+	AN(args);
+
+	AN(args->arg1); // priv_call
+	AN(args->arg2); // priv_vcl
+	AN(args->arg3); // priv_task
+	assert(args->arg1 != args->arg2);
+	assert(args->arg2 != args->arg3);
+
+	if (args->valid_s)
+		AN(args->s);
+	(void)args->valid_b;
+	(void)args->b;
+
+	AN(op);
+	AZ(*op);
+	ALLOC_OBJ(o, VMOD_DEBUG_OBJ_OPT_MAGIC);
+	AN(o);
+	*op = o;
+	REPLACE(o->name, vcl_name);
+	memcpy(&o->args, args, sizeof o->args);
+	if (args->valid_s) {
+		REPLACE(o->freeptr, args->s);
+		o->args.s = o->freeptr;
+	}
+}
+
+VCL_VOID v_matchproto_()
+xyzzy_obj_opt__fini(struct xyzzy_debug_obj_opt **op)
+{
+	struct xyzzy_debug_obj_opt *o;
+
+	AN(op);
+	AN(*op);
+
+	TAKE_OBJ_NOTNULL(o, op, VMOD_DEBUG_OBJ_OPT_MAGIC);
+
+	REPLACE(o->name, NULL);
+	if (o->freeptr) {
+		AN(o->args.valid_s);
+		REPLACE(o->freeptr, NULL);
+	}
+	FREE_OBJ(o);
+	AZ(o);
+}
+
+VCL_STRING v_matchproto_()
+xyzzy_obj_opt_meth_opt(VRT_CTX,
+    struct xyzzy_debug_obj_opt *o,
+    struct VARGS(obj_opt_meth_opt) *args)
+{
+	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);
+	CHECK_OBJ_NOTNULL(o, VMOD_DEBUG_OBJ_OPT_MAGIC);
+
+	AN(args);
+	AN(args->arg1); // priv_call
+	AN(args->arg2); // priv_vcl
+	AN(args->arg3); // priv_task
+	assert(args->arg1 != args->arg2);
+	assert(args->arg2 != args->arg3);
+
+	return (VRT_CollectString(ctx,
+	    "obj ", o->name,
+	    " obj_s ", (o->args.valid_s ? o->args.s : "*undef*"),
+	    " obj_b ", (o->args.valid_b
+			? (o->args.b ? "true" : "false" )
+			: "*undef*"),
+	    " met_s ", (args->valid_s ? args->s : "*undef*"),
+	    " met_b ", (args->valid_b
+			? (args->b ? "true" : "false" )
+			: "*undef*"),
+	    vrt_magic_string_end));
+}
