@@ -36,6 +36,8 @@
 #include "vcc_if.h"
 
 #include "vdir.h"
+#include "vsb.h"
+#include "vbm.h"
 
 struct vmod_directors_fallback {
 	unsigned				magic;
@@ -56,6 +58,93 @@ vmod_fallback_healthy(VRT_CTX, VCL_BACKEND dir, VCL_TIME *changed)
 	return (vdir_any_healthy(ctx, fb->vd, changed));
 }
 
+static void v_matchproto_(vdi_list_f)
+vmod_fallback_list(VRT_CTX, VCL_BACKEND dir, struct vsb *vsb, int pflag,
+    int jflag)
+{
+	struct vmod_directors_fallback *fb;
+	struct vdir *vd;
+	VCL_BACKEND be;
+	VCL_BOOL h;
+	unsigned u, nh;
+
+	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);
+	CHECK_OBJ_NOTNULL(dir, DIRECTOR_MAGIC);
+	CAST_OBJ_NOTNULL(fb, dir->priv, VMOD_DIRECTORS_FALLBACK_MAGIC);
+	CAST_OBJ_NOTNULL(vd, fb->vd, VDIR_MAGIC);
+
+	if (pflag) {
+		if (jflag) {
+			VSB_cat(vsb, "{\n");
+			VSB_indent(vsb, 2);
+			VSB_printf(vsb, "\"sticky\": %s,\n",
+			    fb->st ? "true" : "false");
+			VSB_cat(vsb, "\"backends\": {\n");
+			VSB_indent(vsb, 2);
+		} else {
+			VSB_cat(vsb, "\n\n\tBackend\tCurrent\tHealth\n");
+		}
+	}
+
+	vdir_rdlock(vd);
+	vdir_update_health(ctx, vd);
+	for (u = 0; pflag && u < vd->n_backend; u++) {
+		be = vd->backend[u];
+		CHECK_OBJ_NOTNULL(be, DIRECTOR_MAGIC);
+
+		h = vbit_test(vd->healthy, u);
+
+		if (jflag) {
+			if (u)
+				VSB_cat(vsb, ",\n");
+			VSB_printf(vsb, "\"%s\": {\n", be->vcl_name);
+			VSB_indent(vsb, 2);
+
+			if (fb->cur == u)
+				VSB_cat(vsb, "\"current\": true,\n");
+			else
+				VSB_cat(vsb, "\"current\": false,\n");
+
+			if (h)
+				VSB_cat(vsb, "\"health\": \"healthy\"\n");
+			else
+				VSB_cat(vsb, "\"health\": \"sick\"\n");
+
+			VSB_indent(vsb, -2);
+			VSB_cat(vsb, "}");
+		} else {
+			VSB_cat(vsb, "\t");
+			VSB_cat(vsb, be->vcl_name);
+			if (fb->cur == u)
+				VSB_cat(vsb, "\t*\t");
+			else
+				VSB_cat(vsb, "\t\t");
+			VSB_cat(vsb, h ? "healthy" : "sick");
+			VSB_cat(vsb, "\n");
+		}
+	}
+	nh = vd->n_healthy;
+	u = vd->n_backend;
+	vdir_unlock(vd);
+
+	if (jflag && (pflag)) {
+		VSB_cat(vsb, "\n");
+		VSB_indent(vsb, -2);
+		VSB_cat(vsb, "}\n");
+		VSB_indent(vsb, -2);
+		VSB_cat(vsb, "},\n");
+	}
+
+	if (pflag)
+		return;
+
+	if (jflag)
+		VSB_printf(vsb, "[%u, %u, \"%s\"]", nh, u,
+		    nh ? "healthy" : "sick");
+	else
+		VSB_printf(vsb, "%u/%u\t%s", nh, u, nh ? "healthy" : "sick");
+}
+
 static VCL_BACKEND v_matchproto_(vdi_resolve_f)
 vmod_fallback_resolve(VRT_CTX, VCL_BACKEND dir)
 {
@@ -99,7 +188,8 @@ static const struct vdi_methods vmod_fallback_methods[1] = {{
 	.type =			"fallback",
 	.healthy =		vmod_fallback_healthy,
 	.resolve =		vmod_fallback_resolve,
-	.destroy =		vmod_fallback_destroy
+	.destroy =		vmod_fallback_destroy,
+	.list =			vmod_fallback_list
 }};
 
 
