@@ -29,11 +29,10 @@
 #include "config.h"
 
 #include <stdlib.h>
+#include <string.h>
 
 #include "cache/cache.h"
-#include "cache/cache_director.h"
 
-#include "vrt.h"
 #include "vend.h"
 #include "vsha256.h"
 
@@ -47,7 +46,24 @@ struct vmod_directors_hash {
 	struct vdir				*vd;
 };
 
-VCL_VOID __match_proto__()
+static void v_matchproto_(vdi_destroy_f)
+vmod_hash_destroy(VCL_BACKEND dir)
+{
+	struct vmod_directors_hash *rr;
+
+	CAST_OBJ_NOTNULL(rr, dir->priv, VMOD_DIRECTORS_HASH_MAGIC);
+	vdir_delete(&rr->vd);
+	FREE_OBJ(rr);
+}
+
+static const struct vdi_methods vmod_hash_methods[1] = {{
+	.magic =		VDI_METHODS_MAGIC,
+	.type =			"hash",
+	.destroy =		vmod_hash_destroy
+}};
+
+
+VCL_VOID v_matchproto_()
 vmod_hash__init(VRT_CTX, struct vmod_directors_hash **rrp,
     const char *vcl_name)
 {
@@ -59,49 +75,50 @@ vmod_hash__init(VRT_CTX, struct vmod_directors_hash **rrp,
 	ALLOC_OBJ(rr, VMOD_DIRECTORS_HASH_MAGIC);
 	AN(rr);
 	*rrp = rr;
-	vdir_new(&rr->vd, "hash", vcl_name, NULL, NULL, rr);
+	vdir_new(ctx, &rr->vd, vcl_name, vmod_hash_methods, rr);
 }
 
-VCL_VOID __match_proto__()
+VCL_VOID v_matchproto_()
 vmod_hash__fini(struct vmod_directors_hash **rrp)
 {
 	struct vmod_directors_hash *rr;
 
-	rr = *rrp;
-	*rrp = NULL;
-	CHECK_OBJ_NOTNULL(rr, VMOD_DIRECTORS_HASH_MAGIC);
-	vdir_delete(&rr->vd);
-	FREE_OBJ(rr);
+	// XXX 2297
+	if (*rrp == NULL)
+		return;
+
+	TAKE_OBJ_NOTNULL(rr, rrp, VMOD_DIRECTORS_HASH_MAGIC);
+	VRT_DelDirector(&rr->vd->dir);
 }
 
-VCL_VOID __match_proto__()
+VCL_VOID v_matchproto_()
 vmod_hash_add_backend(VRT_CTX,
     struct vmod_directors_hash *rr, VCL_BACKEND be, double w)
 {
 
 	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);
 	CHECK_OBJ_NOTNULL(rr, VMOD_DIRECTORS_HASH_MAGIC);
-	(void)vdir_add_backend(rr->vd, be, w);
+	vdir_add_backend(ctx, rr->vd, be, w);
 }
 
-VCL_VOID __match_proto__()
+VCL_VOID v_matchproto_()
 vmod_hash_remove_backend(VRT_CTX,
     struct vmod_directors_hash *rr, VCL_BACKEND be)
 {
 
 	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);
 	CHECK_OBJ_NOTNULL(rr, VMOD_DIRECTORS_HASH_MAGIC);
-	vdir_remove_backend(rr->vd, be, NULL);
+	vdir_remove_backend(ctx, rr->vd, be, NULL);
 }
 
-VCL_BACKEND __match_proto__()
+VCL_BACKEND v_matchproto_()
 vmod_hash_backend(VRT_CTX, struct vmod_directors_hash *rr,
     const char *arg, ...)
 {
-	struct SHA256Context sha_ctx;
+	struct VSHA256Context sha_ctx;
 	va_list ap;
 	const char *p;
-	unsigned char sha256[SHA256_LEN];
+	unsigned char sha256[VSHA256_LEN];
 	VCL_BACKEND be;
 	double r;
 
@@ -109,20 +126,20 @@ vmod_hash_backend(VRT_CTX, struct vmod_directors_hash *rr,
 	CHECK_OBJ_ORNULL(ctx->bo, BUSYOBJ_MAGIC);
 
 	CHECK_OBJ_NOTNULL(rr, VMOD_DIRECTORS_HASH_MAGIC);
-	SHA256_Init(&sha_ctx);
+	VSHA256_Init(&sha_ctx);
 	va_start(ap, arg);
 	p = arg;
 	while (p != vrt_magic_string_end) {
 		if (p != NULL && *p != '\0')
-			SHA256_Update(&sha_ctx, p, strlen(p));
+			VSHA256_Update(&sha_ctx, p, strlen(p));
 		p = va_arg(ap, const char *);
 	}
 	va_end(ap);
-	SHA256_Final(sha256, &sha_ctx);
+	VSHA256_Final(sha256, &sha_ctx);
 
 	r = vbe32dec(sha256);
 	r = scalbn(r, -32);
 	assert(r >= 0 && r <= 1.0);
-	be = vdir_pick_be(rr->vd, r, ctx->bo);
+	be = vdir_pick_be(ctx, rr->vd, r);
 	return (be);
 }
