@@ -40,6 +40,7 @@
 #include "vcc_if.h"
 #include "shard_dir.h"
 #include "shard_cfg.h"
+#include "vsb.h"
 
 /* -------------------------------------------------------------------------
  *  shard director: LAZY mode (vdi resolve function), parameter objects
@@ -113,6 +114,8 @@ enum vmod_directors_shard_param_scope {
 
 struct vmod_directors_shard_param;
 
+#define VMOD_SHARD_SHARD_PARAM_BLOB		0xdf5ca116
+
 struct vmod_directors_shard_param {
 	unsigned				magic;
 #define VMOD_SHARD_SHARD_PARAM_MAGIC		0xdf5ca117
@@ -157,7 +160,7 @@ shard_param_task(VRT_CTX, const void *id,
     const struct vmod_directors_shard_param *pa);
 
 static const struct vmod_directors_shard_param *
-shard_param_blob(const VCL_BLOB blob);
+shard_param_blob(VCL_BLOB blob);
 
 static const struct vmod_directors_shard_param *
 vmod_shard_param_read(VRT_CTX, const void *id,
@@ -169,6 +172,7 @@ vmod_shard_param_read(VRT_CTX, const void *id,
  */
 static vdi_healthy_f vmod_shard_healthy;
 static vdi_resolve_f vmod_shard_resolve;
+static vdi_list_f vmod_shard_list;
 
 struct vmod_directors_shard {
 	unsigned				magic;
@@ -180,7 +184,7 @@ struct vmod_directors_shard {
 static enum by_e
 parse_by_e(VCL_ENUM e)
 {
-#define VMODENUM(n) if (e == vmod_enum_ ## n) return(BY_ ## n);
+#define VMODENUM(n) if (e == VENUM(n)) return(BY_ ## n);
 #include "tbl_by.h"
        WRONG("illegal by enum");
 }
@@ -188,7 +192,7 @@ parse_by_e(VCL_ENUM e)
 static enum healthy_e
 parse_healthy_e(VCL_ENUM e)
 {
-#define VMODENUM(n) if (e == vmod_enum_ ## n) return(n);
+#define VMODENUM(n) if (e == VENUM(n)) return(n);
 #include "tbl_healthy.h"
        WRONG("illegal healthy enum");
 }
@@ -196,7 +200,7 @@ parse_healthy_e(VCL_ENUM e)
 static enum resolve_e
 parse_resolve_e(VCL_ENUM e)
 {
-#define VMODENUM(n) if (e == vmod_enum_ ## n) return(n);
+#define VMODENUM(n) if (e == VENUM(n)) return(n);
 #include "tbl_resolve.h"
        WRONG("illegal resolve enum");
 }
@@ -241,7 +245,8 @@ static const struct vdi_methods vmod_shard_methods[1] = {{
 	.type =		"shard",
 	.resolve =	vmod_shard_resolve,
 	.healthy =	vmod_shard_healthy,
-	.destroy =	vmod_shard_destroy
+	.destroy =	vmod_shard_destroy,
+	.list =		vmod_shard_list
 }};
 
 
@@ -341,7 +346,7 @@ vmod_shard_associate(VRT_CTX,
 
 VCL_BOOL v_matchproto_(td_directors_shard_add_backend)
 vmod_shard_add_backend(VRT_CTX, struct vmod_directors_shard *vshard,
-    struct vmod_shard_add_backend_arg *args)
+    struct VARGS(shard_add_backend) *args)
 {
 	CHECK_OBJ_NOTNULL(vshard, VMOD_SHARD_SHARD_MAGIC);
 
@@ -359,7 +364,7 @@ vmod_shard_add_backend(VRT_CTX, struct vmod_directors_shard *vshard,
 
 VCL_BOOL v_matchproto_(td_directors_shard_remove_backend)
 vmod_shard_remove_backend(VRT_CTX, struct vmod_directors_shard *vshard,
-    struct vmod_shard_remove_backend_arg *args)
+    struct VARGS(shard_remove_backend) *args)
 {
 	VCL_BACKEND be = args->valid_backend ? args->backend : NULL;
 	VCL_STRING ident = args->valid_ident ? args->ident : NULL;
@@ -400,7 +405,7 @@ shard_get_key(VRT_CTX, const struct vmod_directors_shard_param *p)
 	switch (p->by) {
 	case BY_HASH:
 		if (ctx->bo) {
-			CHECK_OBJ_NOTNULL(ctx->bo, BUSYOBJ_MAGIC);
+			CHECK_OBJ(ctx->bo, BUSYOBJ_MAGIC);
 			return (vbe32dec(ctx->bo->digest));
 		}
 		/* FALLTHROUGH */
@@ -469,19 +474,19 @@ static uint32_t
 shard_blob_key(VCL_BLOB key_blob)
 {
 	uint8_t k[4] = { 0 };
-	uint8_t *b;
+	const uint8_t *b;
 	int i, ki;
 
-	assert(key_blob);
+	AN(key_blob);
+	AN(key_blob->blob);
 	assert(key_blob->len > 0);
-	assert(key_blob->priv != NULL);
 
 	if (key_blob->len >= 4)
 		ki = 0;
 	else
 		ki = 4 - key_blob->len;
 
-	b = key_blob->priv;
+	b = key_blob->blob;
 	for (i = 0; ki < 4; i++, ki++)
 		k[ki] = b[i];
 	assert(i <= key_blob->len);
@@ -496,7 +501,7 @@ shard_blob_key(VCL_BLOB key_blob)
 #define tobit(args, name) ((args)->valid_##name ? arg_##name : 0)
 
 static uint32_t
-shard_backend_arg_mask(const struct vmod_shard_backend_arg * const a)
+shard_backend_arg_mask(const struct VARGS(shard_backend) * const a)
 {
 	return (tobit(a, by)		|
 		tobit(a, key)		|
@@ -509,7 +514,7 @@ shard_backend_arg_mask(const struct vmod_shard_backend_arg * const a)
 		tobit(a, resolve));
 }
 static uint32_t
-shard_param_set_mask(const struct vmod_shard_param_set_arg * const a)
+shard_param_set_mask(const struct VARGS(shard_param_set) * const a)
 {
 	return (tobit(a, by)		|
 		tobit(a, key)		|
@@ -569,8 +574,8 @@ shard_param_args(VRT_CTX,
 					 who, p->vcl_name, by_s);
 				return (NULL);
 			}
-			if (key_blob == NULL || key_blob->len <= 0 ||
-			    key_blob->priv == NULL) {
+			if (key_blob == NULL || key_blob->len == 0 ||
+			    key_blob->blob == NULL) {
 				sharddir_err(ctx, SLT_Error, "%s %s: "
 					     "by=BLOB but no or empty key_blob "
 					     "- using key 0",
@@ -638,7 +643,7 @@ shard_param_args(VRT_CTX,
 
 VCL_BACKEND v_matchproto_(td_directors_shard_backend)
 vmod_shard_backend(VRT_CTX, struct vmod_directors_shard *vshard,
-		   struct vmod_shard_backend_arg *a)
+		   struct VARGS(shard_backend) *a)
 {
 	struct vmod_directors_shard_param pstk;
 	struct vmod_directors_shard_param *pp = NULL;
@@ -752,6 +757,111 @@ vmod_shard_resolve(VRT_CTX, VCL_BACKEND dir)
 				 pp->rampup, pp->healthy));
 }
 
+static void v_matchproto_(vdi_list_f)
+vmod_shard_list(VRT_CTX, VCL_BACKEND dir, struct vsb *vsb, int pflag, int jflag)
+{
+	struct sharddir *shardd;
+	struct shard_backend *sbe;
+	VCL_TIME c, changed = 0;
+	VCL_DURATION rampup_d, d;
+	VCL_BACKEND be;
+	VCL_BOOL h;
+	unsigned u, nh = 0;
+	double rampup_p;
+
+	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);
+	CHECK_OBJ_NOTNULL(dir, DIRECTOR_MAGIC);
+	CAST_OBJ_NOTNULL(shardd, dir->priv, SHARDDIR_MAGIC);
+
+	if (pflag) {
+		if (jflag) {
+			VSB_cat(vsb, "{\n");
+			VSB_indent(vsb, 2);
+			VSB_printf(vsb, "\"warmup\": %f,\n", shardd->warmup);
+			VSB_printf(vsb, "\"rampup_duration\": %f,\n",
+			    shardd->rampup_duration);
+			VSB_cat(vsb, "\"backends\": {\n");
+			VSB_indent(vsb, 2);
+		} else {
+			VSB_cat(vsb, "\n\n\tBackend\tIdent\tHealth\t"
+			    "Rampup  Remaining\n");
+		}
+	}
+
+	sharddir_rdlock(shardd);
+	for (u = 0; u < shardd->n_backend; u++) {
+		sbe = &shardd->backend[u];
+		AN(sbe);
+		be = sbe->backend;
+		CHECK_OBJ_NOTNULL(be, DIRECTOR_MAGIC);
+
+		c = 0;
+		h = VRT_Healthy(ctx, be, &c);
+		if (h)
+			nh++;
+		if (c > changed)
+			changed = c;
+		if ((pflag) == 0)
+			continue;
+
+		d = ctx->now - c;
+		rampup_d = shardcfg_get_rampup(shardd, u);
+		if (! h) {
+			rampup_p = 0.0;
+			rampup_d = 0.0;
+		} else if (d < rampup_d) {
+			rampup_p = d / rampup_d;
+			rampup_d -= d;
+		} else {
+			rampup_p = 1.0;
+			rampup_d = 0.0;
+		}
+
+		if (jflag) {
+			if (u)
+				VSB_cat(vsb, ",\n");
+			VSB_printf(vsb, "\"%s\": {\n",
+			    be->vcl_name);
+			VSB_indent(vsb, 2);
+			VSB_printf(vsb, "\"ident\": \"%s\",\n",
+			    sbe->ident ? sbe->ident : be->vcl_name);
+			VSB_printf(vsb, "\"health\": \"%s\",\n",
+			    h ? "healthy" : "sick");
+			VSB_printf(vsb, "\"canon_point\": %u,\n",
+			    sbe->canon_point);
+			VSB_printf(vsb, "\"rampup\": %f,\n", rampup_p);
+			VSB_printf(vsb, "\"rampup_remaining\": %.3f\n",
+			    rampup_d);
+			VSB_indent(vsb, -2);
+			VSB_cat(vsb, "}");
+		} else {
+			VSB_printf(vsb, "\t%s\t%s\t%s\t%6.2f%% %8.3fs\n",
+			    be->vcl_name,
+			    sbe->ident ? sbe->ident : be->vcl_name,
+			    h ? "healthy" : "sick",
+			    rampup_p * 100, rampup_d);
+		}
+	}
+	sharddir_unlock(shardd);
+
+	if (jflag && (pflag)) {
+		VSB_cat(vsb, "\n");
+		VSB_indent(vsb, -2);
+		VSB_cat(vsb, "}\n");
+		VSB_indent(vsb, -2);
+		VSB_cat(vsb, "},\n");
+	}
+
+	if (pflag)
+		return;
+
+	if (jflag)
+		VSB_printf(vsb, "[%u, %u, \"%s\"]", nh, u,
+		    nh ? "healthy" : "sick");
+	else
+		VSB_printf(vsb, "%u/%u\t%s", nh, u, nh ? "healthy" : "sick");
+}
+
 VCL_VOID v_matchproto_(td_directors_shard_backend)
 vmod_shard_debug(VRT_CTX, struct vmod_directors_shard *vshard,
     VCL_INT i)
@@ -882,7 +992,7 @@ shard_param_prep(VRT_CTX, struct vmod_directors_shard_param *p,
 
 VCL_VOID v_matchproto_(td_directors_shard_param_set)
 vmod_shard_param_set(VRT_CTX, struct vmod_directors_shard_param *p,
-		     struct vmod_shard_param_set_arg *a)
+		     struct VARGS(shard_param_set) *a)
 {
 	uint32_t args = shard_param_set_mask(a);
 
@@ -1014,12 +1124,17 @@ vmod_shard_param_get_healthy(VRT_CTX,
 }
 
 static const struct vmod_directors_shard_param *
-shard_param_blob(const VCL_BLOB blob)
+shard_param_blob(VCL_BLOB blob)
 {
-	if (blob && blob->priv &&
-	    blob->len == sizeof(struct vmod_directors_shard_param) &&
-	    *(unsigned *)blob->priv == VMOD_SHARD_SHARD_PARAM_MAGIC)
-		return (blob->priv);
+	const struct vmod_directors_shard_param *p;
+
+	if (blob && blob->type == VMOD_SHARD_SHARD_PARAM_BLOB &&
+	    blob->blob != NULL &&
+	    blob->len == sizeof(struct vmod_directors_shard_param)) {
+		CAST_OBJ_NOTNULL(p, blob->blob, VMOD_SHARD_SHARD_PARAM_MAGIC);
+		return (p);
+	}
+
 	return (NULL);
 }
 
@@ -1027,20 +1142,9 @@ VCL_BLOB v_matchproto_(td_directors_shard_param_use)
 vmod_shard_param_use(VRT_CTX,
     struct vmod_directors_shard_param *p)
 {
-	struct vmod_priv *blob;
-
 	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);
 	CHECK_OBJ_NOTNULL(p, VMOD_SHARD_SHARD_PARAM_MAGIC);
 
-	blob = (void *)WS_Alloc(ctx->ws, sizeof *blob);
-	if (blob == NULL) {
-		VRT_fail(ctx, "Workspace overflow (param.use())");
-		return (NULL);
-	}
-
-	memset(blob, 0, sizeof *blob);
-	blob->len = sizeof *p;
-	blob->priv = p;
-
-	return (blob);
+	return (VRT_blob(ctx, "xshard_param.use()", p, sizeof *p,
+	    VMOD_SHARD_SHARD_PARAM_BLOB));
 }
